@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: December 2019 */
 
 
 /* This file contains debugging functions. */
@@ -37,7 +37,7 @@ static const char *accstrings[] = {
 static const char *ornstrings[] = {
   "fermata", "trill", "trill+sharp", "trill+flat", "trill+natural",
   "tremolo1", "tremolo2", "tremolo3", "mordent", "double mordent",
-  "inverted mordent", "double inverted mordent", "turn", 
+  "inverted mordent", "double inverted mordent", "turn",
   "inverted turn", "arpeggio", "arpeggio + uparrow", "arpeggio + downarrow",
   "spread" };
 
@@ -64,8 +64,12 @@ uschar buff[256];
 va_list ap;
 if (debug_file == NULL) return;
 va_start(ap, format);
+
 format_vsprintf(buff, format, ap);
 fprintf(debug_file, "%s", CS buff);
+
+
+
 fflush(debug_file);
 va_end(ap);
 }
@@ -98,7 +102,7 @@ if (n > main_lastmovement ||
     s > movement[n]->laststave ||
     b > (movement[n]->stavetable[s])->lastbar)
   {
-  debug_printf("Movement %d, stave %d, bar %b does not exist\n", n, s, b); 
+  debug_printf("Movement %d, stave %d, bar %b does not exist\n", n, s, b);
   return;
   }
 
@@ -162,7 +166,7 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
       case b_clef:
       debug_printf("Clef %s\n", clefstrings[((b_clefstr *)p)->trueclef]);
       break;
-      
+
       /* b_chord is with b_note */
 
       case b_comma:
@@ -176,10 +180,10 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
       case b_dotbar:
       debug_printf("Dotbar\n");
       break;
-      
+
       case b_dotright:
       debug_printf("Dotright %f\n", ((b_dotrightstr *)p)->value);
-      break;   
+      break;
 
       case b_draw:
         {
@@ -191,7 +195,7 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
       case b_dynmove:
         {
         b_dynmovestr *pp = (b_dynmovestr *)p;
-        debug_printf("Dyn move %d x=%f y=%f bflags=%d\n", pp->dynamic, pp->x, 
+        debug_printf("Dyn move %d x=%f y=%f bflags=%d\n", pp->dynamic, pp->x,
           pp->y, pp->bflags);
         }
       break;
@@ -217,18 +221,18 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
 
       case b_ensure:
       debug_printf("Ensure %f\n", ((b_ensurestr *)p)->value);
-      break;   
-      
+      break;
+
       case b_footnote:
-      debug_printf("Footnote\n"); 
-      break;  
+      debug_printf("Footnote\n");
+      break;
 
       case b_hairpin:
         {
         b_hairpinstr *pp = (b_hairpinstr *)p;
-        debug_printf("Hairpin %c flags=%d x=%f y=%f su=%f h=%f\n",
+        debug_printf("Hairpin %c flags=%d x=%f y=%f offset=%f su=%f h=%f\n",
          (pp->opt == 0)? '*' : pp->opt, pp->flags,
-            pp->x, pp->y, pp->su, pp->h);
+            pp->x, pp->y, pp->offset, pp->su, pp->h);
         }
       break;
 
@@ -254,10 +258,10 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
         debug_printf("Key %k%s\n", pp->key, pp->warn? "" : " nowarn");
         }
       break;
-      
+
       case b_linegap:
       debug_printf("Linegap\n");
-      break;  
+      break;
 
       case b_lrepeat:
       debug_printf("Left repeat\n");
@@ -302,9 +306,10 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
         debug_printf("%s ", (pp->spitch == 0)? "Rest " :
           (type == b_note)? "Note ":"Chord");
         debug_printf("%s %6d ", ntstrings[pp->notetype], pp->length);
-        debug_printf("%s al=%6f ", accstrings[pp->acc], pp->accleft);
-        debug_printf("p=%3d=%3d ", pp->spitch, pp->truepitch);
-        debug_printf("f=%8x yx=%f\n", pp->flags, pp->yextra);
+        debug_printf("%s ", accstrings[pp->acc]);
+        debug_printf("p=%03d=%03d ", pp->spitch, pp->truepitch);
+        debug_printf("f=%08x yx=%f ", pp->flags, pp->yextra);
+        debug_printf("al=%f\n", pp->accleft);
         }
       break;
 
@@ -351,15 +356,15 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
       case b_ornament:
         {
         b_ornamentstr *pp = (b_ornamentstr *)p;
-        debug_printf("Ornament %s x=%f y=%f bflags=%d\n", 
+        debug_printf("Ornament %s x=%f y=%f bflags=%d\n",
           ornstrings[pp->ornament], pp->x, pp->y, pp->bflags);
         }
       break;
-      
+
       case b_page:
       debug_printf("Page %s%d\n", (((b_pagestr *)p)->relative == 0)? "" : "+",
-        ((b_pagestr *)p)->value);  
-      break;  
+        ((b_pagestr *)p)->value);
+      break;
 
       case b_pagebots:
       debug_printf("Bottommargin %f\n", ((b_pagebotsstr *)p)->value);
@@ -368,10 +373,10 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
       case b_pagetops:
       debug_printf("Topmargin %f\n", ((b_pagetopsstr *)p)->value);
       break;
-      
+
       case b_playchange:
       debug_printf("Playchange\n");
-      break;   
+      break;
 
       case b_plet:
         {
@@ -389,12 +394,12 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
         debug_printf(" %f, %f, %f\n", pp->x, pp->yleft, pp->yright);
         }
       break;
-      
+
       case b_prevbar:
-      debug_printf("Prevbar %d %d %d\n", ((b_prevbarstr *)p)->dbar,  
+      debug_printf("Prevbar %d %d %d\n", ((b_prevbarstr *)p)->dbar,
         ((b_prevbarstr *)p)->ibar, ((b_prevbarstr *)p)->style);
       break;
-        
+
       case b_reset:
       debug_printf("Reset\n");
       break;
@@ -449,25 +454,27 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
         debug_printf(" ally=%f\n", pp->ally);
         }
       break;
-      
+
       case b_slurgap:
       debug_printf("Slurgap\n");
-      break;   
+      break;
 
       case b_slurmod:
       debug_printf("Slurmod\n");
-      break;   
+      break;
 
       case b_space:
       debug_printf("Space %f\n", ((b_spacestr *)p)->value);
       break;
 
+      case b_ssabove:
       case b_sshere:
       case b_ssnext:
         {
         b_ssstr *pp = (b_ssstr *)p;
-        debug_printf("%s %d %c %f\n", (type == b_sshere)? "Sshere":"Ssnext",
-          pp->stave, (pp->opt)? pp->opt : ' ', pp->value);
+        debug_printf("%s %d %c %f\n", (type == b_ssabove)? "Ssabove" :
+          (type == b_sshere)? "Sshere":"Ssnext",
+            pp->stave, (pp->opt)? pp->opt : ' ', pp->value);
         }
       break;
 
@@ -518,11 +525,11 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
         if ((flags & text_centre) != 0) debug_printf("/c");
         if ((flags & text_endalign) != 0) debug_printf("/e");
         if ((flags & text_middle) != 0) debug_printf("/m");
-        if ((flags & text_rehearse) != 0) debug_printf("/r");
+        if ((flags & text_rehearse) != 0) debug_printf("/R");
         if ((flags & text_ps) != 0) debug_printf("/ps");
         if ((flags & text_ring) != 0) debug_printf("/ring");
 
-        if (pp->size) debug_printf("/s%d", pp->size + 1);
+        if (pp->size != 0) debug_printf("/s%d", pp->size + 1);
 
         if (pp->x < 0) debug_printf("/l%f", -pp->x);
         if (pp->x > 0) debug_printf("/r%f",  pp->x);
@@ -537,11 +544,11 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
         }
       debug_printf("\n");
       break;
-      
+
       case b_textX:
-      debug_printf("TextX %f %f\n", ((b_textXstr *)p)->rotate,
-        ((b_textXstr *)p)->halfway);
-      break;   
+      debug_printf("TextX %f %f %f\n", ((b_textXstr *)p)->rotate,
+        ((b_textXstr *)p)->halfway, ((b_textXstr *)p)->offset);
+      break;
 
       case b_tick:
       debug_printf("Tick\n");
@@ -592,10 +599,10 @@ if (p == NULL) debug_printf("Empty bar\n\n"); else
       case b_unbreakbarline:
       debug_printf("Unbreakbarline\n");
       break;
-      
+
       case b_zcopy:
       debug_printf("Zcopy %d\n", ((b_zcopystr *)p)->value);
-      break;   
+      break;
 
       default:
       debug_printf("**** UNKNOWN ITEM TYPE %d ****\n", type);
@@ -671,7 +678,7 @@ else if (p == NULL) debug_printf("empty\n"); else
 *            Display a stave map                 *
 *************************************************/
 
-static int debug_print_map(uschar *s, unsigned int *map)
+static int debug_print_map(uschar *s, usint *map)
 {
 int i;
 int last;
@@ -1108,12 +1115,12 @@ while (s != NULL)
 
     debug_printf("\nSYSTEM\n"
                  "warnkey = %B\n"
-                 "warntime = %B\n" 
+                 "warntime = %B\n"
                  "stretch = %B\n"
                  "noadvance = %B\n",
         flags & sysblock_warnkey,
         flags & sysblock_warntime,
-        flags & sysblock_stretch, 
+        flags & sysblock_stretch,
         flags & sysblock_noadvance);
     debug_printf("bars %d to %d\n", s->barstart, s->barend);
     debug_printf("xjustify = %f\n", s->xjustify);
