@@ -2,15 +2,13 @@
 *    The PMW Music Typesetter - 2nd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: December 2020 */
 
 
-/* This file contains code for handling errors. There are some spares in the
-list of messages. They were Acorn-specific ones that are no longer in use.
-These numbers can be re-used for new errors. */
+/* This file contains code for handling errors. */
 
 #include "pmwhdr.h"
 #include "readhdr.h"
@@ -30,20 +28,20 @@ typedef struct {
 static error_struct error_data[] = {
 
 /* 0-4 */
-{ rc_disaster, "Failed to decode command line: \"%s\" %s\n" },
-{ rc_disaster, "Ran out of memory: increase free memory (e.g. by removing tasks)\n" },
-{ rc_disaster, "Internal failure - store overlap upwards (%x %d %x)\n" },
-{ rc_disaster, "Internal failure - store overlap downwards (%x %d %x)\n" },
-{ rc_serious,  "Unable to open file \"%s\": %s\n" },
+{ rc_failed,   "Failed to decode command line: \"%s\" %s\n" },
+{ rc_failed,   "Ran out of memory: increase free memory (e.g. by removing tasks)\n" },
+{ rc_failed,   "Internal failure - store overlap upwards (%x %d %x)\n" },
+{ rc_failed,   "Internal failure - store overlap downwards (%x %d %x)\n" },
+{ rc_failed,   "Unable to open file \"%s\": %s\n" },
 /* 5-9 */
 { rc_serious,  "Mis-placed %s (does not immediately follow a note)\n" },
 { rc_serious,  "Wrong data type on stack for \"%s\" while drawing \"%s\" %s\n" },
-{ rc_disaster, "Unable to open font file \"%s\": %s (check PMW installation)\n" },
+{ rc_serious,  "Duplicate Unicode code point U+%04x in %s\n   Output will be unpredictable\n" },
 { rc_serious,  "Macro name or string repetition expected after \"&\"\n" },
 { rc_serious,  "The name \"%s\" has not been defined\n" },
 /* 10-14 */
-{ rc_serious,  "%s expected" },
-{ rc_disaster, "Bad -printscale value on command line\n" },
+{ rc_serious,  "%s expected" },  /* Note: no newline (see #37) */
+{ rc_failed,   "Bad -printscale value on command line\n" },
 { rc_serious,  "Pre-processing directive expected\n" },
 { rc_serious,  "Unknown pre-processing directive \"%s\"\n" },
 { rc_serious,  "The name \"%s\" is already defined\n" },
@@ -54,16 +52,16 @@ static error_struct error_data[] = {
 { rc_serious,  "Missing \"*fi\" at end of file\n" },
 { rc_serious,  "Unknown heading directive \"%s\"" },
 /* 20-24 */
-{ rc_serious,  "This directive is permitted only in the first movement's\nheading" },
+{ rc_serious,  "This directive is permitted only in the first movement's heading\nsection" },
 { rc_serious,  "Numbers out of order\n" },
 { rc_failed,   "Stave number too large - must be less than %d\n" },
 { rc_failed,   "Unexpected end of file while reading string\n" },
 { rc_serious,  "Unsupported key signature\n" },
 /* 25-29 */
-{ rc_disaster, "Unable to find basic font definitions - check font list file\n" },
+{ rc_failed,   "Unable to find basic font definitions - check font list file\n" },
 { rc_failed,   "Too many different fonts declared (max is %d)\n" },
 { rc_serious,  "Malformed hexadecimal number after \"\\x\": unexpected '%c'\n" },
-{ rc_failed,   "Incorrect PMW version (%f expected, %f used)\n" },
+{ rc_failed,   "Incorrect PMW version (%c%f expected, %f used)\n" },
 { rc_failed,   "Start of stave or new movement expected\n" },
 /* 30-34 */
 { rc_failed,   "%s expected\n" },
@@ -74,12 +72,12 @@ static error_struct error_data[] = {
 /* 35-39 */
 { rc_serious,  "[reset] may not appear inside an irregular note group\n" },
 { rc_failed,   "More than %d bars read%M - use BARCOUNT to increase the limit\n" },
-{ rc_serious,  "%s expected\n" },
+{ rc_serious,  "%s expected\n" },  /* Note: with newline (see #10) */
 { rc_serious,  "Mis-matched curly brackets\n" },
 { rc_serious,  "Invalid text size (a number in the range 1 to %d was expected)\n" },
 /* 40-44 */
 { rc_serious,  "Unknown escape character \"%c\" follows \"\\\" - skipping rest of string\n" },
-{ rc_disaster, "Unable to access %s: %s\n" },
+{ rc_failed,   "Unable to access %s: %s\n" },
 { rc_serious,  "Stave number too large - maximum is %d\n" },
 { rc_serious,  "A chord may not contain a rest\n" },
 { rc_serious,  "The notes of a chord must all be the same length\n" },
@@ -102,14 +100,14 @@ static error_struct error_data[] = {
                "   be compressed to fit within the line width of %f points.\n" },
 { rc_warning,  "Warning: There is insufficient space to print a cautionary key or\n"
                "   time signature at the end of the line, following bar %b%M\n" },
-{ rc_disaster, "Internal error - unknown item %d in bar data\n" },
+{ rc_failed,   "Internal error - unknown item %d in bar data\n" },
 { rc_warning,  "Warning: Bar %b of stave %d%M has a different length to "
                "bar %b\n   on an earlier stave\n"
                "The expected length of bar %b is %l\n"
                "The actual length of bar %b on stave %d is %l\n" },
-{ rc_failed,   "Metrics file for font \"%s\" not found in \"%s\"\n" },
+{ rc_failed,   "%s file for font \"%s\" not found in \"%s\"\n" },
 /* 60-64 */
-{ rc_disaster, "Internal failure - position data missing in bar %b, stave %d%M.\n"
+{ rc_failed,   "Internal failure - position data missing in bar %b, stave %d%M.\n"
                "The musical offset is %d, which is %l.\n" },
 { rc_serious,  "Insufficient space to print notes on opposite sides of beam\n"
                "   in bar %b of stave %d%M\n" },
@@ -122,9 +120,9 @@ static error_struct error_data[] = {
 /* 65-69 */
 { rc_warning,  "Warning: There is underlay or overlay text left over at the end of\n"
                "   stave %d%M\n" },
-{ rc_disaster, "-printside must specify 1 or 2 on command line\n" },
+{ rc_failed,   "-printside must specify 1 or 2 on command line\n" },
 { rc_serious,  "[reset] may not occur before the first note of a bar\n" },
-{ rc_serious,  "Accidentals, dynamics and ornaments may not be specified for rests\n" },
+{ rc_serious,  "Accidentals, dynamics, ornaments, and irrelevant options may not be specified for rests\n" },
 { rc_serious,  "\"%s\" is not a known variable or operator name in a draw item\n" },
 /* 70-74 */
 { rc_serious,  "The drawing function \"%s\" has not been defined\n" },
@@ -134,11 +132,11 @@ static error_struct error_data[] = {
 { rc_serious,  "Malformed stave selection on command line\n" },
 /* 75-79 */
 { rc_serious,  "Malformed page list on command line\n" },
-{ rc_disaster, "Page range out of order on command line\n" },
-{ rc_disaster, "Malformed -dsb option data\n" },
-{ rc_warning,  "Character value %d is greater than 255 for a non-standardly encoded\n"
-               "   font - changed to %d\n" },
-{ rc_warning,  "Unsupported Unicode code point U+%04X changed to U+%04X\n" },
+{ rc_failed,   "Page range out of order on command line\n" },
+{ rc_failed,   "Malformed -dsb option data\n" },
+{ rc_warning,  "For a non-standardly encoded font, the following invalid Unicode code\n"
+               "   point%s been changed to U+%04X:" },
+{ rc_warning,  "The following unsupported Unicode code point%s been changed to U+%04X:" },
 /* 80-84 */
 { rc_serious,  "Too many noteheads at the same or adjacent levels in a chord\n" },
 { rc_failed,   "An input line is too long - please split and reprocess\n" },
@@ -155,7 +153,7 @@ static error_struct error_data[] = {
                "   was intended? (Perhaps \"*\" has been omitted?)\n" },
 /* 90-94 */
 { rc_warning,  "Warning: Hyphen string setting on non-underlay/overlay string ignored\n" },
-{ rc_disaster, "Internal failure - hyphen type not found in bar %b of stave %d %M\n" },
+{ rc_failed,   "Internal failure - hyphen type not found in bar %b of stave %d %M\n" },
 { rc_serious,  "Only two subsidiary strings are allowed\n" },
 { rc_serious,  "[Linegap] cannot be applied to a slur\n"
                "Detected in bar %b of stave %d%M\n" },
@@ -195,31 +193,53 @@ static error_struct error_data[] = {
 { rc_serious,  "[Slurgap] cannot be applied to a line\n"
                "Detected in bar %b of stave %d%M\n" },
 /* 120-124 */
-{ rc_disaster, "Too many movements (maximum %d)\n" },
+{ rc_failed,   "Too many movements (maximum %d)\n" },
 { rc_serious,  "Stack overflow for \"%s\" while drawing \"%s\" %s\n" },
 { rc_failed,   "Error while reading font metrics for \"%s\": %s%s\n" },
 { rc_serious,  "Malformed MIDI bar selection on command line\n" },
-{ rc_disaster, "Malformed options in .pmwrc file: \"%s\" %s\n" },
+{ rc_failed,   "Malformed options in .pmwrc file: \"%s\" %s\n" },
 /* 125-129 */
 { rc_serious,  "Unrecognized escape sequence \"%s\"\n" },
 { rc_serious,  "Unrecognized clef name \"%s\"\n" },
-{ rc_failed,   "Metrics file for font \"%s\" not found in \"%s\" or \"%s\"\n" },
+{ rc_failed,   "%s file for font \"%s\" not found in \"%s\" or \"%s\"\n" },
 { rc_serious,  "Unknown accent number %d\n" },
 { rc_serious,  "Can't have both staccato and staccatissimo\n" },
 /* 130-134 */
-{ rc_disaster, "PostScript font \"%s\" not found in \"%s\" or \"%s\"\n" },
-{ rc_disaster, "Too many text strings before a note (%d maximum)\n" },
-{ rc_disaster, "Too many notes in a chord (%d maximum)\n" },
+{ rc_failed,   "PostScript font \"%s\" not found in \"%s\" or \"%s\"\n" },
+{ rc_failed,   "Too many text strings before a note (%d maximum)\n" },
+{ rc_failed,   "Too many notes in a chord (%d maximum)\n" },
 { rc_serious,  "Missing backslash after note options" },
 { rc_serious,  "A &* replication must have only one argument - others ignored\n" },
 /* 135-139 */
 { rc_serious,  "No options are allowed after \"x\" note repetition" },
-{ rc_disaster, "Macro %s is too long (max %d)\n" },
-{ rc_disaster, "Too many macro arguments (max %d)\n" },
-{ rc_warning,  "Warning: %s repeat at %s of bar - misplaced bar line?\n" }
+{ rc_failed,   "%s is too long (max %d)\n" },
+{ rc_failed,   "Too many macro arguments (max %d)\n" },
+{ rc_warning,  "Warning: %s repeat at %s of bar - misplaced bar line?\n" },
+{ rc_failed,   "%sransposition value (%d) is too large (max %d)\n" },
+/* 140 - 144 */
+{ rc_warning,  "/h and /r or /l with 'c' specified - the latter ignored\n" },
+{ rc_serious,  "Follow-on string not permitted here\n" },
+{ rc_warning,  "Follow-on ignored for %s\n" },
+{ rc_serious,  "Too many text sizes (maximum %d)\n" },
+{ rc_serious,  "Custom key name X1 or X2 or ... X%d expected" },
+/* 145 - 149 */
+{ rc_failed,   "Cannot transpose custom key X%d by %d without KeyTranspose instruction\n" },
+{ rc_failed,   "Letter change value %d is too large for transpose value %d\n" },
+{ rc_serious,  "Too many Unicode translations in %s (max %d)\n" },
+{ rc_serious,  "Missing %s code value in line %d of %s\n%s" },
+{ rc_serious,  "Invalid font code value (> 255) in line %d of %s\n%s" },
+/* 150 - 154 */
+{ rc_serious,  "! may only follow an upper case letter\n" },
+{ rc_failed,   "This version of PMW was compiled without B2PF support\n" },
+{ rc_serious,  "Unknown B2PF option \"%s\"\n" },
+{ rc_failed,   "-norc must be given as the first option\n" },
+{ rc_failed,   "Font is already configured for B2PF\n" },
+/* 155 - 159 */
+{ rc_failed,   "B2PF context creation failed: %s\n" },
+{ rc_failed,   "B2PF processing failed for \"%s\": %s\n" }
 };
 
-#define error_maxerror 138   /* Tests are in misctests/Errors */
+#define error_maxerror (int)(sizeof(error_data)/sizeof(error_struct))
 
 /* Save some typing */
 
@@ -243,7 +263,7 @@ error_moan(int n, ...)
 {
 int rc;
 BOOL do_skip = FALSE;
-uschar buff[256];
+uschar buff[2*WORD_BUFFERSIZE];   /* Allow for inclusion of very long word */
 va_list ap;
 va_start(ap, n);
 
@@ -259,7 +279,7 @@ if (!main_shownlogo)
 if (n > error_maxerror)
   {
   sprintf(CS buff, "** Unknown error number %d\n", n);
-  rc = rc_disaster;
+  rc = rc_failed;
   }
 else
   {
@@ -272,7 +292,8 @@ else
 info to the error message here; the actual skip happens later after the message
 is output (so that the right position in the input is shown). */
 
-if (n == 10 || n == 19 || n == 20 || n == 32 || n == 133 || n == 135)
+if (n == 10 || n == 19 || n == 20 || n == 32 || n == 133 || n == 135 ||
+    n == 144)
   {
   do_skip = TRUE;
   switch(error_skip)
@@ -291,7 +312,7 @@ if (n == 10 || n == 19 || n == 20 || n == 32 || n == 133 || n == 135)
 
     case skip_BAR:
     Ustrcat(buff, " - skipping to next \"|\" or end of line");
-    stave_checklength = FALSE;   /* Don't check bar length */ 
+    stave_checklength = FALSE;   /* Don't check bar length */
     break;
 
     default:
@@ -307,7 +328,7 @@ eprintf(buff);
 
 /* Any error before initialization is complete is a disaster */
 
-if (!main_initialized) rc = rc_disaster;
+if (!main_initialized) rc = rc_failed;
 
 /* If the line number is non-zero, show it, but only in the reading state. We
 used to add the file name only for included files, but now it is always
@@ -370,10 +391,10 @@ above, when the message was modified). */
 if (do_skip) switch(error_skip)
   {
   case skip_EOL:
-  read_chptr = read_endptr;       
+  read_chptr = read_endptr;
   read_ch = ' ';
-  break;                    
- 
+  break;
+
   case skip_KET:
   while (read_ch != ']' && read_ch != '\n' && read_ch != EOF) next_ch();
   sigch();
@@ -383,14 +404,14 @@ if (do_skip) switch(error_skip)
   while (read_ch != '\\' && read_ch != '\n' && read_ch != EOF) next_ch();
   sigch();
   break;
-  
+
   case skip_BAR:
   while (read_ch != '|' && read_ch != '\n' && read_ch != EOF) next_ch();
   sigch();
   break;
-  
+
   default:
-  break;  
+  break;
   }
 
 /* Disaster if too many errors */
@@ -398,7 +419,7 @@ if (do_skip) switch(error_skip)
 if (rc > rc_warning)
   {
   error_count++;
-  if (error_count > 40)
+  if (error_count > error_maximum)
     {
     eprintf(US"** Too many errors - rest of file not scanned\n");
     if (rc < rc_failed) rc = rc_failed;
