@@ -2,11 +2,11 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
 /* Re-coded for Unicode support: August 2005 */
-/* This file last modified: May 2018 */
+/* This file last modified: November 2020 */
 
 
 /* This file contains code for handling text fonts */
@@ -97,6 +97,7 @@ static an2uencod an2ulist[] = {
   { US"Lslash",          0x0141, -1 },
   { US"M",               0x004d, -1 },
   { US"N",               0x004e, -1 },
+  { US"NBspace",         0x00a0, -1 },  /* Added Nov-2019 */
   { US"Nacute",          0x0143, -1 },
   { US"Ncaron",          0x0147, -1 },
   { US"Ncommaaccent",    0x0145, -1 },
@@ -398,6 +399,7 @@ static an2uencod an2ulist[] = {
 static int an2ucount = sizeof(an2ulist)/sizeof(an2uencod);
 
 
+
 /*************************************************
 *     Convert character name to Unicode value    *
 *************************************************/
@@ -432,7 +434,7 @@ while (top > bot)
   if (c > 0) bot = mid + 1; else top = mid;
   }
 
-if (warn) error_moan(21, cname, fname);   /* Warning */
+if (warn) error_moan(ERR122, fname, cname, US" not found");  /* Hard */
 return -1;
 }
 
@@ -453,11 +455,34 @@ Returns      difference between their "pair" values
 */
 
 static int
-table_cmp(const void *a, const void *b)
+kern_table_cmp(const void *a, const void *b)
 {
 kerntablestr *ka = (kerntablestr *)a;
 kerntablestr *kb = (kerntablestr *)b;
-return ka->pair - kb->pair;
+return (int)(ka->pair - kb->pair);
+}
+
+
+/*************************************************
+*         UTR table sorting comparison           *
+*************************************************/
+
+/* This is the auxiliary routine used for comparing UTR table entries when
+sorting them.
+
+Arguments:
+  a          pointer to utrtable structure
+  b          pointer to utrtable structure
+
+Returns      difference between their "pair" values
+*/
+
+static int
+utr_table_cmp(const void *a, const void *b)
+{
+utrtablestr *ka = (utrtablestr *)a;
+utrtablestr *kb = (utrtablestr *)b;
+return (int)(ka->unicode - kb->unicode);
 }
 
 
@@ -489,10 +514,79 @@ return p;
 
 
 /*************************************************
-*      Load width and kern tables for a font     *
+*         Find AFM or other file for a font      *
 *************************************************/
 
-/* This is an externally-called function.
+/* This is an externally-called function, also called from below.
+
+Arguments:
+  psname     the font's PostScript name
+  ext        ".afm", ".utr", ".pfa", etc., or ""
+  fextras    list of extra directories to search
+  fdefault   the default directory
+  filename   where to return the successful file name
+  mandatory  if TRUE, give hard error on failure
+
+Returns:   the opened file or NULL (when not mandatory)
+*/
+
+FILE *
+font_finddata(uschar *psname, const char *ext, uschar *fextras,
+  uschar *fdefault, uschar *filename, BOOL mandatory)
+{
+FILE *f = NULL;
+
+/* First look in any additional directories. */
+
+if (fextras != NULL)
+  {
+  uschar *pp = fextras;
+  while (*pp != 0)
+    {
+    int len;
+    uschar *ep = Ustrchr(pp, ':');
+    if (ep == NULL) ep = Ustrchr(pp, 0);
+
+    /* Using sprintf() for the whole string causes a compiler warning if
+    -Wformat-overflow is set. */
+
+    len = ep - pp;
+    (void)memcpy(filename, pp, len);
+    sprintf(CS (filename + len), "/%s%s", psname, ext);
+    f = Ufopen(filename, "r");
+    if (f != NULL || *ep == 0) break;
+    pp = ep + 1;
+    }
+  }
+
+/* Try the default directory if not yet found. */
+
+if (f == NULL)
+  {
+  sprintf(CS filename, "%s/%s%s", fdefault, psname, ext);
+  f = Ufopen(filename, "r");
+  if (mandatory && f == NULL)
+    {
+    const char *s = (*ext == 0)? "Data" : ext;
+    if (fextras == NULL)
+      error_moan(ERR59, s, psname, fdefault);            /* Hard error */
+    else
+      error_moan(ERR127, s, psname, fextras, fdefault);  /* Hard error */
+    }
+  }
+
+return f;
+}
+
+
+
+/*************************************************
+*  Load width, kern, & Unicode tables for a font *
+*************************************************/
+
+/* This is an externally-callable function. It looks for a mandatory AFM file,
+containing character widths and kerning information, and also checks for an
+optional UTR file if the font is not standardly-encoded.
 
 Argument:   the font id (offset in font_List)
 Returns:    nothing
@@ -501,7 +595,7 @@ Returns:    nothing
 void
 font_loadtables(int fontid)
 {
-FILE *f = NULL;
+FILE *f;
 int kerncount = 0;
 int finalcount = 0;
 int *widths, *r2ladjusts, *heights;
@@ -513,29 +607,8 @@ uschar line[256];
 
 DEBUG(("Loading AFM for %s\n", fs->psname));
 
-/* First look in any additional directory */
-
-if (font_metrics_extra != NULL)
-  {
-  sprintf(CS filename, "%s/%s.afm", font_metrics_extra, fs->psname);
-  f = Ufopen(filename, "r");
-  }
-
-/* Try the default directory if not yet found. */
-
-if (f == NULL)
-  {
-  sprintf(CS filename, "%s/%s.afm", font_metrics_default, fs->psname);
-  f = Ufopen(filename, "r");
-  if (f == NULL)                     /* These are hard errors */
-    {
-    if (font_metrics_extra == NULL)  
-      error_moan(59, fs->psname, font_metrics_default);
-    else
-      error_moan(127, fs->psname, font_metrics_extra, font_metrics_default);
-    }       
-  } 
-
+f = font_finddata(fs->psname, ".afm", font_data_extra, font_data_default,
+  filename, TRUE);   /* Hard error if not found */
 widths = fs->widths = malloc(LOWCHARLIMIT * sizeof(int));
 memset(widths, 0, LOWCHARLIMIT * sizeof(int));
 r2ladjusts = fs->r2ladjusts = malloc(LOWCHARLIMIT * sizeof(int));
@@ -549,7 +622,7 @@ for the standard encoding scheme and for fixed pitch. */
 for (;;)
   {
   if (Ufgets(line, sizeof(line), f) == NULL)
-    error_moan(122, filename, "no metric data found", "");  /* Hard */
+    error_moan(ERR122, filename, "no metric data found", "");  /* Hard */
   if (memcmp(line, "EncodingScheme AdobeStandardEncoding", 36) == 0)
     fs->stdencoding = TRUE;
   if (memcmp(line, "IsFixedPitch true", 17) == 0)
@@ -561,27 +634,27 @@ for (;;)
 
 for (;;)
   {
-  uschar *ppb; 
+  uschar *ppb;
   int width, code;
   int poffset = -1;
-  int r2ladjust = 0; 
+  int r2ladjust = 0;
 
   if (Ufgets(line, sizeof(line), f) == NULL)
-    error_moan(122, filename, "unexpected end of metric data", "");  /* Hard */
+    error_moan(ERR122, filename, "unexpected end of metric data", "");  /* Hard */
   if (memcmp(line, "EndCharMetrics", 14) == 0) break;
 
   if (memcmp(line, "C ", 2) != 0)
-    error_moan(122, filename, "unrecognized metric data line: ", line); /* Hard */
+    error_moan(ERR122, filename, "unrecognized metric data line: ", line); /* Hard */
 
   pp = line + 2;
   while (memcmp(pp, "WX", 2) != 0) pp++;
   pp = read_number(&width, pp+2);
-  
-  /* Look for a bounding box, but use a new pointer, because N comes first. If 
-  a bounding box is found, compute a value by which to adjust the printing 
-  position of this character when printing right-to-left. This is used for the 
-  last character of every string, instead of the stringwidth character. 
-  
+
+  /* Look for a bounding box, but use a new pointer, because N comes first. If
+  a bounding box is found, compute a value by which to adjust the printing
+  position of this character when printing right-to-left. This is used for the
+  last character of every string, instead of the stringwidth character.
+
         |          |--------|
         |          | char   |
         |          |  glyph |
@@ -597,14 +670,14 @@ for (;;)
 
   ppb = pp;
   while (*ppb != 0 && Ustrncmp(ppb, "B ", 2) != 0) ppb++;
-  if (*ppb != 0) 
+  if (*ppb != 0)
     {
-    int x0, x1;  
+    int x0, x1;
     ppb = read_number(&x0, ppb+2);   /* x-left */
     ppb = read_number(&x1, ppb);     /* y-bottom */
     ppb = read_number(&x1, ppb);     /* x-right */
     r2ladjust = x1 + x0;
-    }   
+    }
 
   /* If this is a StandardEncoding font, scan the list of characters so as to
   get the Unicode value for this character. */
@@ -648,12 +721,12 @@ for (;;)
   if (code == CHAR_FI) fs->hasfi = TRUE;
 
   /* Now put the widths in an appropriate place. */
-  
-  if (code < LOWCHARLIMIT) 
+
+  if (code < LOWCHARLIMIT)
     {
-    widths[code] = width; 
-    r2ladjusts[code] = r2ladjust; 
-    } 
+    widths[code] = width;
+    r2ladjusts[code] = r2ladjust;
+    }
   else
     {
     tree_node *tc = malloc(sizeof(tree_node));
@@ -661,7 +734,7 @@ for (;;)
     tc->name[misc_ord2utf8(code, tc->name)] = 0;
     tc->val[0] = poffset;
     tc->val[1] = width;
-    tc->val[2] = r2ladjust; 
+    tc->val[2] = r2ladjust;
     (void)Tree_InsertNode(&(fs->high_tree), tc);
     }
   }
@@ -671,11 +744,7 @@ AFM file. */
 
 for (;;)
   {
-  if (Ufgets(line, sizeof(line), f) == NULL)
-    {
-    (void)fclose(f);
-    return;        /* No kerning data */
-    }
+  if (Ufgets(line, sizeof(line), f) == NULL) goto ENDKERN;
   if (memcmp(line, "StartKernPairs", 14) == 0) break;
   }
 
@@ -700,7 +769,7 @@ while (kerncount--)
   int b = -1;
 
   if (Ufgets(line, sizeof(line), f) == NULL)
-    error_moan(122, filename, "unexpected end of kerning data");  /* Hard */
+    error_moan(ERR122, filename, "unexpected end of kerning data");  /* Hard */
   if (memcmp(line, "EndKernPairs", 12) == 0) break;
 
   /* Skip blank lines */
@@ -742,12 +811,88 @@ while (kerncount--)
 /* Adjust the count and sort the table into ascending order */
 
 fs->kerncount = finalcount;  /* true count */
-qsort(kerntable, fs->kerncount, sizeof(kerntablestr), table_cmp);
+qsort(kerntable, fs->kerncount, sizeof(kerntablestr), kern_table_cmp);
 
 /* Finished with the AFM file */
 
+ENDKERN:
 (void)fclose(f);
 
+
+/* If this font does not have standard encoding, see if there is a .utr file
+that contains Unicode translations. */
+
+if (!fs->stdencoding &&
+    (f = font_finddata(fs->psname, ".utr", font_data_extra, font_data_default,
+      filename, FALSE)) != NULL)
+  {
+  int ucount = 0;
+  int lineno = 0;
+  utrtablestr utable[MAX_UTRANSLATE];
+
+  DEBUG(("Loading UTR for %s\n", fs->psname));
+
+  while (Ufgets(line, sizeof(line), f) != NULL)
+    {
+    uschar *epp;
+
+    lineno++;
+    pp = line;
+    while (isspace(*pp)) pp++;
+    if (*pp == 0 || *pp == '#') continue;
+
+    if (ucount >= MAX_UTRANSLATE)
+      {
+      error_moan(ERR147, filename, MAX_UTRANSLATE);
+      break;
+      }
+
+    if (Ustrncmp(pp, "U+", 2) == 0) pp += 2;
+    utable[ucount].unicode = (unsigned int)Ustrtoul(pp, &epp, 16);
+    if (epp == pp)
+      {
+      error_moan(ERR148, "Unicode", lineno, filename, line);
+      continue;
+      }
+    pp = epp;
+    utable[ucount].pscode = (unsigned int)Ustrtoul(pp, &epp, 0);
+    if (epp == pp)
+      {
+      error_moan(ERR148, "font", lineno, filename, line);
+      continue;
+      }
+    if (utable[ucount].pscode >= 256)
+      {
+      error_moan(ERR149, lineno, filename, line);
+      continue;
+      }
+
+    ucount++;
+    }
+
+  (void)fclose(f);
+
+  /* Sort the data, check for duplicates, and remember with the font. */
+
+  if (ucount > 0)
+    {
+    int i;
+    qsort(utable, ucount, sizeof(utrtablestr), utr_table_cmp);
+    for (i = 1; i < ucount; i++)
+      {
+      if (utable[i].unicode == utable[i-1].unicode)
+        {
+        error_moan(ERR7, utable[i].unicode, filename);
+        while(i < ucount - 1 && utable[i].unicode == utable[i+1].unicode) i++;
+        }
+      }
+    fs->utr = store_Xget(ucount * sizeof(utrtablestr));
+    memcpy(fs->utr, utable, ucount * sizeof(utrtablestr));
+    fs->utrcount = ucount;
+    }
+  }
+
+
 /* Early checking debugging code; retained in the source in case it is ever
 needed again. */
 
@@ -802,7 +947,7 @@ static void
 font_addbasefont(uschar *psfont)
 {
 fontstr *fs = &(font_List[font_basecount]);
-if (font_basecount >= MAX_FONTS) { error_moan(26, MAX_FONTS); return; }
+if (font_basecount >= MAX_FONTS) { error_moan(ERR26, MAX_FONTS); return; }
 fs->psname = malloc(Ustrlen(psfont)+1);
 Ustrcpy(fs->psname, psfont);
 fs->widths = NULL;
@@ -811,6 +956,7 @@ fs->heights = NULL;
 fs->kerns = NULL;
 fs->kerncount = -1;
 fs->stdencoding = fs->fixedpitch = fs->hasfi = FALSE;
+fs->include = FALSE;
 font_loadtables(font_basecount++);
 }
 
@@ -848,7 +994,7 @@ Argument:  the font name (e.g. "Times-Roman")
 Returns:   the font id (offset in font_List) or -1 if not found
 */
 
-int 
+int
 font_search(uschar *name)
 {
 int i;
@@ -858,6 +1004,42 @@ return -1;
 }
 
 
+/*************************************************
+*      Manually translate Unicode code point     *
+*************************************************/
+
+/* Called for non-standardly encoded fonts. If there is no translation table,
+or if the character is absent from the table, return the code point unchanged.
+
+Arguments:
+  c           the Unicode code point
+  fs          the font structure
+
+Returns:      possibly a different code point
+*/
+
+int
+font_utranslate(int c, fontstr *fs)
+{
+int top, bot, mid;
+
+if (fs->utrcount == 0) return c;
+
+bot = 0;
+top = fs->utrcount;
+
+while (bot < top)
+  {
+  mid = (bot + top)/2;
+  if (c == fs->utr[mid].unicode) return fs->utr[mid].pscode;
+  if (c > fs->utr[mid].unicode) bot = mid + 1;
+    else top = mid;
+  }
+
+return c;   /* Not found */
+}
+
+
 
 /*************************************************
 *         Find the width of a string             *
@@ -883,32 +1065,64 @@ font_stringwidth(uschar *s, int font, int pointsize)
 int hwidth = 0;
 int vheight = 0;
 int spacecount = 0;
-int *widths, *heights;
-int fontid = font_table[font];
 int c;
 int lastc = -1;
-int kerncount;
-BOOL stdencoding;
-fontstr *fs = &(font_List[fontid]);
-kerntablestr *ktable;
-tree_node *high_tree;
 
-widths = fs->widths;
-high_tree = fs->high_tree;
-heights = fs->heights;
-ktable = fs->kerns;
-kerncount = fs->kerncount;
-stdencoding = fs->stdencoding;
+#ifdef SUPPORT_B2PF
+uschar b2pf_stack_buffer[B2PF_STACKBSIZE];
+#endif
+
+int fontid = font_table[font];
+fontstr *fs = &(font_List[fontid]);
+int *widths = fs->widths;
+int *heights = fs->heights;
+tree_node *high_tree = fs->high_tree;
+kerntablestr *ktable = fs->kerns;
+int kerncount = fs->kerncount;
+BOOL stdencoding = fs->stdencoding;
 
 DEBUG(("font_stringwidth %d %d \"%s\"\n", fontid, pointsize, s));
 
-/* Open and close quotes are fudged. When kerning we have to note the kern
-values. */
+/* If we have B2PF support, process this string if its font is set up for it.
+For the moment, just use the on-stack output buffer. If ever there's a
+complaint, we could retry the process using a larger heap buffer. */
+
+#ifdef SUPPORT_B2PF
+if (font_b2pf_contexts[font] != NULL)
+  {
+  size_t ilen = Ustrlen(s);
+  size_t used, eoffset;
+  int rc;
+
+  /* Reduce output size by 1 to allow for a terminating zero to be added. */
+
+  rc = b2pf_format_string(font_b2pf_contexts[font], (void *)s, ilen,
+    (void *)b2pf_stack_buffer, B2PF_STACKBSIZE - 1, &used,
+    font_b2pf_options[fontid], &eoffset);
+
+  if (rc != B2PF_SUCCESS)
+    {
+    size_t buffused;
+    char buffer[128];
+    (void)b2pf_get_error_message(rc, buffer, sizeof(buffer), &buffused, 0);
+    buffer[buffused] = 0;
+    error_moan(ERR156, s, buffer);  /* Hard */
+    }
+
+  b2pf_stack_buffer[used] = 0;
+  s = b2pf_stack_buffer;
+  }
+#endif
+
+/* Now scan the string. For non-standardly encoded fonts there may be a .utr
+translation from Unicode to the font's encoding. When kerning we have to note
+the kern values. */
 
 while (*s != 0)
   {
   GETCHARINC(c, s);
   if (c == ' ') spacecount++;
+  if (!stdencoding) c = font_utranslate(c, fs);
 
   /* Deal with characters >= 256 */
 
@@ -945,7 +1159,7 @@ while (*s != 0)
       lastc >= 0 && lastc <= 0xffff && c <= 0xffff)
     {
     int top, bot, mid;
-    unsigned int pair = (unsigned int)((unsigned int)lastc << 16) | c;
+    usint pair = (usint)((usint)lastc << 16) | c;
 
     bot = 0;
     top = kerncount;
@@ -1001,24 +1215,23 @@ Returns:   font_rm, font_it, etc, or -1 on error
 int
 font_fontword(BOOL soft)
 {
-uschar word[100];
-read_word(word);
-if (Ustrcmp(word, "roman") == 0) return font_rm;
-if (Ustrcmp(word, "italic") == 0) return font_it;
-if (Ustrcmp(word, "bold") == 0) return font_bf;
-if (Ustrcmp(word, "bolditalic") == 0) return font_bi;
-if (Ustrcmp(word, "symbol") == 0) return font_sy;
-if (Ustrcmp(word, "music") == 0) return font_mu;
-if (Ustrcmp(word, "bigmusic") == 0) return font_mf;
-if (Ustrcmp(word, "extra") == 0)
+next_word();
+if (Ustrcmp(read_word, "roman") == 0) return font_rm;
+if (Ustrcmp(read_word, "italic") == 0) return font_it;
+if (Ustrcmp(read_word, "bold") == 0) return font_bf;
+if (Ustrcmp(read_word, "bolditalic") == 0) return font_bi;
+if (Ustrcmp(read_word, "symbol") == 0) return font_sy;
+if (Ustrcmp(read_word, "music") == 0) return font_mu;
+if (Ustrcmp(read_word, "bigmusic") == 0) return font_mf;
+if (Ustrcmp(read_word, "extra") == 0)
   {
   int x;
   if (!read_expect_integer(&x, FALSE, FALSE)) return -1;
   if (x >= 1 && x <= MaxExtraFont) return font_xx + x - 1;
-  error_moan(26, MaxExtraFont);
+  error_moan(ERR26, MaxExtraFont);
   }
 else if (!soft)
-  error_moan(10, "\"roman\", \"italic\", \"bold\", \"bolditalic\", or \"extra\"");
+  error_moan(ERR10, "\"roman\", \"italic\", \"bold\", \"bolditalic\", or \"extra\"");
 return -1;
 }
 
