@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: November 2020 */
 
 
 /* This file contains all the global data */
@@ -81,15 +81,22 @@ int draw_oy;
 int draw_thickness;
 
 int  error_count;
+int  error_maximum = 40;
 int  error_ptr_correction = 0;
 int  error_skip;
 BOOL error_111 = FALSE;
 
+#ifdef SUPPORT_B2PF
+b2pf_context **font_b2pf_contexts;
+uint32_t *font_b2pf_options;
+#endif
 int  font_basecount = 0;
 int  font_count;
 fontstr *font_List;
-uschar *font_metrics_default;
-uschar *font_metrics_extra;
+uschar *font_data_default;
+uschar *font_data_extra;
+uschar *font_music_default;
+uschar *font_music_extra;
 int  font_sinr;
 int  font_cosr;
 int  font_stringheight;
@@ -105,6 +112,7 @@ BOOL main_format_tested;
 htypestr *main_htypes;
 int  main_initialized = FALSE;
 BOOL main_kerning;
+keytransposestr *main_keytranspose = NULL;
 int  main_lastmovement = 0;
 int  main_lastpage;
 int  main_magnification;
@@ -127,7 +135,7 @@ int  main_sheetheight;
 int  main_sheetwidth;
 BOOL main_shownlogo = FALSE;
 int  main_stavemagn;
-int  main_staves[STAVE_BITVEC_SIZE] = { -1, -1 };
+usint  main_staves[STAVE_BITVEC_SIZE] = { -1, -1 };
 int  main_storetotal = 0;
 int  main_storestaves = 0;
 int  main_stretchrespacethresh = 1075;
@@ -135,10 +143,11 @@ int  main_stretchthreshnum = 1;
 int  main_stretchthreshden = 2;
 int  main_totalbars;
 int  main_tracepos = (-2);
-int  main_transpose = max_transpose;
+int  main_transpose = no_transpose;
 BOOL main_transposedaccforce;
 trkeystr *main_transposedkeys;
 int  main_truepagelength;
+uschar main_xkeys[MAX_XKEYS][10];
 
 uschar *midi_filename = NULL;
 BOOL    midi_for_notes_off = FALSE;
@@ -154,11 +163,11 @@ BOOL n_beamed;
 int  n_cueadjust;
 int  n_chordcount;
 int  n_chordflags;
-int  n_chordacflags;
+usint n_chordacflags;
 int  n_dotxadjust;
 int  n_firstacc;
 int  n_flags;
-int  n_acflags;
+usint n_acflags;
 int  n_fontsize;
 int  n_gracecount;
 int  n_gracemoff;
@@ -220,6 +229,7 @@ int  out_joinxposition;
 int  out_keycount;
 int  out_lastbarlinex;
 BOOL out_lastbarwide;
+int  out_lastmoff;
 BOOL out_lastnotebeamed;
 int  out_lastnotex;
 int  out_laststave;
@@ -257,6 +267,8 @@ int  out_stave;
 int  out_stavelines;
 int  out_stavebottom;
 int  out_stavetop;
+int  out_string_endx;
+int  out_string_endy;
 sysblock *out_sysblock;
 int  out_textnextabove;
 int  out_textnextbelow;
@@ -280,7 +292,7 @@ int  out_ystave;
 
 int  output_copies = 1;
 BOOL output_duplex = FALSE;
-BOOL output_includefont = FALSE;
+BOOL output_incPMWfont = FALSE;
 BOOL output_manualfeed = FALSE;
 stave_list *output_pagelist = NULL;
 BOOL output_tumble = FALSE;
@@ -325,7 +337,7 @@ workposstr *page_posptr;
 workposstr *page_postable;
 BOOL page_savehadmovt;
 int  page_sgnext;
-unsigned int page_showtimes[STAVE_BITVEC_SIZE];
+usint page_showtimes[STAVE_BITVEC_SIZE];
 int *page_ssnext;
 int *page_ssehere;
 int *page_ssenext;
@@ -333,7 +345,7 @@ BOOL page_startchangetime;
 startlinestr *page_startline;
 BOOL page_startlinebar;
 int  page_stave;
-unsigned int  page_stavemap[STAVE_BITVEC_SIZE];
+usint  page_stavemap[STAVE_BITVEC_SIZE];
 uschar *page_sysclef;
 sysblock *page_sysblock;
 sysblock **page_sysprevptr;
@@ -377,8 +389,6 @@ int  print_xpageoffset = 72000;
 int  print_ypageoffset = 53000;
 
 FILE *ps_file;
-uschar *ps_fontdir_default;
-uschar *ps_fontdir_extra;
 uschar *ps_header = NULL;
 
 BOOL reading_input = FALSE;
@@ -400,12 +410,13 @@ int  read_prev_had_dbar;
 int  read_prev_had_ibar;
 int  read_prev_barlinestyle;
 int  read_skipdepth;
-uschar *read_stavedir;
+BOOL read_stavedir = FALSE;
+uschar *read_word;
 
 stavestr *stavehead;
 stavestr **stavetable;
 
-int  stave_accentflags;
+usint stave_accentflags;
 int  stave_accritvalue;
 int  stave_barlength;
 int  stave_barlinestyle;
@@ -478,7 +489,8 @@ int  stave_stemforce;
 int  stave_stemlength;
 b_notestr **stave_stemstack;
 int  stave_stemstackptr;
-int  *stave_stemswaplevel;
+int *stave_stemswaplevel;
+BOOL stave_string_followOK;
 BOOL stave_suspended;
 int  stave_textabsolute;
 int  stave_textflags;
@@ -489,6 +501,7 @@ int  stave_ties;
 int  stave_totalnocount;
 int  stave_transpose;
 int  stave_transpose_letter;
+BOOL stave_transpose_letter_is_auto;
 BOOL stave_transposedaccforce;
 BOOL stave_tripletize;
 int  stave_ulfont;
