@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: August 2020 */
 
 
 /* This file contains initializing code, including the main program, which is
@@ -26,7 +26,7 @@ writing as flexibly as possible, so that future ports are easier.
 ------------------------------------------------------------------------------
 
 The future has arrived. This port of PMW for Unix-like systems runs only as a
-single command, processing a single input file (though that may include 
+single command, processing a single input file (though that may include
 other files). Much of the old apparatus for the previous windowing version has
 been tidied away, but there are still a few quirks in the way the code works
 that hark back to the original, event-driven implementation. */
@@ -47,24 +47,26 @@ static const char *arg_pattern =
   "a5ona4/s,"
   "c/k/n,"
   "debug/s,"
-  "dbl=drawbarlines/s," 
-  "dsl=drawstavelines=drawstafflines/n=3," 
+  "dbl=drawbarlines/s,"
+  "dsl=drawstavelines=drawstafflines/n=3,"
   "dsb/k,"
   "dtp/k/n,"
   "duplex/s,"
+  "em=errormaximum/n,"
   "eps/s,"
   "F/k,"
   "f/k,"
   "H/k,"
   "-help=help/s,"
-  "includefont/s,"
-  "MF/k," 
+  "incPMWfont=incpmwfont=ipf=includefont/s,"
+  "MF/k,"
   "MP/k,"
   "MV/k,"
   "manualfeed/s,"
   "midi/k,"
   "mb=midibars/k,"
   "mm=midimovement/k/n,"
+  "norc=nopmwrc/s,"
   "nr=norepeats=norepeat/s,"
   "nw=nowidechars/s,"
   "o/k,"
@@ -90,24 +92,26 @@ enum {
   arg_a5ona4,
   arg_c,
   arg_debug,
-  arg_drawbarlines, 
-  arg_drawstavelines, 
+  arg_drawbarlines,
+  arg_drawstavelines,
   arg_dsb,
-  arg_dtp, 
+  arg_dtp,
   arg_duplex,
+  arg_em,
   arg_eps,
   arg_F,
   arg_f,
   arg_H,
   arg_help,
-  arg_includefont,
-  arg_MF, 
+  arg_incPMWfont,
+  arg_MF,
   arg_MP,
   arg_MV,
   arg_manualfeed,
   arg_midi,
   arg_midibars,
   arg_midimovement,
+  arg_norc,
   arg_norepeats,
   arg_nowidechars,
   arg_o,
@@ -142,7 +146,7 @@ static char **newargv;
 *          Given help on command syntax          *
 *************************************************/
 
-static void 
+static void
 givehelp(void)
 {
 printf("\nPMW version %s\n%s\n", version_string, copyright);
@@ -160,13 +164,16 @@ printf("-dsb <m>,<s>,<b>      write debugging bar data (movement, stave, bar) \n
 printf("-dsl [<n>]            synonym for -drawstavelines\n");
 printf("-dtp <bar>            write debugging position data (-1 for all bars)\n");
 printf("-duplex               set duplex printing in the PostScript\n");
+printf("-em <n>               synonym for -errormaximum\n");
 printf("-eps                  output encapsulated PostScript\n");
-printf("-F <directory>        specify fontmetrics directory\n");
+printf("-errormaximum <n>     set maximum number of errors (for testing)\n");
+printf("-F <directory-list>   specify fontmetrics and/or .utr directories\n");
 printf("-f <name>             specify format name\n");
 printf("-H <file>             specify PostScript header file\n");
 printf("-help                 output this information\n");
-printf("-includefont          include PMW font in the output\n");
-printf("-MF <directory>       specify PostScript music fonts directory\n");
+printf("-incPMWfont           include PMW font in the output\n");
+printf("-ipf                  synonym for -incPMWfont\n");
+printf("-MF <directory-list>  specify PostScript music fonts directories\n");
 printf("-MP <file>            specify MIDIperc file\n");
 printf("-MV <file>            specify MIDIvoices file\n");
 printf("-manualfeed           set manualfeed in the PostScript\n");
@@ -176,7 +183,7 @@ printf("-midibars <range>     limit MIDI output to given bar range\n");
 printf("-midimovement <n>     specifies movement for MIDI output\n");
 printf("-mm <n>               synonym for -midimovement\n");
 #ifndef NO_PMWRC
-printf("-norc                 don't read .pmwrc (must be first option)\n");
+printf("-norc or -nopmwrc     don't read .pmwrc (must be first option)\n");
 #endif
 printf("-norepeats            do not play repeats in MIDI output\n");
 printf("-nowidechars          don't use 100-point stave chars\n");
@@ -214,17 +221,17 @@ printf("pmw -s 1 -midi zz.mid -mm 2 -mb 10-20 sonata\n");
 
 /* This could just be replaced by fprintf() to stderr nowadays, but we keep the
 separate function just in case in the future we want do so something else with
-all this output. The function is global because it is also called from 
+all this output. The function is global because it is also called from
 setdraw.c to show the contents of the draw stack.
 
 Arguments:
   format      a format
   ...         data for the format
-  
-Returns:      nothing  
+
+Returns:      nothing
 */
 
-void 
+void
 info_printf(const char *format, ...)
 {
 uschar buff[256];
@@ -247,7 +254,7 @@ Arguments:  none
 Returns:    nothing
 */
 
-static void 
+static void
 display_info(void)
 {
 pagestr *p = main_pageanchor;
@@ -380,18 +387,18 @@ while (p != NULL)
 /* The turns strings like "1,3,4-6,10" into a bitmap.
 For some
 reason gcc gives a weird warning if ss is used directly with strtol, even with
-the right casts. That's why I use another variable of type char *. 
+the right casts. That's why I use another variable of type char *.
 
 Arguments:
   ss         the string
   endptr     where to return a pointer to the char after the last used
-  map        pointer to bitmap, held as ints
-  
-Returns:     nothing   
+  map        pointer to bitmap, held as unsigned ints
+
+Returns:     nothing
 */
 
-static void 
-init_strtomap(uschar *ss, uschar **endptr, int *map)
+static void
+init_strtomap(uschar *ss, uschar **endptr, usint *map)
 {
 long int i;
 char *sss = (char *)ss;
@@ -419,24 +426,28 @@ while (isdigit(*sss))
 *             Decode command line                *
 *************************************************/
 
-/* -V and -help act immediately; otherwise the values from the command line 
+/* -V and -help act immediately; otherwise the values from the command line
 options are place in appropriate global variables.
 
 Arguments:
   argc        the (possibly modified) command line argc
   argv        the (possibly modified) command line argv
-  
-Returns:      nothing
-*/  
 
-static void 
+Returns:      nothing
+*/
+
+static void
 decode_command(int argc, char **argv)
 {
 arg_result results[80];
 int rc = rdargs(argc, argv, arg_pattern, results);
 
-if (rc != 0) 
-  error_moan(0, results[0].text, results[1].text);  /* Hard */
+if (rc != 0)
+  error_moan(ERR0, results[0].text, results[1].text);  /* Hard */
+
+/* -norc is invalid here; give an explanation. */
+
+if (results[arg_norc].number != 0) error_moan(ERR153);
 
 /* Deal with -V */
 
@@ -468,36 +479,36 @@ if (results[arg_debug].number != 0)
 if (results[arg_dsb].text != NULL)
   {
   if (strspn(results[arg_dsb].text, "0123456789,") !=
-      strlen(results[arg_dsb].text)) error_moan(77);   /* Hard */ 
-  switch (sscanf(results[arg_dsb].text, "%d,%d,%d", &dsb_movement, 
+      strlen(results[arg_dsb].text)) error_moan(ERR77);   /* Hard */
+  switch (sscanf(results[arg_dsb].text, "%d,%d,%d", &dsb_movement,
           &dsb_stave, &dsb_bar))
     {
     case 1:                           /* One value is a bar number */
     dsb_bar = dsb_movement;
     dsb_movement = dsb_stave = 1;
-    break;  
-    
+    break;
+
     case 2:                           /* Two values are stave, bar */
     dsb_bar = dsb_stave;
     dsb_stave = dsb_movement;
     dsb_movement = 1;
     break;
-    
+
     case 3:                           /* Three values are movt, stave, bar */
-    break;       
-     
-    default:
-    error_moan(77);   /* Hard */        
     break;
-    }  
+
+    default:
+    error_moan(ERR77);   /* Hard */
+    break;
+    }
   debug_file = stderr;
   }
-  
+
 if (results[arg_dtp].presence != arg_present_not)
   {
   debug_file = stderr;
-  main_tracepos = results[arg_dtp].number; 
-  } 
+  main_tracepos = results[arg_dtp].number;
+  }
 
 /* Deal with -from and -o */
 
@@ -511,13 +522,13 @@ if (results[arg_o].text != NULL)
 MIDIvoices files */
 
 if (results[arg_F].text != NULL)
-  font_metrics_extra = US results[arg_F].text;
+  font_data_extra = US results[arg_F].text;
 
 if (results[arg_H].text != NULL)
   ps_header = US results[arg_H].text;
-  
+
 if (results[arg_MF].text != NULL)
-  ps_fontdir_extra = US results[arg_MF].text; 
+  font_music_extra = US results[arg_MF].text;
 
 if (results[arg_MP].text != NULL)
   midi_perc = US results[arg_MP].text;
@@ -536,25 +547,30 @@ if (results[arg_midibars].text != NULL)
   play_startbar = Ustrtoul(results[arg_midibars].text, &endptr, 10);
   if (*endptr == 0) play_endbar = play_startbar; else
     {
-    if (*endptr++ != '-') error_moan(123);     /* Hard error */
+    if (*endptr++ != '-') error_moan(ERR123);     /* Hard error */
     play_endbar = Ustrtoul(endptr, &endptr, 10);
-    if (*endptr != 0) error_moan(123);         /* Hard error */
+    if (*endptr != 0) error_moan(ERR123);         /* Hard error */
     }
   }
 
 if (results[arg_midimovement].presence != arg_present_not)
   play_movt_number = results[arg_midimovement].number;
 
+/* Error limit is adjustable, mainly for testing */
+
+if (results[arg_em].presence != arg_present_not)
+  error_maximum = results[arg_em].number;
+
 /* Some BOOL options */
 
 if (results[arg_norepeats].number != 0) play_repeats = FALSE;
 if (results[arg_nowidechars].number != 0) stave_use_widechars = FALSE;
 if (results[arg_drawbarlines].number != 0) bar_use_draw = TRUE;
 
-/* Draw stave lines instead of using font characters: the thickness can 
+/* Draw stave lines instead of using font characters: the thickness can
 optionally be altered. */
 
-if (results[arg_drawstavelines].presence != arg_present_not) 
+if (results[arg_drawstavelines].presence != arg_present_not)
   stave_use_draw = results[arg_drawstavelines].number;
 
 /* Deal with stave selection */
@@ -564,7 +580,7 @@ if (results[arg_s].text != NULL)
   uschar *endptr;
   init_strtomap(US results[arg_s].text, &endptr, main_staves);
   mac_setstave(main_staves, 0);
-  if (*endptr != 0) error_moan(74);  /* Hard error */
+  if (*endptr != 0) error_moan(ERR74);  /* Hard error */
   }
 
 /* Deal with page selection */
@@ -579,13 +595,13 @@ if (results[arg_p].text != NULL)
     {
     int first, last, n;
     int count = sscanf(CS s, "%d%n-%d%n", &first, &n, &last, &n);
-    if (count == 0) error_moan(75); else   /* Hard error */
+    if (count == 0) error_moan(ERR75); else   /* Hard error */
       {
       p = malloc(sizeof(stave_list));
       p->first = first;
       if (count == 1) p->last = first; else
         {
-        if (first > last) error_moan(76);  /* Hard error */
+        if (first > last) error_moan(ERR76);  /* Hard error */
           else p->last = last;
         }
       p->next = NULL;
@@ -600,7 +616,11 @@ if (results[arg_p].text != NULL)
 /* Deal with transposition */
 
 if (results[arg_t].presence != arg_present_not)
+  {
   main_transpose = results[arg_t].number;
+  if (abs(main_transpose) > max_transpose)
+    error_moan(ERR139, "T", main_transpose, max_transpose);  /* Hard error */
+  }
 
 /* Deal with format */
 
@@ -624,7 +644,7 @@ if (results[arg_a4sideways].number != 0) print_pagefeed = pc_a4sideways;
 if (results[arg_a4ona3].number != 0) print_imposition = pc_a4ona3;
 if (results[arg_a5ona4].number != 0) print_imposition = pc_a5ona4;
 
-if (results[arg_includefont].number != 0) output_includefont = TRUE;
+if (results[arg_incPMWfont].number != 0) output_incPMWfont = TRUE;
 if (results[arg_manualfeed].number != 0) output_manualfeed = TRUE;
 if (results[arg_duplex].number != 0) output_duplex = TRUE;
 if (results[arg_tumble].number != 0) output_tumble = TRUE;
@@ -658,7 +678,7 @@ if (results[arg_printscale].text != NULL)
   float d;
   sscanf(results[arg_printscale].text, "%g", &d);
   print_magnification = (int)(1000.0 * d);
-  if (print_magnification == 0) error_moan(11);  /* Hard */
+  if (print_magnification == 0) error_moan(ERR11);  /* Hard */
   }
 
 if (results[arg_printside].presence != arg_present_not)
@@ -666,7 +686,7 @@ if (results[arg_printside].presence != arg_present_not)
   int n = results[arg_printside].number;
   if (n == 1) print_side2 = FALSE;
     else if (n == 2) print_side1 = FALSE;
-      else error_moan(66);  /* Hard */
+      else error_moan(ERR66);  /* Hard */
   }
 }
 
@@ -681,6 +701,13 @@ if (results[arg_printside].presence != arg_present_not)
 static void
 clean_up(void)
 {
+#ifdef SUPPORT_B2PF
+int i;
+for (i = 0; i < font_tablen; i++)
+  {
+  if (font_b2pf_contexts[i] != NULL) b2pf_context_free(font_b2pf_contexts[i]);
+  }
+#endif
 free(newargv);
 }
 
@@ -689,18 +716,18 @@ free(newargv);
 *                Entry Point                     *
 *************************************************/
 
-int 
+int
 main(int argc, char **argv)
 {
-int newargc;
+int i, newargc;
 uschar to_buffer[256];
 verify = FALSE;
 ps_file = stdout;
 
-font_metrics_default = US FONTMETRICS;
-font_metrics_extra = NULL;
-ps_fontdir_default = US FONTDIR;
-ps_fontdir_extra = NULL;
+font_data_default = US FONTMETRICS;
+font_data_extra = NULL;
+font_music_default = US FONTDIR;
+font_music_extra = NULL;
 ps_header = US PSHEADER;
 midi_voices = US MIDIVOICES;
 midi_perc = US MIDIPERC;
@@ -711,6 +738,16 @@ version_init();
 font_List = malloc(MAX_FONTS * sizeof(fontstr));
 font_table = malloc(font_tablen * sizeof(int));
 
+#ifdef SUPPORT_B2PF
+font_b2pf_contexts = malloc(font_tablen * sizeof(b2pf_context *));
+font_b2pf_options = malloc(font_tablen * sizeof(uint32_t));
+for (i = 0; i < font_tablen; i++)
+  {
+  font_b2pf_contexts[i] = NULL;
+  font_b2pf_options[i] = 0;
+  }
+#endif
+
 beam_stemadjusts = malloc(MAX_BEAMSIZE);
 movement = malloc((main_max_movements+1) * sizeof(movtstr *));
 
@@ -751,10 +788,8 @@ else
   main_filename = arg_from_name;
   input_file = Ufopen(main_filename, "r");
   if (input_file == NULL)
-    {
-    error_moan(4, main_filename, strerror(errno));
-    return rc_disaster;
-    }
+    error_moan(ERR4, main_filename, strerror(errno));  /* Hard error */
+
   if (arg_to_name == NULL)
     {
     uschar *p;
@@ -797,10 +832,7 @@ if (arg_to_name != NULL && Ustrcmp(arg_to_name, "-") != 0)
     fprintf(stderr, "\nWriting PostScript file \"%s\" ...\n", arg_to_name);
   ps_file = Ufopen(arg_to_name, "w");
   if (ps_file == NULL)
-    {
-    error_moan(4, arg_to_name, strerror(errno));
-    return main_rc;
-    }
+    error_moan(ERR4, arg_to_name, strerror(errno));  /* Hard error */
   }
 else if (verify) fprintf(stderr, "\nWriting Postscript to stdout ...\n");
 
@@ -833,15 +865,15 @@ if (ps_file != stdout) fclose(ps_file);
 
 /* Output warning if coupled staves were not spaced by a multiple of 4 */
 
-if (error_111) error_moan(111);
+if (error_111) error_moan(ERR111);
 
 /* Write MIDI output if required */
 
-if (midi_filename != NULL) 
+if (midi_filename != NULL)
   {
   if (verify) fprintf(stderr, "Writing MIDI file \"%s\" ...\n", midi_filename);
   midi_write();
-  } 
+  }
 
 if (verify) fprintf(stderr, "PMW done\n");
 return main_rc;
