@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: April 2020 */
 
 
 /* This file contains code for writing a MIDI file */
@@ -47,7 +47,7 @@ static movtstr *play_movt;
 static int      play_nextbar;
 static int      play_nextbar_moff;
 static BOOL     play_onebar_only = FALSE;
-static int      play_staves[STAVE_BITVEC_SIZE] = { -1, -1 };
+static usint    play_staves[STAVE_BITVEC_SIZE] = { ~0, ~0 };
 static int      play_tempo;
 static int      play_volume = 127;
 
@@ -355,7 +355,7 @@ if (ptc != NULL && play_bar >= *ptc)
   while (play_bar >= *ptc) ptc += 2;
   if (ptc[-1] != play_tempo)
     {
-    int temp;
+    usint temp;
     play_tempo = ptc[-1];
     temp = 60000000/play_tempo;     /* Microseconds per crotchet */
     next_event->time = 0;
@@ -364,9 +364,9 @@ if (ptc != NULL && play_bar >= *ptc)
     next_event->data[1] = 0xff;
     next_event->data[2] = 0x51;
     next_event->data[3] = 0x03;
-    next_event->data[4] = temp >> 16;
-    next_event->data[5] = temp >> 8;
-    next_event->data[6] = temp;
+    next_event->data[4] = (uschar)((temp >> 16) & 0xffu);
+    next_event->data[5] = (uschar)((temp >> 8) & 0xffu);
+    next_event->data[6] = (uschar)(temp & 0xffu);
     next_event++;
     }
   }
@@ -425,12 +425,12 @@ for (stave = 1; stave <= play_movt->laststave; stave++)
                 {
                 case HR_PLAYON:
                 break;
-                
+
                 case HR_REPEATED:
                 goto NEXT_STAVE;
-                
+
                 default:
-                case HR_NONE:      
+                case HR_NONE:
                 if (repeat_count == 1)
                   {
                   hadrepeat = HR_REPEATED;
@@ -439,14 +439,14 @@ for (stave = 1; stave <= play_movt->laststave; stave++)
                   repeat_endbar = play_bar;
                   repeat_count++;
                   goto NEXT_STAVE;   /* Skip rest of bar */
-                  } 
-                else 
+                  }
+                else
                   {
-                  hadrepeat = HR_PLAYON; 
+                  hadrepeat = HR_PLAYON;
                   if (play_bar == repeat_endbar) repeat_count = 1;
                   }
-                break;   
-                }    
+                break;
+                }
               }
             }
           break;
@@ -787,10 +787,7 @@ int temp;
 midi_file = Ufopen(midi_filename, "w");
 
 if (midi_file == NULL)
-  {
-  error_moan(4, midi_filename, strerror(errno));
-  return;
-  }
+  error_moan(ERR4, midi_filename, strerror(errno));  /* Hard error */
 
 DEBUG(("midi_write()\n"));
 
