@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: April 2020 */
 
 
 /* This file contains code for miscellanous functions used when paginating,
@@ -96,18 +96,37 @@ for (pk = main_printkey; pk != NULL; pk = pk->next)
     break;
   }
 
+/* Printkey applies */
+
 if (pk != NULL)
   {
   width = string_width((key>63)? pk->cstring : pk->string, font_mf, 10000);
   }
 
-else
+/* Conventional key signature */
+
+else if (key63 < X_key)
   {
   int n = main_keysigtable[key63];
   width = abs(n) *
     (curmovt->accspacing)[(key>63)? ac_natural : (n>0)?ac_sharp : ac_flat];
   }
 
+/* Custom key signature */
+
+else
+  {
+  int i;
+  uschar *p = main_xkeys[key63 - X_key];
+  width = 0;
+  for (i = 0; i < 10; i++)
+    {
+    int ch = p[i] & 0x7f;
+    if (ch == ac_none) continue;
+    width += curmovt->accspacing[ch];
+    }
+  }
+
 DEBUG(("misc_keywidth() = %d\n", width));
 return width;
 }
@@ -702,7 +721,7 @@ for (stave = 0; stave <= page_lastwanted; stave++)
               {
               slurstr *s = misc_getendslur(p);
               if (s == NULL)
-                error_moan(62, ((b_endslurstr *)p)->id, bar, stave);
+                error_moan(ERR62, ((b_endslurstr *)p)->id, bar, stave);
               else store_free(s);
               }
             break;
@@ -758,8 +777,8 @@ for (stave = 0; stave <= page_lastwanted; stave++)
             if (((b_notestr *)p)->spitch != 0)
               {
               b_notestr *tp = (b_notestr *)p;
+              usint acflags = tp->acflags;
               int flags = tp->flags;
-              int acflags = tp->acflags;
               int apitch = tp->spitch;
               int bpitch = tp->spitch;
               int acc = tp->acc;
@@ -978,10 +997,10 @@ for (stave = 0; stave <= page_lastwanted; stave++)
                 while (pb->next != NULL) pb = pb->next;  /* Find last block */
                 if(pb->type == sh_system)                /* Ignore if heading */
                   {
-                  curpage->spaceleft += pb->systemgap; 
+                  curpage->spaceleft += pb->systemgap;
                   if (pp->opt == '+') pb->systemgap += v;
                     else pb->systemgap = v;
-                  curpage->spaceleft -= pb->systemgap; 
+                  curpage->spaceleft -= pb->systemgap;
                   }
                 }
               }
@@ -1004,6 +1023,35 @@ for (stave = 0; stave <= page_lastwanted; stave++)
               }
             break;
 
+            case b_ssabove:           /* change ensured stave spacing above */
+              {
+              int i;
+              b_ssstr *pp = (b_ssstr *)p;
+              int s = pp->stave;
+              int v = pp->value;
+
+              if (page_ssehere == NULL)
+                {
+                page_ssehere = store_Xget((page_lastwanted+1)*sizeof(int));
+                for (i = 0; i <= page_lastwanted; i++) page_ssehere[i] = 0;
+                }
+
+              if (s == 0)  /* Stave 0 => all staves */
+                {
+                for (i = 1; i < page_lastwanted; i++)
+                  {
+                  if (pp->opt == '+') page_ssehere[i] += v;
+                    else page_ssehere[i] = v;
+                  }
+                }
+              else
+                {
+                if (pp->opt == '+') page_ssehere[s] += v;
+                  else page_ssehere[s] = v;
+                }
+              }
+            break;
+
             case b_sshere:            /* immediate change of stave spacing */
               {
               b_ssstr *pp = (b_ssstr *)p;
@@ -1017,7 +1065,7 @@ for (stave = 0; stave <= page_lastwanted; stave++)
                 hadsshere = TRUE;
                 }
 
-              if (s == 0)
+              if (s == 0)  /* Stave 0 => all staves */
                 {
                 int i;
                 for (i = 1; i < page_lastwanted; i++)
@@ -1106,7 +1154,7 @@ for (stave = 0; stave <= page_lastwanted; stave++)
               int value = pg->value;
               if (pg->relative == '+') value += curpage->number;
               if (value < curpage->number)
-                error_moan(112, value, curpage->number, bar, stave);
+                error_moan(ERR112, value, curpage->number, bar, stave);
               else
                 curpage->number = value;
               }
@@ -1357,7 +1405,7 @@ int yield;
 int extra = 0;
 int accextra = 0;
 int flags = n_flags;
-int acflags = n_acflags;
+usint acflags = n_acflags;
 
 /* If this is a rest, the only parameter of interest is the rest level */
 
@@ -1453,15 +1501,8 @@ if (tie != NULL)
 /* Allow for dynamics. First of all, get rid of bowing marks if they are not
 relevant. */
 
-if ((bar_cont->flags & cf_bowingabove) == 0)
-  {
-  if (!below) acflags &= ~(af_up | af_down);
-  }
-else
-  {
-  if (below) acflags &= ~(af_up | af_down);
-  }
-
+if (((bar_cont->flags & cf_bowingabove) != 0) == below)
+  acflags &= ~(af_up | af_down);
 
 if ((acflags & af_dynamics) != 0 && (dynflag || (acflags & af_dyninside) != 0))
   {
