@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: June 2020 */
 
 
 /* This file contains code for outputting a page */
@@ -64,7 +64,7 @@ for (;;)
 
   GETCHARINC(c, s);
   if (c == 0) break;
-  
+
   /* If the string buffer is getting full, output the string so far. This
   leaves plenty of room for escapes (which in practice are only a few bytes
   long). */
@@ -92,7 +92,7 @@ for (;;)
         c = CHAR_FI;
         s++;
         }
-      }   
+      }
     utf[misc_ord2utf8(c, utf)] = 0;
     Ustrcpy(ss + i, utf);
     i += Ustrlen(utf);
@@ -153,13 +153,12 @@ for (;;)
   i += Ustrlen(xs);
   }
 
-/* If there are any pending characters, output them. We only need update the
-position if boxing or ringing is going to happen. */
+/* If there are any pending characters, output them. */
 
 if (i)
   {
   ss[i] = 0;
-  ps_string(ss, f, pointsize, &x, &y, boxring != 0);
+  ps_string(ss, f, pointsize, &x, &y, TRUE);
   }
 
 /* Deal with boxed and/or ringed strings. Messy stuff added to cope with
@@ -170,7 +169,7 @@ if (boxring != 0)
   int y0 = out_ystave - ystart - 2*main_stavemagn;
   int y1 = y0 + pointsize + main_stavemagn;
 
-  /* Compute length of string along string. Putting the (int) cast next to the 
+  /* Compute length of string along string. Putting the (int) cast next to the
   sqrt function gets a compiler warning. */
 
   if (font_sinr != 0)
@@ -178,7 +177,7 @@ if (boxring != 0)
     double xx = (double)x - (double)xstart;
     double yy = (double)y - (double)ystart;
     double zz = sqrt(xx*xx + yy*yy);
-    x = xstart + (int)zz;  
+    x = xstart + (int)zz;
     }
 
   x += 2*main_stavemagn - pointsize/20;
@@ -261,7 +260,12 @@ if (boxring != 0)
     }
   }
 
-/* Return the current font, with font_sc added if we are in small caps. */
+/* Return the current font, with font_sc added if we are in small caps.
+Remember the string's end point in global variables, for use with follow-on
+strings. */
+
+out_string_endx = x;
+out_string_endy = y;
 
 return (pointsize == orig_pointsize)? f : f + font_sc;
 }
@@ -273,7 +277,7 @@ return (pointsize == orig_pointsize)? f : f + font_sc;
 *             Output one head/foot line          *
 *************************************************/
 
-/* Called from out_heading below.
+/* Called from out_heading() below.
 
 Argument:   pointer to headstr
 Returns:    nothing
@@ -283,16 +287,17 @@ static void
 out_headfootline(headstr *p)
 {
 int f = p->font;
-int i = 0;
+int i;
 int *matrix = p->matrix;
 
 uschar *s = p->text;
-uschar left[256];
-uschar centre[256];
-uschar right[256];
+uschar left[READ_BUFFERSIZE];    /* No part can be longer than the maximum */
+uschar centre[READ_BUFFERSIZE];  /* input buffer size as the original string */
+uschar right[READ_BUFFERSIZE];   /* must be all on one line. */
 
 if (matrix != NULL) memcpy(font_transform, matrix, 4*sizeof(int));
 
+i = 0;
 while (*s != 0 && *s != '|') left[i++] = *s++;
 left[i] = 0;
 
@@ -362,11 +367,11 @@ while (p != NULL)
 
   if (p->size > 0)
     {
-    int descender = (4 * p->size)/10; 
+    int descender = (4 * p->size)/10;
     out_yposition += p->spaceabove;
-    if (out_yposition + descender > out_bbox[1]) 
+    if (out_yposition + descender > out_bbox[1])
       out_bbox[1] = out_yposition + descender;
-    if (out_yposition - p->size < out_bbox[3]) 
+    if (out_yposition - p->size < out_bbox[3])
       out_bbox[3] = out_yposition - p->size;
     out_headfootline(p);
     out_yposition += p->space;
@@ -452,14 +457,16 @@ for (stave = 1; stave <= out_laststave; stave++)
 
     mac_setstavesize(stave);
 
-    /* Deal with stave name */
+    /* Deal with stave name; there may be additional strings hung off the
+    "extra" field. */
 
-    if (sname != NULL)
+    for (; sname != NULL; sname = sname->extra)
       {
       /* Deal with textual stave name */
       if (sname->text != NULL)
         {
-        int yoffset;
+        BOOL vcentred = FALSE;
+        int yoffset= -8*main_stavemagn;
         int size = ((curmovt->fontsizes)->fontsize_text)[sname->offset];
         int linedepth = size;
         int *matrix = ((curmovt->fontsizes)->fontmatrix_text)[sname->offset];
@@ -472,13 +479,27 @@ for (stave = 1; stave <= out_laststave; stave++)
 
         if ((sname->flags & snf_vcentre) != 0)
           {
-          int gap = out_sysblock->stavespacing[stave];
-          int st = stave + 1;
-          while (gap == 0 && mac_teststave(out_sysblock->notsuspend, stave))
-            gap = out_sysblock->stavespacing[st++];
-          yoffset = gap/2 - 8*main_stavemagn - 2000;
+          int gap;
+          int st;
+
+          /* Scan from the current stave, looking for a stave with a non-zero
+          stavespacing. That is, look for the distance down from this stave.
+          We use this for additional vertical spacing only if neither it nor
+          the stave that follows are suspended. */
+
+          for (st = stave; st < out_laststave; st++)
+            {
+            gap = out_sysblock->stavespacing[st];
+            if (gap != 0)
+              {
+              if (mac_teststave(out_sysblock->notsuspend, st))
+                vcentred = mac_teststave(out_sysblock->notsuspend, (st+1));
+              break;
+              }
+            }
+
+          if (vcentred) yoffset += gap/2 - 2000;
           }
-        else yoffset = -8*main_stavemagn;
 
         /* Now adjust the offset according to the number of lines */
 
@@ -489,8 +510,7 @@ for (stave = 1; stave <= out_laststave; stave++)
 
         if ((sname->flags & snf_vertical) != 0)
           {
-          if ((sname->flags & snf_vcentre) != 0)
-            yoffset += string_width(t, font_rm, size)/2 - 2000;
+          if (vcentred) yoffset += string_width(t, font_rm, size)/2 - 2000;
           font_rotate(90000);
           out_string(t, font_rm, size, out_sysblock->xjustify,
             out_ystave + yoffset, 0);
@@ -633,7 +653,7 @@ staves, unless there is only one stave. */
 
 if (out_sysblock->systemdepth > 0)
   {
-  int bracketed[STAVE_BITVEC_SIZE];
+  usint bracketed[STAVE_BITVEC_SIZE];
   int bar = out_bar;
 
   DEBUG(("joining signs\n"));
