@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: April 2020 */
 
 
 /* This file contains miscellaneous subroutines for outputting a page */
@@ -39,7 +39,26 @@ static int clef_chars[] = {
 
 static uschar clef_adjusts[] = { 4, 0, 4, 8, 12, 16, 8, 12, 16, 0, 0, 4, 4, 4, 12, 12 };
 
+/* These are the clef adjustments for custom key signatures. */
 
+static int xclef_adjusts[] = {
+  -1,   /* treble */
+  -6,   /* soprano */
+  -4,   /* mezzo */
+  -2,   /* alto */
+   0,   /* tenor */
+   2,   /* cbaritone */
+  -5,   /* baritone */
+  -3,   /* bass */
+  -1,   /* deepbass */
+  -1,   /* h-clef */
+  -1,   /* none */
+  -1,   /* trebledescant */
+  -1,   /* trebletenor */
+  -1,   /* trebletenorB */
+  -3,   /* soprabass */
+  -3    /* contrabass */
+};
 
 
 /************************************************
@@ -51,16 +70,16 @@ start of a system. It returns a bit map of the staves it referenced. For braces
 and thin brackets, prev is the bit map of the (thick) bracketed staves. */
 
 
-/* Local subroutine to determine if a stave is being printed. 
+/* Local subroutine to determine if a stave is being printed.
 
 Arguments:
   stave      stave number
   bar        bar number
-  
-Returns:     TRUE if the bar is to be printed  
+
+Returns:     TRUE if the bar is to be printed
 */
 
-static BOOL 
+static BOOL
 is_printing(int stave, int bar)
 {
 stavestr *ss = curmovt->stavetable[stave];
@@ -69,7 +88,7 @@ return mac_teststave(out_sysblock->notsuspend, stave) &&
 }
 
 
-/* If yield is not null, bits are set for those staves that are included in the 
+/* If yield is not null, bits are set for those staves that are included in the
 joining item.
 
 Arguments:
@@ -84,9 +103,9 @@ Arguments:
 Returns:     nothing
 */
 
-void 
-out_dojoinsign(stave_list *list, int *prev, int which, int bartype, int bar, 
-  int *yield)
+void
+out_dojoinsign(stave_list *list, usint *prev, int which, int bartype, int bar,
+  usint *yield)
 {
 if (yield != NULL) mac_initstave(yield, 0);
 
@@ -94,9 +113,9 @@ for (; list != NULL; list = list->next)
   {
   int pb1 = list->first;
   int pb2 = list->last;
-  
+
   if (pb1 > curmovt->laststave) continue;
-  if (pb2 > curmovt->laststave) pb2 = curmovt->laststave;  
+  if (pb2 > curmovt->laststave) pb2 = curmovt->laststave;
   while (pb1 < out_laststave && pb1 < pb2 && !is_printing(pb1, bar)) pb1++;
   while (pb2 > pb1 && !is_printing(pb2, bar)) pb2--;
 
@@ -164,16 +183,16 @@ for (; list != NULL; list = list->next)
 
 
 /************************************************
-*              Output a clef                    *
+*               Output a clef                   *
 ************************************************/
 
 /* The clef characters all print a little to the right of the given position.
 The reason is historical. It does not matter at all, except when staves of
 different sizes are printed - then the different size causes an uneven spacing
-at the start of a line. So attempt to correct for this. 
+at the start of a line. So attempt to correct for this.
 
 We must also adjust the vertical position for clefs that are not of full size,
-so that they appear at the correct position on the stave. 
+so that they appear at the correct position on the stave.
 
 Arguments:
   x           x-position
@@ -181,11 +200,11 @@ Arguments:
   clef        which clef
   size        size
   midbar      TRUE if a mid-bar clef
-  
-Returns:      nothing     
+
+Returns:      nothing
 */
 
-void 
+void
 out_writeclef(int x, int y, int clef, int size, BOOL midbar)
 {
 switch(clef)
@@ -232,69 +251,110 @@ ps_muschar(x,
 *              Output a key                     *
 ************************************************/
 
-/*
+/* This handles both standard and custom keys, taking note of any "printkey"
+settings. The 0x40 bit is set when a naturalizing key is required.
 
 Arguments:
   x         x-position
   y         y-position
   clef      the current clef
   key       the key signature
-  
-Returns:    nothing
-*/    
 
-void 
+Returns:    nothing
+*/
+
+void
 out_writekey(int x, int y, int clef, int key)
 {
 int i, ch, chwidth, n, *order;
 int key63 = key & 63;
+BOOL use_naturals = (key & 0x40) != 0;
 pkeystr *pk;
 
 for (pk = main_printkey; pk != NULL; pk = pk->next)
-  { 
-  if (key63 == pk->key && clef == pk->clef && 
-      pk->movt_number <= curmovt->number) 
-    break; 
+  {
+  if (key63 == pk->key && clef == pk->clef &&
+      pk->movt_number <= curmovt->number)
+    break;
   }
-  
+
 /* There is a special string for this key. */
 
 if (pk != NULL)
   {
-  out_string((key >63)? pk->cstring : pk->string, font_mf, 10*main_stavemagn, 
+  out_string((key > 63)? pk->cstring : pk->string, font_mf, 10*main_stavemagn,
     x, y, 0);
   return;
-  } 
- 
-/* Handle "normal" key signature printing */
-
-if ((n = main_keysigtable[key63]) > 0)
-  {
-  ch = mc_sharp;
-  chwidth = (curmovt->accspacing)[ac_sharp];
-  order = main_sharporder;
   }
+
+/* Handle standard key signature printing */
+
+if (key63 < X_key)
+  {
+  if ((n = main_keysigtable[key63]) > 0)
+    {
+    ch = mc_sharp;
+    chwidth = (curmovt->accspacing)[ac_sharp];
+    order = main_sharporder;
+    }
+  else
+    {
+    ch = mc_flat;
+    chwidth = (curmovt->accspacing)[ac_flat];
+    order = main_flatorder;
+    n = -n;
+    }
+
+  if (use_naturals)
+    {
+    ch = mc_natural;
+    chwidth = (curmovt->accspacing)[ac_natural];
+    }
+
+  order += main_clefoffset[clef];
+  chwidth = (chwidth * main_stavemagn)/1000;
+
+  for (i = 0; i < n; i++)
+    {
+    ps_muschar(x, y-(order[i]*main_stavemagn)/1000, ch, 10*main_stavemagn);
+    x += chwidth;
+    }
+  }
+
+/* Handle custom key signature */
+
 else
   {
-  ch = mc_flat;
-  chwidth = (curmovt->accspacing)[ac_flat];
-  order = main_flatorder;
-  n = -n;
-  }
+  uschar *p = main_xkeys[key63 - X_key];
+  int clef_offset = xclef_adjusts[clef];
 
-if (key > 63)
-  {
-  ch = mc_natural;
-  chwidth = (curmovt->accspacing)[ac_natural];
-  }
+  for (i = 0; i < 10; i++)
+    {
+    int level, offset;
+    int baseacc = p[i] & 0x7f;
 
-order += main_clefoffset[clef];
-chwidth = (chwidth * main_stavemagn)/1000;
+    if (baseacc == ac_none) continue;
 
-for (i = 0; i < n; i++)
-  {
-  ps_muschar(x, y-(order[i]*main_stavemagn)/1000, ch, 10*main_stavemagn);
-  x += chwidth;
+    level = i + clef_offset;
+    if (level > 9) level -= 7;
+      else if (level < 0) level += 7;
+
+    if (use_naturals)
+      {
+      offset = baseacc = ac_natural;
+      }
+    else
+      {
+      offset = baseacc;
+      if ((p[i] & 0x80) != 0)
+        offset += ((offset == ac_sharp)? (curmovt->hsharpstyle == 0) :
+          (curmovt->hflatstyle == 0))? 18:36;
+      }
+
+    ps_muschar(x, y-(level*2*main_stavemagn), out_acctable[offset],
+      10*main_stavemagn);
+    x += (curmovt->accspacing[baseacc] * main_stavemagn)/1000;
+    }
   }
 }
 
@@ -304,16 +364,16 @@ for (i = 0; i < n; i++)
 *              Output a time                    *
 ************************************************/
 
-/* 
+/*
 Arguments:
   x         x-position
   y         y-position
   ts        time signature
-  
-Returns:    nothing
-*/   
 
-void 
+Returns:    nothing
+*/
+
+void
 out_writetime(int x, int y, int ts)
 {
 ptimestr *pt = main_printtime;
@@ -329,7 +389,7 @@ uschar *botstring = vd;
 if (!curmovt->showtime) return;
 
 /* First see if this time signature has special strings specified for its
-printing. The printtime directive must have happened in this movement or 
+printing. The printtime directive must have happened in this movement or
 earlier for it to be applicable. */
 
 while (pt != NULL)
@@ -458,11 +518,11 @@ Arguments:
   x          x-position
   type       type of repeat
   magn       magnification
-  
-Returns:     nothing
-*/   
 
-void 
+Returns:     nothing
+*/
+
+void
 out_writerepeat(int x, int type, int magn)
 {
 int *xx = repspacing + type + curmovt->repeatstyle * 3;
@@ -475,7 +535,7 @@ if (xx[1] >= 0)
     (curmovt->repeatstyle == 2)? bar_dotted : bar_single);
 
 out_string((curmovt->repeatstyle != 3)? US"xI" : US"IxxyyyyyyI", font_mf,
-  10*main_stavemagn, x + (xx[2]*magn)/10 + (65*(magn - main_stavemagn))/100, 
+  10*main_stavemagn, x + (xx[2]*magn)/10 + (65*(magn - main_stavemagn))/100,
   out_ystave, 0);
 }
 
@@ -495,7 +555,7 @@ Argument:  the musical offset in the bar
 Returns:   the x offset in the bar
 */
 
-int 
+int
 out_findXoffset(int moff)
 {
 if (!beam_overbeam || moff < out_poslast->moff)
@@ -514,7 +574,7 @@ else
   barposstr *bp = curmovt->posvector + out_bar + 1;
   new_postable = new_posptr = bp->vector;
   new_poslast = new_postable + bp->count - 1;
-  
+
   while (newmoff > new_posptr->moff && new_posptr < new_poslast) new_posptr++;
   while (newmoff < new_posptr->moff && new_posptr > new_postable) new_posptr--;
 
@@ -525,7 +585,7 @@ else
 /* Cannot find a position for this moff. This error is hard, but need to keep
 the compiler happy. */
 
-error_moan(60, out_bar, out_stave, moff, moff);
+error_moan(ERR60, out_bar, out_stave, moff, moff);
 return 0;
 }
 
@@ -535,17 +595,17 @@ return 0;
 *  Find X offset for one of two given M offsets  *
 *************************************************/
 
-/* This is used only in the non-overbeaming case. It returns the x offset of 
+/* This is used only in the non-overbeaming case. It returns the x offset of
 the first moff if it exists, otherwise the x offset of the second moff.
 
 Arguments:
   moff1       the first music offset
   moff2       the second music offset
-  
-Returns:      the x offset
-*/  
 
-int 
+Returns:      the x offset
+*/
+
+int
 out_findGoffset(int moff1, int moff2)
 {
 while (moff1 > out_posptr->moff && out_posptr < out_poslast) out_posptr++;
@@ -574,4 +634,50 @@ while (moff < out_posptr->moff && out_posptr > out_postable) out_posptr--;
 return (moff == out_posptr->moff)? out_posptr : NULL;
 }
 
+
+
+/*************************************************
+*   Find X offset or interpolate in current bar  *
+*************************************************/
+
+/* This is used to implement MusicXML-styole "offsets", which may be positive
+or negative. Don't alter out_posptr. If we don't find an exact moff,
+interpolate between two entries or, if off the end, fudge assuming 16 points
+per crotchet.
+
+Argument: the music offset
+Returns:  an x offset
+*/
+
+int
+out_findAoffset(int moff)
+{
+posstr *a, *b;
+posstr *p = out_posptr;
+while (moff > p->moff && p < out_poslast) p++;
+while (moff < p->moff && p > out_postable) p--;
+if (moff == p->moff) return p->xoff;
+
+if (moff > p->moff)
+  {
+  if (p == out_poslast)  /* After last entry */
+    return p->xoff + mac_muldiv(16000, moff - p->moff, len_crotchet);
+  a = p;
+  b = p + 1;
+  }
+
+else
+  {
+  if (p == out_postable)  /* Before first entry */
+    return p->xoff - mac_muldiv(16000, p->moff - moff, len_crotchet);
+  b = p;
+  a = p - 1;
+  }
+
+/* In between two entries; interpolate. */
+
+return a->xoff +
+  mac_muldiv(b->xoff - a->xoff, moff - a->moff, b->moff - a->moff);
+}
+
 /* End of out2.c */
