@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: August 2019 */
 
 /* This is the header file for the pagination routines. It contains mostly
 definitions of shared variables. Many of the structures used are more global,
@@ -75,7 +75,7 @@ extern workposstr *page_posptr;       /* pointer to end of following */
 extern workposstr *page_postable;     /* working position table */
 extern BOOL page_savehadmovt;         /* temp save */
 extern int  page_sgnext;              /* system gap for next system */
-extern unsigned int page_showtimes[STAVE_BITVEC_SIZE]; /* which staves require time sigs */
+extern usint page_showtimes[STAVE_BITVEC_SIZE]; /* which staves require time sigs */
 extern int *page_ssnext;              /* spacing table for next system */
 extern int *page_ssehere;             /* ensure table for this system */
 extern int *page_ssenext;             /* ensure table for next system */
@@ -83,7 +83,7 @@ extern BOOL page_startchangetime;     /* flags change of ts at start line */
 extern startlinestr *page_startline;  /* current start line data */
 extern BOOL page_startlinebar;        /* flags start of line */
 extern int  page_stave;               /* current stave */
-extern unsigned int page_stavemap[STAVE_BITVEC_SIZE]; /* copy of movement stave bits */
+extern usint page_stavemap[STAVE_BITVEC_SIZE]; /* copy of movement stave bits */
 extern uschar *page_sysclef;          /* clef maintenance */
 extern sysblock *page_sysblock;       /* current block */
 extern sysblock **page_sysprevptr;    /* points to where to hang the next on */
@@ -105,6 +105,6 @@ extern void page_newpage(headstr *, headstr *);
 extern int  page_makepostable(BOOL);
 extern void page_setcont(void);
 extern void page_setsignatures(void);
-extern int  page_startwidth(pagedatastr *, unsigned int *, unsigned int *);
+extern int  page_startwidth(pagedatastr *, usint *, usint *);
 
 /* End of pagehdr.h */
