@@ -5,7 +5,7 @@
 /* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: November 2019 */
 
 
 /* This file contains subroutines for the paginating code. */
@@ -23,13 +23,13 @@
 
 /* This function is called at the start of a system, to see if the
 first bar contains clef, key, or time items that should be transferred
-into the basic setting for the bar. 
+into the basic setting for the bar.
 
 Arguments: none
 Returns:   nothing
 */
 
-void 
+void
 page_setsignatures(void)
 {
 int stave;
@@ -85,7 +85,7 @@ for (stave = 0; stave <= page_lastwanted; stave++)
             b_timestr *t = (b_timestr *)p;
             (page_sysblock->cont[stave]).time = t->time;
             t->suppress = hadtime = TRUE;
-            mac_setstave(page_showtimes, stave); 
+            mac_setstave(page_showtimes, stave);
             }
           break;
 
@@ -110,18 +110,18 @@ for (stave = 0; stave <= page_lastwanted; stave++)
 *          Find width of name string             *
 *************************************************/
 
-/* This function is called by page_startwidth() below. Multiple lines in a name 
+/* This function is called by page_startwidth() below. Multiple lines in a name
 string are indicated by '|'. We return the width of the widest.
 
 Arguments:
   s            the string
   font         the font
   size         the font size
-  
-Returns:       a width   
+
+Returns:       a width
 */
 
-static int 
+static int
 page_namestringwidth(uschar *s, int font, int size)
 {
 int yield = 0;
@@ -145,22 +145,22 @@ return yield;
 *          Compute width of stave names          *
 *************************************************/
 
-/* This function returns the width needed for names to be printed at the start 
-of staves on a page. Two stave bitmaps are supplied: the name is counted only 
-if the stave is selected in both bitmaps. Typically, the first map gives the 
-staves that are active in a system, and the second is those that are currently 
+/* This function returns the width needed for names to be printed at the start
+of staves on a page. Two stave bitmaps are supplied: the name is counted only
+if the stave is selected in both bitmaps. Typically, the first map gives the
+staves that are active in a system, and the second is those that are currently
 suspended.
 
 Arguments:
   pagedata    the page data block
   map         the first bitmap
   map2        the second bitmap
-  
-Returns:      the maximum width needed   
+
+Returns:      the maximum width needed
 */
 
-int 
-page_startwidth(pagedatastr *pagedata, unsigned int *map, unsigned int *map2)
+int
+page_startwidth(pagedatastr *pagedata, usint *map, usint *map2)
 {
 int x = 0;
 int *f = (curmovt->fontsizes)->fontsize_text;
@@ -174,10 +174,10 @@ if (pagedata->stavenames != NULL)
     {
     if (mac_teststave2(map, map2, i))
       {
-      if (stavenames[i] != NULL)
+      snamestr *s;
+      for (s = stavenames[i]; s != NULL; s = s->extra)
         {
         int w = 0;
-        snamestr *s = stavenames[i];
         if ((s->flags & snf_vertical) != 0) w = 6000; else if (s->text != NULL)
           {
           int *matrix = m[s->offset];
@@ -192,7 +192,7 @@ if (pagedata->stavenames != NULL)
       }
     }
   }
-  
+
 /* If there are stave names, add extra if the system is braced or has a thin
 bracket, to leave space between them and the names. */
 
@@ -200,8 +200,8 @@ if (x != 0)
   {
   if (curmovt->bracelist != NULL) x += 6500;
     else if (curmovt->thinbracketlist != NULL) x += 4000;
-  }  
-   
+  }
+
 return x;
 }
 
@@ -213,8 +213,8 @@ return x;
 *              Deal with a page's heading        *
 *************************************************/
 
-/* This function sets up a headblock and adds it to the chain of headings and 
-systems for the page. It also subtracts the space needed for the heading lines 
+/* This function sets up a headblock and adds it to the chain of headings and
+systems for the page. It also subtracts the space needed for the heading lines
 from the space left on the page. Remember to include an additional stave depth
 + 1pt after any headings at the top of a page. This function is not called if
 there are no headings.
@@ -223,7 +223,7 @@ Argument:   the heading structure
 Returns:    nothing
 */
 
-void 
+void
 page_dopageheading(headstr *page_heading)
 {
 int used = 0;
@@ -258,83 +258,83 @@ curpage->spaceleft -= used;
 /* This function scans a heading text and splits up the "left hand" part into
 two or more heading lines. For all but the last line the stretch field is set
 to cause the text to be printed with right justification. (Actually the last
-line is justified too, if it is near enough to the end.) 
+line is justified too, if it is near enough to the end.)
 
 Argument:  a heading structure
 Returns:   nothing
 */
 
-void 
+void
 page_justifyheading(headstr *h)
 {
 while (h != NULL)
   {
   if (h->size > 0)  /* Text heading (as opposed to drawing or postscript) */
-    { 
+    {
     uschar *lastsplit = NULL;
     uschar *p = h->text;
     int spacecount = 0;
-    int splitfont = 0; 
+    int splitfont = 0;
     int lastwidth = 0;
-    
+
     if (h->matrix != NULL) memcpy(font_transform, h->matrix, 4*sizeof(int));
-    
+
     for (;;)
       {
       if (*p == '|') break;
-      
+
       if (*p == ' ' || *p == 0)
         {
         int width;
-        int c = *p; 
-     
-        *p = 0;  
+        int c = *p;
+
+        *p = 0;
         width = string_width(h->text, h->font, h->size);
-        *p = c; 
-        
+        *p = c;
+
         if (width > curmovt->linelength)
           {
-          headstr *new; 
+          headstr *new;
           if (lastsplit == NULL) break;
           p = lastsplit;
-          *p++ = 0;  
+          *p++ = 0;
           new = store_Xget(sizeof(headstr));
           memcpy(new, h, sizeof(headstr));
           new->text = p;
-          new->font = splitfont; 
-          new->spaceabove = 0; 
+          new->font = splitfont;
+          new->spaceabove = 0;
           new->stretch = 0;
-          if (spacecount > 1) 
+          if (spacecount > 1)
             h->stretch = (curmovt->linelength - lastwidth)/(spacecount - 1);
-          h->next = new; 
-          h = new;     
+          h->next = new;
+          h = new;
           spacecount = 0;
-          lastsplit = NULL; 
+          lastsplit = NULL;
           }
-        else if (*p == 0) 
+        else if (*p == 0)
           {
-          if (curmovt->linelength - width < 5000 && spacecount > 0) 
+          if (curmovt->linelength - width < 5000 && spacecount > 0)
             h->stretch = (curmovt->linelength - width)/spacecount;
-          break;   
-          } 
+          break;
+          }
         else
           {
-          spacecount++; 
-          lastsplit = p++; 
+          spacecount++;
+          lastsplit = p++;
           lastwidth = width;
           splitfont = string_font;   /* Set by string_width */
           while (*p == ' ') { p++; spacecount++; }
-          }     
-        }   
-    
+          }
+        }
+
       else p++;
-      }  
-      
+      }
+
     font_reset();        /* To set no transformation */
     }
-      
+
   h = h->next;           /* Loop through all headings */
-  }  
+  }
 }
 
 
@@ -345,17 +345,17 @@ while (h != NULL)
 *              Set up a new page                 *
 *************************************************/
 
-/* This is not called for the very first page; curpage is always set to the 
+/* This is not called for the very first page; curpage is always set to the
 current page.
 
 Arguments:
   heading        a movement heading, or NULL if not a movement start
   page_heading   a page heading, or NULL if no page heading
-  
+
 Returns:         nothing; curpage is updated
 */
 
-void 
+void
 page_newpage(headstr *heading, headstr *page_heading)
 {
 pagestr *newpage = store_Xget(sizeof(pagestr));
@@ -390,13 +390,13 @@ if (heading != NULL) page_dopageheading(heading);
 *************************************************/
 
 /* Correct the final spaceleft value (for information use), then justify
-the page if necessary, and set up the footnotes & footing. 
+the page if necessary, and set up the footnotes & footing.
 
 Argument:   TRUE for the final page; causes it to use lastfooting
 Returns:    nothing
 */
 
-void 
+void
 page_endpage(BOOL final)
 {
 headstr *footing;
@@ -417,9 +417,9 @@ if (page_footnotes != NULL)
   h->movt = curmovt;
   h->headings = page_footnotes;
   *page_sysprevptr = (sysblock *)h;
-  page_lastsystem->systemgap = page_footnotespacing + 10000; 
+  page_lastsystem->systemgap = page_footnotespacing + 10000;
   spaceleft -= page_footnotedepth + page_footnotespacing;
-  }  
+  }
 
 if (spaceleft < 0) spaceleft = 0;    /* can happen on deep systems */
 
@@ -485,11 +485,11 @@ if (page_footing != NULL)     /* Start of movement page footing */
   footing = page_footing;
   page_footing = NULL;
   }
-else 
+else
   {
   footing = (final && curmovt->lastfooting != NULL)?
     curmovt->lastfooting : curmovt->pagefooting;
-  }   
+  }
 
 if (footing != NULL)
   {
