@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: April 2020 */
 
 
 /* This file contains code for splitting the music bars into systems and making
@@ -167,7 +167,7 @@ while (!page_done) switch(page_state)
 
   page_lastwanted = -1;
     {
-    int x;
+    usint x;
     for (i = STAVE_BITVEC_SIZE - 1; i >= 0; i--)
       {
       if ((x = curmovt->staves[i]) != 0)
@@ -176,7 +176,7 @@ while (!page_done) switch(page_state)
         break;
         }
       }
-    while (x)
+    while (x != 0)
       {
       page_lastwanted++;
       if (page_lastwanted >= curmovt->laststave) break;
@@ -251,14 +251,14 @@ while (!page_done) switch(page_state)
     p->flags = cf_default;
     p->clef = clef_treble;
     p->time = curmovt->time;
-    p->key = transpose_key(curmovt->key, curmovt->transpose, FALSE);
+    p->key = transpose_key(curmovt->key, curmovt->transpose);
     }
 
   /* Final set up */
 
   if (curmovt->startnotime) mac_initstave(page_showtimes, 0); else
     {
-    mac_initstave(page_showtimes, -1);
+    mac_initstave(page_showtimes, ~0);
     mac_clearstave(page_showtimes, 0);
     }
 
@@ -317,6 +317,15 @@ while (!page_done) switch(page_state)
     if (page_ssenext != NULL)
       memcpy(page_ssehere, page_ssenext, (page_lastwanted+1)*sizeof(int));
 
+    /* If page_ssenext is NULL, but page_sshere is not, it means that
+    page_ssehere was set up in the previous system by [ssabove]. */
+
+    else if (page_ssehere != NULL)
+      {
+      store_free(page_ssehere);
+      page_ssehere = NULL;
+      }
+
     /* Get a new system block and initialize */
 
     page_sysblock = store_Xget(sizeof(sysblock));
@@ -612,7 +621,7 @@ while (!page_done) switch(page_state)
     lengthwarn = 1;       /* Don't warn when we reprocess the bar */
 
     /* See if cautionary signature(s) are needed. */
-    
+
     if (!page_startlinebar && (page_warnkey || page_warntime))
       {
       xposition = page_accepteddata->xposition + page_xxwidth;
@@ -635,7 +644,7 @@ while (!page_done) switch(page_state)
           }
         else
           {
-          error_moan(56, page_previousdata->endbar);
+          error_moan(ERR56, page_previousdata->endbar);
           overrun = 25500;
           }
         }
@@ -654,7 +663,7 @@ while (!page_done) switch(page_state)
 
     overrun = (overrun + 999)/1000;
     if (overrun > 255) overrun = 255;
-    page_sysblock->overrun = overrun;
+    page_sysblock->overrun = (uschar)overrun;
 
     /* In all cases we've finished the system */
 
@@ -675,7 +684,7 @@ while (!page_done) switch(page_state)
 
     if (page_accepteddata->count == 0 && xposition > curmovt->linelength)
       {
-      error_moan(55, page_accepteddata->endbar + 1, nextbarwidth,
+      error_moan(ERR55, page_accepteddata->endbar + 1, nextbarwidth,
         xposition-nextbarwidth, curmovt->linelength);
       }
 
@@ -1098,7 +1107,7 @@ while (!page_done) switch(page_state)
     if (sysfootdepth > main_pagelength)
       {
       int overflow = sysfootdepth - main_pagelength;
-      error_moan(64, page_sysblock->barstart, page_movtnumber,
+      error_moan(ERR64, page_sysblock->barstart, page_movtnumber,
         overflow, (overflow == 1000)? "" : "s");
       }
 
