@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: November 2020 */
 
 /* This is the main header file, imported by all other sources. */
 
@@ -27,6 +27,12 @@
 #include <sys/types.h>
 #include <errno.h>
 
+/* Optionsl B2PF include */
+
+#ifdef SUPPORT_B2PF
+#include <b2pf.h>
+#endif
+
 /* Debugging */
 
 #define DEBUG(x) if (debugging) debug_printf x
@@ -36,7 +42,9 @@
 *                Program parameters              *
 *************************************************/
 
+#define B2PF_STACKBSIZE    2000   /* On-stack buffer size */
 #define DEF_MAX_BARS        500   /* Default max bars per movement */
+#define DEF_MAX_MOVEMENTS    50   /* Default max movements */
 #define DRAW_BLOCKSIZE      256   /* Chunk size for draw routines */
 #define DRAW_STACKSIZE      100   /* Size of draw stack */
 #define LAYOUT_MAXSIZE      200   /* Max items in layout list */
@@ -45,14 +53,16 @@
 #define MAX_DRAW_VARIABLE    19   /* Max number of a draw variable */
 #define MAX_FONTS            50   /* Maximum number of different fonts */
 #define MAX_INCLUDE          10   /* Max depth of included files */
-#define MAX_MACRONAME       256   /* Max length of macro name */
-#define MAX_MACRODEFAULT    256   /* Max length of macro default argument */
 #define MAX_MACROARGS        20   /* Max number of macro arguments */
-#define DEF_MAX_MOVEMENTS    50   /* Default max movements */
+#define MAX_MACRODEFAULT    256   /* Max length of macro default argument */
 #define MAX_POSTABLESIZE    400   /* Max number of entries */
-#define READ_BUFFERSIZE    2000   /* Input buffer size */
+#define MAX_UTRANSLATE     1000   /* Max Unicode translations for a font */
+#define MAX_XKEYS            10   /* Max number of custom keys (must be <= 21)*/
 #define MIDI_MAXCHANNEL      16   /* Maximum MIDI channel number */
+#define READ_BUFFERSIZE    2000   /* Input buffer size */
 #define TEXT_QUEUE_SIZE      50   /* Max texts before a note */
+#define WORD_BUFFERSIZE     256   /* Buffer for reading words */
+
 
 /* Maximum stave is defined in terms of the length of the bit vector
 that maps stave presence, suspension, etc. If STAVE_BITVEC_SIZE is
@@ -69,7 +79,7 @@ suitably modified. */
 
 
 /*************************************************
-*        Macros etc for character handling       *
+*  Macros etc for types and character handling   *
 *************************************************/
 
 /* Some operating systems (naughtily, imo) include a definition for "uchar" in
@@ -77,6 +87,7 @@ the standard header files, so we use "uschar". Solaris has u_char in
 sys/types.h. This is just a typing convenience, of course. */
 
 typedef unsigned char uschar;
+typedef unsigned int  usint;
 
 /* These macros save typing for the casting that is needed to cope with the
 mess that is "char" in ISO/ANSI C. */
@@ -204,7 +215,7 @@ addresses. They used to be punned, stuffing pointers into ints, but this
 doesn't work in 64-bit environments. */
 
 typedef struct {
-  unsigned int dtype;             /* data type, eg dd_number */
+  usint dtype;             /* data type, eg dd_number */
   union {
     void *ptr;
     int   val;
@@ -242,7 +253,7 @@ typedef struct headstr {
   int  size;           /* Font size: 0 => PostScript; < 0 => drawing */
   int  space;          /* Space to follow; must be 0 except for text */
   uschar *text;        /* String for text or PostScript */
-  /* --- Text string --- */ 
+  /* --- Text string --- */
   int *matrix;         /* points to transformation or NULL */
   int  font;           /* initial font */
   int  stretch;        /* amount to stretch spaces by */
@@ -282,7 +293,6 @@ typedef struct {
 #define rc_warning     1
 #define rc_serious     2
 #define rc_failed      3
-#define rc_disaster    4
 
 #define time_common  255
 #define time_cut     254
@@ -293,7 +303,7 @@ sufficient identifying strings, and that ps_IdStrings in ps.c is of the right
 size too. If MaxFontSizes is changed, the initializing table init_fontsizes in
 tables.c must be made to be the correct size. */
 
-#define MaxFontSizes   12   /* for different text sizes */
+#define MaxFontSizes   20   /* for different text sizes */
 #define MaxExtraFont   12   /* for different faces */
 
 
@@ -350,6 +360,7 @@ accidental on the relevant line or space. */
 #define text_absolute   0x0800
 #define text_centre     0x1000
 #define text_timealign  0x2000
+#define text_followon   0x4000
 
 /* Flags for stave name texts */
 
@@ -363,9 +374,21 @@ accidental on the relevant line or space. */
 #define clef_oldbass    0x01
 #define clef_oldalto    0x02
 
-/* Conventional value for "no transposition" */
+/* A maximum is set for transposition - 5 octaves should be ample - and a
+conventional value for "no transposition". */
 
-#define max_transpose 1000
+#define max_transpose   60
+#define no_transpose  1000
+
+/* Names for C major and the special key 'N', which behaves like C major (no
+key signature) but does not transpose. There are 42 normal keys (7 letters
+times 2 modes times 3 accidentals) whose values are 0-41. There are also up to
+MAX_XKEYS custom-defined keys, starting at 43. Note that the 64-bit is used in
+some contexts to indicate "reset with naturals". */
+
+#define C_key   2
+#define N_key  42  /* The answer to life, the universe, and everything. */
+#define X_key  43  /* The first custom key */
 
 
 
@@ -397,7 +420,7 @@ accidental on the relevant line or space. */
 
 /* Needs manual adjustment when STAVE_BITVEC_SIZE changes */
 #define mac_initstave(a, b) \
-  a[0] = a[1] = b
+  a[0] = a[1] = (usint)b
 
 #define mac_setstave(a, z) \
   a[z >> 5] |= main_bit[z & 0x1f]
@@ -501,6 +524,24 @@ enum { font_rm, font_it, font_bf, font_bi,
 
 #define font_sc 256
 
+/* Give names to error numbers so they can be more easily tracked. */
+
+enum { ERR0,  ERR1,  ERR2,  ERR3,  ERR4,  ERR5,  ERR6,  ERR7,  ERR8,  ERR9,
+       ERR10, ERR11, ERR12, ERR13, ERR14, ERR15, ERR16, ERR17, ERR18, ERR19,
+       ERR20, ERR21, ERR22, ERR23, ERR24, ERR25, ERR26, ERR27, ERR28, ERR29,
+       ERR30, ERR31, ERR32, ERR33, ERR34, ERR35, ERR36, ERR37, ERR38, ERR39,
+       ERR40, ERR41, ERR42, ERR43, ERR44, ERR45, ERR46, ERR47, ERR48, ERR49,
+       ERR50, ERR51, ERR52, ERR53, ERR54, ERR55, ERR56, ERR57, ERR58, ERR59,
+       ERR60, ERR61, ERR62, ERR63, ERR64, ERR65, ERR66, ERR67, ERR68, ERR69,
+       ERR70, ERR71, ERR72, ERR73, ERR74, ERR75, ERR76, ERR77, ERR78, ERR79,
+       ERR80, ERR81, ERR82, ERR83, ERR84, ERR85, ERR86, ERR87, ERR88, ERR89,
+       ERR90, ERR91, ERR92, ERR93, ERR94, ERR95, ERR96, ERR97, ERR98, ERR99,
+       ERR100,ERR101,ERR102,ERR103,ERR104,ERR105,ERR106,ERR107,ERR108,ERR109,
+       ERR110,ERR111,ERR112,ERR113,ERR114,ERR115,ERR116,ERR117,ERR118,ERR119,
+       ERR120,ERR121,ERR122,ERR123,ERR124,ERR125,ERR126,ERR127,ERR128,ERR129,
+       ERR130,ERR131,ERR132,ERR133,ERR134,ERR135,ERR136,ERR137,ERR138,ERR139,
+       ERR140,ERR141,ERR142,ERR143,ERR144,ERR145,ERR146,ERR147,ERR148,ERR149,
+       ERR150,ERR151,ERR152,ERR153,ERR154,ERR155,ERR156 };
 
 
 /*************************************************
@@ -524,12 +565,20 @@ typedef struct {
 /* Items in a kerning table, pointed to from a fontstr */
 
 typedef struct kerntablestr {
-  unsigned int pair;
+  usint pair;
   int kwidth;
 } kerntablestr;
 
 
-/* Font data read from AFM file */
+/* Items in Unicode translation table, pointed to from a fontstr */
+
+typedef struct utrtablestr {
+  int unicode;
+  int pscode;
+} utrtablestr;
+
+
+/* Font data */
 
 typedef struct {
   uschar *psname;                 /* PostScript font name */
@@ -537,11 +586,14 @@ typedef struct {
   int *r2ladjusts;                /* Pointer to right-to-left adjusts */
   int *heights;                   /* Pointer to height table, if any */
   int kerncount;                  /* Size of kern table */
-  tree_node *high_tree;           /* Tree for data for high val chars */
+  int utrcount;                   /* Size of utr table */
+  utrtablestr *utr;               /* Unicode translation for nonstd font */
+  tree_node *high_tree;           /* Tree for data for high val stdenc chars */
   kerntablestr *kerns;            /* Pointer to kerning table */
   BOOL stdencoding;               /* True if std encoding */
   BOOL fixedpitch;                /* True if fixed pitch */
   BOOL hasfi;                     /* True if has "fi" */
+  BOOL include;                   /* True if to include in output */
 } fontstr;
 
 
@@ -570,7 +622,7 @@ typedef struct {
   int fontsize_triplet;
   int fontsize_repno;                 /* repeat bar number */
   int fontsize_restct;                /* long rest count */
-  int fontsize_trill;                 
+  int fontsize_trill;
   int fontsize_vertacc;               /* vertical accidentals */
   int fontsize_text[SIZES_SIZE];      /* vector of sizes */
 
@@ -617,6 +669,7 @@ typedef struct stave_list {
 
 typedef struct snamestr {
   struct snamestr *next;
+  struct snamestr *extra;     /* additional string(s); maybe different options */
   uschar *text;
   tree_node *drawing;
   drawitem  *args;            /* for drawing */
@@ -630,7 +683,7 @@ typedef struct snamestr {
 
 typedef struct ptimestr {
   struct ptimestr *next;
-  int movt_number;          /* movement in which defined */ 
+  int movt_number;          /* movement in which defined */
   int time;
   uschar *top;
   uschar *bot;
@@ -642,11 +695,11 @@ typedef struct ptimestr {
 
 typedef struct pkeystr {
   struct pkeystr *next;
-  int movt_number;          /* movement in which defined */ 
+  int movt_number;          /* movement in which defined */
   int key;
   int clef;
   uschar *string;
-  uschar *cstring; 
+  uschar *cstring;
 } pkeystr;
 
 /* One transposed key masquerade item */
@@ -657,6 +710,15 @@ typedef struct trkeystr {
   int newkey;
 } trkeystr;
 
+/* One KeyTranspose item */
+
+typedef struct keytransposestr {
+  struct keytransposestr *next;
+  int oldkey;
+  int newkeys[12];
+  int letterchanges[12];
+} keytransposestr;
+
 
 /* Data pertaining to one stave */
 
@@ -692,12 +754,12 @@ routines which use this we cheat by defining a macro "posstaves" instead. */
 typedef struct {
   int  moff;
   int  xoff;
-  int  space;                       /* additional space required */
-  int  stemup[STAVE_BITVEC_SIZE];   /* bitmap of staves where there is an */
-                                    /*  actual stem up, aka "posstaves" */
-  int  stemdown[STAVE_BITVEC_SIZE]; /* ditto for stem down */
-  int  auxid;                       /* identification of auxiliary items */
-                                    /*  (may be -ve) */
+  int  space;                        /* additional space required */
+  usint stemup[STAVE_BITVEC_SIZE];   /* bitmap of staves where there is an */
+                                     /*  actual stem up, aka "posstaves" */
+  usint stemdown[STAVE_BITVEC_SIZE]; /* ditto for stem down */
+  int  auxid;                        /* identification of auxiliary items */
+                                     /*  (may be -ve) */
  } workposstr;
 
 
@@ -726,7 +788,7 @@ typedef struct {
   int   endbar;                /* end bar number */
   int   count;                 /* count of bars */
   int   notespacing[8];        /* notespacing values */
-  unsigned int notsuspend[STAVE_BITVEC_SIZE];  /* not suspended staves */
+  usint notsuspend[STAVE_BITVEC_SIZE];  /* not suspended staves */
   snamestr **stavenames;       /* stave name vector */
   CBOOL endkey;                /* line ends with key signature */
   CBOOL endtime;               /* line ends with time signature */
@@ -839,7 +901,7 @@ typedef struct {
   int   maxbeamslope1;
   int   maxbeamslope2;
   int   movt_opt;
-  int   number; 
+  int   number;
   int   overlaydepth;
   int   play_tempo;
   int   rehearsalstyle;
@@ -859,9 +921,9 @@ typedef struct {
   int   underlaydepth;
   int   underlaystyle;
 
-  unsigned int breakbarlines[STAVE_BITVEC_SIZE];
-  unsigned int staves[STAVE_BITVEC_SIZE];
-  unsigned int suspend[STAVE_BITVEC_SIZE];
+  usint breakbarlines[STAVE_BITVEC_SIZE];
+  usint staves[STAVE_BITVEC_SIZE];
+  usint suspend[STAVE_BITVEC_SIZE];
 
   int   clefwidths[clef_count];
   int   tailadjusts[8];
@@ -870,11 +932,12 @@ typedef struct {
   CBOOL breverests;
   CBOOL check;
   CBOOL checkdoublebars;
-  CBOOL codemultirests; 
+  CBOOL codemultirests;
   CBOOL fullbarend;
   CBOOL startjoin;
   CBOOL keydoublebar;
   CBOOL keywarn;
+  CBOOL rehearsallsleft;
   CBOOL showtime;
   CBOOL showtimebase;
   CBOOL tiesoverwarnings;
@@ -901,14 +964,15 @@ typedef struct slurstr {
   struct slurstr *next;
   b_slurstr *slur;
   gapstr *gaps;
-  int  x;
-  int  y;
-  int  maxy;
-  int  miny;
-  int  lastx;
-  int  lasty;
-  int  slopeleft;
-  int  sloperight;
+  int moff;
+  int x;
+  int y;
+  int maxy;
+  int miny;
+  int lastx;
+  int lasty;
+  int slopeleft;
+  int sloperight;
   short int count;
   short int section;
 } slurstr;
@@ -1026,9 +1090,9 @@ typedef struct sysblock {
   int       keyxposition;       /* position for key signature */
   int       timexposition;      /* position for time signature */
   int       firstnoteposition;  /* sic */
-  unsigned int notsuspend[STAVE_BITVEC_SIZE];
+  usint     notsuspend[STAVE_BITVEC_SIZE];
                                 /* staves not suspended in this system */
-  unsigned int showtimes[STAVE_BITVEC_SIZE];
+  usint     showtimes[STAVE_BITVEC_SIZE];
                                 /* staves which have starting time signatures */
   short int barstart;           /* starting bar number */
   short int barend;             /* ending bar number */
@@ -1079,7 +1143,7 @@ typedef struct mfstr {
   struct mfstr *next;
   int x;
   int y;
-  unsigned int ch;  /* Holds up to 4 chars */
+  usint ch;                 /* Holds up to 4 chars */
 } mfstr;
 
 
@@ -1146,15 +1210,22 @@ extern tree_node *draw_variable_tree;  /* Tree of draw variables */
 extern int  draw_thickness;            /* Line thickness */
 
 extern int  error_count;               /* Count of errors in one file */
+extern int  error_maximum;             /* Limit (raiseable for testing) */
 extern int  error_ptr_correction;      /* For pointing within strings */
 extern int  error_skip;                /* Skip option */
 
+#ifdef SUPPORT_B2PF
+extern b2pf_context **font_b2pf_contexts;  /* Table of B2PF context pointers */
+extern uint32_t *font_b2pf_options;    /* Table of B2PF options */
+#endif
 extern int  font_count;                /* Count of defined fonts */
 extern int  font_basecount;            /* Ditto, as read from file */
 extern uschar *font_IdStrings[];       /* 2-uschar ID strings */
 extern fontstr *font_List;             /* In-store list of fonts */
-extern uschar *font_metrics_default;   /* Default font metrics directory */
-extern uschar *font_metrics_extra;     /* Additional font metrics directory */
+extern uschar *font_data_default;      /* Default font data directory */
+extern uschar *font_data_extra;        /* Additional font data directory */
+extern uschar *font_music_default;     /* Default music font directory */
+extern uschar *font_music_extra;       /* Additional music font directory */
 extern int  font_sinr;                 /* Sin of rotation angle */
 extern int  font_cosr;                 /* Cos of rotation angle */
 extern int  font_stringheight;         /* 2nd value from width function */
@@ -1174,7 +1245,7 @@ extern uschar length_table[];          /* Table of item lengths */
 extern int  longrest_barwidths[];      /* Fixed widths for longrest bars */
 extern int  longrest_widths[];         /* Widths of longrest symbols */
 
-extern unsigned int main_bit[];        /* Table of bits in a word */
+extern usint main_bit[];               /* Table of bits in a word */
 extern uschar *main_caesurastrings[];  /* For printing caesuras */
 extern int  main_clefoffset[];         /* Offsets in tables for clefs */
 extern int  main_cleftypes[];          /* Types of shape indicators for next */
@@ -1187,6 +1258,7 @@ extern htypestr *main_htypes;          /* Chain of hyphen types */
 extern BOOL main_initialized;          /* True when initialized */
 extern BOOL main_kerning;              /* Kerning flag */
 extern int  main_keysigtable[];        /* Table of sharps and flats */
+extern keytransposestr *main_keytranspose; /* List of explicit transpositions */
 extern int  main_lastmovement;         /* Number of last movement */
 extern int  main_lastpage;             /* Number of last page */
 extern int  main_magnification;        /* Overall magnification */
@@ -1210,7 +1282,7 @@ extern int  main_sheetheight;          /* Height of paper */
 extern int  main_sheetwidth;           /* Width of paper */
 extern BOOL main_shownlogo;            /* FALSE if need to show logo on error */
 extern int  main_stavemagn;            /* Magnification for current stave */
-extern int  main_staves[STAVE_BITVEC_SIZE]; /* External stave selection */
+extern usint main_staves[STAVE_BITVEC_SIZE]; /* External stave selection */
 extern int  main_storetotal;           /* Total store used */
 extern int  main_storestaves;          /* Store used for staves */
 extern int  main_stretchrespacethresh; /* Threshold for re-spacing */
@@ -1222,6 +1294,7 @@ extern int  main_transpose;            /* External transpose value */
 extern BOOL main_transposedaccforce;   /* TRUE => retain accs when transposed */
 extern trkeystr *main_transposedkeys;  /* Transposed keys masquerades */
 extern int  main_truepagelength;       /* Actual page length */
+extern uschar main_xkeys[MAX_XKEYS][10]; /* Custom key signatures */
 
 extern uschar *midi_filename;          /* name of MIDI file */
 extern BOOL    midi_for_notes_off;     /* If TRUE, do MIDI even if [notes off] */
@@ -1262,8 +1335,6 @@ extern BOOL print_pamphlet;            /* Printing options */
 extern BOOL print_reverse;
 extern BOOL print_side1;
 extern BOOL print_side2;
-extern uschar *ps_fontdir_default;     /* Default PS font directory */
-extern uschar *ps_fontdir_extra;       /* Additional PS font directory */
 extern uschar *ps_header;              /* Name of PostScript header file */
 
 extern BOOL reading_input;             /* TRUE while reading */
@@ -1308,6 +1379,8 @@ extern uschar *define_find(uschar *);
 
 extern void  error_moan(int, ...);
 
+extern FILE *font_finddata(uschar *, const char *, uschar *, uschar *,
+               uschar *, BOOL);
 extern int   font_fontword(BOOL);
 extern void  font_init(void);
 extern void  font_loadtables(int);
@@ -1315,6 +1388,7 @@ extern void  font_reset(void);
 extern void  font_rotate(int);
 extern int   font_search(uschar *);
 extern int   font_stringwidth(uschar *, int, int);
+extern int   font_utranslate(int, fontstr *);
 
 extern int   format_sprintf(uschar *, const char *, ...);      /* 2nd is format */
 extern int   format_vsprintf(uschar *, const char *, va_list); /* ditto */
@@ -1352,7 +1426,7 @@ extern uschar *store_copystring(uschar *);
 extern void  store_debug_print(void);
 extern void  store_debug_print(void);
 extern void  store_free(void *);
-extern void *store_get(unsigned int);
+extern void *store_get(usint);
 extern void *store_getitem(int);
 extern void *store_nextitem(void);
 extern void *store_Xget(int);
@@ -1363,9 +1437,9 @@ extern uschar *string_read(void);
 extern void    string_stavestring(BOOL);
 extern int     string_width(uschar *, int, int);
 
-extern void  sys_relativize(uschar *);
+extern void  sys_relativize(uschar *, int);
 
-extern int   transpose_key(int, int, BOOL);
+extern int   transpose_key(int, int);
 extern int   transpose_note(int, int *, int *, int, BOOL, BOOL, BOOL, int);
 
 extern BOOL  Tree_InsertNode(tree_node **, tree_node *);
