@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: April 2020 */
 
 
 /* This file contains the main code for creating a position table for a bar. It
@@ -251,11 +251,11 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
             b_notestr *note = (b_notestr *)p;
 
             int length = note->length;
-            int flags = note->flags;
             int notetype = note->notetype;
             int pitch = note->spitch;
-            int upflags[STAVE_BITVEC_SIZE];
-            int downflags[STAVE_BITVEC_SIZE];
+            usint flags = note->flags;
+            usint upflags[STAVE_BITVEC_SIZE];
+            usint downflags[STAVE_BITVEC_SIZE];
 
             mac_initstave(upflags, 0);
             mac_initstave(downflags, 0);
@@ -465,7 +465,7 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
           break;
 
           default:
-          if (type >= b_baditem) error_moan(57, type);
+          if (type >= b_baditem) error_moan(ERR57, type);
           break;
           }
 
@@ -475,17 +475,45 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
         type = p->type;
         }
 
-      /* We are now at the end of the bar; set the current offset to the
-      maximum encountered at [reset]s and handle any rounding problems caused
-      by tuplets. */
+      /* We are now at the end of the bar. If MaxMoff is greater than zero, it
+      means there has been a [reset]. The following notes/rests may not have
+      filled the bar, and may not have coincided with note positions before the
+      [reset]. Therefore, we might need to insert one more position for where
+      we ended up. Then set the current position to the maximum encountered. */
 
-      if (moff < MaxMoff) moff = MaxMoff;
+      if (moff < MaxMoff)
+        {
+        while (pp < page_posptr && pp->moff < moff) pp++;
+        if (pp->moff != moff)
+          {
+          workposstr *q;
+          workposstr *prev = pp - 1;
+          workposstr *next = pp + 1;
 
-      /* If the current length differs from a multiple of hemidemisemiquavers
-      by only a small amount, round it. This can happen when note lengths are
-      divided by non-factors for tuplets. For example, in 5/8 time, if one of
-      the quavers in {5-/6 g-g-g-g-g-g-} is turned into a hemidemisemiquaver,
-      the bar ends up off by 1. */
+          for (q = page_posptr; q >= pp; q--) q[1] = q[0];
+
+          pp->moff = moff;
+          pp->space = 0;
+          pp->auxid = 0;
+
+          pp->xoff = mac_muldiv(next->xoff, moff - prev->moff,
+            next->moff - prev->moff);
+          next->xoff -= pp->xoff;
+
+          /* Move pointer to include one more entry */
+
+          page_posptr++;
+          }
+
+        moff = MaxMoff;
+        }
+
+      /* Handle rounding problems caused by tuplets. If the current length
+      differs from a multiple of hemidemisemiquavers by only a small amount,
+      round it. This can happen when note lengths are divided by non-factors
+      for tuplets. For example, in 5/8 time, if one of the quavers in {5-/6
+      g-g-g-g-g-g-} is turned into a hemidemisemiquaver, the bar ends up off by
+      1. */
 
       moffdelta = moff % len_hdsquaver;
       if (moffdelta <= TUPLET_ROUND) moff -= moffdelta;
@@ -517,7 +545,7 @@ for (page_stave = 0; page_stave <= page_lastwanted; page_stave++)
         if (endbarmoff == BIGNUMBER) endbarmoff = 0; else
           {
           if (moff != endbarmoff && lengthwarn)
-            error_moan(58, barnumber, page_stave, barnumber,
+            error_moan(ERR58, barnumber, page_stave, barnumber,
               barnumber, endbarmoff, barnumber, page_stave, moff);
           }
 
@@ -1552,34 +1580,34 @@ else if (page_postable->auxid == posx_keyfirst ||
 
   page_xxwidth += page_postable->xoff;
 
-  for (; t <= page_posptr && 
+  for (; t <= page_posptr &&
          t->auxid >= posx_keyfirst &&
          t->auxid <= posx_timelast; t++)
     {
     if (t->auxid <= posx_keylast)
       {
       t->xoff -= 3*largestmagn;
-      if (page_warnkey) page_xxwidth += t->xoff; 
+      if (page_warnkey) page_xxwidth += t->xoff;
       }
     else
       {
       t->xoff += page_startline->timespace - 4*largestmagn;
-      if (page_warntime) page_xxwidth += t->xoff; 
+      if (page_warntime) page_xxwidth += t->xoff;
       }
     }
 
-  /* Add in space for the final item. MaxTimeWidth has extra space added to it 
-  which seems to be too much in the case when a key signature could have been 
-  present, but was suppressed by [nowarn]. In that case (only), we reduce the 
-  value. This is a fudge because I don't want to mess with the rest of the code 
+  /* Add in space for the final item. MaxTimeWidth has extra space added to it
+  which seems to be too much in the case when a key signature could have been
+  present, but was suppressed by [nowarn]. In that case (only), we reduce the
+  value. This is a fudge because I don't want to mess with the rest of the code
   when adding the independent [nowarn] facility. */
 
   if (!page_warnkey && MaxKeyWidth > 0) MaxTimeWidth -= 8000;
-  
+
   page_xxwidth += (page_warnkey && page_warntime)?
     ((MaxKeyWidth > MaxTimeWidth)? MaxKeyWidth:MaxTimeWidth) :
-    (page_warnkey? MaxKeyWidth : MaxTimeWidth); 
-    
+    (page_warnkey? MaxKeyWidth : MaxTimeWidth);
+
   /* Add notespace to midline bars */
 
   /***  This would make the spacing as at line start, but stretching makes it
