@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: December 2020 */
 
 
 /* This file contains code for outputting things in PostScript */
@@ -15,6 +15,8 @@
 #include "outhdr.h"
 
 
+#define UUSIZE 100  /* For list of unsupported Unicode codes */
+
 
 
 /************************************************
@@ -36,6 +38,13 @@ static int  ps_ymax;
 static uschar *ps_IdStrings[font_tablen+1];
 static int  ps_curfonttransform[6];
 
+static int  unsupported_unicode[UUSIZE];
+static int  uunext = 0;
+static int  invalid_unicode[UUSIZE];
+static int  uinvnext = 0;
+static BOOL uuoverflow = FALSE;
+static BOOL uinvoverflow = FALSE;
+
 
 
 /************************************************
@@ -228,14 +237,51 @@ ps_basic_string(uschar *s, int f, int pointsize, BOOL absolute, int x, int y,
   int w)
 {
 int font = font_table[f];
-kerntablestr *ktable;
 fontstr *fs = &(font_List[font]);
+kerntablestr *ktable = fs->kerns;
 BOOL instring = FALSE;
 BOOL stdencoding = fs->stdencoding;
-uschar *p = s;
-uschar *endp = s + Ustrlen(s);
+uschar *p, *endp;
 
-ktable = fs->kerns;
+#ifdef SUPPORT_B2PF
+uschar b2pf_stack_buffer[B2PF_STACKBSIZE];
+#endif
+
+/* If we have B2PF support, process this string if its font is set up for it.
+For the moment, just use the on-stack output buffer. If ever there's a
+complaint, we could retry the process using a larger heap buffer. */
+
+#ifdef SUPPORT_B2PF
+if (font_b2pf_contexts[f] != NULL)
+  {
+  size_t ilen = Ustrlen(s);
+  size_t used, eoffset;
+  int rc;
+
+  /* Reduce output size by 1 to allow for a terminating zero to be added. */
+
+  rc = b2pf_format_string(font_b2pf_contexts[f], (void *)s, ilen,
+    (void *)b2pf_stack_buffer, B2PF_STACKBSIZE - 1, &used,
+    font_b2pf_options[f], &eoffset);
+
+  if (rc != B2PF_SUCCESS)
+    {
+    size_t buffused;
+    char buffer[128];
+    (void)b2pf_get_error_message(rc, buffer, sizeof(buffer), &buffused, 0);
+    buffer[buffused] = 0;
+    error_moan(ERR156, s, buffer);  /* Hard */
+    }
+
+  b2pf_stack_buffer[used] = 0;
+  s = b2pf_stack_buffer;
+  }
+#endif
+
+/* Initialize start and end */
+
+p = s;
+endp = s + Ustrlen(s);
 
 /* Check top/bottom of bbox for EPS. Allow 0.4*pointsize for descenders below
 and pointsize above. */
@@ -267,20 +313,64 @@ if (main_righttoleft || ps_EPS)
     GETCHARINC(c, p);
     if (c == 0) break;
 
-    if (c >= 256)
+    if (!stdencoding)
       {
-      if (!stdencoding)
+      c = font_utranslate(c, fs);
+
+      if (c >= 256)
         {
         last_width = fs->widths[UNKNOWN_CHAR_N];
         last_r2ladjust = fs->r2ladjusts[UNKNOWN_CHAR_N];
         }
 
-      else if (c < LOWCHARLIMIT)
+      else
+        {
+        int cj = c;
+
+        /* Fudges for various special cases in the music font, where the
+        adjustment bounding box is taken from another character. */
+
+        if (f == font_mf)
+          {
+          switch(c)
+            {
+            case 'J':       /* Additional stem characters - adjust as for note */
+            case 'K':
+            case 'o':
+            case 'p':
+            case 'q':
+            case 'r':
+            cj = '5';
+            break;
+
+            case '7':       /* Up quaver - adjust as for down quaver */
+            cj = '8';
+            break;
+
+            case '9':       /* Up semiquaver - adjust as for down semiquaver */
+            cj = ':';
+            break;
+
+            default:
+            break;
+            }
+          }
+
+        last_width = fs->widths[c];
+        last_r2ladjust = fs->r2ladjusts[cj];
+        }
+      }
+
+
+    /* Standardly encoded font */
+
+    else
+      {
+      if (c < LOWCHARLIMIT)
         {
         last_width = fs->widths[c];
         last_r2ladjust = fs->r2ladjusts[c];
         }
-
       else
         {
         uschar utf[8];
@@ -300,43 +390,6 @@ if (main_righttoleft || ps_EPS)
         }
       }
 
-    else
-      {
-      int cj = c;
-
-      /* Fudges for various special cases in the music font, where the
-      adjustment bounding box is taken from another character. */
-
-      if (f == font_mf)
-        {
-        switch(c)
-          {
-          case 'J':       /* Additional stem characters - adjust as for note */
-          case 'K':
-          case 'o':
-          case 'p':
-          case 'q':
-          case 'r':
-          cj = '5';
-          break;
-
-          case '7':       /* Up quaver - adjust as for down quaver */
-          cj = '8';
-          break;
-
-          case '9':       /* Up semiquaver - adjust as for down semiquaver */
-          cj = ':';
-          break;
-
-          default:
-          break;
-          }
-        }
-
-      last_width = fs->widths[c];
-      last_r2ladjust = fs->r2ladjusts[cj];
-      }
-
     /* Amass the total string width */
 
     swidth += last_width;
@@ -348,7 +401,7 @@ if (main_righttoleft || ps_EPS)
       int bot = 0;
       int top = fs->kerncount;
       int cc;
-      unsigned int pair;
+      usint pair;
 
       GETCHAR(cc, p);
       pair = (c << 16) | cc;
@@ -389,13 +442,38 @@ if (main_righttoleft || ps_EPS)
 p = s;
 for (;;)
   {
-  int c, pc;
+  int c, pc;               /* pc is the code value to print */
   BOOL extended = FALSE;
 
   GETCHARINC(c, p);
   if (c == 0) break;
 
-  pc = c;    /* The character value to print */
+  /* For a non-standardly encoded font, see if there's a Unicode translation.
+  If not, values greater than 255 are erroneous. Remember them, to be listed at
+  the end. */
+
+  if (!stdencoding)
+    {
+    pc = font_utranslate(c, fs);
+    if (pc >= 256)
+      {
+      pc = UNKNOWN_CHAR_N;  /* Known to be < 256 */
+      if (uinvnext <= UUSIZE)
+        {
+        int i;
+        for (i = 0; i < uinvnext; i++) if (invalid_unicode[i] == c) break;
+        if (i >= uinvnext)
+          {
+          if (uinvnext < UUSIZE) invalid_unicode[uinvnext++] = c;
+            else uinvoverflow = TRUE;
+          }
+        }
+      }
+    }
+
+  /* Standardly encoded font */
+
+  else if (c < 256) pc = c;
 
   /* For standardly encoded fonts, code points >= 256 and < LOWCHARLIMIT are
   encoded in the second of the two PostScript fonts, using the Unicode encoding
@@ -405,15 +483,9 @@ for (;;)
   large). To find this encoding, we search for the character in the widths tree
   for the font, where the offset is also stored. */
 
-  if (c >= 256)
+  else
     {
-    if (!stdencoding)
-      {
-      pc = UNKNOWN_CHAR_N;  /* Known to be < 256 */
-      error_moan(78, c, UNKNOWN_CHAR_N);
-      }
-
-    else if (c < LOWCHARLIMIT)
+    if (c < LOWCHARLIMIT)
       {
       pc = c - 256;
       extended = TRUE;
@@ -428,7 +500,16 @@ for (;;)
       if (t == NULL)
         {
         pc = UNKNOWN_CHAR_S;
-        error_moan(79, c, UNKNOWN_CHAR_S);
+        if (uunext <= UUSIZE)
+          {
+          int i;
+          for (i = 0; i < uunext; i++) if (unsupported_unicode[i] == c) break;
+          if (i >= uunext)
+            {
+            if (uunext < UUSIZE) unsupported_unicode[uunext++] = c;
+              else uuoverflow = TRUE;
+            }
+          }
         }
       else
         {
@@ -451,7 +532,7 @@ for (;;)
     ps_setfont(f, pointsize, extended);
     }
 
-  /* Arrange to print the byte */
+  /* Arrange to print the code */
 
   if (!instring)
     {
@@ -469,7 +550,7 @@ for (;;)
     ps_chcount += fprintf(ps_file, "\\%c", pc);
   else if (pc >= 32 && pc <= 126)
     {
-    fputc(c, ps_file);
+    fputc(pc, ps_file);
     ps_chcount++;
     }
   else ps_chcount += fprintf(ps_file, "\\%03o", pc);
@@ -482,10 +563,10 @@ for (;;)
     int bot = 0;
     int top = fs->kerncount;
     int cc;
-    unsigned int pair;
+    usint pair;
 
     GETCHAR(cc, p);
-    pair = ((unsigned int)c << 16) | cc;
+    pair = ((usint)c << 16) | cc;
 
     while (bot < top)
       {
@@ -784,7 +865,7 @@ Returns:       nothing
 void
 ps_stave(int leftx, int y, int rightx, int stavelines)
 {
-uschar sbuff[8];
+uschar sbuff[16];
 uschar buff[256];
 int ch, i;
 int chwidth = 0;
@@ -898,7 +979,7 @@ int xfudge = 0;
 
 for (p = out_mftable_ps[ch]; p != NULL; p = p->next)
   {
-  unsigned int c = p->ch;
+  usint c = p->ch;
   int i = 0;
   int nxfudge = 0;
 
@@ -1496,10 +1577,10 @@ static void
 ps_include(uschar *s, BOOL relativize)
 {
 FILE *f;
-uschar name[256];
+uschar name[WORD_BUFFERSIZE];
 
 Ustrcpy(name, s);
-if (relativize) sys_relativize(name);
+if (relativize) sys_relativize(name, sizeof(name));
 f = Ufopen(name, "r");
 
 if (f != NULL)
@@ -1534,7 +1615,7 @@ if (f != NULL)
   ps_chcount = 0;
   }
 
-else error_moan(4, name, strerror(errno));
+else error_moan(ERR4, name, strerror(errno));  /* Hard error */
 }
 
 
@@ -1564,61 +1645,64 @@ ps_chcount = 0;
 
 
 /*************************************************
-*        Include a Music font in the output      *
+*          Include a font in the output          *
 *************************************************/
 
-/* This function is called if either PMW-Music or PMW-Alpha is to be included
-in the output.
+/*
+Argument:
+  name     the name of the font
+  ext      a file extension or empty string
 
-Argument:  the name of the font
 Returns:   nothing
 */
 
 static void
-include_font(const char *name)
+include_font(uschar *name, const char *ext)
 {
 FILE *f = NULL;
+char *s;
+uschar *fextra, *fdefault;
 uschar buff[256];
 
-/* First look in any additional directory */
+/* If this is one of the PMW fonts, seek it in the psfonts directories,
+otherwise look in the general fonts directories. */
 
-if (ps_fontdir_extra != NULL)
+if (Ustrncmp(name, "PMW-", 4) == 0)
   {
-  sprintf(CS buff, "%s/%s", ps_fontdir_extra, name);
-  f = Ufopen(buff, "r");
+  fextra = font_music_extra;
+  fdefault = font_music_default;
+  }
+else
+  {
+  fextra = font_data_extra;
+  fdefault = font_data_default;
   }
 
-/* Try the default directory if not yet found. */
+/* font_finddata(..., TRUE) gives a hard error if the file cannot be found. */
 
-if (f == NULL)
+f = font_finddata(name, ext, fextra, fdefault, buff, TRUE);
+
+/* Copy from "%%BeginResource:" or the start of the file to "%%EndResource" or
+the end of the file. */
+
+while ((s = Ufgets(buff, sizeof(buff), f)) != NULL)
+  if (Ustrncmp(buff, "%%BeginResource:", 16) == 0) break;
+
+if (s == NULL)
   {
-  sprintf(CS buff, "%s/%s", ps_fontdir_default, name);
-  f = Ufopen(buff, "r");
-  if (f == NULL)                     /* These are hard errors */
-    {
-    if (ps_fontdir_extra == NULL)
-      error_moan(7, buff, strerror(errno));
-    else
-      error_moan(130, name, ps_fontdir_extra, ps_fontdir_default);
-    }
+  fprintf(ps_file, "\n%%%%BeginResource: font %s\n", name);
+  rewind(f);
   }
+else fprintf(ps_file, "%s", CS buff);
 
-while (Ufgets(buff, sizeof(buff), f) != NULL)
-  {
-  if (Ustrncmp(buff, "%%BeginResource:", 16) == 0)
-    {
-    fprintf(ps_file, "%s", CS buff);
-    break;
-    }
-  }
-
-while (Ufgets(buff, sizeof(buff), f) != NULL)
+while ((s = Ufgets(buff, sizeof(buff), f)) != NULL)
   {
   fprintf(ps_file, "%s", CS buff);
   if (Ustrncmp(buff, "%%EndResource", 13) == 0) break;
   }
-}
 
+if (s == NULL) fprintf(ps_file, "\n%%%%EndResource\n\n");
+}
 
 
 
@@ -1637,9 +1721,9 @@ Returns:   nothing
 void
 ps_go(void)
 {
-BOOL used_pmw_music = FALSE;
-BOOL used_pmw_alpha = FALSE;
 time_t timer;
+int fonts_to_include[MAX_FONTS];
+int fonts_to_include_count = 0;
 int i, w = 0, d = 0;
 int count = 0;
 int fcount = 1;
@@ -1707,32 +1791,54 @@ if (ps_EPS) fprintf(ps_file, "%%%%BoundingBox: (atend)\n");
   else fprintf(ps_file, "%%%%Pages: (atend)\n");
 fprintf(ps_file, "%%%%DocumentNeededResources: font ");
 
+/* Scan the fonts, set the ps id and process each unique one, remembering those
+that are to be included in the output. */
+
 for (i = 0; i < font_tablen; i++)
   {
-  int j;
+  fontstr *fs;
+  int fontid, j;
+
   for (j = 0; j < i; j++) if (font_table[i] == font_table[j]) break;
-  if (j == i)
-    {
-    if (++fcount > 3)
-      {
-      fprintf(ps_file, "\n%%%%+ font ");
-      fcount = 1;
-      }
-    fprintf(ps_file, "%s ", font_List[font_table[i]].psname);
-    /* Remember which music fonts have been used */
-    if (Ustrcmp(font_List[font_table[i]].psname, "PMW-Music") == 0)
-      used_pmw_music = TRUE;
-    if (Ustrcmp(font_List[font_table[i]].psname, "PMW-Alpha") == 0)
-      used_pmw_alpha = TRUE;
-    }
   ps_IdStrings[i] = font_IdStrings[j];
+  if (j != i) continue;   /* Seen this one already */
+
+  if (++fcount > 3)
+    {
+    fprintf(ps_file, "\n%%%%+ font ");
+    fcount = 1;
+    }
+
+  fontid = font_table[i];
+  fs = font_List + fontid;
+  fprintf(ps_file, "%s ", fs->psname);
+
+  /* Remember which fonts are to be included. If -incPMWfont was set, do
+  this automatically for music fonts. */
+
+  if (fs->include || (output_incPMWfont &&
+       (Ustrcmp(fs->psname, "PMW-Music") == 0 ||
+        Ustrcmp(fs->psname, "PMW-Alpha") == 0)))
+    fonts_to_include[fonts_to_include_count++] = fontid;
   }
 fprintf(ps_file, "\n");
 
-if (output_includefont && (used_pmw_music || used_pmw_alpha))
+/* List the included fonts */
+
+if (fonts_to_include_count > 0)
   {
-  fprintf(ps_file, "%%%%DocumentSuppliedResources: font%s%s\n",
-    used_pmw_music? " PMW-Music":"", used_pmw_alpha? " PMW-Alpha":"");
+  fcount = 1;
+  fprintf(ps_file, "%%%%DocumentSuppliedResources: font");
+  for (i = 0; i < fonts_to_include_count; i++)
+    {
+    if (++fcount > 3)
+      {
+      fprintf(ps_file, "\n%%%%+ font");
+      fcount = 1;
+      }
+    fprintf(ps_file, " %s", (font_List[fonts_to_include[i]]).psname);
+    }
+  fprintf(ps_file, "\n");
   }
 
 if (!ps_EPS) fprintf(ps_file,
@@ -1795,18 +1901,18 @@ if (main_pssetup != NULL)
 fprintf(ps_file, "%%%%EndProlog\n\n");
 
 /* The setup section sets up the printing device. We include the font finding
-in here, as it seems the right place. If output_includefont is set, include the
-music font(s) in the output file. */
+in here, as it seems the right place. Include any relevant fonts in the output
+file. */
 
 fprintf(ps_file, "%%%%BeginSetup\n");
 
-if (output_includefont)
+for (i = 0; i < fonts_to_include_count; i++)
   {
-  if (used_pmw_music) include_font("/PMW-Music.pfa");
-  if (used_pmw_alpha) include_font("/PMW-Alpha");
+  uschar *psname = font_List[fonts_to_include[i]].psname;
+  include_font(psname, (Ustrcmp(psname, "PMW-Alpha") == 0)? "" : ".pfa");
   }
 
-/* Now list the other fonts */
+/* Now set up the fonts */
 
 for (i = 0; i < font_tablen; i++)
   {
@@ -2025,6 +2131,30 @@ if (ps_EPS)
       mac_muldiv(psytran(out_bbox[3]), main_magnification, 1000));
   }
 else fprintf(ps_file, "%%%%Pages: %d\n", count);
+
+/* Warn for unsupported Unicode code points */
+
+if (uunext > 0)
+  {
+  error_moan(ERR79, (uunext == 1)? " has":"s have", UNKNOWN_CHAR_S);
+  fprintf(stderr, "  ");
+  while (uunext > 0) fprintf(stderr, " U+%04X", unsupported_unicode[--uunext]);
+  if (uuoverflow) fprintf(stderr, " ...");
+  fprintf(stderr, "\n");
+  if (main_rc < rc_warning) main_rc = rc_warning;
+  }
+
+/* Warn for invalid Unicode code points in non-standardly encode fonts*/
+
+if (uinvnext > 0)
+  {
+  error_moan(ERR78, (uinvnext == 1)? " has":"s have", UNKNOWN_CHAR_N);
+  fprintf(stderr, "  ");
+  while (uinvnext > 0) fprintf(stderr, " U+%04X", invalid_unicode[--uinvnext]);
+  if (uinvoverflow) fprintf(stderr, " ...");
+  fprintf(stderr, "\n");
+  if (main_rc < rc_warning) main_rc = rc_warning;
+  }
 }
 
 /* End of ps.c */
