@@ -7,11 +7,11 @@
 typedef struct {
   int presence;
   int number;
-  char *text;
+  const char *text;
 } arg_result;
 
 enum { arg_present_not, arg_present_unkeyed, arg_present_keyed };
 
-extern int rdargs(int, char **, char *, arg_result *);
+extern int rdargs(int, char **, const char *, arg_result *);
 
 /* End of rdargs.h */
