@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 2nd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: December 2020 */
 
 
 /* This file contains part I of the code for reading in a PMW score file. */
@@ -23,12 +23,12 @@
 *         Local static variables                 *
 *************************************************/
 
-static int  read_count;
-static int  read_staves[STAVE_BITVEC_SIZE];   /* Map of staves actually read */
-static int  read_prevstave;                   /* Previous stave number */
+static usint read_staves[STAVE_BITVEC_SIZE];   /* Map of staves actually read */
+static int   read_prevstave;                   /* Previous stave number */
+static int   read_count;
 
-static BOOL read_in_quotes;
-static BOOL read_stavestart;   /* True if expecting staff start */
+static BOOL  read_in_quotes;
+static BOOL  read_stavestart;   /* True if expecting staff start */
 
 /* A dummy page structure specially for the string escape mechanism which is
 called to check strings and may need to handle a page number. */
@@ -100,32 +100,33 @@ while (inptr < inend)
     if (*inptr == '&') *temp++ = *inptr++; else
       {
       macrostr *mm = NULL;
-      BOOL had_semicolon = FALSE; 
-      unsigned int count = 1;
+      BOOL had_semicolon = FALSE;
+      usint count = 1;
+
+      /* Set up for a macro call. Note that macro names may start with a digit
+      and are case-sensitive, so we can't use next_word(). */
 
-      /* Set up for a macro call */
-      
       if (isalnum(*inptr))
         {
         int i = 0;
         tree_node *s;
-        uschar name[MAX_MACRONAME+1];
+        uschar name[WORD_BUFFERSIZE];
 
         name[i++] = *inptr++;
-        while (isalnum(*inptr)) 
+        while (isalnum(*inptr))
           {
-          if (i >= MAX_MACRONAME) 
-            error_moan(136, "name", MAX_MACRONAME);  /* Hard */
+          if (i >= WORD_BUFFERSIZE - 1)
+            error_moan(ERR136, "Macro name", WORD_BUFFERSIZE - 1);  /* Hard */
           name[i++] = *inptr++;
-          } 
+          }
         name[i] = 0;
-        
+
         if (*inptr == ';')
           {
           inptr++;                /* Optional semicolon after name is */
           had_semicolon = TRUE;   /*   skipped, and no args allowed */
-          }    
- 
+          }
+
         if ((s = Tree_Search(define_tree, name)) != NULL)
           {
           mm = (macrostr *)s->data;  /* Will be NULL for no replacement */
@@ -133,7 +134,7 @@ while (inptr < inend)
           }
         else
           {
-          error_moan(9, name);              /* Couldn't find name */
+          error_moan(ERR9, name);              /* Couldn't find name */
           goto END_MACRO;
           }
         }
@@ -145,7 +146,7 @@ while (inptr < inend)
         int len;
         if (sscanf((char *)(++inptr), "%u(%n", &count, &len) <= 0)
           {
-          error_moan(10, "Unsigned number followed by \"(\"");
+          error_moan(ERR10, "Unsigned number followed by \"(\"");
           goto END_MACRO;
           }
         inptr += len - 1;
@@ -154,7 +155,7 @@ while (inptr < inend)
 
       else
         {
-        error_moan(8);                      /* Bad uschar after '&' */
+        error_moan(ERR8);                      /* Bad uschar after '&' */
         goto END_MACRO;
         }
 
@@ -169,7 +170,7 @@ while (inptr < inend)
         {
         int k = Ustrlen(mm->text);
         if (outlen - 2 - (temp - outbuffer) < k + (inend - inptr))
-          error_moan(88);   /* hard error - buffer overflow */
+          error_moan(ERR88);   /* hard error - buffer overflow */
         Ustrcpy(temp, mm->text);
         temp += k;
         }
@@ -201,10 +202,10 @@ while (inptr < inend)
             int bracount = 0;
             BOOL inquotes = FALSE;
             uschar *ss = ap;
-            
-            if (argcount >= MAX_MACROARGS) 
-              error_moan(137, MAX_MACROARGS);  /* Hard */
- 
+
+            if (argcount >= MAX_MACROARGS)
+              error_moan(ERR137, MAX_MACROARGS);  /* Hard */
+
             while (++inptr < inend &&
                   ((*inptr != ',' && *inptr != ')') ||
                     bracount > 0 || inquotes))
@@ -224,33 +225,33 @@ while (inptr < inend)
                 }
               }
 
-            if (inptr >= inend) error_moan(99);
-             
-            if (i >= argcount) 
-              { 
-              args[i] = NULL; 
-              argcount++; 
+            if (inptr >= inend) error_moan(ERR99);
+
+            if (i >= argcount)
+              {
+              args[i] = NULL;
+              argcount++;
               }
-               
-            if (ap - ss > 0) 
-              { 
-              *ap++ = 0; 
-              args[i] = ss; 
+
+            if (ap - ss > 0)
+              {
+              *ap++ = 0;
+              args[i] = ss;
               }
 
             if (inptr >= inend || *inptr == ')')
-              { 
-              inptr++; 
-              break; 
+              {
+              inptr++;
+              break;
               }
             }
 
           /* Only one argument is currently allowed for a replication. Any
           others are ignored. */
 
-          if (mm == &replicate_macro && argcount > 1) error_moan(134);
+          if (mm == &replicate_macro && argcount > 1) error_moan(ERR134);
           }
-          
+
         /* Process the arguments for nested macro calls */
 
         for (i = 0; i < argcount; i++)
@@ -283,7 +284,7 @@ while (inptr < inend)
                 if (ss != NULL)
                   {
                   if (READ_BUFFERSIZE - 2 - (temp - outbuffer) <
-                    Ustrlen(ss)) error_moan(88);  /* hard */
+                    Ustrlen(ss)) error_moan(ERR88);  /* hard */
                   Ustrcpy(temp, ss);
                   temp += Ustrlen(ss);
                   }
@@ -292,7 +293,7 @@ while (inptr < inend)
             else
               {
               if (temp - outbuffer + 2 >= outlen)
-                error_moan(88);    /* hard error - buffer overflow */
+                error_moan(ERR88);    /* hard error - buffer overflow */
               *temp++ = *pp++;
               }
             }
@@ -373,7 +374,7 @@ for (;;)   /* Loop until a character is obtained or very end is reached */
       int size = READ_BUFFERSIZE - 4;
 
       len = 0;
-      for (;;)
+      for (;;)  /* Loop for concatenated lines */
         {
         if (Ufgets(tbuffer, size, input_file) != NULL)
           {
@@ -407,7 +408,7 @@ for (;;)   /* Loop until a character is obtained or very end is reached */
     read_chptr = this_buffer + 19;                /* just in case */
     read_endptr = read_chptr + 1;                 /* nothing left */
 
-    if (read_skipdepth > 0 || read_okdepth > 0) error_moan(18);
+    if (read_skipdepth > 0 || read_okdepth > 0) error_moan(ERR18);
 
     /* Real end */
 
@@ -437,7 +438,7 @@ for (;;)   /* Loop until a character is obtained or very end is reached */
   read_chptr  = this_buffer;
   read_endptr = this_buffer + len;
 
-  if (len >= READ_BUFFERSIZE - 5) error_moan(81);  /* give-up error */
+  if (len >= READ_BUFFERSIZE - 5) error_moan(ERR81);  /* give-up error */
 
   if (len > 0 && read_endptr[-1] == '\n') read_endptr--;
 
@@ -469,7 +470,7 @@ for (;;)   /* Loop until a character is obtained or very end is reached */
   while (*read_chptr == ' ' || *read_chptr == '\t') read_chptr++;
   if (*read_chptr++ == '*')
     {
-    if (isalpha(read_ch = *read_chptr++)) pre_process(); else error_moan(12);
+    if (isalpha(read_ch = *read_chptr++)) pre_process(); else error_moan(ERR12);
     read_chptr = read_endptr;
     read_in_quotes = FALSE;
     }
@@ -482,16 +483,15 @@ for (;;)   /* Loop until a character is obtained or very end is reached */
 
 
 /*************************************************
-*                Read next word                  *
+*          Read and lowercase next word          *
 *************************************************/
 
 /*
-Argument:   where to put the word
 Returns:    nothing
 */
 
 void
-read_word(uschar *word)
+next_word(void)
 {
 int i = 0;
 sigch();
@@ -499,12 +499,14 @@ if (isalpha(read_ch))
   {
   do
     {
-    word[i++] = tolower(read_ch);
+    if (i >= WORD_BUFFERSIZE - 1)
+      error_moan(ERR136, "Word", WORD_BUFFERSIZE - 1);  /* Hard */
+    read_word[i++] = tolower(read_ch);
     next_ch();
     }
-  while (isalnum(read_ch));
+  while (isalnum(read_ch) || read_ch == '_');
   }
-word[i] = 0;
+read_word[i] = 0;
 }
 
 
@@ -516,26 +518,28 @@ word[i] = 0;
 
 /* This procedure is used for reading file names and the like
 in heading directives. These are in quotes, but are not to be
-interpreted as PMW strings.
+interpreted as PMW strings. They are stored in read_word.
 
-Argument:    where to put the string
-Returns:     TRUE if OK, FALSE if quote missing
+Returns:     TRUE if OK, FALSE if starting quote missing
 */
 
 BOOL
-read_plainstring(uschar *s)
+read_plainstring(void)
 {
+int i = 0;
 sigch();
 if (read_ch != '\"') return FALSE;
 next_ch();
 while (read_ch != '\"' && read_ch != '\n')
   {
-  *s++ = read_ch;
+  if (i >= WORD_BUFFERSIZE - 1)
+    error_moan(ERR136, "String", WORD_BUFFERSIZE - 1);  /* Hard */
+  read_word[i++] = read_ch;
   next_ch();
   }
-*s = 0;
+read_word[i] = 0;
 if (read_ch == '\"') next_ch();
-  else error_moan(16, "Terminating quote missing");
+  else error_moan(ERR16, "Terminating quote missing");
 return TRUE;
 }
 
@@ -612,7 +616,7 @@ if (allowsign)
 
 if (!isdigit(read_ch))
   {
-  error_moan(10, allowsign? "Number" : "Unsigned number");
+  error_moan(ERR10, allowsign? "Number" : "Unsigned number");
   return FALSE;
   }
 
@@ -650,15 +654,17 @@ return (read_expect_integer(&x, TRUE, FALSE))? x : 0;
 
 /*
 Arguments:  none
-Returns:    the keysignature or zero on error
+Returns:    the key signature (C major after an error)
 */
 
 int
 read_key(void)
 {
-int key = 0;
+int key = C_key;
+
 sigch();
 read_ch = tolower(read_ch);
+
 if ('a' <= read_ch && read_ch <= 'g')
   {
   key = read_ch - 'a';
@@ -666,9 +672,21 @@ if ('a' <= read_ch && read_ch <= 'g')
   if (read_ch == '#')      { key += 7; next_ch(); }
     else if (read_ch == '$') { key += 14; next_ch(); }
   if (tolower(read_ch) == 'm') { key += 21; next_ch(); }
-  if (main_keysigtable[key] == -100) { key = 2; error_moan(24); }
+  if (main_keysigtable[key] == -100) { key = C_key; error_moan(ERR24); }
   }
-else error_moan(10, "Key signature");
+else if (read_ch == 'n') { key = N_key; next_ch(); }
+
+else if (read_ch == 'x')
+  {
+  int n;
+  next_ch();
+  if (!read_expect_integer(&n, FALSE, FALSE) || n == 0 || n > MAX_XKEYS)
+    error_moan(ERR144, MAX_XKEYS);
+  else
+    key = X_key + n - 1;
+  }
+
+else error_moan(ERR10, "Key signature");
 return key;
 }
 
@@ -703,7 +721,7 @@ if (d == time_common || d == time_cut)
   if (main_noteden > 1)
     {
     if (m%main_noteden == 0) m /= main_noteden;
-      else error_moan(102);
+      else error_moan(ERR102);
     }
   return (m << 16) | d;
   }
@@ -762,13 +780,13 @@ if (!gotnum)
 sigch();
 if (read_ch != '/')
   {
-  error_moan(10, "\"/\"");
+  error_moan(ERR10, "\"/\"");
   return 0;
   }
 
 next_ch();
 if (!read_expect_integer(&d, FALSE, FALSE)) return 0;
-if (d < 1 || d > 64 || d != (d & (-d))) error_moan(87); /* hard error */
+if (d < 1 || d > 64 || d != (d & (-d))) error_moan(ERR87); /* hard error */
 return read_scaletime((m << 16) | (n << 8) | d);
 }
 
@@ -832,7 +850,7 @@ if (list != NULL) while (*list)
 
 if (yield < 0)
   {
-  if (midi_filename != NULL) error_moan(110, text, string);
+  if (midi_filename != NULL) error_moan(ERR110, text, string);
   yield = 0;
   }
 
@@ -869,7 +887,7 @@ curmovt->barcount = 0;
 curmovt->baroffset = 0;
 curmovt->barnovector = NULL;           /* Got when first stave encountered */
 curmovt->heading = curmovt->footing = NULL;
-curmovt->key = 2;                      /* C major */
+curmovt->key = C_key;                  /* C major */
 curmovt->laststave = -1;
 curmovt->movt_opt = opt;
 curmovt->notespacing = store_Xget(8*sizeof(int));
@@ -904,7 +922,7 @@ appropriate stave data is fudged. */
 
 stave_key = curmovt->key;
 stave_transpose = curmovt->transpose;
-stave_key_tp = transpose_key(stave_key, stave_transpose, TRUE);
+stave_key_tp = transpose_key(stave_key, stave_transpose);
 }
 
 
@@ -981,7 +999,6 @@ READ_HEADING:
 for (;;)
   {
   next_heading();
-  if (main_rc >= rc_failed) break;
 
   /* The appearance of '[' indicates the start of the stave data. Set up things
   that can't be done till now. */
@@ -1003,38 +1020,37 @@ for (;;)
 
 /* Loop to deal with reading the stave part of a movement. */
 
-if (main_rc < rc_failed) for (;;)
+for (;;)
   {
   sigch();
-  if (read_ch == EOF || main_rc >= rc_failed) break;
+  if (read_ch == EOF) break;
 
   /* Deal with starting a new stave or movement. */
 
   if (read_stavestart)
     {
-    uschar word[100];
-    if (read_ch != '[') { error_moan(29); break; }
+    if (read_ch != '[') { error_moan(ERR29); break; }
     next_ch();
-    read_word(word);
+    next_word();
 
     /* It's another stave. Check that a valid number is given. If so, set up
     the index for its bars and get ready to read the rest of the data. */
 
-    if (Ustrcmp(word, "stave") == 0 || Ustrcmp(word, "staff") == 0)
+    if (Ustrcmp(read_word, "stave") == 0 || Ustrcmp(read_word, "staff") == 0)
       {
       int k;
       if (!read_expect_integer(&curstave, FALSE, FALSE))
         {
-        error_moan(30, "Stave number");  /* Failed */
+        error_moan(ERR30, "Stave number");  /* Failed */
         break;
         }
 
       /* Error if number too big or stave supplied twice */
 
       if (curstave > MAX_STAVE)
-        { error_moan(22, MAX_STAVE+1); break; }
+        { error_moan(ERR22, MAX_STAVE+1); break; }
       if (stavetable[curstave] != NULL)
-        { error_moan(31, curstave); break; }
+        { error_moan(ERR31, curstave); break; }
 
       /* Staves must be in order; missing staves are inserted as empty, except
       for stave 0, which is just omitted. This is historical really. The rest
@@ -1046,7 +1062,8 @@ if (main_rc < rc_failed) for (;;)
       for (k = read_prevstave+1; k <= curstave; k++)
         {
         int j;
-        mac_setstave(read_staves, k);
+
+        mac_setstave(read_staves, (usint)k);
         if (k > curmovt->laststave) curmovt->laststave = k;
 
         /* Set up the data structure for the stave */
@@ -1102,7 +1119,7 @@ if (main_rc < rc_failed) for (;;)
       /* We have to call transpose_key in order to get the value for the letter
       transformation set, even if there is a forced key signature */
 
-      stave_key_tp = transpose_key(stave_key, stave_transpose, TRUE);
+      stave_key_tp = transpose_key(stave_key, stave_transpose);
 
       stave_requiredbarlength = read_compute_barlength(curmovt->time);
 
@@ -1128,7 +1145,6 @@ if (main_rc < rc_failed) for (;;)
       read_prev_had_dbar = read_prev_had_ibar = FALSE;
       read_prev_barlinestyle = stave_barlinestyle;
 
-
       /* Set the bar number, and enter the normal reading state, with a
       pre-fixed "name" directive if necessary. */
 
@@ -1136,17 +1152,18 @@ if (main_rc < rc_failed) for (;;)
       read_stavestart = read_endstave = FALSE;
       sigch();
       if (read_ch == '\"' || (read_ch == 'd' && Ustrncmp(read_chptr, "raw ", 4) == 0))
-        Ustrcpy(read_stavedir, "name");
-          else read_stavedir[0] = 0;
+        {
+        Ustrcpy(read_word, "name");
+        read_stavedir = TRUE;
+        }
       read_chptr--;
       read_ch = '[';
       }
 
-
     /* It's another movement. Check for options, set up a new movement control
     block, and change state. */
 
-    else if (Ustrcmp(word, "newmovement") == 0)
+    else if (Ustrcmp(read_word, "newmovement") == 0)
       {
       int opt = movt_default;
       int ph = 0;
@@ -1154,19 +1171,19 @@ if (main_rc < rc_failed) for (;;)
       movtstr *newcurmovt;
 
       if (main_lastmovement >= main_max_movements)
-        { error_moan(120, main_max_movements); break; }   /* disaster */
+        { error_moan(ERR120, main_max_movements); break; }   /* disaster */
 
       sigch();
       while (read_ch != ']')
         {
-        read_word(word);
+        next_word();
         sigch();
-        if (Ustrcmp(word, "thispage") == 0) opt = movt_thispage;
-        else if (Ustrcmp(word, "thisline") == 0) opt = movt_thisline;
-        else if (Ustrcmp(word, "newpage") == 0) opt = movt_newpage;
-        else if (Ustrcmp(word, "nopageheading") == 0) ph = movt_nopageheading;
-        else if (Ustrcmp(word, "uselastfooting") == 0) lf = movt_uselastfooting;
-        else error_moan(10,
+        if (Ustrcmp(read_word, "thispage") == 0) opt = movt_thispage;
+        else if (Ustrcmp(read_word, "thisline") == 0) opt = movt_thisline;
+        else if (Ustrcmp(read_word, "newpage") == 0) opt = movt_newpage;
+        else if (Ustrcmp(read_word, "nopageheading") == 0) ph = movt_nopageheading;
+        else if (Ustrcmp(read_word, "uselastfooting") == 0) lf = movt_uselastfooting;
+        else error_moan(ERR10,
           "\"thispage\", \"thisline\", \"newpage\", \"nopageheading\", or \"uselastfooting\"");
         }
       next_ch();
@@ -1180,7 +1197,7 @@ if (main_rc < rc_failed) for (;;)
 
     /* It's a disastrous error */
 
-    else { error_moan(29); break; }
+    else { error_moan(ERR29); break; }
     }
 
   /* Otherwise we are in the middle of reading a stave's data. Read another
@@ -1198,7 +1215,7 @@ if (main_rc < rc_failed) for (;;)
     while (stave_barrepeatcount--)
       {
       if (++stave_barnumber > curmovt->maxbarcount)
-        { error_moan(36, curmovt->maxbarcount); more = FALSE; break; }
+        { error_moan(ERR36, curmovt->maxbarcount); more = FALSE; break; }
 
       if (stave_totalnocount > (curmovt->barnovector)[stave_barnumber])
         (curmovt->barnovector)[stave_barnumber] = stave_totalnocount;
@@ -1234,7 +1251,7 @@ if (main_rc < rc_failed) for (;;)
 
       if (stave_pendulay != NULL)
         {
-        error_moan(65, curstave);   /* Give warning */
+        error_moan(ERR65, curstave);   /* Give warning */
         while (stave_pendulay != NULL)
           {
           ulaypend *next = stave_pendulay->next;
@@ -1252,7 +1269,7 @@ read_endmovement();            /* Terminate the final movement */
 store_free(next_buffer);
 store_free(this_buffer);
 store_free(prev_buffer);
-store_free(read_stavedir);
+store_free(read_word);
 
 if (input_file != NULL) fclose(input_file);
 while (read_filestackptr > 0) fclose(read_filestack[--read_filestackptr].file);
@@ -1261,7 +1278,7 @@ while (read_filestackptr > 0) fclose(read_filestack[--read_filestackptr].file);
 resetting reading_input, it won't try to reflect the input line. */
 
 reading_input = FALSE;
-if (main_format[0] != 0 && !main_format_tested) error_moan(104, main_format);
+if (main_format[0] != 0 && !main_format_tested) error_moan(ERR104, main_format);
 
 DEBUG(("read_go() end\n"));
 }
@@ -1289,7 +1306,7 @@ read_filestack = store_Xget(MAX_INCLUDE*sizeof(filestr));
 
 /* Set up buffers for line reading */
 
-read_stavedir = store_Xget(64);
+read_word = store_Xget(WORD_BUFFERSIZE);
 next_buffer = store_Xget(READ_BUFFERSIZE);
 this_buffer = store_Xget(READ_BUFFERSIZE);
 prev_buffer = store_Xget(READ_BUFFERSIZE);
@@ -1301,6 +1318,7 @@ read_filestackptr = 0;
 read_linenumber = read_count = error_count = main_rc = 0;
 read_skipdepth = read_okdepth = 0;
 read_EOF = FALSE;
+read_stavedir = FALSE;
 read_in_quotes = FALSE;
 read_chptr = this_buffer + 1;
 read_endptr = this_buffer;
@@ -1310,6 +1328,10 @@ read_ch = ' ';
 
 memcpy(main_notespacing, init_notespacing, 8*sizeof(int));
 
+/* All custom keys default to no key signature. */
+
+memset(main_xkeys, 0, sizeof(main_xkeys));
+
 /* Initialize movement table for first movement */
 
 main_lastmovement = 1;
@@ -1332,11 +1354,11 @@ font_table[font_sy] = font_search(US"Symbol");
 font_table[font_mf] = font_search(main_musicchoice);
 font_table[font_mu] = font_search(main_musicchoice);
 
-for (i = font_xx; i < font_xx + MaxExtraFont; i++)
+for (i = font_xx; i < font_tablen; i++)
   font_table[i] = font_search(US"Times-Roman");
 
 if (font_table[font_rm] < 0 || font_table[font_mf] < 0)
-  error_moan(25);    /* Hard */
+  error_moan(ERR25);    /* Hard */
 
 /* Point current page at a dummy (relevant for string escape checking) */
 
