@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: November 2020 */
 
 
 /* This file contains part II of the code for reading in a PMW score file. It
@@ -20,13 +20,31 @@ them. */
 
 #define oo offsetof
 
+/* Options for reading integer values */
+
 #define int_u       1    /* unsigned */
 #define int_s       2    /* signed */
 #define int_rs      3    /* relative if signed */
 #define int_less1   4    /* take away 1 (flag) */
 #define int_f       8    /* fixed point (flag, must be last) */
 
+/* Data for the b2pffont directive */
 
+#ifdef SUPPORT_B2PF
+typedef struct b2pfopt {
+  const char *name;
+  uint32_t option;
+} b2pfopt;
+
+static b2pfopt b2pf_options[] = {
+  { "input_backchars", B2PF_INPUT_BACKCHARS },
+  { "input_backcodes", B2PF_INPUT_BACKCODES },
+  { "output_backchars", B2PF_OUTPUT_BACKCHARS },
+  { "output_backcodes", B2PF_OUTPUT_BACKCODES }
+};
+
+#define b2pf_options_size (sizeof(b2pf_options)/sizeof(b2pfopt))
+#endif
 
 
 /*************************************************
@@ -169,7 +187,7 @@ if (isdigit(read_ch))
       next_ch();
       if (!read_expect_integer(&shear, TRUE, TRUE)) return;
       }
-    if (!stretchOK) error_moan(97, "allowed", "with this directive"); else
+    if (!stretchOK) error_moan(ERR97, "allowed", "with this directive"); else
       {
       int *matrix = store_Xget(4*sizeof(int));
       *matrixptr = matrix;
@@ -202,7 +220,7 @@ movt_fszproc(void)
 {
 sigch();
 if (isdigit(read_ch)) set_fontsize(read_dir->arg1, read_dir->arg2);
-  else error_moan(10, "Number");
+  else error_moan(ERR10, "Number");
 }
 
 
@@ -240,7 +258,7 @@ opt_boolproc(void)
 {
 if (main_lastmovement == 1)
   *global_vars[read_dir->arg1] = read_dir->arg2;
-else error_moan(20);
+else error_moan(ERR20);
 }
 
 
@@ -280,7 +298,7 @@ sigch();
 if (arg2 == int_rs && (read_ch == '+' || read_ch == '-')) value = *address;
 if (read_expect_integer(&x, read_dir->arg2 >= int_f, arg2 != int_u))
   *address = value + x;
-if (main_lastmovement != 1) error_moan(20);
+if (main_lastmovement != 1) error_moan(ERR20);
 }
 
 
@@ -306,8 +324,8 @@ while (isdigit(read_ch))
     if (!read_expect_integer(&t, FALSE, TRUE)) return;
     }
 
-  if (t < s) error_moan(21);
-    else if (t > MAX_STAVE) error_moan(22, MAX_STAVE+1);
+  if (t < s) error_moan(ERR21);
+    else if (t > MAX_STAVE) error_moan(ERR22, MAX_STAVE+1);
   else
     {
     stave_list *q = store_Xget(sizeof(stave_list));
@@ -346,8 +364,8 @@ while (isdigit(read_ch))
     if (!read_expect_integer(&t, FALSE, TRUE)) return;
     }
 
-  if (t < s) error_moan(21);
-    else if (t > MAX_STAVE) error_moan(22, MAX_STAVE + 1);
+  if (t < s) error_moan(ERR21);
+    else if (t > MAX_STAVE) error_moan(ERR22, MAX_STAVE + 1);
       else while (s <= t) { mac_setstave(read_map, s); s++; }  /* Can't put */
                                                                /* s++ inside */
   sigchNL();
@@ -422,12 +440,11 @@ if (!is_pshdr && isalpha(read_ch))
   int argcount = 0;
   drawitem *drawargs = NULL;
   drawitem args[20];
-  uschar word[80];
 
-  read_word(word);
-  if (Ustrcmp(word, "draw") != 0)
+  next_word();
+  if (Ustrcmp(read_word, "draw") != 0)
     {
-    error_moan(10, "\"draw\"");
+    error_moan(ERR10, "\"draw\"");
     return;
     }
 
@@ -456,11 +473,11 @@ if (!is_pshdr && isalpha(read_ch))
     for (i = 1; i <= argcount; i++) drawargs[i] = args[i];
     }
 
-  read_word(word);
-  node = Tree_Search(draw_tree, word);
+  next_word();
+  node = Tree_Search(draw_tree, read_word);
   if (node == NULL)
     {
-    error_moan(70, word);
+    error_moan(ERR70, read_word);
     return;
     }
   new->drawing = node;
@@ -550,13 +567,13 @@ new->stretch = 0;
 follows the heading; if not, default it. */
 
 sigch();
-if (!is_pshdr)
+if (is_pshdr) new->space = 0; else
   {
-  if (isdigit(read_ch) || ((read_ch == '+' || read_ch == '-') && 
+  if (isdigit(read_ch) || ((read_ch == '+' || read_ch == '-') &&
       isdigit(*read_chptr)))
     read_expect_integer(&(new->space), TRUE, TRUE);
   else new->space = defaultsize;
-  } 
+  }
 }
 
 
@@ -599,6 +616,98 @@ curmovt->accspacing = x;
 }
 
 
+/*************************************************
+*       Set up font for B2PF processing          *
+*************************************************/
+
+/* The b2pffont directive is available only if explicitely selected when PMW is
+built. It is permitted only in the first movement of a file (like textfont). */
+
+static void
+b2pffont(void)
+{
+#ifndef SUPPORT_B2PF
+error_moan(ERR151);   /* Hard error: not supported */
+#else
+
+int fontid, rc;
+unsigned int ln;
+uint32_t options = 0;
+
+if (main_lastmovement != 1) { error_moan(ERR20); return; }
+
+sigch();
+if ((fontid = font_fontword(FALSE)) < 0) return;
+if (font_b2pf_contexts[fontid] != NULL) error_moan(ERR154);  /* Hard */
+
+/* Read B2PF options */
+
+sigch();
+while (read_ch != '\"')
+  {
+  size_t i;
+  next_word();
+  for (i = 0; i < b2pf_options_size; i++)
+    {
+    if (Ustrcmp(read_word, b2pf_options[i].name) == 0)
+      {
+      options |= b2pf_options[i].option;
+      break;
+      }
+    }
+  if (i >= b2pf_options_size)
+    {
+    error_moan(ERR152, read_word);
+    break;
+    }
+  sigch();
+  }
+
+font_b2pf_options[fontid] = options;  /* Remember processing options */
+
+/* Read the B2PF context name */
+
+if (!read_plainstring())
+  {
+  error_moan(ERR10, "B2PF context name (quoted)");
+  return;
+  }
+
+/* Create a B2PF context for this font (there are currently no context create
+options). */
+
+rc = b2pf_context_create((const char *)read_word,
+  (const char *)font_data_extra, 0, font_b2pf_contexts + fontid,
+  NULL, NULL, NULL, &ln);
+
+/* If there are more strings in quotes, add them as extra context rules. */
+
+if (rc == B2PF_SUCCESS)
+  {
+  sigch();
+  while (read_ch == '\"')
+    {
+    (void)read_plainstring();  /* Can't fail if leading quote seen */
+    rc = b2pf_context_add_file(font_b2pf_contexts[fontid],
+      (const char *)read_word, (const char *)font_data_extra, 0, &ln);
+    if (rc != B2PF_SUCCESS) break;
+    sigch();
+    }
+  }
+
+if (rc != B2PF_SUCCESS)
+  {
+  size_t buffused;
+  char buffer[128];
+  (void)b2pf_get_error_message(rc, buffer, sizeof(buffer), &buffused, 0);
+  buffer[buffused] = 0;
+  error_moan(ERR155, buffer);  /* Hard */
+  }
+
+#endif  /* SUPPORT_B2PF */
+}
+
+
 /*************************************************
 *               Barlinespace                     *
 *************************************************/
@@ -609,7 +718,7 @@ barlinespace(void)
 sigch();
 if (read_ch == '*')
   {
-  curmovt->barlinespace = 0x80000000;
+  curmovt->barlinespace = (int)0x80000000;
   next_sigch();
   }
 else movt_intproc();
@@ -626,7 +735,7 @@ static void
 barnumberlevel(void)
 {
 sigch();
-if (read_ch != '+' && read_ch != '-') error_moan(10, "\"+\" or \"-\"");
+if (read_ch != '+' && read_ch != '-') error_moan(ERR10, "\"+\" or \"-\"");
   else movt_intproc();
 }
 
@@ -639,25 +748,24 @@ static void
 barnumbers(void)
 {
 int wordread = FALSE;
-uschar word[80];
 curmovt->barno_textflags = 0;
 sigch();
 if (isalpha(read_ch))
   {
-  read_word(word);
+  next_word();
   sigch();
-  if (Ustrcmp(word, "boxed") == 0) curmovt->barno_textflags = text_box;
-    else if (Ustrcmp(word, "ringed") == 0) curmovt->barno_textflags = text_ring;
+  if (Ustrcmp(read_word, "boxed") == 0) curmovt->barno_textflags = text_box;
+    else if (Ustrcmp(read_word, "ringed") == 0) curmovt->barno_textflags = text_ring;
       else wordread = TRUE;
   }
 
 if (!wordread && isalpha(read_ch))
-  { read_word(word); wordread = TRUE; }
+  { next_word(); wordread = TRUE; }
 
 if (wordread)
   {
-  if (Ustrcmp(word, "line") == 0) curmovt->barno_interval = -1;
-    else { error_moan(10, "\"line\""); return; }
+  if (Ustrcmp(read_word, "line") == 0) curmovt->barno_interval = -1;
+    else { error_moan(ERR10, "\"line\""); return; }
   }
 else
   {
@@ -679,7 +787,7 @@ bracestyle(void)
 movt_intproc();
 if (curmovt->bracestyle > 1)
   {
-  error_moan(37, "Number less than 2");
+  error_moan(ERR37, "Number less than 2");
   curmovt->bracestyle = 0;
   }
 }
@@ -695,7 +803,7 @@ breakbarlines(void)
 sigch();
 if (!isdigit(read_ch))
   {
-  int *map = (int *)(((uschar *)curmovt) + read_dir->arg1);
+  usint *map = (usint *)(((uschar *)curmovt) + read_dir->arg1);
   mac_initstave(map, -1);
   }
 else movt_mapproc();
@@ -725,7 +833,7 @@ caesurastyle(void)
 movt_intproc();
 if (curmovt->caesurastyle > 1)
   {
-  error_moan(37, "Number less than 2");
+  error_moan(ERR37, "Number less than 2");
   curmovt->caesurastyle = 0;
   }
 }
@@ -744,7 +852,7 @@ if (isdigit(read_ch))
   set_fontsize(oo(fontsizestr,fontsize_clefs), FALSE);
   (curmovt->fontsizes)->fontsize_clefs *= 10;
   }
-else error_moan(10, "Number");
+else error_moan(ERR10, "Number");
 }
 
 
@@ -756,7 +864,7 @@ static void
 clefstyle(void)
 {
 movt_intproc();
-if (curmovt->clefstyle > 3) error_moan(10, "Number less than 4");
+if (curmovt->clefstyle > 3) error_moan(ERR10, "Number less than 4");
 }
 
 
@@ -793,7 +901,7 @@ copyzero(void)
 zcopystr **pp = &(curmovt->zcopy);
 sigch();
 
-if (!isdigit(read_ch)) { error_moan(10, "stave number"); return; }
+if (!isdigit(read_ch)) { error_moan(ERR10, "stave number"); return; }
 
 while (isdigit(read_ch))
   {
@@ -886,10 +994,8 @@ curmovt->time = read_scaletime(curmovt->time);
 static void
 hyphenstring(void)
 {
-uschar s[256];
-if (read_plainstring(s))
-  curmovt->hyphenstring = store_copystring(s);
-else error_moan(10, "String");
+if (read_plainstring()) curmovt->hyphenstring = store_copystring(read_word);
+  else error_moan(ERR10, "String");
 }
 
 
@@ -906,15 +1012,14 @@ for (;;)
   sigch();
   if (isalpha(read_ch))
     {
-    uschar word[256];
     uschar *backupptr = read_chptr;
     int  backup = read_ch;
-    read_word(word);
-    if (Ustrcmp(word, "top") == 0)         yield |= just_top;
-    else if (Ustrcmp(word, "bottom") == 0) yield |= just_bottom;
-    else if (Ustrcmp(word, "left") == 0)   yield |= just_left;
-    else if (Ustrcmp(word, "right") == 0)  yield |= just_right;
-    else if (Ustrcmp(word, "all") == 0)    yield |= just_all;
+    next_word();
+    if (Ustrcmp(read_word, "top") == 0)         yield |= just_top;
+    else if (Ustrcmp(read_word, "bottom") == 0) yield |= just_bottom;
+    else if (Ustrcmp(read_word, "left") == 0)   yield |= just_left;
+    else if (Ustrcmp(read_word, "right") == 0)  yield |= just_right;
+    else if (Ustrcmp(read_word, "all") == 0)    yield |= just_all;
     else { read_chptr = backupptr; read_ch = backup; break; }
     }
   else break;
@@ -936,10 +1041,103 @@ key(void)
 {
 curmovt->key = read_key();
 stave_key = curmovt->key;
-stave_key_tp = transpose_key(stave_key, stave_transpose, TRUE);
+stave_key_tp = transpose_key(stave_key, stave_transpose);
 }
 
 
+/*************************************************
+*                 Keytranspose                   *
+*************************************************/
+
+static void
+keytranspose(void)
+{
+int i, oldkey;
+keytransposestr *k;
+
+oldkey = read_key();
+if (oldkey == N_key)
+  {
+  error_moan(ERR10, "Transposable key signature");
+  return;
+  }
+
+for (k = main_keytranspose; k != NULL; k = k->next)
+  {
+  if (k->oldkey == oldkey) break;
+  }
+
+if (k == NULL)
+  {
+  k = store_Xget(sizeof(keytransposestr));
+  k->oldkey = oldkey;
+  k->next = main_keytranspose;
+  main_keytranspose = k;
+  for (i = 0; i < 12; i++)
+    {
+    k->newkeys[i] = -1;
+    k->letterchanges[i] = -1;
+    }
+  }
+
+for (;;)
+  {
+  int sign = 1;
+  next_sigch();
+  if (read_ch == '-')
+    {
+    sign = -1;
+    next_ch();
+    }
+  else if (!isdigit(read_ch)) break;
+
+  i = read_integer(FALSE) * sign;
+  while (i > 11) i -= 12;
+  while (i < 0) i += 12;
+  sigch();
+
+  if (read_ch != '=')
+    {
+    error_moan(ERR10, "'='");
+    goto ERROR;
+    }
+  next_ch();
+  k->newkeys[i] = read_key();
+
+  if (read_ch != '/')
+    {
+    error_moan(ERR10, "'/'");
+    goto ERROR;
+    }
+  else
+    {
+    int x;
+    next_ch();
+    if (!read_expect_integer(&x, FALSE, TRUE)) goto ERROR;
+    if (x < -6 || x > 6)
+      {
+      error_moan(ERR10, "Number in the range -6 to +6");
+      goto ERROR;
+      }
+    if (abs(x) > i + 1)
+      {
+      error_moan(ERR146, x, i);
+      goto ERROR;
+      }
+    k->letterchanges[i] = x;
+    }
+  }
+
+return;
+
+ERROR:
+k->newkeys[i] = oldkey;
+k->letterchanges[i] = 0;
+return;
+}
+
+
+
 /*************************************************
 *                 Landscape                      *
 *************************************************/
@@ -956,7 +1154,7 @@ temp = main_sheetwidth;
 main_sheetwidth = main_sheetheight;
 main_sheetheight = temp;
 opt_landscape = TRUE;
-if (main_lastmovement != 1) error_moan(20);
+if (main_lastmovement != 1) error_moan(ERR20);
 }
 
 
@@ -978,7 +1176,7 @@ for (;;)
   sigch();
   if (!isdigit(read_ch))
     {
-    if (ptr == 0 || level > 0) { error_moan(10, "Number"); return; }
+    if (ptr == 0 || level > 0) { error_moan(ERR10, "Number"); return; }
 
     /* Final item is always repeat back to start */
 
@@ -998,7 +1196,7 @@ for (;;)
   read_expect_integer(&value, FALSE, FALSE);
   if (value == 0)
     {
-    error_moan(16, "Zero value changed to 1");
+    error_moan(ERR16, "Zero value changed to 1");
     value = 1;
     }
 
@@ -1032,7 +1230,7 @@ for (;;)
 
       else if (read_ch == ')')
         {
-        if (level <= 0) { error_moan(10, "Bracket not"); return; } else
+        if (level <= 0) { error_moan(ERR10, "Bracket not"); return; } else
           {
           temp[ptr++] = lv_repeatptr;
           temp[ptr++] = stack[--level];
@@ -1072,6 +1270,93 @@ curmovt->ledger = (curmovt->ledger == 0)? '=' : 184;
 }
 
 
+/*************************************************
+*             Makekey                            *
+*************************************************/
+
+/* Read the definition of a custom key signature. */
+
+static void
+makekey(void)
+{
+int i, n;
+
+sigch();
+if (toupper(read_ch) != 'X' ||
+    (next_ch(), !read_expect_integer(&n, FALSE, FALSE)) ||
+    n == 0 || n > MAX_XKEYS)
+  {
+  error_moan(ERR144, MAX_XKEYS);
+  return;
+  }
+
+n--;  /* Now in range 0-9 */
+for (i = 0; i < 10; i++) main_xkeys[n][i] = 0;  /* Reset all accidentals */
+
+for (;;)
+  {
+  int ac;
+
+  sigch();
+  switch(read_ch)
+    {
+    case '#':
+    next_ch();
+    if (read_ch == '#')
+      {
+      ac = ac_dsharp;
+      next_ch();
+      }
+    else
+      {
+      ac = ac_sharp;
+      if (read_ch == '-')
+        {
+        ac |= 0x80;
+        next_ch();
+        }
+      }
+    break;
+
+    case '$':
+    next_ch();
+    if (read_ch == '$')
+      {
+      ac = ac_dflat;
+      next_ch();
+      }
+    else
+      {
+      ac = ac_flat;
+      if (read_ch == '-')
+        {
+        ac |= 0x80;
+        next_ch();
+        }
+      }
+    break;
+
+    case '%':
+    ac = ac_natural;
+    next_ch();
+    break;
+
+    default:
+    return;
+    }
+
+  if (!read_expect_integer(&i, FALSE, FALSE)) return;
+  if (i > 9)
+    {
+    error_moan(10, "Number between 0 and 9");
+    return;
+    }
+
+  main_xkeys[n][i] = ac;
+  }
+}
+
+
 /*************************************************
 *             Maxbeamslope                       *
 *************************************************/
@@ -1109,7 +1394,6 @@ static void
 midichannel(void)
 {
 int channel;
-uschar string[80];
 
 debug_printf("midichannel start\n");
 
@@ -1119,22 +1403,22 @@ if (!read_expect_integer(&channel, FALSE, FALSE)) return;
 
 if (channel < 1 || channel > MIDI_MAXCHANNEL)
   {
-  error_moan(109, "channel", MIDI_MAXCHANNEL);
+  error_moan(ERR109, "channel", MIDI_MAXCHANNEL);
   channel = 1;
   }
 
 /* Deal with an optional voice setting */
 
-if (read_plainstring(string))
+if (read_plainstring())
   {
   int voicenumber;
-  if (string[0] == 0) voicenumber = 129; else  /* => don't do MIDI voice setting */
+  if (read_word[0] == 0) voicenumber = 129; else  /* => don't do MIDI voice setting */
     {
-    if (string[0] == '#') voicenumber = Uatoi(string+1);
-      else voicenumber = read_getmidinumber(midi_voicenames, string, US"voice");
+    if (read_word[0] == '#') voicenumber = Uatoi(read_word+1);
+      else voicenumber = read_getmidinumber(midi_voicenames, read_word, US"voice");
     if (voicenumber < 1 || voicenumber > 128)
       {
-      if (midi_filename != NULL) error_moan(109, "voice", 128);
+      if (midi_filename != NULL) error_moan(ERR109, "voice", 128);
       voicenumber = 1;
       }
     }
@@ -1150,7 +1434,7 @@ if (read_plainstring(string))
     next_ch();
     if (read_expect_integer(&vol, FALSE, FALSE))
       {
-      if (vol > 15) error_moan(10, "Number between 0 and 15"); else
+      if (vol > 15) error_moan(ERR10, "Number between 0 and 15"); else
         {
         copy_midi_map(rh_midivolume, MIDI_MAXCHANNEL, &(curmovt->midi_volume));
         curmovt->midi_volume[channel-1] = vol;
@@ -1173,11 +1457,11 @@ if (isdigit(read_ch))
 
   /* Deal with optional 'pitch' forcing */
 
-  if (read_plainstring(string))
+  if (read_plainstring())
     {
-    if (string[0] == 0) pitch = 0;  /* => don't do MIDI pitch forcing */
-      else if (string[0] == '#') pitch = Uatoi(string+1);
-        else pitch = read_getmidinumber(midi_percnames, string, US"percussion instrument");
+    if (read_word[0] == 0) pitch = 0;  /* => don't do MIDI pitch forcing */
+      else if (read_word[0] == '#') pitch = Uatoi(read_word+1);
+        else pitch = read_getmidinumber(midi_percnames, read_word, US"percussion instrument");
     copy_midi_map(rh_midinote, MAX_STAVE+1, &(curmovt->midi_note));
     }
 
@@ -1214,7 +1498,7 @@ for (;;)
   sigch();
   if (!isdigit(read_ch))
     {
-    if (count == 0) { error_moan(10, "Number"); return; }
+    if (count == 0) { error_moan(ERR10, "Number"); return; }
     temp[0] = count;
     curmovt->midi_start = temp;
     return;
@@ -1302,7 +1586,7 @@ static void
 page(void)
 {
 main_pageinc = 1;
-if (main_lastmovement != 1) { error_moan(20); return; }
+if (main_lastmovement != 1) { error_moan(ERR20); return; }
 if (!read_expect_integer(&main_firstpage, FALSE, FALSE)) return;
 sigch();
 if (!isdigit(read_ch)) return;
@@ -1340,7 +1624,7 @@ if (isdigit(read_ch))
     int tempo;
     if (read_ch != '/')
       {
-      error_moan(10, "/");
+      error_moan(ERR10, "/");
       break;
       }
     next_ch();
@@ -1348,13 +1632,13 @@ if (isdigit(read_ch))
 
     if (bar <= lastbar)
       {
-      if (!barerror) { error_moan(86); barerror = TRUE; }
+      if (!barerror) { error_moan(ERR86); barerror = TRUE; }
       }
     lastbar = bar;
 
     if (i > 99)
       {
-      error_moan(85, 50);
+      error_moan(ERR85, 50);
       break;
       }
 
@@ -1389,7 +1673,7 @@ while (isdigit(read_ch))
 
   if (read_ch != '/')
     {
-    error_moan(10, "/");
+    error_moan(ERR10, "/");
     return;
     }
 
@@ -1417,7 +1701,7 @@ if (!read_expect_integer(&v, FALSE, FALSE)) return;  /* Default setting */
 
 if (v > 15)
   {
-  error_moan(10, "Number between 0 and 15");
+  error_moan(ERR10, "Number between 0 and 15");
   return;
   }
 
@@ -1434,7 +1718,7 @@ while (isdigit(read_ch))
   int stave = read_integer(FALSE);
   if (read_ch != '/')
     {
-    error_moan(10, "/");
+    error_moan(ERR10, "/");
     break;
     }
   next_ch();
@@ -1442,7 +1726,7 @@ while (isdigit(read_ch))
 
   if (v > 15)
     {
-    error_moan(10, "Number between 0 and 15");
+    error_moan(ERR10, "Number between 0 and 15");
     break;
     }
 
@@ -1460,9 +1744,26 @@ while (isdigit(read_ch))
 static void
 pmwversion(void)
 {
+BOOL ok = FALSE;
+int c = '=';
 int v;
+
+sigch();
+if (read_ch == '>' || read_ch == '<' || read_ch == '=')
+  {
+  c = read_ch;
+  next_ch();
+  }
 if (!read_expect_integer(&v, TRUE, FALSE)) return;
-if (v != version_fixed) error_moan(28, v, version_fixed);
+
+switch (c)
+  {
+  case '<': ok = version_fixed < v; break;
+  case '=': ok = version_fixed == v; break;
+  case '>': ok = version_fixed > v; break;
+  }
+
+if (!ok) error_moan(ERR28, c, v, version_fixed);
 }
 
 
@@ -1493,11 +1794,11 @@ if (read_ch == '/')
     {
     next_ch();
     if (!read_expect_integer(&fo, FALSE, FALSE)) return FALSE;
-    if ((fo -= 1) >= MaxFontSizes) error_moan(39, MaxFontSizes);
+    if ((fo -= 1) >= MaxFontSizes) error_moan(ERR39, MaxFontSizes);
     }
   else
     {
-    error_moan(10, "/s");
+    error_moan(ERR10, "/s");
     return FALSE;
     }
   *offset = fo;
@@ -1549,9 +1850,8 @@ p->cstring = (read_ch == '"')? string_check(string_read()) : US"";
 static void
 pssetup(void)
 {
-if (main_lastmovement != 1) error_moan(20); else
+if (main_lastmovement != 1) error_moan(ERR20); else
   {
-  uschar s[256];
   headstr *h = main_pssetup;
   headstr **hh = &main_pssetup;
 
@@ -1561,17 +1861,17 @@ if (main_lastmovement != 1) error_moan(20); else
     h = *hh;
     }
 
-  if (read_plainstring(s))
+  if (read_plainstring())
     {
     h = store_Xget(sizeof(headstr));
     h->next = NULL;
     h->size = 0;
-    h->text = store_copystring(s);
+    h->text = store_copystring(read_word);
     h->spaceabove = 0;
     *hh = h;
     }
 
-  else error_moan(10, "String");
+  else error_moan(ERR10, "String");
   }
 }
 
@@ -1583,17 +1883,26 @@ if (main_lastmovement != 1) error_moan(20); else
 static void
 rehearsalmarks(void)
 {
-uschar word[80];
 sigch();
-if (isalpha(read_ch))
+
+while (isalpha(read_ch))
   {
-  read_word(word);
+  next_word();
   sigch();
-  if (Ustrcmp(word, "boxed") == 0) curmovt->rehearsalstyle = text_box;
-    else if (Ustrcmp(word, "ringed") == 0) curmovt->rehearsalstyle = text_ring;
-      else if (Ustrcmp(word, "plain") == 0) curmovt->rehearsalstyle = 0;
-        else error_moan(10, "\"boxed\", \"ringed\", or \"plain\"");
+  if (Ustrcmp(read_word, "linestartleft") == 0 ||
+      Ustrcmp(read_word, "nolinestartleft") == 0)
+    {
+    curmovt->rehearsallsleft = (read_word[0] == 'l');
+    continue;
+    }
+
+  if (Ustrcmp(read_word, "boxed") == 0) curmovt->rehearsalstyle = text_box;
+    else if (Ustrcmp(read_word, "ringed") == 0) curmovt->rehearsalstyle = text_ring;
+      else if (Ustrcmp(read_word, "plain") == 0) curmovt->rehearsalstyle = 0;
+        else error_moan(ERR10, "\"boxed\", \"ringed\", or \"plain\"");
+  break;
   }
+
 set_fontsize(oo(fontsizestr, fontsize_rehearse), TRUE);
 set_fontname(oo(movtstr, font_rehearse));
 }
@@ -1618,9 +1927,8 @@ opt_intproc();
 static void
 sheetsize(void)
 {
-uschar word[80];
-read_word(word);
-if (Ustrcmp(word, "a4") == 0 || Ustrcmp(word, "A4") == 0)
+next_word();
+if (Ustrcmp(read_word, "a4") == 0 || Ustrcmp(read_word, "A4") == 0)
   {
   curmovt->truelinelength = 480000;
   main_truepagelength = 720000;
@@ -1628,7 +1936,7 @@ if (Ustrcmp(word, "a4") == 0 || Ustrcmp(word, "A4") == 0)
   main_sheetheight = 842000;
   opt_sheetsize = sheet_A4;
   }
-else if (Ustrcmp(word, "a3") == 0 || Ustrcmp(word, "A3") == 0)
+else if (Ustrcmp(read_word, "a3") == 0 || Ustrcmp(read_word, "A3") == 0)
   {
   curmovt->truelinelength = 730000;
   main_truepagelength = 1060000;
@@ -1636,7 +1944,7 @@ else if (Ustrcmp(word, "a3") == 0 || Ustrcmp(word, "A3") == 0)
   main_sheetheight = 1190000;
   opt_sheetsize = sheet_A3;
   }
-else if (Ustrcmp(word, "a5") == 0 || Ustrcmp(word, "A5") == 0)
+else if (Ustrcmp(read_word, "a5") == 0 || Ustrcmp(read_word, "A5") == 0)
   {
   curmovt->truelinelength = 366000;
   main_truepagelength = 480000;
@@ -1644,7 +1952,7 @@ else if (Ustrcmp(word, "a5") == 0 || Ustrcmp(word, "A5") == 0)
   main_sheetheight = 595000;
   opt_sheetsize = sheet_A5;
   }
-else if (Ustrcmp(word, "b5") == 0 || Ustrcmp(word, "B5") == 0)
+else if (Ustrcmp(read_word, "b5") == 0 || Ustrcmp(read_word, "B5") == 0)
   {
   curmovt->truelinelength = 420000;
   main_truepagelength = 590000;
@@ -1652,7 +1960,7 @@ else if (Ustrcmp(word, "b5") == 0 || Ustrcmp(word, "B5") == 0)
   main_sheetheight = 709000;
   opt_sheetsize = sheet_B5;
   }
-else if (Ustrcmp(word, "letter") == 0)
+else if (Ustrcmp(read_word, "letter") == 0)
   {
   curmovt->truelinelength = 500000;
   main_truepagelength = 670000;
@@ -1660,8 +1968,8 @@ else if (Ustrcmp(word, "letter") == 0)
   main_sheetheight = 792000;
   opt_sheetsize = sheet_letter;
   }
-else error_moan(10, "\"A3\", \"A4\", \"A5\", \"B5\", or \"letter\"");
-if (main_lastmovement != 1) error_moan(20);
+else error_moan(ERR10, "\"A3\", \"A4\", \"A5\", \"B5\", or \"letter\"");
+if (main_lastmovement != 1) error_moan(ERR20);
 }
 
 
@@ -1675,11 +1983,10 @@ startbracketbar(void)
 sigch();
 if (isalpha(read_ch))
   {
-  uschar word[80];
-  read_word(word);
-  if (Ustrcmp(word, "join") == 0) curmovt->startjoin = TRUE;
-    else if (Ustrcmp(word, "nojoin") == 0) curmovt->startjoin = FALSE;
-      else { error_moan(10, "\"join\" or \"nojoin\""); return; }
+  next_word();
+  if (Ustrcmp(read_word, "join") == 0) curmovt->startjoin = TRUE;
+    else if (Ustrcmp(read_word, "nojoin") == 0) curmovt->startjoin = FALSE;
+      else { error_moan(ERR10, "\"join\" or \"nojoin\""); return; }
   }
 
 read_expect_integer(&curmovt->startbracketbar, FALSE, FALSE);
@@ -1702,8 +2009,8 @@ while (isdigit(read_ch))
   {
   int size;
   int stave = read_integer(FALSE);
-  if (stave > MAX_STAVE) { error_moan(22, MAX_STAVE+1); stave = 1; }
-  if (read_ch != '/') { error_moan(10, "/"); return; }
+  if (stave > MAX_STAVE) { error_moan(ERR22, MAX_STAVE+1); stave = 1; }
+  if (read_ch != '/') { error_moan(ERR10, "/"); return; }
   next_ch();
   if (!read_expect_integer(&size, TRUE, FALSE)) return;
   stavesizes[stave] = size;
@@ -1724,7 +2031,7 @@ BOOL first = TRUE;
 int i;
 int *newsp = store_Xget((MAX_STAVE+1) * sizeof(int));
 int *newen = store_Xget((MAX_STAVE+1) * sizeof(int));
-unsigned int done[STAVE_BITVEC_SIZE];
+usint done[STAVE_BITVEC_SIZE];
 
 newsp[0] = newen[0] = 0;
 mac_initstave(done, 0);
@@ -1755,7 +2062,7 @@ while (isdigit(read_ch))
       }
     else
       {
-      error_moan(10, "/");
+      error_moan(ERR10, "/");
       return;
       }
     }
@@ -1770,9 +2077,9 @@ while (isdigit(read_ch))
     if (!read_expect_integer(&space, TRUE, FALSE)) return;
     }
 
-  if (stave == 0) error_moan(107); else
+  if (stave == 0) error_moan(ERR107); else
     {
-    if (mac_teststave(done, stave)) error_moan(106, stave, US"stavespacing");
+    if (mac_teststave(done, stave)) error_moan(ERR106, stave, US"stavespacing");
     mac_setstave(done, stave);
     newsp[stave] = space;
     newen[stave] = ensure;
@@ -1835,13 +2142,12 @@ curmovt->startline = p;
 static void
 stemswap(void)
 {
-uschar word[100];
-read_word(word);
-if (Ustrcmp(word, "up") == 0) curmovt->stemswaptype = stemswap_up;
-else if (Ustrcmp(word, "down") == 0) curmovt->stemswaptype = stemswap_down;
-else if (Ustrcmp(word, "left") == 0) curmovt->stemswaptype = stemswap_left;
-else if (Ustrcmp(word, "right") == 0) curmovt->stemswaptype = stemswap_right;
-else error_moan(10, "\"up\", \"down\", \"left\", or \"right\"");
+next_word();
+if (Ustrcmp(read_word, "up") == 0) curmovt->stemswaptype = stemswap_up;
+else if (Ustrcmp(read_word, "down") == 0) curmovt->stemswaptype = stemswap_down;
+else if (Ustrcmp(read_word, "left") == 0) curmovt->stemswaptype = stemswap_left;
+else if (Ustrcmp(read_word, "right") == 0) curmovt->stemswaptype = stemswap_right;
+else error_moan(ERR10, "\"up\", \"down\", \"left\", or \"right\"");
 }
 
 
@@ -1870,7 +2176,7 @@ if (x > 0 && read_ch == '/')
     sigch();
     if (!isdigit(read_ch)) break;
     x = read_integer(FALSE);
-    if (read_ch != '/') { error_moan(10, "/"); return; }
+    if (read_ch != '/') { error_moan(ERR10, "/"); return; }
     }
   }
 
@@ -1907,43 +2213,60 @@ for (i = 2; i < 8 && isdigit(read_ch); i++)
 static void
 textfont(void)
 {
+BOOL include = FALSE;
 int fontid, n;
-uschar s[100];
 
 /* Font selection is permitted only in the first movement */
 
-if (main_lastmovement != 1) { error_moan(20); return; }
+if (main_lastmovement != 1) { error_moan(ERR20); return; }
 
-/* If font id given, use it, else read a word */
+/* If font id given (a music font), use it, else read a font word or "extra
+<n>", optionally followed by "include". */
 
-if (read_dir->arg1) fontid = read_dir->arg1; else
+if (read_dir->arg1 != 0) fontid = read_dir->arg1; else
   {
   if ((fontid = font_fontword(FALSE)) < 0) return;
+  sigch();
+  if (read_ch != '"')
+    {
+    uschar *save_chptr = read_chptr;
+    next_word();
+    if (Ustrcmp(read_word, "include") == 0) include = TRUE;
+      else read_chptr = save_chptr;
+    }
   }
 
 /* Read the first font name, which must be present, and the
 optional second name for the PostScript font. */
 
-if (!read_plainstring(s)) { error_moan(10, "String"); return; }
+if (!read_plainstring()) { error_moan(ERR10, "String"); return; }
 if (read_ch == ',') next_ch();
 
-/* See if is already in the font list. If it is, we set its
-number in the font table and return. If not, we set it up
-as a new font. */
+/* See if is already in the font list. If it is, we set its number in the font
+table, enable include if required, and return. If not, we set it up as a new
+font. */
 
-n = font_search(s);
-if (n >= 0) font_table[fontid] = n; else
+n = font_search(read_word);
+if (n >= 0)
+  {
+  font_table[fontid] = n;
+  if (include) font_List[n].include = TRUE;
+  }
+else
   {
   fontstr *fs;
-  if (font_count >= MAX_FONTS) { error_moan(26, MAX_FONTS); return; }
+  if (font_count >= MAX_FONTS) { error_moan(ERR26, MAX_FONTS); return; }
   fs = &(font_List[font_count]);
-  fs->psname = store_copystring(s);
+  fs->psname = store_copystring(read_word);
   fs->widths = NULL;
   fs->high_tree = NULL;
+  fs->utr = NULL;
+  fs->utrcount = 0;
   fs->heights = NULL;
   fs->kerns = NULL;
   fs->kerncount = -1;
   fs->stdencoding = fs->fixedpitch = fs->hasfi = FALSE;
+  fs->include = include;
   font_table[fontid] = font_count;
   font_loadtables(font_count++);
   }
@@ -1958,6 +2281,8 @@ if (fontid == font_mf) font_table[font_mu] = font_table[font_mf];
 *            Textsizes                           *
 *************************************************/
 
+/* Note: set_fontsize() does nothing if the next character is not a digit. */
+
 static void
 textsizes(void)
 {
@@ -1967,6 +2292,12 @@ for (i = 0; i < MaxFontSizes; i++)
   set_fontsize(oo(fontsizestr,fontsize_text) + i*sizeof(int), TRUE);
   if (read_ch == ',') next_ch();
   }
+sigch();
+if (isdigit(read_ch))
+  {
+  error_moan(ERR143, MaxFontSizes);
+  while (isdigit(read_ch) || isspace(read_ch) || read_ch == ',') next_ch();
+  }
 }
 
 
@@ -1991,14 +2322,18 @@ static void
 transpose(void)
 {
 int temp = curmovt->transpose;
+if (temp == no_transpose) temp = 0;
 movt_intproc();
-if (temp < max_transpose) curmovt->transpose += temp;
+curmovt->transpose += temp;
+if (abs(curmovt->transpose) > max_transpose)
+  error_moan(ERR139, (temp == 0)? "T":"Accumulated t", curmovt->transpose,
+    max_transpose);  /* Hard error */
 
 /* In case this is followed by heading/footing lines that contain transposed
 text, fudge up some fake stave values. The stave_key will already be set. */
 
 stave_transpose = curmovt->transpose;
-stave_key_tp = transpose_key(stave_key, stave_transpose, TRUE);
+stave_key_tp = transpose_key(stave_key, stave_transpose);
 }
 
 
@@ -2010,10 +2345,8 @@ static void
 transposedkey(void)
 {
 int oldkey = read_key();
-uschar word[80];
-
-read_word(word);
-if (Ustrcmp(word, "use") != 0) error_moan(10, "\"use\""); else
+next_word();
+if (Ustrcmp(read_word, "use") != 0) error_moan(ERR10, "\"use\""); else
   {
   int newkey = read_key();
   trkeystr *k = store_Xget(sizeof(trkeystr));
@@ -2033,11 +2366,10 @@ if (Ustrcmp(word, "use") != 0) error_moan(10, "\"use\""); else
 static void
 transposedacc(void)
 {
-uschar word[80];
-read_word(word);
-if (Ustrcmp(word, "force") == 0) main_transposedaccforce = TRUE; else
-  if (Ustrcmp(word, "noforce") == 0) main_transposedaccforce = FALSE;
-    else error_moan(10, "\"force\" or \"noforce\"");
+next_word();
+if (Ustrcmp(read_word, "force") == 0) main_transposedaccforce = TRUE; else
+  if (Ustrcmp(read_word, "noforce") == 0) main_transposedaccforce = FALSE;
+    else error_moan(ERR10, "\"force\" or \"noforce\"");
 }
 
 
@@ -2048,12 +2380,10 @@ if (Ustrcmp(word, "force") == 0) main_transposedaccforce = TRUE; else
 static void
 trillstring(void)
 {
-uschar s[256];
 sigch();
 if (isdigit(read_ch)) set_fontsize(oo(fontsizestr,fontsize_trill), TRUE);
-if (read_plainstring(s))
-  curmovt->trillstring = store_copystring(s);
-else error_moan(10, "String");
+if (read_plainstring()) curmovt->trillstring = store_copystring(read_word);
+else error_moan(ERR10, "String");
 }
 
 
@@ -2067,6 +2397,7 @@ else error_moan(10, "String");
 static dirstr read_headlist[] = {
   { "accadjusts",       accadjusts,     0, 0 },
   { "accspacing",       accspacing,     0, 0 },
+  { "b2pffont",         b2pffont,       0, 0 },
   { "bar",              movt_intproc,   oo(movtstr,baroffset), int_u+int_less1 },
   { "barcount",         movt_intproc,   oo(movtstr,maxbarcount),  int_u },
   { "barlinesize",      movt_intproc,   oo(movtstr,barlinesize), int_u+int_f },
@@ -2124,6 +2455,7 @@ static dirstr read_headlist[] = {
   { "key",              key,            0, 0 },
   { "keydoublebar",     movt_cboolproc, oo(movtstr,keydoublebar), TRUE },
   { "keysinglebar",     movt_cboolproc, oo(movtstr,keydoublebar), FALSE },
+  { "keytranspose",     keytranspose,   0, 0 },
   { "keywarn",          movt_cboolproc, oo(movtstr,keywarn), TRUE },
   { "landscape",        landscape,      0, 0 },
   { "lastfooting",      movt_headproc,  oo(movtstr,lastfooting), rh_lastfooting },
@@ -2133,6 +2465,7 @@ static dirstr read_headlist[] = {
   { "linelength",       movt_intproc,   oo(movtstr,truelinelength), int_rs+int_f },
   { "longrestfont",     movt_fontproc,  oo(movtstr,font_longrest), oo(fontsizestr, fontsize_restct) },
   { "magnification",    opt_intproc,    glob_magnification, int_u+int_f },
+  { "makekey",          makekey,        0, 0 },
   { "maxbeamslope",     maxbeamslope,   0, 0 },
   { "maxvertjustify",   opt_intproc,    glob_maxvertjustify, int_u+int_f },
   { "midichannel",      midichannel,    0, 0 },
@@ -2247,17 +2580,16 @@ void
 next_heading(void)
 {
 dirstr *first, *last;
-uschar word[80];
 
 sigch();
 if (read_ch == EOF) return;
-read_word(word);
+next_word();
 
 /* A null word is either the end of the heading, or an error */
 
-if (word[0] == 0)
+if (read_word[0] == 0)
   {
-  if (read_ch != '[') error_moan(10, "Heading directive");
+  if (read_ch != '[') error_moan(ERR10, "Heading directive");
   return;
   }
 
@@ -2271,7 +2603,7 @@ while (last > first)
   {
   int c;
   read_dir = first + (last-first)/2;
-  c = Ustrcmp(word, read_dir->name);
+  c = Ustrcmp(read_word, read_dir->name);
   if (c == 0)
     {
     (read_dir->proc)();
@@ -2279,7 +2611,7 @@ while (last > first)
     }
   if (c > 0) first = read_dir + 1; else last = read_dir;
   }
-error_moan(19, word);
+error_moan(ERR19, read_word);
 }
 
 /* End of read2.c */
