@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: November 2020 */
 
 
 /* This file contains part III of the code for reading in a PMW score file. It
@@ -17,23 +17,34 @@ contains top-level code for reading in one bar on one stave. */
 
 
 
-/* Key signatures tables giving the offsets of the accidentals, in order, for
+/* Key signatures tables giving the semitone offsets from C of the white notes
+to which accidentals apply, in order for conventional key signatures, for
 initializing baraccs. */
 
 static uschar tp_sharporder[] = { 5,0,7,2,9,4,11 };
 static uschar tp_flatorder[]  = { 11,4,9,2,7,0,5 };
 
+/* Values for baraccs for the different accidentals. */
+
+static uschar ba_values[] = { 2, 4, 1, 0, 2, 3 };
+
+/* Pitch offsets for custom keys */
+
+static int ba_xkey_offsets[] = { 4, 5, 7, 9, 11, 12, 14, 4, 5, 7 };
+
 
 
 /*************************************************
 *           Initialize accidentals table         *
 *************************************************/
 
-/* The baraccs table is used to hold the current accidental for a given "white
-note" pitch, to enable an absolute pitch to be calculated. To enable the table
-to be a byte table, the values are held offset by 2, so that, for example, 0 =>
-double flat, and 3 => sharp. When transposition is happening, the baraccs table
-is used for the input values, and baraccs_tp for the output values.
+/* The baraccs table is used to hold the number of semitones in the current
+accidental for a given "white note" pitch, to enable an absolute pitch to be
+calculated. To enable the table to be a byte table containing only positive
+values, the values are held offset by 2, so that, for example, 0 => double
+flat (-2), and 3 => sharp (+1). When transposition is happening, the baraccs
+table is used for the input values, and baraccs_tp for the output values. This
+function is called at the start of each bar, and also after [key] and [reset].
 
 Arguments:
   ba         the table that is to be set (baraccs or baraccs_tp)
@@ -47,31 +58,55 @@ read_initbaraccs(uschar *ba, int key)
 {
 int i;
 int value;
-int countacc = main_keysigtable[key];
-uschar *ordertab;
 
-if (countacc >= 0)
+/* Clear out the table to no accidentals */
+
+for (i = 0; i < baraccs_len; i++) ba[i] = ba_values[ac_natural];
+
+/* Handle conventional key signatures */
+
+if (key < X_key)
   {
-  ordertab = tp_sharporder;
-  value = 3;
+  int countacc = main_keysigtable[key];
+  uschar *ordertab;
+
+  if (countacc >= 0)
+    {
+    ordertab = tp_sharporder;
+    value = ba_values[ac_sharp];
+    }
+  else
+    {
+    ordertab = tp_flatorder;
+    value = ba_values[ac_flat];
+    countacc = -countacc;
+    }
+
+  /* Put in the key signature accidentals */
+
+  for (i = 0; i < baraccs_len; i += 12)
+    {
+    int j;
+    for (j = 0; j < countacc; j++) ba[i+ordertab[j]] = value;
+    }
   }
+
+/* Handle custom key signatures. The 10 values in the custom table refer to the
+10 positions on a treble-clef stave; we don't need to know the clef in order to
+set up the right pitches. */
+
 else
-  {
-  ordertab = tp_flatorder;
-  value = 1;
-  countacc = -countacc;
-  }
-
-/* Clear out the table to no accidental */
-
-for (i = 0; i < baraccs_len; i++) ba[i] = 0x02;
-
-/* Put in the key signature accidentals */
-
-for (i = 0; i < baraccs_len; i += 12)
   {
   int j;
-  for (j = 0; j < countacc; j++) ba[i+ordertab[j]] = value;
+  uschar *p = main_xkeys[key - X_key];   /* Custom accidental list */
+
+  for (j = 0; j < 10; j++)
+    {
+    int offset = ba_xkey_offsets[j];
+    int ac = ba_values[p[j] & 0x7f];     /* Ignore half-accidental bit */
+    if (ac == ba_values[ac_natural]) continue;
+    for (i = 0; i < baraccs_len; i += 12) ba[i + offset] = ac;
+    }
   }
 }
 
@@ -102,7 +137,7 @@ switch(read_ch)
   case 'u': sign = 1;  break;
   case 'd': sign = -1; break;
   default:
-  error_moan(37, "/l<n>, /r<n>, /lx, /rx, /lu<n>, /ld<n>, /ru<n> or /rd<n>");
+  error_moan(ERR37, "/l<n>, /r<n>, /lx, /rx, /lu<n>, /ld<n>, /ru<n> or /rd<n>");
   next_ch();
   return;
   }
@@ -133,6 +168,7 @@ read_initbaraccs(baraccs, stave_key);
 read_initbaraccs(baraccs_tp, stave_key_tp);
 
 stave_resetOK = FALSE;
+stave_string_followOK = FALSE;
 
 stave_barrepeatcount = 0;
 stave_barlength = stave_maxbarlength = stave_pletlen = 0;
@@ -153,7 +189,7 @@ read_lastensuredtie = NULL;
 /* Complain if too many bars */
 
 if (stave_barnumber > curmovt->maxbarcount)
-  { error_moan(36, curmovt->maxbarcount); return FALSE; }
+  { error_moan(ERR36, curmovt->maxbarcount); return FALSE; }
 
 /* Update the bar number printing vector if necessary */
 
@@ -199,7 +235,6 @@ index to null instead of inserting an End item. */
 for (;;)
   {
   sigch();
-  if (main_rc >= rc_failed) return FALSE;      /* disaster has struck */
   if (read_endstave || read_ch == EOF) break;  /* exit the loop */
 
   /* Deal with bar lines */
@@ -257,9 +292,9 @@ for (;;)
       {
       /* If a name is already read, handle it, possibly repeatedly. Note that
       at the start of a stave we enter this procedure with '[' set and possible
-      "name" in stavedir. That is why this test comes first. */
+      "name" in read_word. That is why this test comes first. */
 
-      while (read_stavedir[0]) read_stavedirective();
+      while (read_stavedir) read_stavedirective();
 
       sigch();
       if (read_ch == ']') { next_ch(); break; }
@@ -267,7 +302,11 @@ for (;;)
 
       /* Deal with the various things that can occur in brackets */
 
-      if (isalpha(read_ch)) read_word(read_stavedir);
+      if (isalpha(read_ch))
+        {
+        next_word();
+        read_stavedir = TRUE;
+        }
 
       /* Try for accent defaulting */
 
@@ -313,26 +352,26 @@ for (;;)
             {
             int x;
             next_ch();
-            error_skip = skip_OFF;  /* Temporarily no skip */ 
+            error_skip = skip_OFF;  /* Temporarily no skip */
             if (read_expect_integer(&x, FALSE, FALSE))
               {
               if (x <= dyn_max)
                 stave_accentflags |= accent_list[x];
-              else error_moan(128, x);
+              else error_moan(ERR128, x);
               }
-            error_skip = skip_KET;   
+            error_skip = skip_KET;
             }
 
-          else 
-            { 
-            error_moan(45);
+          else
+            {
+            error_moan(ERR45);
             if (read_ch == ']') break;
-            next_sigch(); 
+            next_sigch();
             }
           }
 
         if ((stave_accentflags & (af_staccato|af_staccatiss)) ==
-          (af_staccato|af_staccatiss)) error_moan(129);
+          (af_staccato|af_staccatiss)) error_moan(ERR129);
         }
 
       /* Deal with repeated bars; expect number > 0, but subtract one from it.
@@ -341,17 +380,18 @@ for (;;)
       else if (isdigit(read_ch))
         {
         int n = read_integer(FALSE);
-        if (n)
+
+        if (n > 0)
           {
           int done = FALSE;
           sigch();
           if (isalpha(read_ch))
             {
-            read_word(read_stavedir);
-            if (Ustrcmp(read_stavedir, "st") == 0 ||
-                Ustrcmp(read_stavedir, "nd") == 0 ||
-                Ustrcmp(read_stavedir, "rd") == 0 ||
-                Ustrcmp(read_stavedir, "th") == 0)
+            next_word();
+            if (Ustrcmp(read_word, "st") == 0 ||
+                Ustrcmp(read_word, "nd") == 0 ||
+                Ustrcmp(read_word, "rd") == 0 ||
+                Ustrcmp(read_word, "th") == 0)
               {
               b_nbarstr *p = store_getitem(b_nbar);
               p->x = p->y = 0;
@@ -361,58 +401,58 @@ for (;;)
 
               sigch();
               while (read_ch == '/')
-               {
-               int xsign = 1;
-               int ysign = 1;
+                {
+                int xsign = 1;
+                int ysign = 1;
 
-               next_ch();
+                next_ch();
 
-               switch(read_ch)
-                 {
-                 case 'd':
-                 ysign = -1;
-                 /* Fall through */
+                switch(read_ch)
+                  {
+                  case 'd':
+                  ysign = -1;
+                  /* Fall through */
 
-                 case 'u':
-                 next_ch();
-                 read_expect_integer(&n, TRUE, FALSE);
-                 p->y += n * ysign;
-                 break;
+                  case 'u':
+                  next_ch();
+                  read_expect_integer(&n, TRUE, FALSE);
+                  p->y += n * ysign;
+                  break;
 
-                 case 'l':
-                 xsign = -1;
-                 /* Fall through */
+                  case 'l':
+                  xsign = -1;
+                  /* Fall through */
 
-                 case 'r':
-                 next_ch();
-                 read_expect_integer(&n, TRUE, FALSE);
-                 p->x += n * xsign;
-                 break;
+                  case 'r':
+                  next_ch();
+                  read_expect_integer(&n, TRUE, FALSE);
+                  p->x += n * xsign;
+                  break;
 
-                 /* One day we might want individual sizes for strings, but for
-                 the moment this is not implemented. */
+                  /* One day we might want individual sizes for strings, but for
+                  the moment this is not implemented. */
 
-                 /*****************
-                 case 's':
-                 next_ch();
-                 read_expect_integer(&n, FALSE, FALSE);
-                 p->ssize = n;
-                 break;
-                 *****************/
+                  /*****************
+                  case 's':
+                  next_ch();
+                  read_expect_integer(&n, FALSE, FALSE);
+                  p->ssize = n;
+                  break;
+                  *****************/
 
-                 case '\"':
-                 p->s = string_read();
-                 break;
+                  case '\"':
+                  p->s = string_read();
+                  break;
 
-                 default:
-                 error_moan(10, "\"u\", \"d\", \"l\", \"r\", or quoted string");
-                 break;
-                 }
-               }
+                  default:
+                  error_moan(ERR10, "\"u\", \"d\", \"l\", \"r\", or quoted string");
+                  break;
+                  }
+                }
 
-              read_stavedir[0] = 0;
               done = TRUE;
               }
+            else read_stavedir = TRUE;
             }
 
           /* Repeated bar */
@@ -423,7 +463,7 @@ for (;;)
             stave_barrepeatptr = store_nextitem();
             }
           }
-        else error_moan(10, "Number greater than zero");
+        else error_moan(ERR10, "Number greater than zero");
         }
 
       /* Deal with rehearsal string */
@@ -432,7 +472,7 @@ for (;;)
 
       /* Unrecognized */
 
-      else error_moan(10, "Staff directive");
+      else error_moan(ERR10, "Staff directive");
       }
 
     error_skip = skip_EOL;
@@ -450,7 +490,7 @@ for (;;)
     case '{':
     if (stave_pletlen != 0)
       {
-      error_moan(38);
+      error_moan(ERR38);
       next_ch();
       }
     else
@@ -567,7 +607,7 @@ for (;;)
           break;
 
           default:
-          error_moan(37, "a, b, n, x, u, d, l or r");
+          error_moan(ERR37, "a, b, n, x, u, d, l or r");
           next_sigch();
           break;
           }
@@ -597,7 +637,7 @@ for (;;)
     occasionally arise here if another directive intervenes. */
 
     case '}':
-    if (stave_pletlen == 0) error_moan(38); else
+    if (stave_pletlen == 0) error_moan(ERR38); else
       {
       stave_pletlen = 0;
       (void)store_getitem(b_endplet);
@@ -616,7 +656,7 @@ for (;;)
       int ending = (type == '<' && stave_hairpinbegun == '<') ||
                    (type == '>' && stave_hairpinbegun == '>');
       int y = ending? 0 : stave_hairpiny;
-      int x = 0, h = 0;
+      int x = 0, h = 0, offset = 0;
       int slu = 0, sru = 0;
 
       next_ch();
@@ -639,8 +679,24 @@ for (;;)
           {
           case 'u': y += read_movevalue(); break;
           case 'd': y -= read_movevalue(); break;
-          case 'l': x -= read_movevalue(); break;
-          case 'r': x += read_movevalue(); break;
+
+          case 'l':
+          if (*read_chptr == 'c')
+            {
+            next_ch();
+            offset -= read_movevalue();
+            }
+          else x -= read_movevalue();
+          break;
+
+          case 'r':
+          if (*read_chptr == 'c')
+            {
+            next_ch();
+            offset += read_movevalue();
+            }
+          else x += read_movevalue();
+          break;
 
           case 'h':
           next_ch();
@@ -662,14 +718,14 @@ for (;;)
               }
             else
               {
-              error_moan(37, "u or d");
+              error_moan(ERR37, "u or d");
               break;
               }
             next_ch();
             if (!read_expect_integer(&amount, TRUE, FALSE)) break;
             *a += sign*amount;
             }
-          else error_moan(37, "\"slu\", \"sld\", \"sru\" or \"srd\"");
+          else error_moan(ERR37, "\"slu\", \"sld\", \"sru\" or \"srd\"");
           break;
 
           default:
@@ -708,12 +764,12 @@ for (;;)
               break;
 
               default:
-              error_moan(37,
+              error_moan(ERR37,
                 "\"u\", \"d\", \"l\", \"r\", \"a\", \"b\", \"m\", \"w\", \"slu\", \"sru\" or \"h\"");
               break;
               }
             }
-          else error_moan(37, "\"u\", \"d\", \"l\", \"r\", \"su\" or \"h\"");
+          else error_moan(ERR37, "\"u\", \"d\", \"l\", \"r\", \"su\" or \"h\"");
           break;
           }
         }
@@ -727,10 +783,11 @@ for (;;)
           p->flags = flags;
           p->x = x;
           p->y = y;
+          p->offset = offset;
           p->su = stave_hairpinsru;
           p->h = h;
           }
-        else p->flags = p->x = p->y = p->su = p->h = 0;
+        else p->flags = p->x = p->y = p->offset = p->su = p->h = 0;
         }
 
       if (ending) stave_hairpinbegun = 0; else
@@ -742,6 +799,7 @@ for (;;)
         p->y = y;
         p->su = slu;
         p->h = h;
+        p->offset = offset;
         p->width = width;
         stave_hairpinsru = sru;
         stave_resetOK = FALSE;
@@ -753,7 +811,7 @@ for (;;)
 
     case '/':
     next_ch();
-    if (read_ch != '/') error_moan(37, "\"/\""); else
+    if (read_ch != '/') error_moan(ERR37, "\"/\" (to make // for caesura)"); else
       {
       (void)store_getitem(b_caesura);
       next_ch();
@@ -774,7 +832,7 @@ for (;;)
       {
       (void)store_getitem(b_rrepeat);
       next_ch();
-      if (stave_barlength == 0) error_moan(138, "End", "start");  /* Warning */ 
+      if (stave_barlength == 0) error_moan(ERR138, "End", "start");  /* Warning */
       }
     else (void)store_getitem(b_dotbar);
     break;
@@ -787,7 +845,7 @@ for (;;)
     anyone will every feed PMW files with that set, but you never know). */
 
     case ';':
-    if (!opt_oldbeambreak) error_moan(5, "semicolon");
+    if (!opt_oldbeambreak) error_moan(ERR5, "semicolon");
     next_sigch();
     break;
 
@@ -795,7 +853,7 @@ for (;;)
     enough to be tied up with old beam breaks). */
 
     case ',':
-    error_moan(5, "comma");
+    error_moan(ERR5, "comma");
     next_sigch();
     break;
 
@@ -808,7 +866,7 @@ for (;;)
       store_getitem(b_lrepeat);
       next_ch();
       next_sigch();
-      if (read_ch == '|') error_moan(138, "Start", "end");  /* Warning */
+      if (read_ch == '|') error_moan(ERR138, "Start", "end");  /* Warning */
       break;
       }
 
@@ -905,7 +963,7 @@ else
     if (abs(extra) > TUPLET_ROUND)
       {
       format_movt = curmovt;
-      error_moan(49, stave_barnumber, curstave,
+      error_moan(ERR49, stave_barnumber, curstave,
         (extra > 0)? "long":"short", abs(extra));
       }
     }
@@ -968,7 +1026,7 @@ else
 
 /* Grumble if unclosed plet */
 
-if (stave_pletlen != 0) error_moan(83);
+if (stave_pletlen != 0) error_moan(ERR83);
 
 /* Check for end of file and return FALSE at end of stave */
 
