@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: November 2020 */
 
 
 /* This file contains part IV of the code for reading in a PMW score file. It
@@ -61,10 +61,10 @@ p_above(void)
 {
 int flag = FALSE;
 b_charvaluestr *p = store_getitem(read_dir->arg1);
-uschar word[80];
-read_word(word);
-if (Ustrcmp(word, "above") == 0) flag = TRUE;
-  else if (Ustrcmp(word, "below") != 0) error_moan(10, "\"above\" or \"below\"");
+next_word();
+if (Ustrcmp(read_word, "above") == 0) flag = TRUE;
+  else if (Ustrcmp(read_word, "below") != 0)
+    error_moan(ERR10, "\"above\" or \"below\"");
 p->value = flag;
 }
 
@@ -82,7 +82,7 @@ if (isdigit(read_ch))
   b_intvaluestr *p = store_getitem(read_dir->arg1);
   p->value = read_integer(TRUE);
   }
-else error_moan(10, "Number");
+else error_moan(ERR10, "Number");
 }
 
 
@@ -129,7 +129,7 @@ int *p = (read_dir->arg1 == 0)? &stave_fbsize :
          (read_dir->arg1 == 2)? &stave_ulsize : &stave_olsize;
 if (!read_expect_integer(&size, FALSE, FALSE)) return;
 if (--size < 0 || size >= MaxFontSizes)
-  { error_moan(39, MaxFontSizes); return; }
+  { error_moan(ERR39, MaxFontSizes); return; }
 *p = size;
 }
 
@@ -159,7 +159,7 @@ sigch();
 if (isdigit(read_ch))
   {
   int n = read_integer(FALSE);
-  if (n != 2 && n != 3) error_moan(10, "2 or 3");
+  if (n != 2 && n != 3) error_moan(ERR10, "2 or 3");
     else stave_accritvalue = n;
   }
 p->value = stave_accritvalue;
@@ -210,15 +210,16 @@ static const char *assume_list[] = {
 static void
 p_assume(void)
 {
-unsigned int i;
-read_word(read_stavedir);
-
+usint i;
+next_word();
 for (i = 0; i < (sizeof(assume_list)/sizeof(uschar *)); i++)
-  if (Ustrcmp(read_stavedir, assume_list[i]) == 0)
-    { read_assumeflag = TRUE; return; }
-
-read_stavedir[0] = 0;
-error_moan(10, "Clef, key, or time setting");
+  if (Ustrcmp(read_word, assume_list[i]) == 0)
+    {
+    read_assumeflag = TRUE;
+    read_stavedir = TRUE;
+    return;
+    }
+error_moan(ERR10, "Clef, key, or time setting");
 }
 
 
@@ -269,7 +270,7 @@ if (read_ch == '/')
 
       default:
       sign = 0;
-      error_moan(10, "/u, /d, /l, or /r");
+      error_moan(ERR10, "/u, /d, /l, or /r");
       break;
       }
 
@@ -280,14 +281,12 @@ if (read_ch == '/')
     *a = *a + sign *b;
     }
   }
+
 else if (isalpha(read_ch))
   {
-  read_word(read_stavedir);
-  if (Ustrcmp(read_stavedir, "off") == 0)
-    {
-    flag = FALSE;
-    read_stavedir[0] = 0;
-    }
+  next_word();
+  if (Ustrcmp(read_word, "off") == 0) flag = FALSE;
+    else read_stavedir = TRUE;
   }
 
 p->flag = flag;
@@ -303,12 +302,11 @@ p->y = y;
 static void
 p_couple(void)
 {
-uschar word[80];
-read_word(word);
-if (Ustrcmp(word, "up") == 0) stave_couplestate = +1;
-else if (Ustrcmp(word, "down") == 0) stave_couplestate = -1;
-else if (Ustrcmp(word, "off") == 0) stave_couplestate = 0;
-else error_moan(10, "\"up\", \"down\", or \"off\"");
+next_word();
+if (Ustrcmp(read_word, "up") == 0) stave_couplestate = +1;
+else if (Ustrcmp(read_word, "down") == 0) stave_couplestate = -1;
+else if (Ustrcmp(read_word, "off") == 0) stave_couplestate = 0;
+else error_moan(ERR10, "\"up\", \"down\", or \"off\"");
 }
 
 
@@ -324,11 +322,10 @@ stave_noteflags |= nf_cuesize;
 sigch();
 if (read_ch == '/')
   {
-  uschar word[80];
   next_ch();
-  read_word(word);
-  if (Ustrcmp(word, "dotalign") == 0) stave_noteflags |= nf_cuedotalign;
-  else error_moan(10, "\"dotalign\"");
+  next_word();
+  if (Ustrcmp(read_word, "dotalign") == 0) stave_noteflags |= nf_cuedotalign;
+    else error_moan(ERR10, "\"dotalign\"");
   }
 }
 
@@ -340,11 +337,10 @@ if (read_ch == '/')
 static void
 p_dots(void)
 {
-uschar word[80];
-read_word(word);
-if (Ustrcmp(word, "above") == 0) stave_noteflags &= ~nf_lowdot;
-else if (Ustrcmp(word, "below") == 0) stave_noteflags |= nf_lowdot;
-else error_moan(10, "\"above\" or \"below\"");
+next_word();
+if (Ustrcmp(read_word, "above") == 0) stave_noteflags &= ~nf_lowdot;
+else if (Ustrcmp(read_word, "below") == 0) stave_noteflags |= nf_lowdot;
+else error_moan(ERR10, "\"above\" or \"below\"");
 }
 
 
@@ -369,7 +365,6 @@ p_draw(void)
 tree_node *node;
 int argcount = 0;
 drawitem args [20];
-uschar word[80];
 
 sigch();
 while (isdigit(read_ch) || read_ch == '-' || read_ch == '+' || read_ch == '\"')
@@ -387,9 +382,9 @@ while (isdigit(read_ch) || read_ch == '-' || read_ch == '+' || read_ch == '\"')
   sigch();
   }
 
-read_word(word);
-node = Tree_Search(draw_tree, word);
-if (node == NULL) error_moan(70, word); else
+next_word();
+node = Tree_Search(draw_tree, read_word);
+if (node == NULL) error_moan(ERR70, read_word); else
   {
   b_drawstr *d = store_getitem(b_draw);
   d->overflag = read_dir->arg1;
@@ -429,7 +424,7 @@ sigch();
 if (read_ch == '/')
   {
   next_ch();
-  if (read_ch != '=') error_moan(10, "\"=\""); else
+  if (read_ch != '=') error_moan(ERR10, "\"=\""); else
     {
     next_ch();
     id = read_ch;
@@ -444,7 +439,7 @@ if (stave_slurcount-- > 0)
   }
 else
   {
-  error_moan(17, "end of slur or line - ignored");
+  error_moan(ERR17, "end of slur or line - ignored");
   stave_slurcount = 0;
   }
 }
@@ -484,15 +479,14 @@ read_headfootingtext(&(f->h), rh_footnote);
 static void
 p_hairpins(void)
 {
-uschar word[80];
 stave_hairpinflags = stave_hairpiny = 0;
-read_word(word);
-if (Ustrcmp(word, "below") == 0) stave_hairpinflags = hp_below;
-  else if (Ustrcmp(word, "middle") == 0)
+next_word();
+if (Ustrcmp(read_word, "below") == 0) stave_hairpinflags = hp_below;
+  else if (Ustrcmp(read_word, "middle") == 0)
     stave_hairpinflags = hp_below | hp_middle;
-      else if (Ustrcmp(word, "above") != 0)
+      else if (Ustrcmp(read_word, "above") != 0)
         {
-        error_moan(10, "\"above\", \"below\", or \"middle\"");
+        error_moan(ERR10, "\"above\", \"below\", or \"middle\"");
         return;
         }
 
@@ -552,20 +546,21 @@ int oldwidth, newwidth;
 stave_key = read_key();
 read_initbaraccs(baraccs, stave_key);
 
-stave_key_tp = transpose_key(stave_key, stave_transpose, TRUE);
+stave_key_tp = transpose_key(stave_key, stave_transpose);
 read_initbaraccs(baraccs_tp, stave_key_tp);
 
-read_word(read_stavedir);
-if (Ustrcmp(read_stavedir, "nowarn") == 0)
+sigch();
+if (isalpha(read_ch))
   {
-  read_stavedir[0] = 0;
-  warn = FALSE;
+  next_word();
+  if (Ustrcmp(read_word, "nowarn") == 0) warn = FALSE;
+    else read_stavedir = TRUE;
   }
 
 oldwidth = misc_keywidth(oldkey | 64, stave_clef);
 newwidth = misc_keywidth(stave_key_tp, stave_clef);
 
-/* If both the old (cancellation) width and the new key width are zero, there's 
+/* If both the old (cancellation) width and the new key width are zero, there's
 nothing we can do about warning. */
 
 if (oldwidth == 0 && newwidth == 0) warn = FALSE;
@@ -608,7 +603,6 @@ else
 static void
 p_justify(void)
 {
-uschar word[80];
 sigch();
 if (read_ch == '+' || read_ch == '-')
   {
@@ -618,15 +612,15 @@ if (read_ch == '+' || read_ch == '-')
     int opt = read_ch;
     int side;
     next_ch();
-    read_word(word);
+    next_word();
     sigch();
-    if (Ustrcmp(word, "top") == 0) side = just_top;
-    else if (Ustrcmp(word, "bottom") == 0) side = just_bottom;
-    else if (Ustrcmp(word, "left") == 0) side = just_left;
-    else if (Ustrcmp(word, "right") == 0) side = just_right;
+    if (Ustrcmp(read_word, "top") == 0) side = just_top;
+    else if (Ustrcmp(read_word, "bottom") == 0) side = just_bottom;
+    else if (Ustrcmp(read_word, "left") == 0) side = just_left;
+    else if (Ustrcmp(read_word, "right") == 0) side = just_right;
     else
       {
-      error_moan(10, "\"top\", \"bottom\", \"left\", or \"right\"");
+      error_moan(ERR10, "\"top\", \"bottom\", \"left\", or \"right\"");
       return;
       }
     p = store_getitem(b_justify);
@@ -634,7 +628,7 @@ if (read_ch == '+' || read_ch == '-')
     p->side = side;
     }
   }
-else error_moan(10, "\"+\" or \"-\"");
+else error_moan(ERR10, "\"+\" or \"-\"");
 }
 
 
@@ -672,7 +666,6 @@ while (read_ch == '/')
       {
       int argcount = 0;
       drawitem args[20];
-      uschar word[80];
       read_chptr += 4;
       read_ch = ' ';
       sigch();
@@ -698,11 +691,11 @@ while (read_ch == '/')
         drawargs[0].d.val = argcount;
         for (i = 1; i <= argcount; i++) drawargs[i] = args[i];
         }
-      read_word(word);
-      draw = Tree_Search(draw_tree, word);
-      if (draw == NULL) error_moan(70, word);
+      next_word();
+      draw = Tree_Search(draw_tree, read_word);
+      if (draw == NULL) error_moan(ERR70, read_word);
       }
-    else error_moan(10, "\"draw\"");
+    else error_moan(ERR10, "\"draw\"");
     break;
 
     case 'h':
@@ -755,7 +748,7 @@ while (read_ch == '/')
           next_ch();
           gaptext->flags |= text_box;
           }
-        else error_moan(10, "/box, /ring, or /s");
+        else error_moan(ERR10, "/box, /ring, or /s");
         break;
 
         case 'd':
@@ -770,7 +763,7 @@ while (read_ch == '/')
         next_ch();
         read_expect_integer(&size, FALSE, FALSE);
         if (--size < 0 || size >= MaxFontSizes)
-          { error_moan(39, MaxFontSizes); size = 0; }
+          { error_moan(ERR39, MaxFontSizes); size = 0; }
         gaptext->size = size;
         break;
 
@@ -791,14 +784,14 @@ while (read_ch == '/')
         break;
 
         default:
-        error_moan(10, "/box, /ring, or /s");
+        error_moan(ERR10, "/box, /ring, or /s");
         break;
         }
       }
     break;
 
     default:
-    error_moan(10, "=, l, r, or w");
+    error_moan(ERR10, "=, l, r, or w");
     break;
     }
 
@@ -806,7 +799,7 @@ while (read_ch == '/')
   }
 
 if (stave_slurcount <= 0)
-  error_moan(17, "%sgap directive", (read_dir->arg1)? "slur":"line");
+  error_moan(ERR17, "%sgap directive", (read_dir->arg1)? "slur":"line");
 
 /* Width defaults to width of text or 4 points */
 
@@ -880,23 +873,22 @@ p_midichannel(void)
 int channel;
 int voicenumber;
 int volume = 128;
-uschar string[80];
 if (!read_expect_integer(&channel, FALSE, FALSE)) return;
 if (channel < 1 || channel > MIDI_MAXCHANNEL)
   {
-  error_moan(109, MIDI_MAXCHANNEL);
+  error_moan(ERR109, MIDI_MAXCHANNEL);
   return;
   }
 
-if (read_plainstring(string))
+if (read_plainstring())
   {
-  if (string[0] == 0) voicenumber = 129; else /* => no change */
+  if (read_word[0] == 0) voicenumber = 129; else /* => no change */
     {
-    if (string[0] == '#') voicenumber = Uatoi(string+1);
-      else voicenumber = read_getmidinumber(midi_voicenames, string, US"voice");
+    if (read_word[0] == '#') voicenumber = Uatoi(read_word+1);
+      else voicenumber = read_getmidinumber(midi_voicenames, read_word, US"voice");
     if (voicenumber < 1 || voicenumber > 128)
       {
-      error_moan(109, "voice", 128);
+      error_moan(ERR109, "voice", 128);
       voicenumber = 1;
       }
     }
@@ -908,7 +900,7 @@ if (read_plainstring(string))
     if (read_expect_integer(&vol, FALSE, FALSE))
       {
       if (vol > 15)
-        error_moan(10, "Number between 0 and 15"); else volume = vol;
+        error_moan(ERR10, "Number between 0 and 15"); else volume = vol;
       }
     }
   }
@@ -923,22 +915,21 @@ static void
 p_midivoice(void)
 {
 int voicenumber;
-uschar string[80];
-if (read_plainstring(string))
+if (read_plainstring())
   {
-  if (string[0] == 0) voicenumber = 129; else  /* => no change */
+  if (read_word[0] == 0) voicenumber = 129; else  /* => no change */
     {
-    if (string[0] == '#') voicenumber = Uatoi(string+1);
-      else voicenumber = read_getmidinumber(midi_voicenames, string, US"voice");
+    if (read_word[0] == '#') voicenumber = Uatoi(read_word+1);
+      else voicenumber = read_getmidinumber(midi_voicenames, read_word, US"voice");
     if (voicenumber < 1 || voicenumber > 128)
       {
-      error_moan(109, "voice", 128);
+      error_moan(ERR109, "voice", 128);
       voicenumber = 1;
       }
     }
   makechange(128, voicenumber - 1, 128, 128, 0);  /* 128 => no change */
   }
-else error_moan(10, "string");
+else error_moan(ERR10, "string");
 }
 
 /*** Midipitch ***/
@@ -947,15 +938,14 @@ static void
 p_midipitch(void)
 {
 int note;
-uschar string[80];
-if (read_plainstring(string))
+if (read_plainstring())
   {
-  if (string[0] == 0) note = 0;  /* => no more forcing */
-    else if (string[0] == '#') note = Uatoi(string+1);
-      else note = read_getmidinumber(midi_percnames, string, US"percussion instrument");
+  if (read_word[0] == 0) note = 0;  /* => no more forcing */
+    else if (read_word[0] == '#') note = Uatoi(read_word+1);
+      else note = read_getmidinumber(midi_percnames, read_word, US"percussion instrument");
   makechange(128, 128, note, 128, 0);  /* 128 => no change */
   }
-else error_moan(10, "string");
+else error_moan(ERR10, "string");
 }
 
 /*** Playtranspose ***/
@@ -977,7 +967,7 @@ int volume;
 if (!read_expect_integer(&volume, FALSE, FALSE)) return;
 if (volume > 15)
   {
-  error_moan(10, "Number between 0 and 15");
+  error_moan(ERR10, "Number between 0 and 15");
   return;
   }
 makechange(128, 128, 128, volume, 0);
@@ -1024,6 +1014,9 @@ the fixed size is used. */
 static void
 p_name(void)
 {
+snamestr **pp, **savepp;
+BOOL reading_extra_strings = FALSE;
+
 sigch();
 
 /* Handle [name <n>] */
@@ -1036,29 +1029,40 @@ if (isdigit(read_ch))
   }
 
 /* Else handle any number of <string> <draw> pairs; either or both may be
-present in each case. */
+present in each case. Add onto any existing chain. This code is a mess because
+it was hacked to allow for drawings and then extra strings. */
+
+pp = &(stavehead->stave_name);
+while (*pp != NULL) pp = &((*pp)->next);
 
 for (;;)
   {
+  int size;
   uschar *ss;
   snamestr *p;
-  snamestr **pp;
-  int size = ff_offset_init;
+
+  /* Check for end of strings and drawings */
 
   sigch();
   if (read_ch != '\"' && (read_ch != 'd' || Ustrncmp(read_chptr, "raw ", 4) != 0))
     break;
 
+  /* Set up a stave name structure, either for a string or a drawing. Restart
+  here for extra strings that are associated with a "main" string. */
+
+  RESTART_STRING:
+  size = ff_offset_init;
   p = store_Xget(sizeof(snamestr));
-  pp = &(stavehead->stave_name);
-  while (*pp != NULL) pp = &((*pp)->next);
+  *pp = p;
+  pp = &(p->next);
 
   p->next = NULL;
+  p->extra = NULL;
   p->text = NULL;
   p->drawing = NULL;
   p->flags = 0;
 
-  /* Handle a text string */
+  /* Read a string with possible extra strings attached. */
 
   if (read_ch == '\"')
     {
@@ -1092,7 +1096,7 @@ for (;;)
         if (read_expect_integer(&size, FALSE, FALSE))
           {
           if (--size < 0 || size >= MaxFontSizes)
-            error_moan(39, MaxFontSizes);
+            error_moan(ERR39, MaxFontSizes);
           }
         else return;
         }
@@ -1101,14 +1105,36 @@ for (;;)
         p->flags |= snf_vertical;
         next_ch();
         }
+
+      /* One or more additional strings may hang off the extra field; this
+      allows for different options. Such strings must follow immediately. */
+
+      else if (read_ch == '\"')
+        {
+        if (!reading_extra_strings)
+          {
+          savepp = pp;
+          reading_extra_strings = TRUE;
+          }
+
+        p->offset = size;
+        pp = &(p->extra);
+        goto RESTART_STRING;
+        }
+
       else
         {
-        error_moan(10, "/c, /e, /m, /s or /v");
+        error_moan(ERR10, "/c, /e, /m, /s or /v");
         return;
         }
-      }
+      }   /* End of '/' loop */
 
     p->offset = size;
+    if (reading_extra_strings)
+      {
+      pp = savepp;
+      reading_extra_strings = FALSE;
+      }
     }
 
   /* Handle a drawing; might follow a string, so check again */
@@ -1116,7 +1142,6 @@ for (;;)
   sigch();
   if (read_ch == 'd' && Ustrncmp(read_chptr, "raw ", 4) == 0)
     {
-    uschar word[80];
     drawitem args[20];
     drawitem *drawargs = NULL;
     int argcount = 0;
@@ -1153,17 +1178,12 @@ for (;;)
       for (i = 1; i <= argcount; i++) drawargs[i] = args[i];
       }
 
-    read_word(word);
-    node = Tree_Search(draw_tree, word);
-    if (node == NULL) error_moan(70, word);
+    next_word();
+    node = Tree_Search(draw_tree, read_word);
+    if (node == NULL) error_moan(ERR70, read_word);
     p->drawing = node;
     p->args = drawargs;
     }
-
-  /* Set up to move on to another one */
-
-  *pp = p;
-  pp = &(p->next);
   }
 }
 
@@ -1177,7 +1197,7 @@ for (;;)
 static void
 p_newmovement(void)
 {
-error_moan(73);  /* this stops processing */
+error_moan(ERR73);  /* this stops processing */
 }
 
 
@@ -1220,29 +1240,28 @@ static void
 p_noteheads(void)
 {
 int nh;
-uschar word[80];
 b_noteheadsstr *p;
 
-read_word(word);
+next_word();
 stave_stemflag = nf_stem;
 
-if (Ustrcmp(word, "only") == 0)
+if (Ustrcmp(read_word, "only") == 0)
   {
   stave_stemflag = 0;
   nh = nh_only;
   }
-else if (Ustrcmp(word, "direct") == 0)
+else if (Ustrcmp(read_word, "direct") == 0)
   {
   stave_stemflag = 0;
   nh = nh_direct;
   }
-else if (Ustrcmp(word, "normal") == 0) nh = nh_normal;
-else if (Ustrcmp(word, "harmonic") == 0) nh = nh_harmonic;
-else if (Ustrcmp(word, "cross") == 0) nh = nh_cross;
-else if (Ustrcmp(word, "none") == 0) nh = nh_none;
+else if (Ustrcmp(read_word, "normal") == 0) nh = nh_normal;
+else if (Ustrcmp(read_word, "harmonic") == 0) nh = nh_harmonic;
+else if (Ustrcmp(read_word, "cross") == 0) nh = nh_cross;
+else if (Ustrcmp(read_word, "none") == 0) nh = nh_none;
 else
   {
-  error_moan(10, "\"normal\", \"harmonic\", \"cross\", \"none\", \"only\", or \"direct\"");
+  error_moan(ERR10, "\"normal\", \"harmonic\", \"cross\", \"none\", \"only\", or \"direct\"");
   return;
   }
 
@@ -1261,10 +1280,9 @@ p_notes(void)
 {
 int flag = FALSE;
 b_charvaluestr *p = store_getitem(b_notes);
-uschar word[80];
-read_word(word);
-if (Ustrcmp(word, "on") == 0) flag = TRUE;
-  else if (Ustrcmp(word, "off") != 0) error_moan(10, "\"on\" or \"off\"");
+next_word();
+if (Ustrcmp(read_word, "on") == 0) flag = TRUE;
+  else if (Ustrcmp(read_word, "off") != 0) error_moan(ERR10, "\"on\" or \"off\"");
 stave_notes = p->value = flag;
 }
 
@@ -1308,7 +1326,7 @@ else if (isdigit(read_ch) || read_ch == '+' || read_ch == '-')
     p->ns[i] = x;
     if (read_ch == ',') next_ch();
     }
-  if (i == 1) error_moan(89);  /* Single change only may be a typo: warn */
+  if (i == 1) error_moan(ERR89);  /* Single change only may be a typo: warn */
   }
 else store_getitem(b_ens);     /* Reset */
 }
@@ -1400,8 +1418,8 @@ if (stave_beaming) read_setbeamstems();
 if (stave_barlength > stave_maxbarlength) stave_maxbarlength = stave_barlength;
 
 if (!stave_resetOK)
-  error_moan((stave_barlength == 0)? 67 : 34);
-    else if (stave_pletlen) error_moan(35);
+  error_moan((stave_barlength == 0)? ERR67 : ERR34);
+    else if (stave_pletlen) error_moan(ERR35);
 
 /* We do the action anyway, to prevent spurious over-long line errors */
 
@@ -1547,7 +1565,7 @@ while (read_ch == '/')
 
   if (mods != NULL && mods->sequence != 0)
     {
-    if (strchr("=abeshiow", read_ch) != NULL) error_moan(113, read_ch);
+    if (strchr("=abeshiow", read_ch) != NULL) error_moan(ERR113, read_ch);
     }
 
   switch (read_ch)
@@ -1592,7 +1610,12 @@ while (read_ch == '/')
     case 'c':
     next_ch();
     if (mods == NULL) mods = findmods(0, &modchain);
-    if (read_ch == 'i' || read_ch == 'o')
+    if (read_ch == 'x')
+      {
+      next_ch();
+      flags |= sflag_cx;
+      }
+    else if (read_ch == 'i' || read_ch == 'o')
       {
       int s = read_ch == 'o'? +1 : -1;
       next_ch();
@@ -1627,9 +1650,9 @@ while (read_ch == '/')
           break;
           }
         }
-      else error_moan(10, "clu, cld, cll, clr, cru, crd, crl, or crr");
+      else error_moan(ERR10, "clu, cld, cll, clr, cru, crd, crl, or crr");
       }
-    else error_moan(10, "ci, co, clu, cld, cll, clr, cru, crd, crl, or crr");
+    else error_moan(ERR10, "ci, co, clu, cld, cll, clr, cru, crd, crl, or crr");
     break;
 
     case 'u':
@@ -1660,17 +1683,88 @@ while (read_ch == '/')
     case 'l':
     case 'r':
     if (mods == NULL) mods = findmods(0, &modchain);
-    a = (read_ch == 'l')? &(mods->ly) : &(mods->ry);
-    b = (read_ch == 'l')? &(mods->lx) : &(mods->rx);
+
+    x = (read_ch == 'l')? 0 : 1;
     next_ch();
-    if (read_ch != 'u' && read_ch != 'd' && read_ch != 'l' && read_ch != 'r')
-      error_moan(10, "lu, ld, ll, lr, ru, rd, rl, or rr");
+
+    switch(read_ch)
+      {
+      case 'l':
+      break;
+
+      case 'r':
+      x |= 2;
+      break;
+
+      case 'd':
+      x |= 4;
+      break;
+
+      case 'u':
+      x |= 6;
+      break;
+
+      default:
+      x = -1;
+      break;
+      }
+
+    next_ch();
+    if (read_ch == 'c')
+      {
+      next_ch();
+      if ((x & 4) == 0) x |= 8; else x = -1;
+      }
+
+    if (x < 0)
+      {
+      error_moan(ERR10, "lu, ld, ll, llc, lr, lrc, ru, rd, rl, rlc, rr, or rrc");
+      }
     else
       {
-      int  s = (read_ch == 'u' || read_ch == 'r')? +1 : -1;
-      int *z = (read_ch == 'l' || read_ch == 'r')? b : a;
-      next_ch();
-      if (!read_expect_integer(&x, TRUE, FALSE)) return;
+      int s;
+      int *z = NULL;  /* Stop compiler unset warning */
+
+      switch (x)   /* 12-15 won't occur because c is only with left/right */
+        {
+        case 0:  /* ll */
+        case 2:  /* lr */
+        z = &(mods->lx);
+        x -= 1;  /* -1 or +1 */
+        break;
+
+        case 1:  /* rl */
+        case 3:  /* rr */
+        z = &(mods->rx);
+        x -= 2;  /* -1 or +1 */
+        break;
+
+        case 4:  /* ld */
+        case 6:  /* lu */
+        z = &(mods->ly);
+        x -= 5;  /* -1 or +1 */
+        break;
+
+        case 5:  /* rd */
+        case 7:  /* ru */
+        z = &(mods->ry);
+        x -= 6;  /* -1 or +1 */
+        break;
+
+        case 8:  /* llc */
+        case 10: /* lrc */
+        z = &(mods->lxoffset);
+        x -= 9;  /* -1 or +1 */
+        break;
+
+        case 9:  /* rlc */
+        case 11: /* rrc */
+        z = &(mods->rxoffset);
+        x -= 10; /* -1 or +1 */
+        break;
+        }
+
+      if (!read_expect_integer(&s, TRUE, FALSE)) return;
       *z += x*s;
       }
     break;
@@ -1710,14 +1804,14 @@ while (read_ch == '/')
         {
         next_ch();
         if (read_ch != 'i' && read_ch != 'o')
-          error_moan(10, "slci or slco");
+          error_moan(ERR10, "slci or slco");
         else
           {
           s = (read_ch == 'o')? +1 : -1;
           z = b;
           }
         }
-      else error_moan(10, "u, d, ci or co");
+      else error_moan(ERR10, "u, d, ci or co");
 
       if (z != NULL)
         {
@@ -1726,7 +1820,7 @@ while (read_ch == '/')
         *z += s*x;
         }
       }
-    else error_moan(10, "sl.. or sr..");
+    else error_moan(ERR10, "sl.. or sr..");
     break;
 
     case 'h':
@@ -1748,7 +1842,7 @@ while (read_ch == '/')
     next_ch();
     if (read_ch == 'l') flags |= sflag_ol;
       else if (read_ch == 'r') flags |= sflag_or;
-        else error_moan(10, "ol or or");
+        else error_moan(ERR10, "ol or or");
     next_ch();
     break;
 
@@ -1761,11 +1855,11 @@ while (read_ch == '/')
     if (isdigit(read_ch))
       {
       int n = read_integer(FALSE);
-      if (n == 0) error_moan(37, "number greater than zero");
+      if (n == 0) error_moan(ERR37, "number greater than zero");
       mods = findmods(n, &modchain);
       sigch();
       }
-    else error_moan(10, "=, a, b, w, ci, co, d, e, u, lu, ld, ru, rd, h, i, ol, or, or number");
+    else error_moan(ERR10, "=, a, b, w, ci, co, d, e, u, lu, ld, ru, rd, h, i, ol, or, or number");
     break;
     }
 
@@ -1776,7 +1870,7 @@ while (read_ch == '/')
 
 if ((flags & sflag_w) != 0)
   {
-  if ((flags & sflag_l) != 0) error_moan(33, "lines");
+  if ((flags & sflag_l) != 0) error_moan(ERR33, "lines");
   }
 
 /* We don't support editorial marks on dotted or dashed slurs */
@@ -1786,7 +1880,7 @@ drawing the editorial mark through a space ...
 
 if ((flags & sflag_e) != 0)
   {
-  if ((flags & sflag_i) != 0) error_moan(94);
+  if ((flags & sflag_i) != 0) error_moan(ERR94);
   }
 */
 
@@ -1826,7 +1920,7 @@ p_stavelines(void)
 {
 int n;
 if (!read_expect_integer(&n, FALSE, FALSE)) return;
-if (n > 6) error_moan(10, "Number in the range 0-6");
+if (n > 6) error_moan(ERR10, "Number in the range 0-6");
   else stavehead->stavelines = n;
 }
 
@@ -1838,7 +1932,7 @@ if (n > 6) error_moan(10, "Number in the range 0-6");
 static void
 p_ss(void)
 {
-unsigned int done[STAVE_BITVEC_SIZE];
+usint done[STAVE_BITVEC_SIZE];
 
 sigch();
 mac_initstave(done, 0);
@@ -1853,7 +1947,7 @@ for (;;)
     {
     if (opt != ' ' || spacing < 0 || (spacing%1000) != 0)
       {
-      error_moan(10, "Stave number");
+      error_moan(ERR10, "Stave number");
       return;
       }
     stave = spacing/1000;
@@ -1862,7 +1956,7 @@ for (;;)
     if (!read_expect_integer(&spacing, TRUE, TRUE)) return;
     }
 
-  if (stave > MAX_STAVE) error_moan(42, MAX_STAVE); else
+  if (stave > MAX_STAVE) error_moan(ERR42, MAX_STAVE); else
     {
     b_ssstr *p = store_getitem(read_dir->arg1);
     p->opt = opt;
@@ -1870,7 +1964,7 @@ for (;;)
     p->value = spacing;
     }
 
-  if (mac_teststave(done, stave)) error_moan(106, stave, read_dir->name);
+  if (mac_teststave(done, stave)) error_moan(ERR106, stave, read_dir->name);
   mac_setstave(done, stave);
   sigch();
   if (read_ch == ',') next_sigch();
@@ -1887,12 +1981,11 @@ static void
 p_stems(void)
 {
 int *p = (read_dir->arg1 == 1)? &stave_stemforce : &stave_ties;
-uschar word[80];
-read_word(word);
-if (Ustrcmp(word, "auto") == 0) *p = 0;
-else if (Ustrcmp(word, "up") == 0 || Ustrcmp(word, "above") == 0) *p = +1;
-else if (Ustrcmp(word, "down") == 0 || Ustrcmp(word, "below") == 0) *p = -1;
-else error_moan(10, "\"auto\", \"above\", \"up\", \"below\", or \"down\"");
+next_word();
+if (Ustrcmp(read_word, "auto") == 0) *p = 0;
+else if (Ustrcmp(read_word, "up") == 0 || Ustrcmp(read_word, "above") == 0) *p = +1;
+else if (Ustrcmp(read_word, "down") == 0 || Ustrcmp(read_word, "below") == 0) *p = -1;
+else error_moan(ERR10, "\"auto\", \"above\", \"up\", \"below\", or \"down\"");
 }
 
 
@@ -1948,25 +2041,25 @@ static void
 p_text(void)
 {
 int sign = 1;
-uschar word[80];
-read_word(word);
+
+next_word();
 stave_textabsolute = 0;
 
-if (Ustrcmp(word, "underlay") == 0) stave_textflags = text_ul;
-else if (Ustrcmp(word, "overlay") == 0) stave_textflags = text_ul | text_above;
-else if (Ustrcmp(word, "fb") == 0) stave_textflags = text_fb;
+if (Ustrcmp(read_word, "underlay") == 0) stave_textflags = text_ul;
+else if (Ustrcmp(read_word, "overlay") == 0) stave_textflags = text_ul | text_above;
+else if (Ustrcmp(read_word, "fb") == 0) stave_textflags = text_fb;
 
 else
   {
-  if (Ustrcmp(word, "above") == 0) stave_textflags = text_above;
-  else if (Ustrcmp(word, "below") == 0)
+  if (Ustrcmp(read_word, "above") == 0) stave_textflags = text_above;
+  else if (Ustrcmp(read_word, "below") == 0)
     {
     stave_textflags = 0;
     sign = -1;
     }
   else
     {
-    error_moan(10, "\"underlay\", \"fb\", \"above\", or \"below\"");
+    error_moan(ERR10, "\"underlay\", \"fb\", \"above\", or \"below\"");
     return;
     }
 
@@ -2020,12 +2113,12 @@ if (t == tt) stave_matchnum = 0; else
 
 /* Now test for "nowarn". */
 
-read_word(read_stavedir);
-
-if (Ustrcmp(read_stavedir, "nowarn") == 0)
+sigch();
+if (isalpha(read_ch))
   {
-  read_stavedir[0] = 0;
-  warn = FALSE;
+  next_word();
+  if (Ustrcmp(read_word, "nowarn") == 0) warn = FALSE;
+    else read_stavedir = TRUE;
   }
 
 if (!read_assumeflag)
@@ -2057,9 +2150,12 @@ p_transpose(void)
 {
 int x;
 if (!read_expect_integer(&x, FALSE, TRUE)) return;
-if (stave_transpose >= max_transpose) stave_transpose = 0;
+if (stave_transpose == no_transpose) stave_transpose = 0;
 stave_transpose += x;
-(void)transpose_key(stave_key, stave_transpose, TRUE);
+if (abs(stave_transpose) > max_transpose)
+  error_moan(ERR139, (stave_transpose == x)? "T":"Accumulated t", stave_transpose,
+    max_transpose);  /* Hard error */
+(void)transpose_key(stave_key, stave_transpose);
 stave_lastbasenoteptr = NULL;
 }
 
@@ -2071,11 +2167,10 @@ stave_lastbasenoteptr = NULL;
 static void
 p_transposedacc(void)
 {
-uschar word[80];
-read_word(word);
-if (Ustrcmp(word, "force") == 0) stave_transposedaccforce = TRUE;
-  else if (Ustrcmp(word, "noforce") == 0) stave_transposedaccforce = FALSE;
-    else error_moan(10, "\"force\" or \"noforce\"");
+next_word();
+if (Ustrcmp(read_word, "force") == 0) stave_transposedaccforce = TRUE;
+  else if (Ustrcmp(read_word, "noforce") == 0) stave_transposedaccforce = FALSE;
+    else error_moan(ERR10, "\"force\" or \"noforce\"");
 }
 
 
@@ -2100,7 +2195,7 @@ while (read_ch == '/')
     next_ch();
     if (!read_expect_integer(xp, FALSE, FALSE)) return;
     }
-  else error_moan(10, "\"x\" or \"j\"");
+  else error_moan(ERR10, "\"x\" or \"j\"");
   sigch();
   }
 
@@ -2118,18 +2213,19 @@ p->join = join;
 static void
 p_tripletize(void)
 {
-read_word(read_stavedir);
-if (Ustrcmp(read_stavedir, "off") == 0)
+sigch();
+if (isalpha(read_ch)) next_word();
+  else Ustrcpy(read_word, "on");
+if (Ustrcmp(read_word, "off") == 0)
   {
-  read_stavedir[0] = 0;
   stave_noteflags &= ~nf_tripletize;
   }
 else
   {
+  if (Ustrcmp(read_word, "on") != 0) read_stavedir = TRUE;
   stave_noteflags |= nf_tripletize;
   stave_tripletize = TRUE;   /* Check bar for tripletizing */
   }
-if (Ustrcmp(read_stavedir, "on") == 0) read_stavedir[0] = 0;
 }
 
 
@@ -2146,15 +2242,15 @@ b_charvaluestr *p = store_getitem(b_tripsw);
 
 for (;;)
   {
-  read_word(read_stavedir);
+  next_word();
 
-  if (Ustrcmp(read_stavedir, "above") == 0)
+  if (Ustrcmp(read_word, "above") == 0)
     {
     stave_pletflags &= ~plet_b;
     stave_pletflags |=  plet_a;
     goto ADJUST;
     }
-   else if (Ustrcmp(read_stavedir, "below") == 0)
+   else if (Ustrcmp(read_word, "below") == 0)
     {
     stave_pletflags &= ~plet_a;
     stave_pletflags |=  plet_b;
@@ -2171,30 +2267,33 @@ ADJUST:
       if ((stave_pletflags & plet_b) != 0) stave_plety = -stave_plety;
       }
     }
-  else if (Ustrcmp(read_stavedir, "auto") == 0)
+  else if (Ustrcmp(read_word, "auto") == 0)
     {
     stave_pletflags &= ~(plet_a | plet_b | plet_abs | plet_bn | plet_by);
     stave_plety = 0;
     }
-  else if (Ustrcmp(read_stavedir, "bracket") == 0)
+  else if (Ustrcmp(read_word, "bracket") == 0)
     {
     stave_pletflags &= ~plet_bn;
     stave_pletflags |=  plet_by;
     }
-  else if (Ustrcmp(read_stavedir, "nobracket") == 0)
+  else if (Ustrcmp(read_word, "nobracket") == 0)
     {
     stave_pletflags &= ~plet_by;
     stave_pletflags |=  plet_bn;
     }
-  else if (Ustrcmp(read_stavedir, "off") == 0) flag = FALSE;
-  else if (Ustrcmp(read_stavedir, "on") == 0) flag = TRUE;
+  else if (Ustrcmp(read_word, "off") == 0) flag = FALSE;
+  else if (Ustrcmp(read_word, "on") == 0) flag = TRUE;
   else break;
 
   hadone = TRUE;
   }
 
 if (!hadone)
-  error_moan(10, "\"above\", \"below\", \"auto\", \"[no]bracket\", \"on\", or \"off\"");
+  {
+  read_stavedir = TRUE;
+  error_moan(ERR10, "\"above\", \"below\", \"auto\", \"[no]bracket\", \"on\", or \"off\"");
+  }
 
 p->value = flag;
 }
@@ -2330,6 +2429,7 @@ static dirstr read_stavedirlist[] = {
   { "soprabass",     p_clef,       clef_soprabass, FALSE },
   { "soprano",       p_clef,       clef_soprano, FALSE },
   { "space",         p_rspace,     FALSE, FALSE },
+  { "ssabove",       p_ss,         b_ssabove, TRUE },
   { "sshere",        p_ss,         b_sshere, TRUE },
   { "ssnext",        p_ss,         b_ssnext, TRUE },
   { "stafflines",    p_stavelines, 0, TRUE },
@@ -2372,9 +2472,8 @@ static int read_stavedirsize = sizeof(read_stavedirlist)/sizeof(dirstr);
 *            Handle a stave directive            *
 *************************************************/
 
-/* The directive name is already in read_stavedir. If processing
-this one reads one word ahead, it leaves that word in read_stavedir
-for next time.
+/* The directive name is already in read_word. If processing this one reads one
+word ahead, it sets read_stavedir TRUE.
 
 Arguments:  none
 Returns:    nothing
@@ -2386,14 +2485,14 @@ read_stavedirective(void)
 dirstr *first = read_stavedirlist;
 dirstr *last  = first + read_stavedirsize;
 
+read_stavedir = FALSE;
 while (last > first)
   {
   int c;
   read_dir = first + (last-first)/2;
-  c = Ustrcmp(read_stavedir, read_dir->name);
+  c = Ustrcmp(read_word, read_dir->name);
   if (c == 0)
     {
-    read_stavedir[0] = 0;
     (read_dir->proc)();
     if (!read_dir->arg2) stave_resetOK = FALSE;
     return;
@@ -2401,8 +2500,7 @@ while (last > first)
   if (c > 0) first = read_dir + 1; else last = read_dir;
   }
 
-error_moan(32, read_stavedir);
-read_stavedir[0] = 0;
+error_moan(ERR32, read_word);
 }
 
 
@@ -2420,11 +2518,10 @@ Returns:  the key
 
 int read_clef(void)
 {
-uschar word[256];
 dirstr *first, *last;
 
-read_word(word);
-if (word[0] == 0) { error_moan(105); return 0; }
+next_word();
+if (read_word[0] == 0) { error_moan(ERR105); return 0; }
 
 first = read_stavedirlist;
 last  = first + read_stavedirsize;
@@ -2433,7 +2530,7 @@ while (last > first)
   {
   int c;
   dirstr *d = first + (last-first)/2;
-  c = Ustrcmp(word, d->name);
+  c = Ustrcmp(read_word, d->name);
   if (c == 0)
     {
     if (d->proc == p_clef) return d-> arg1;
@@ -2442,7 +2539,7 @@ while (last > first)
   if (c > 0) first = d + 1; else last = d;
   }
 
-error_moan(126, word);
+error_moan(ERR126, read_word);
 return 0;
 }
 
