@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: August 2020 */
 
 
 /* This file contains part V of the code for reading in a PMW score file - code
@@ -86,7 +86,7 @@ int pitch;
 read_ch = tolower(read_ch);
 if ('a' > read_ch || read_ch > 'g')
   {
-  error_moan(10, "note letter");
+  error_moan(ERR10, "Note letter");
   return 128;
   }
 pitch = stave_octave + 36 + read_basicpitch[read_ch - 'a'];
@@ -128,14 +128,14 @@ while (read_ch == '/')
     case 'l':
     case 'r':
     if (ax == NULL)
-      error_moan(10, "/u or /d");
+      error_moan(ERR10, "/u or /d");
     else
       *ax += ((read_ch == 'r')?(+1):(-1)) * read_movevalue();
     break;
 
     default:
     if (af == NULL)
-      error_moan(10, "/u, /d, /l, or /r");
+      error_moan(ERR10, "/u, /d, /l, or /r");
     else switch (read_ch)
       {
       case 'b': *af &= ~(DO_SBRA|DO_SKET); *af |= DO_RBRA | DO_RKET; break;
@@ -146,7 +146,7 @@ while (read_ch == '/')
       case ']': *af &= ~DO_RKET; *af |= DO_SKET; break;
 
       default:
-      error_moan(10, "/u, /d, /l, /r, /b, /B, /(, /), /[, or /]");
+      error_moan(ERR10, "/u, /d, /l, /r, /b, /B, /(, /), /[, or /]");
       break;
       }
     if (read_ch != '\\') next_ch();  /* \ means there was an error */
@@ -202,19 +202,22 @@ while (p != NULL)
   uschar *s = p->string;
   b_textstr *q;
 
-  if (p->halfway)
+  if (p->halfway != 0 || p->offset != 0)
     {
     b_textXstr *xx = store_getitem(b_textX);
     xx->rotate = 0;
     xx->halfway = p->halfway;
+    xx->offset = p->offset;
     p->halfway = 0;
+    p->offset = 0;
     }
 
   q = store_getitem(b_text);
-  q->ulevel = p->level;
+  q->font = endfont = p->font;
   q->size = p->size;
   q->htype = p->htype;
-  q->font = endfont = p->font;
+  q->ulevel = p->level;
+
   q->flags = p->flags;
   q->string = s;
   q->x = p->x;
@@ -447,10 +450,10 @@ else if (length == 0)
 
 /* For a rest, unset grace stem forcing */
 
-else 
+else
   {
   stave_lastgracestem = 0;
-  } 
+  }
 
 /* We now need to deal with ties, glissando marks, beam breaks, and the ends of
 plet groups. We permit the plet group ending to come before any of the other
@@ -495,7 +498,7 @@ if (read_ch == '_')
       if (isdigit(read_ch)) count = read_integer(FALSE);
       if (read_ch == 'b') { bcount = count; acount = 0; }
         else if (read_ch == 'a') { acount = count; bcount = 0; }
-          else error_moan(37, "/a /b /e /g /i /p /s or /t");
+          else error_moan(ERR37, "/a /b /e /g /i /p /s or /t");
       flags |= tief_slur;
       }
     next_ch();
@@ -503,10 +506,10 @@ if (read_ch == '_')
 
   if ((flags & tief_editorial) != 0)
     {
-    if ((flags & (tief_dotted | tief_dashed)) != 0) error_moan(94);
+    if ((flags & (tief_dotted | tief_dashed)) != 0) error_moan(ERR94);
     }
 
-  if (minpitch == 0) error_moan(51); else
+  if (minpitch == 0) error_moan(ERR51); else
     {
     b_tiestr *p = store_getitem(b_tie);
     p->flags = flags;
@@ -564,7 +567,7 @@ else  /* Not oldbeambreak */
     }
   else if (read_ch == ',')
     {
-    unsigned int v;
+    usint v;
     SECBEAMBREAK:
     next_ch();
     if (isdigit(read_ch))
@@ -592,7 +595,7 @@ if (read_ch == '}')
 
 if (pletending)
   {
-  if (stave_pletlen == 0) error_moan(52); else
+  if (stave_pletlen == 0) error_moan(ERR52); else
     {
     stave_pletlen = 0;
     (void)store_getitem(b_endplet);
@@ -662,26 +665,32 @@ its predecessor). */
 
 if (stave_firstinbar) onlytieddup = TRUE;
 
-/* Handle exact repetition of the previous note. Repetition of the previous
-pitch(es) only is handled by the 'p' letter, and is mixed up with other
-interpretation below. */
+/* Handle exact duplication of the previous note or chord. Duplication of a
+previous note's pitch(es) only is handled by the 'p' letter, and is mixed up
+with other interpretation below. */
 
 if (read_ch == 'x')
   {
   int count = 1;
   next_sigch();
   if (isdigit(read_ch)) count = read_integer(FALSE);
-  
+
   if (read_ch == '\\')  /* No options are allowed */
     {
-    next_ch(); 
+    next_ch();
     error_skip = skip_BACKSLASH;
-    error_moan(135);
-    error_skip = skip_EOL;  
-    next_ch(); 
-    }  
- 
-  if (stave_lastbasenoteptr == NULL) { error_moan(115); return; }
+    error_moan(ERR135);
+    error_skip = skip_EOL;
+    next_ch();
+    }
+
+  if (stave_lastbasenoteptr == NULL)
+    {
+    error_moan(ERR115);
+    return;
+    }
+
+  /* Duplicate a note or chord */
 
   for (; count > 0; count--)
     {
@@ -735,7 +744,7 @@ if (read_ch == 'x')
     /* This error should never occur because the original chord should be
     diagnosed. Paranoia. */
 
-    if (chordcount >= MAX_CHORDSIZE) error_moan(132, MAX_CHORDSIZE); /* Hard */
+    if (chordcount >= MAX_CHORDSIZE) error_moan(ERR132, MAX_CHORDSIZE); /* Hard */
     if (new->spitch > maxpitch) maxpitch = new->spitch;
     if (new->spitch < minpitch) minpitch = new->spitch;
 
@@ -799,8 +808,8 @@ for (;;)
   BOOL acc_onenote = FALSE;
   BOOL duplicating = FALSE;
   uschar *acc_above = NULL;
+  usint acflags = 0;
   int flags = stave_noteflags | stave_stemflag | inchord;
-  int acflags = 0;
   int transposedaccforce = stave_transposedaccforce;
   int acc = 0;
   int accleft = 0;
@@ -913,7 +922,7 @@ for (;;)
           acc_invis = TRUE;
           acc_onenote = TRUE;
           }
-        else error_moan(96);
+        else error_moan(ERR96);
         }
 
       /* Deal with visible, normally placed, accidentals */
@@ -934,7 +943,8 @@ for (;;)
       }
     }
 
-  /* Now read note letter. */
+  /* Now read note letter. Non-letters will be assigned as crotchets. The only
+  relevant one currently is '/'. */
 
   if (isupper(read_ch))
     {
@@ -961,13 +971,13 @@ for (;;)
 
   if (read_ch == 'p')
     {
-    if (acc != 0) error_moan(114);
-    if (inchord) { error_moan(37, "note letter"); read_ch = 'a'; }
-    else if (stave_lastbasenoteptr == NULL) { error_moan(115); read_ch = 'r'; }
+    if (acc != 0) error_moan(ERR114);
+    if (inchord) { error_moan(ERR37, "Note letter"); read_ch = 'a'; }
+    else if (stave_lastbasenoteptr == NULL) { error_moan(ERR115); read_ch = 'r'; }
     else
       {
       int x = stave_lastbasenoteptr->truepitch;
-      if (stave_transpose < max_transpose) x -= stave_transpose;
+      if (stave_transpose != no_transpose) x -= stave_transpose;
       dup_octave = 12 - stave_octave;
       while (x < 48) { x += 12; dup_octave -= 12; }
       while (x > 59) { x -= 12; dup_octave += 12; }
@@ -995,7 +1005,7 @@ for (;;)
       }
     }
 
-  /* Deal with non-rests */
+  /* Deal with non-rests, i.e. notes */
 
   if ('a' <= read_ch && read_ch <= 'g')
     {
@@ -1005,7 +1015,9 @@ for (;;)
     if (!duplicating) onlytieddup = FALSE;
     }
 
-  /* Deal with rests */
+  /* Deal with rests. R is a normal rest, Q is a quiet (invisible) rest, S is
+  like R but whole bars do not pack up, and / is a "repeat beat" mark that is
+  handle as a differently printed crotchet rest. */
 
   else
     {
@@ -1013,16 +1025,17 @@ for (;;)
     acflags &= ~af_dynamics;         /* Ignore any default dynamics */
     yextra = stave_restlevel;
     if (read_ch == 'q') flags |= nf_hidden;
-      else if (read_ch == 's') nopack = TRUE;
-        else if (read_ch != 'r')
-          {
-          error_moan(37, inchord? "Note letter or end of chord" : 
-                                  "Note or rest letter");
-          pitch = 30;                 /* Take a random pitch */
-          inchord = FALSE;            /* Terminate a chord, in case missing ')' */
-          stave_checklength = FALSE;  /* Disable length check for this bar */
-          }
-    if (inchord && pitch == 0) error_moan(43);
+    else if (read_ch == 's') nopack = TRUE;
+    else if (read_ch == 't') flags |= nf_restrep;
+    else if (read_ch != 'r')
+      {
+      error_moan(ERR37, inchord? "Note letter or end of chord" :
+                                 "Note or rest letter");
+      pitch = 30;                 /* Take a random pitch */
+      inchord = FALSE;            /* Terminate a chord, in case missing ')' */
+      stave_checklength = FALSE;  /* Disable length check for this bar */
+      }
+    if (inchord && pitch == 0) error_moan(ERR43);
     }
 
   /* If suspended and not a rest, resume automatically */
@@ -1051,10 +1064,10 @@ for (;;)
     which may have been in the previous bar. Save the information in case this
     note is tied. */
 
-    if (chordcount >= MAX_CHORDSIZE) error_moan(132, MAX_CHORDSIZE); /* Hard */
+    if (chordcount >= MAX_CHORDSIZE) error_moan(ERR132, MAX_CHORDSIZE); /* Hard */
     this_tiedata[chordcount].pitch = pitch;
 
-    if (acc)
+    if (acc != 0)
       {
       int x = read_accpitch[acc];
       if (!acc_onenote) baraccs[pitch] = x;
@@ -1090,7 +1103,7 @@ TRANSPOSE:
 
     /* Transpose the note and its accidental if required */
 
-    if (stave_transpose < max_transpose)
+    if (stave_transpose != no_transpose)
       {
       abspitch = transpose_note(abspitch, &pitch, &acc, transposeacc,
         transposedaccforce, acc_onenote, FALSE, tiedcount);
@@ -1173,19 +1186,25 @@ TRANSPOSE:
     }
 
   /* If the accidental is invisible, cancel it now that the true pitch has been
-  determined. */
+  determined. Don't cancel for a rest so that it gets an error below. */
 
-  if (acc_invis) acc = 0;
+  if (acc_invis && pitch != 0) acc = 0;
 
-  /* Note or rest length adjustment by modifiers */
+  /* Note or rest length adjustment by modifiers. */
 
   if (read_ch == '!')
     {
+    if (notetype != minim) error_moan(ERR150);
     notetype = semibreve;
     length = stave_requiredbarlength;
-    if (pitch == 0) flags |= nf_centre;
+    if (pitch == 0)
+      {
+      flags |= nf_centre;
+      if ((flags & nf_restrep) != 0) nopack = TRUE;   /* Was T! */
+      }
     next_ch();
     }
+
   else
     {
     while (read_ch == '=' )
@@ -1194,6 +1213,7 @@ TRANSPOSE:
       { next_ch(); notetype += 1; length /= 2; }
     while (read_ch == '+')
       { next_ch(); notetype -= 1; length *= 2; }
+    if (notetype != crotchet) flags &= ~nf_restrep;  /* Ignore if not crotchet */
     }
 
   /* Set left movement for any accidental. We can't do this earlier, because
@@ -1230,11 +1250,11 @@ TRANSPOSE:
         {
         d->value = read_integer(TRUE);
         if (read_ch != '>' || *read_chptr != '.')
-          error_moan(37, "\">.\" (augmentation dot movement)");
+          error_moan(ERR37, "\">.\" (augmentation dot movement)");
         next_ch();
         }
       }
-    else error_moan(48, "Augmentation dot movement");
+    else error_moan(ERR48, "Augmentation dot movement");
     }
 
   /* Lengthen the note according to .+ . or .. */
@@ -1271,7 +1291,7 @@ TRANSPOSE:
   /* Can't handle anything longer than a breve or shorter than a
   hemidemisemiquaver. */
 
-  if (notetype < 0 || notetype > 7) error_moan(103);
+  if (notetype < 0 || notetype > 7) error_moan(ERR103);
 
   /* Now we have the final length for the note, adjust it if necessary to allow
   for triplet vs duplet time signatures, etc. */
@@ -1293,17 +1313,17 @@ TRANSPOSE:
     minimise error cascades when it is missing. */
 
     for (end_options = read_chptr;; end_options++)
-      { 
+      {
       if (end_options >= read_endptr ||
           (*end_options == '|' && end_options[-1] != '~' &&
             end_options[-1] != 't'))
         {
         error_skip = skip_BAR;
-        error_moan(133);
+        error_moan(ERR133);
         goto ENDACCENT;
         }
       if (*end_options == '\\') break;
-      } 
+      }
 
     /* Now process the options */
 
@@ -1325,6 +1345,11 @@ TRANSPOSE:
         else flags ^= nf_lowdot;
         break;
 
+        case ')':
+        next_ch();
+        flags |= nf_headbra;
+        break;
+
         case 'c':
         next_ch();
         explicit_couple = nf_couple;
@@ -1376,7 +1401,7 @@ TRANSPOSE:
                 else break;
           }
         if (masq < breve || masq > hdsquaver)
-          { error_moan(45); masq = -1; }
+          { error_moan(ERR45); masq = -1; }
 
         if (read_ch == '.')
           {
@@ -1390,17 +1415,24 @@ TRANSPOSE:
           }
         break;
 
+        case 'n':
+        next_ch();
+        if (read_ch == 'h') { flags |= nf_nhharmonic; next_ch(); }
+          else if (read_ch == 'x') { flags |= nf_nhcross; next_ch(); }
+            else error_moan(ERR45);
+        break;
+
         case 's':
         next_ch();
         if (read_ch == 'u')
           {
-          if (stemforce < 0) error_moan(46, inchord? "in chord" : "");
+          if (stemforce < 0) error_moan(ERR46, inchord? "in chord" : "");
           stemforce = 1;
           next_ch();
           }
         else if (read_ch == 'd')
           {
-          if (stemforce > 0) error_moan(46, inchord? "in chord" : "");
+          if (stemforce > 0) error_moan(ERR46, inchord? "in chord" : "");
           stemforce = -1;
           next_ch();
           }
@@ -1408,11 +1440,11 @@ TRANSPOSE:
           {
           if (stave_beaming && notetype >= quaver && stave_beamstemforce != 0)
             {
-            if (stemforce) error_moan(46, inchord? "in chord" : "");
+            if (stemforce) error_moan(ERR46, inchord? "in chord" : "");
             stave_beamstemforce = -stave_beamstemforce;
             if (abs(stave_beamstemforce) == 1) stave_beamstemforce *= 2;
             }
-          else error_moan(47);
+          else error_moan(ERR47);
           next_ch();
           }
         else if (read_ch == 'l')
@@ -1438,7 +1470,7 @@ TRANSPOSE:
           }
         else
           {
-          error_moan(10, "su, sd, sw, sl, sm, or sp");  /* skips to '\' or EOL */
+          error_moan(ERR10, "su, sd, sw, sl, sm, or sp");  /* skips to '\' or EOL */
           if (read_ch != '\\') goto ENDACCENT;
           }
         break;
@@ -1482,7 +1514,7 @@ TRANSPOSE:
               case or_turn:
               case or_iturn:
               if ((ornset & ornset_mordturn) != 0)
-                error_moan(45);   /* But carry on */
+                error_moan(ERR45);   /* But carry on */
               else ornset |= ornset_mordturn;
               break;
 
@@ -1490,7 +1522,7 @@ TRANSPOSE:
               case or_trem2:
               case or_trem3:
               if ((ornset & ornset_trem) != 0)
-                error_moan(45);   /* But carry on */
+                error_moan(ERR45);   /* But carry on */
               else ornset |= ornset_trem;
               break;
 
@@ -1499,7 +1531,7 @@ TRANSPOSE:
               case or_trfl:
               case or_trnat:
               if ((ornset & ornset_trill) != 0)
-                error_moan(45);   /* But carry on */
+                error_moan(ERR45);   /* But carry on */
               else ornset |= ornset_trill;
               break;
               }
@@ -1539,7 +1571,7 @@ TRANSPOSE:
               else ornmove(&(p->x), &(p->y),
                 (ap->flag >= or_arp)? NULL : &(p->bflags));
               }
-            else error_moan(48, "Ornaments");
+            else error_moan(ERR48, "Ornaments");
             }
 
           else    /* Accents */
@@ -1557,7 +1589,7 @@ TRANSPOSE:
                   }
                 }
               }
-            else error_moan(48, "Accents");
+            else error_moan(ERR48, "Accents");
             }
           }
 
@@ -1577,19 +1609,19 @@ TRANSPOSE:
                 acflags |= accent_list[x];
                 dynmove(x-1);
                 }
-              else error_moan(128, x);
+              else error_moan(ERR128, x);
               }
-            else error_moan(48, "Accents");
+            else error_moan(ERR48, "Accents");
             }
           error_skip = skip_BACKSLASH;
           }
 
-        /* Unknown ornament or expression. Give up after two successive errors. 
+        /* Unknown ornament or expression. Give up after two successive errors.
         */
 
         else
           {
-          error_moan(45);
+          error_moan(ERR45);
           if (had_error)
             {
             fprintf(stderr, "** Assuming missing backslash after two "
@@ -1605,7 +1637,7 @@ TRANSPOSE:
         }
 
       if ((acflags & (af_staccato|af_staccatiss)) ==
-        (af_staccato|af_staccatiss)) error_moan(129);
+        (af_staccato|af_staccatiss)) error_moan(ERR129);
       sigch();
       }                            /* end of loop */
 
@@ -1615,14 +1647,19 @@ TRANSPOSE:
   ENDACCENT:                       /* some error states jump here */
   error_skip = skip_EOL;           /* default setting */
 
-  /* Rests should not have ornaments or accidentals */
+  /* Rests should not have ornaments or accidentals or certain options. */
 
-  if (pitch == 0 && ((ornset != 0 && (flags & nf_hidden) == 0) ||
-    acc != 0 || (acflags & af_dynamics) != 0)) error_moan(68);
+  if (pitch == 0 && (
+      (ornset != 0 && (flags & nf_hidden) == 0) ||
+      (flags & nf_notrests) != 0 ||
+      acc != 0 ||
+      stemforce != 0 ||
+      (acflags & af_dynamics) != 0))
+    error_moan(ERR68);
 
   /* A small notehead may not be specified for a grace note. */
 
-  if (length == 0 && (flags & nf_smallhead) != 0) error_moan(108);
+  if (length == 0 && (flags & nf_smallhead) != 0) error_moan(ERR108);
 
   /* Accumulate bar length and check that chord notes are all same length. If
   not, and the current one is a crotchet (i.e. just a lower case letter) force
@@ -1648,7 +1685,7 @@ TRANSPOSE:
       notetype = stave_firstnoteptr->notetype;
       flags = (flags & ~nf_dotted) | (stave_firstnoteptr->flags & nf_dotted);
       }
-    else error_moan(44);
+    else error_moan(ERR44);
     }
 
   /* Allow automatic coupling flags through only if appropriately coupled, but
@@ -1702,13 +1739,13 @@ TRANSPOSE:
   if (pitch == 0 && length == stave_requiredbarlength && stave_checklength)
     flags |= nf_centre;
 
-  /* Now create a data block for the note, saving the address of the first one
-  in a chord for exact duplications ('x') and the first one in a
+  /* Now create a data block for the note or rest, saving the address of the
+  first one in a chord for exact duplications ('x') and the first one in a
   non-p-duplicated chord for subsequent 'p' duplications, except in the case of
   above/below accidentals. */
 
   noteptr = store_getitem(item);
-  if (stave_firstnoteptr == NULL)
+  if (stave_firstnoteptr == NULL)  /* Rest, single, or first note in a chord */
     {
     stave_firstnoteptr = noteptr;
     if (pitch != 0)
@@ -1785,21 +1822,23 @@ TRANSPOSE:
 
       this_tiedata[i] = stave_tiedata[i];
       }
-
-    noteptr[1].type = b_End;   /* Mark end for sorting */
     }
 
-  /* Check for end of chord; when found, insert a b_End item immediately after
-  the last note so that the function for sorting chords detects the end
+  /* Insert a b_End item immediately after this note in case it is the last
+  note of a chord, so that the function for sorting chords detects the end
   correctly. As there is always space for a b_Jump item after any item, this is
-  safe to do. */
+  safe to do. This is done unconditionally, because after an error inchord may
+  not be set at this point. */
+
+  noteptr[1].type = b_End;   /* Mark end for sorting */
+
+  /* Check for end of chord */
 
   if (inchord)
     {
     sigch();
     if (read_ch == ')' || read_ch == EOF)
       {
-      noteptr[1].type = b_End;
       next_ch();
       break;
       }
