@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2008 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: September 2008 */
+/* This file last modified: April 2020 */
 
 
 /* This file contains part VI of the code for reading a PMW score file -
@@ -41,20 +41,20 @@ the one that gets the stems, we must arrange for certain notes to be printed
 on the "wrong" side of the stem, and we must arrange the positioning of any
 accidentals. If we are sorting the last chord that has been read, we must sort
 stave_tiedata along with it. We must NOT do this when sorting other chords
-(those that were stacked up). 
+(those that were stacked up).
 
 Arguments:
   w             point to the first note of the chord
   upflag        TRUE for stem up
-  
-Returns:        nothing  
+
+Returns:        nothing
 */
 
-void 
+void
 read_sortchord(b_notestr *w, int upflag)
 {
+usint dynamics = 0;
 int SecondsExist = FALSE;
-int dynamics = 0;
 int fuq = 0;
 
 int acc_count = 0;
@@ -81,19 +81,19 @@ while (ww->type == b_chord)
   {
   b_notestr *insertptr = sortvec;
   tiedata *tieinsertptr = sorttievec;
- 
+
   int pitch = ww->spitch;
   int flags = ww->flags;
-  int acflags = ww->acflags; 
-  
+  usint acflags = ww->acflags;
+
   if (ww->acc) acc_count++;
   if ((flags & nf_accleft) != 0) acc_explicit = TRUE;
 
   dynamics |= acflags & (af_dynamics | af_opposite);
   fuq |= flags & nf_fuq;
 
-  ww->flags = upflag | (flags & ~(nf_dotright | nf_invert | nf_stemup));    
-  ww->acflags = acflags & ~(af_dynamics | af_opposite); 
+  ww->flags = upflag | (flags & ~(nf_dotright | nf_invert | nf_stemup));
+  ww->acflags = acflags & ~(af_dynamics | af_opposite);
 
   while (insertptr < sorttop)
     {
@@ -101,13 +101,13 @@ while (ww->type == b_chord)
     insertptr++;
     tieinsertptr++;
     }
-    
+
   memmove(tieinsertptr+1, tieinsertptr, (sorttop-insertptr)*sizeof(tiedata));
   *tieinsertptr = *tt++;
 
   memmove(insertptr+1, insertptr, (sorttop-insertptr)*sizeof(b_notestr));
   *insertptr = *ww;
-   
+
   sorttop++;
   mac_advancechord(ww);
   }
@@ -168,7 +168,7 @@ while (ww < sorttop - 1)
         wwA = wwB++;
         }
 
-    if (count > 1 && samelevel) error_moan(80);
+    if (count > 1 && samelevel) error_moan(ERR80);
 
     count = (count + 1)/2;   /* number of pairs to consider */
 
@@ -269,10 +269,10 @@ if (!acc_explicit && acc_count > 1)
     if (ww->acc == 0 && (flags & nf_invert) == 0) continue;
 
     row->pitch = ww->spitch;
-    
+
     if ((flags & nf_couple) != 0)
-      row->pitch += ((flags & nf_coupleU) != 0)? 16 : -16; 
- 
+      row->pitch += ((flags & nf_coupleU) != 0)? 16 : -16;
+
     row->inverted = !upflag && ((flags & nf_invert) != 0);
     row->acc = ww->acc;
 
@@ -575,7 +575,7 @@ if (upflag)
   for (wwP = sorttop-1; wwP >= sortvec; wwP--)
     {
     *ww = *wwP;
-    www = ww;   /* save last */ 
+    www = ww;   /* save last */
     mac_advancechord(ww);
     if (w == stave_firstnoteptr) *tt++ = *ttP--;
     }
@@ -587,7 +587,7 @@ else
   for (wwP = sortvec; wwP < sorttop; wwP++)
     {
     *ww = *wwP;
-    www = ww;   /* save last */ 
+    www = ww;   /* save last */
     mac_advancechord(ww);
     if (w == stave_firstnoteptr) *tt++ = *ttP++;
     }
@@ -619,16 +619,16 @@ Before we reset the flag for a chord, we must reset the offsets of any
 accidentals, unless there is an explicit setting, because when the stem of a
 chord is set, the accidental positions are calculated, assuming that what is
 there already is the basic width of the accidental. As this is an error
-situation, we don't have to get it perfect. 
+situation, we don't have to get it perfect.
 
 Arguments:
   noteptr        pointer
   flag           TRUE for stem up
-  
-Returns:         nothing  
+
+Returns:         nothing
 */
 
-void 
+void
 read_resetstemflag(b_notestr *noteptr, int flag)
 {
 b_notestr *p = noteptr;
@@ -661,13 +661,13 @@ even called for single notes that might have been the start of a beam, so we
 use the call to set the fuq flag when the stem direction is known.
 
 If the option for the stem swap level is "right", we can't take a decision
-here, so the notes are transferred on to the ordinary note pending stack. 
+here, so the notes are transferred on to the ordinary note pending stack.
 
 Arguments:  none
 Returns:    nothing
 */
 
-void 
+void
 read_setbeamstems(void)
 {
 if (stave_beamstackptr > 0)
