@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: November 2020 */
 
 /* This is the header file for the score reading routines;
 imported by error.c in order to access the input buffers. */
@@ -123,21 +123,22 @@ typedef struct ulaypend {
   int  x;
   int  y;
   int  halfway;    /* "halfway" fraction */
+  int  offset;     /* "offset" fraction */
 } ulaypend;
 
 /* Structure for list of accent/ornament codings */
 
 typedef struct accent {
   uschar *string;
-  unsigned int flag;
-} accent;    
+  usint flag;
+} accent;
 
 
 /*************************************************
 *             Global variables                   *
 *************************************************/
 
-extern unsigned int accent_list[];   /* List of accent flags, in order */
+extern usint   accent_list[];        /* List of accent flags, in order */
 extern accent  accent_chars[];       /* List of accents/ornaments & flags */
 
 extern bstr  **barvector;           /* Points to bar index */
@@ -170,9 +171,10 @@ extern int     read_prev_had_dbar;  /* For setting up previous bar block */
 extern int     read_prev_had_ibar;
 extern int     read_prev_barlinestyle;
 extern int     read_skipdepth;      /* For if nesting */
-extern uschar *read_stavedir;       /* Holds next stave directive */
+extern BOOL    read_stavedir;       /* TRUE if next stave directive is read */
+extern uschar *read_word;           /* Holds word read from input */
 
-extern int     stave_accentflags;   /* Default for next note */
+extern usint   stave_accentflags;   /* Default for next note */
 extern int     stave_accritvalue;   /* holds 2 or 3 */
 extern int     stave_barlength;     /* Current bar length */
 extern int     stave_barlinestyle;  /* Current bar line style */
@@ -205,7 +207,7 @@ extern int     stave_key;           /* Current key */
 extern int     stave_key_tp;        /* Transposed key */
 extern int     stave_lastgracestem; /* Stemforce value for previous grace */
 extern b_notestr *stave_lastbasenoteptr; /* Points to start of previous original note */
-extern b_notestr *stave_lastnoteptr; /* Points to start of previous note */
+extern b_notestr *stave_lastnoteptr; /* Points to start of previous note or rest */
 extern BOOL    stave_laststemup;    /* Flag */
 extern int     stave_lasttiepitch;  /* Pitch of single tied note */
 extern BOOL    stave_lastwasdouble; /* Last bar had double bar line */
@@ -246,6 +248,7 @@ extern int     stave_stemlength;    /* Default stem length */
 extern b_notestr **stave_stemstack; /* Stack of pending stems */
 extern int     stave_stemstackptr;  /* Pointer therein */
 extern int    *stave_stemswaplevel; /* Pointer to current table */
+extern BOOL    stave_string_followOK; /* OK for a followon string */
 extern BOOL    stave_suspended;     /* For automatic resume */
 extern int     stave_textabsolute;  /* Default absolute position */
 extern int     stave_textflags;     /* Default text flags */
@@ -257,6 +260,7 @@ extern int     stave_totalnocount;  /* Count uncounted bars */
 extern int     stave_transpose;     /* Current transposition */
 extern BOOL    stave_transposedaccforce; /* Default acc forcing */
 extern int     stave_transpose_letter; /* Current letter transposition */
+extern BOOL    stave_transpose_letter_is_auto; /* Automatically set */
 extern BOOL    stave_tripletize;    /* Tripletize dotted quaver pairs */
 extern int     stave_ulfont;        /* Default initial font */
 extern int     stave_ulsize;        /* Default size */
@@ -267,14 +271,12 @@ extern uschar *prev_buffer;         /* The previous line */
 
 
 
-
-
-
 /*************************************************
 *               Global procedures                *
 *************************************************/
 
 extern void next_ch(void);
+extern void next_word(void);
 extern void next_heading(void);
 
 extern void pre_process(void);
@@ -293,7 +295,7 @@ extern int  read_integer(BOOL);
 extern int  read_key(void);
 extern int  read_movevalue(void);
 extern void read_note(void);
-extern BOOL read_plainstring(uschar *);
+extern BOOL read_plainstring(void);
 extern void read_resetstemflag(b_notestr *, int);
 extern int  read_scaletime(int);
 extern void read_setbeamstems(void);
@@ -302,6 +304,5 @@ extern void read_sortchord(b_notestr *, int);
 extern void read_stavedirective(void);
 extern int  read_stavepitch(void);
 extern int  read_time(void);
-extern void read_word(uschar *);
 
 /* End of readhdr.h */
