@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: January 2020 */
 
 
 /* This file contains code for outputting one bar */
@@ -36,7 +36,7 @@ int i;
 BOOL notfirst = FALSE;
 bstr *lastp = NULL;
 int numberwanted = 0;
-unsigned int *notsuspend = out_sysblock->notsuspend;
+usint *notsuspend = out_sysblock->notsuspend;
 contstr *out_cont_save = out_cont;
 barposstr *bp = curmovt->posvector + out_bar;
 zcopystr *zcopy = curmovt->zcopy;
@@ -58,7 +58,7 @@ out_lastbarwide = FALSE;
 out_manyrest = bp->multi;
 out_lineendflag = (out_bar + out_manyrest - 1) == out_sysblock->barend;
 
-for (i = 0; i < dyn_max; i++) 
+for (i = 0; i < dyn_max; i++)
   out_dynmovef[i] = out_dynmovex[i] = out_dynmovey[i] = 0;
 
 /* Decide if a bar number is wanted. If there was a [nocount] directive in this
@@ -120,7 +120,7 @@ for (out_stave = out_laststave; out_stave >= 0; out_stave--)
   /* Reset hairpin starting halfway flag */
 
   out_hairpinhalf = FALSE;
-  
+
   /* Set the gaps for coupling - must find previous printing stave. As the gaps
   are in pitch units, they must be made relative to the stave magnification. */
 
@@ -259,7 +259,7 @@ for (out_stave = out_laststave; out_stave >= 0; out_stave--)
   out_passedreset = out_lastnotebeamed = FALSE;
   out_lastnotex = -100000;                  /* for accidental stretching */
 
-  out_moff = out_Xadjustment = out_Yadjustment = 0;
+  out_moff = out_lastmoff = out_Xadjustment = out_Yadjustment = 0;
 
   out_gracenotes = FALSE;
   n_gracecount = n_gracemoff = out_keycount = out_timecount = 0;
@@ -469,7 +469,7 @@ for (out_stave = out_laststave; out_stave >= 0; out_stave--)
           int ttype = pp->type;
           while (ttype != b_End)
             {
-            if (ttype == b_Jump) 
+            if (ttype == b_Jump)
               pp = (bstr *)(((b_Jumpstr *)pp)->next);
             else if (ttype == b_nbar)
               {
@@ -689,7 +689,7 @@ for (out_stave = out_laststave; out_stave >= 0; out_stave--)
 
         /* Deal with bar line styles that involve either or both of lines on
         the stave and lines between the staves. */
-        
+
         if (out_barchar == bar_thick)
           ps_barline(out_barlinex - x, ybarstart, out_ybarend, bar_single);
         else if (barlinestyle < 4 || out_barchar == bar_double)
@@ -703,7 +703,7 @@ for (out_stave = out_laststave; out_stave >= 0; out_stave--)
         else
           {
           ps_musstring((barlinestyle == 4)?
-            US"~x\302\211yyyyyyx\302\211" : US"|\302\211yyyyyyxxxxx\302\211", 
+            US"~x\302\211yyyyyyx\302\211" : US"|\302\211yyyyyyxxxxx\302\211",
             10*main_stavemagn, out_barlinex, out_ystave);
           }
         }
@@ -754,7 +754,7 @@ for (out_stave = out_laststave; out_stave >= 0; out_stave--)
             else
               {
               ps_musstring((style == 4)?
-                US"~x\302\211yyyyyyx\302\211" : US"|\302\211yyyyyyxxxxx\302\211", 
+                US"~x\302\211yyyyyyx\302\211" : US"|\302\211yyyyyyxxxxx\302\211",
                 10*main_stavemagn, out_lastbarlinex, out_ystave);
               }
             }
