@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2016 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: June 2016 */
+/* This file last modified: April 2020 */
 
 
 /* This file contains code for setting up and outputting beams */
@@ -1137,7 +1137,7 @@ for (i = 0; i < 2; i++)
 
   else
     {
-    int abs_slope; 
+    int abs_slope;
     int slope = findslope();
     int yslope = beam_firstY + mac_muldiv(slope, xright-beam_firstX, 1000);
 
@@ -1154,7 +1154,7 @@ for (i = 0; i < 2; i++)
     keep the slope down to a smaller maximum to enable more cases to be fitted
     in without retries. */
 
-    abs_slope = abs(beam_slope); 
+    abs_slope = abs(beam_slope);
     if (lastopposite && abs_slope > 200)
       beam_slope = mac_muldiv(beam_slope, 200, abs_slope);
     }
@@ -1179,16 +1179,16 @@ for (i = 0; i < 2; i++)
   if (offsetOK) break; else
     {
     b_notestr *bfp = beam_first;
-    int flag = beam_upflag? nf_stemup : 0;
+    usint flag = beam_upflag? nf_stemup : 0;
 
     for (;;)
       {
       if ((bfp->flags & nf_stemup) != flag) read_resetstemflag(bfp, flag);
       if (bfp == beam_last) break;
-      bfp = misc_nextnote(p, NULL);
+      bfp = misc_nextnote(bfp, NULL);
       }
 
-    error_moan(61, out_bar, out_stave);
+    error_moan(ERR61, out_bar, out_stave);
     lastopposite = FALSE;
     }
   }
