@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: November 2020 */
 
 
 /* This file contains code for supporting the "draw" directives in PMW. It
@@ -54,7 +54,7 @@ enum {
 };
 
 /* This table contains the number and type of entries required to be on the
-stack for each operator. Up to 8 values are catered for (though the maximum 
+stack for each operator. Up to 8 values are catered for (though the maximum
 used is 6). Each entry uses one nibble, starting from the least significant end
 of the word. The values are the dd_* values defined in pmwhdr, namely:
 
@@ -65,18 +65,18 @@ of the word. The values are the dd_* values defined in pmwhdr, namely:
   dd_varname  5   variable name
 */
 
-static unsigned int stack_rqd[] = {
+static usint stack_rqd[] = {
   0,                 /* accleft */
   0x00000022,        /* add */
-  0x00000022,        /* and */  
+  0x00000022,        /* and */
   0,                 /* barnumber */
   0,                 /* bra */
   0x00000001,        /* copy */
   0,                 /* currentgray */
   0,                 /* currentlinewidth */
-  0,                 /* currentpoint */     
+  0,                 /* currentpoint */
   0x00222222,        /* curveto */
-  0x00000023,        /* cvs */  
+  0x00000023,        /* cvs */
   0x00000051,        /* def */
   0x00000022,        /* div */
   0,                 /* draw */
@@ -91,22 +91,22 @@ static unsigned int stack_rqd[] = {
   0x00000003,        /* fontsize */
   0,                 /* gaptype */
   0x00000022,        /* ge */
-  0x00000022,        /* gt */              
-  0,                 /* headbottom */ 
-  0,                 /* headleft */ 
-  0,                 /* headright */ 
-  0,                 /* headtop */ 
+  0x00000022,        /* gt */
+  0,                 /* headbottom */
+  0,                 /* headleft */
+  0,                 /* headright */
+  0,                 /* headtop */
   0x00000024,        /* if */
-  0x00000244,        /* ifelse */  
+  0x00000244,        /* ifelse */
   0,                 /* jump */
   0,                 /* ket */
   0x00000022,        /* le */
-  0,                 /* leftbarx */ 
-  0,                 /* linebottom */ 
-  0,                 /* linegapx */ 
-  0,                 /* linegapy */ 
-  0,                 /* linelength */ 
-  0x00000022,        /* lineto */ 
+  0,                 /* leftbarx */
+  0,                 /* linebottom */
+  0,                 /* linegapx */
+  0,                 /* linegapy */
+  0,                 /* linelength */
+  0x00000022,        /* lineto */
   0,                 /* linetop */
   0x00000004,        /* loop */
   0x00000022,        /* lt */
@@ -118,26 +118,26 @@ static unsigned int stack_rqd[] = {
   0x00000002,        /* not */
   0,                 /* number */
   0x00000022,        /* or */
-  0,                 /* origin */            
-  0,                 /* originx */            
-  0,                 /* originy */            
-  0,                 /* pagelength */ 
-  0,                 /* pagenumber */ 
+  0,                 /* origin */
+  0,                 /* originx */
+  0,                 /* originy */
+  0,                 /* pagelength */
+  0,                 /* pagenumber */
   0x00000001,        /* pop */
-  0,                 /* pstack: no checking done */  
+  0,                 /* pstack: no checking done */
   0x00222222,        /* rcurveto */
   0x00000024,        /* repeat */
   0x00000022,        /* rlineto */
-  0x00000022,        /* rmoveto */    
+  0x00000022,        /* rmoveto */
   0x00000022,        /* roll */
-  0x00000002,        /* setgray */  
-  0x00000002,        /* setlinewidth */  
+  0x00000002,        /* setgray */
+  0x00000002,        /* setlinewidth */
   0x00000003,        /* show */
   0,                 /* stavesize */
-  0,                 /* stavespace */   
-  0,                 /* stavestart */   
-  0,                 /* stembottom */ 
-  0,                 /* stemtop */ 
+  0,                 /* stavespace */
+  0,                 /* stavestart */
+  0,                 /* stembottom */
+  0,                 /* stemtop */
   0x00000003,        /* stringwidth */
   0,                 /* stroke */
   0x00000022,        /* sub */
@@ -148,7 +148,7 @@ static unsigned int stack_rqd[] = {
   0,                 /* true */
   0,                 /* varname */
   0,                 /* varref */
-  0x00000022         /* xor */           
+  0x00000022         /* xor */
 };
 
 
@@ -273,8 +273,8 @@ static int draw_operator_count = sizeof(draw_operators)/sizeof(draw_op);
 
 Arguments:
   s1         introductory string
-  s2         terminating string 
-   
+  s2         terminating string
+
 Returns:     nothing
 */
 
@@ -282,42 +282,42 @@ static void
 do_pstack(const char *s1, const char *s2)
 {
 int i;
-drawtextstr *d; 
+drawtextstr *d;
 info_printf("%s", s1);
 for (i = 0; i < out_drawstackptr; i++)
   {
   switch (draw_stack[i].dtype)
     {
     case dd_text:
-    d = draw_stack[i].d.ptr; 
-    if (d->ident != magic) 
+    d = draw_stack[i].d.ptr;
+    if (d->ident != magic)
       info_printf("<corrupt-text> ");
     else
-      {    
-      info_printf("\"%s\"", d->text); 
+      {
+      info_printf("\"%s\"", d->text);
       if (d->xdelta != 0) info_printf("/%c%f", (d->xdelta > 0)? 'r':'l',
         d->xdelta);
       if (d->ydelta != 0) info_printf("/%c%f", (d->ydelta > 0)? 'u':'d',
         d->ydelta);
       if (d->size != 0) info_printf("/s%d", d->size + 1);
       if (d->rotate != 0) info_printf("/rot%f", d->rotate);
-      if ((d->flags & text_box) != 0) info_printf("/box");   
-      if ((d->flags & text_centre) != 0) info_printf("/c");   
-      if ((d->flags & text_endalign) != 0) info_printf("/e");   
-      if ((d->flags & text_ring) != 0) info_printf("/ring");   
-      info_printf(" "); 
-      } 
+      if ((d->flags & text_box) != 0) info_printf("/box");
+      if ((d->flags & text_centre) != 0) info_printf("/c");
+      if ((d->flags & text_endalign) != 0) info_printf("/e");
+      if ((d->flags & text_ring) != 0) info_printf("/ring");
+      info_printf(" ");
+      }
     break;
-    
+
     case dd_code:
-    info_printf("<code-block> "); 
+    info_printf("<code-block> ");
     break;
-    
-    default:        
+
+    default:
     info_printf("%f ", draw_stack[i].d.val);
     break;
-    }  
-  } 
+    }
+  }
 info_printf("\n%s", s2);
 }
 
@@ -328,7 +328,7 @@ info_printf("\n%s", s2);
 *************************************************/
 
 /* This is also called when reading arguments for draw calls. It reads the
-string and parameters into a store block, and yields its address. 
+string and parameters into a store block, and yields its address.
 
 Arguments:  none
 Returns:    pointer to a textstr
@@ -361,7 +361,7 @@ while (read_ch == '/')
       next_ch();
       textptr->flags |= text_box;
       }
-    else error_moan(10, "/box, /c, /e, /ring, /rot, or /s");
+    else error_moan(ERR10, "/box, /c, /e, /ring, /rot, or /s");
     break;
 
     case 'c':
@@ -369,11 +369,11 @@ while (read_ch == '/')
     textptr->flags |= text_centre;
     next_ch();
     break;
-    
+
     case 'd':
     next_ch();
     if (read_expect_integer(&delta, TRUE, TRUE)) textptr->ydelta -= delta;
-    break; 
+    break;
 
     case 'e':
     textptr->flags &= ~text_centre;
@@ -384,24 +384,24 @@ while (read_ch == '/')
     case 'l':
     next_ch();
     if (read_expect_integer(&delta, TRUE, TRUE)) textptr->xdelta -= delta;
-    break; 
+    break;
 
     case 'n':
     next_ch();
     if (read_ch == 'c') textptr->flags &= ~text_centre;
     else if (read_ch == 'e') textptr->flags &= ~text_endalign;
-    else error_moan(37, "/nc or /ne");
+    else error_moan(ERR37, "/nc or /ne");
     next_ch();
     break;
 
     case 's':
     next_ch();
     if (read_expect_integer(&size, FALSE, FALSE))
-      { 
-      if (--size < 0 || size >= MaxFontSizes) 
-        { error_moan(39, MaxFontSizes); size = 0; }
+      {
+      if (--size < 0 || size >= MaxFontSizes)
+        { error_moan(ERR39, MaxFontSizes); size = 0; }
       textptr->size = size;
-      } 
+      }
     break;
 
     case 'r':
@@ -423,20 +423,20 @@ while (read_ch == '/')
       textptr->flags |= text_ring;
       break;
       }
-    else 
+    else
       {
       next_ch();
       if (read_expect_integer(&delta, TRUE, TRUE)) textptr->xdelta += delta;
-      } 
-    break; 
-    
+      }
+    break;
+
     case 'u':
     next_ch();
     if (read_expect_integer(&delta, TRUE, TRUE)) textptr->ydelta += delta;
-    break; 
- 
+    break;
+
     default:
-    error_moan(10, "/box, /c, /d, /e, /l, /r, /ring, /rot, /s, or /u");
+    error_moan(ERR10, "/box, /c, /d, /e, /l, /r, /ring, /rot, /s, or /u");
     break;
     }
   }
@@ -463,12 +463,11 @@ tree_node *drawnode = store_Xget(sizeof(tree_node));
 drawitem *ptr;
 int left;
 int bracount = 0;
-uschar word[80];
 
-read_word(word);
-if (word[0] == 0) { error_moan(10, "name"); return; }
+next_word();
+if (read_word[0] == 0) { error_moan(ERR10, "name"); return; }
 
-drawnode->name = store_copystring(word);
+drawnode->name = store_copystring(read_word);
 drawnode->data = store_Xget(DRAW_BLOCKSIZE);
 
 ptr = (drawitem *)drawnode->data;
@@ -480,7 +479,7 @@ for (;;)
   {
   int type = -1;
   int value = 0;
-  void *pointer = NULL; 
+  void *pointer = NULL;
 
   sigch();
 
@@ -507,7 +506,7 @@ for (;;)
     {
     next_ch();
     type = dr_ket;
-    if (--bracount < 0) error_moan(118, drawnode->name);
+    if (--bracount < 0) error_moan(ERR118, drawnode->name);
     }
 
   /* Deal with variable names; data put into "value" is the variable index */
@@ -517,20 +516,22 @@ for (;;)
     tree_node *node;
     type = dr_varname;
     next_ch();
-    read_word(word);
-    if (word[0] == 0) error_moan(37, "Variable name"); else
+    if (!isspace(read_ch)) next_word();
+      else read_word[0] = 0;
+
+    if (read_word[0] == 0) error_moan(ERR37, "Variable name"); else
       {
-      node = Tree_Search(draw_variable_tree, word);
+      node = Tree_Search(draw_variable_tree, read_word);
       if (node == NULL)
         {
         if (draw_nextvariable > MAX_DRAW_VARIABLE)
           {
-          error_moan(100, MAX_DRAW_VARIABLE + 1);
+          error_moan(ERR100, MAX_DRAW_VARIABLE + 1);
           draw_nextvariable--;
           }
         value = draw_nextvariable++;
         node = store_Xget(sizeof(tree_node));
-        node->name = store_copystring(word);
+        node->name = store_copystring(read_word);
         node->val[0] = value;
         Tree_InsertNode(&draw_variable_tree, node);
         }
@@ -542,18 +543,22 @@ for (;;)
 
   else if (isalpha(read_ch))
     {
-    read_word(word);
-    if (Ustrcmp(word, "enddraw") == 0) { ptr->d.val = dr_end; break; }
+    next_word();
+    if (Ustrcmp(read_word, "enddraw") == 0)
+      {
+      ptr->d.val = dr_end;
+      break;
+      }
 
     /* Deal with "subroutine" call; value put into "pointer" */
 
-    if (Ustrcmp(word, "draw") == 0)
+    if (Ustrcmp(read_word, "draw") == 0)
       {
       tree_node *node;
       type = dr_draw;
-      read_word(word);
-      node = Tree_Search(draw_tree, word);
-      if (node == NULL) error_moan(70, word); else pointer = node;
+      next_word();
+      node = Tree_Search(draw_tree, read_word);
+      if (node == NULL) error_moan(ERR70, read_word); else pointer = node;
       }
 
     /* Deal with normal operators and variables */
@@ -562,15 +567,15 @@ for (;;)
       {
       draw_op *first = draw_operators;
       draw_op *last = first + draw_operator_count;
-      
-      /* Search for a standard variable or operator; if found, the type is set 
+
+      /* Search for a standard variable or operator; if found, the type is set
       but there is no data. */
 
       while (last > first)
         {
         int c;
         draw_op *middle = first + (last - first)/2;
-        c = Ustrcmp(middle->name, word);
+        c = Ustrcmp(middle->name, read_word);
         if (c == 0) { type = middle->value; break; }
         if (c > 0) last = middle; else first = middle + 1;
         }
@@ -581,7 +586,7 @@ for (;;)
 
     if (type < 0)
       {
-      tree_node *node = Tree_Search(draw_variable_tree, word);
+      tree_node *node = Tree_Search(draw_variable_tree, read_word);
       if (node != NULL)
         {
         type = dr_varref;
@@ -591,12 +596,16 @@ for (;;)
 
     /* Grumble if unrecognized word */
 
-    if (type < 0) error_moan(69, word);
+    if (type < 0) error_moan(ERR69, read_word);
     }
 
-  /* Grumble if unrecognized input */
+  /* Grumble if unrecognized input; error 10 skips to end of line */
 
-  else { error_moan(10, "number, string, name, or curly bracket"); break; }
+  else
+    {
+    error_moan(ERR10, "number, string, name, or curly bracket");
+    continue;
+    }
 
   /* Extend to new block if necessary */
 
@@ -608,8 +617,8 @@ for (;;)
     left = DRAW_BLOCKSIZE/sizeof(drawitem);
     }
 
-  /* Add this item to the "program". Numbers, variable names, and variable 
-  references have a numerical argument; strings and draw subroutine calls have 
+  /* Add this item to the "program". Numbers, variable names, and variable
+  references have a numerical argument; strings and draw subroutine calls have
   an address argument. */
 
   (ptr++)->d.val = type;
@@ -628,7 +637,7 @@ for (;;)
 
 /* Insert into tree; give error if duplicate */
 
-if (!Tree_InsertNode(&draw_tree, drawnode)) error_moan(14, drawnode->name);
+if (!Tree_InsertNode(&draw_tree, drawnode)) error_moan(ERR14, drawnode->name);
 }
 
 
@@ -736,7 +745,7 @@ memcpy(pp+cp+cp, c, cp*sizeof(int));
 *************************************************/
 
 /* This function checks the validity of the argument of a conditional or
-looping command, which is a sequence of draw commands, ending at the current 
+looping command, which is a sequence of draw commands, ending at the current
 command pointer.
 
 Arguments:
@@ -848,15 +857,15 @@ while (pp != p && pp->d.val != dr_end)
     default:
       {
       char buff[40];
-      sprintf(buff, " (bad value %d) ", pp[-1].d.val);   
-      draw_error(117, buff, t); 
-      } 
+      sprintf(buff, " (bad value %d) ", pp[-1].d.val);
+      draw_error(ERR117, buff, t);
+      }
     return FALSE;
     }
   }
-  
+
 if (p == pp) return TRUE;
-draw_error(117, " (ended too soon) ", t);
+draw_error(ERR117, " (ended too soon) ", t);
 return FALSE;
 }
 
@@ -882,36 +891,36 @@ static BOOL
 sub_draw(tree_node *t, drawitem *p, int *x, int *y, int *c, BOOL overflag)
 {
 if (p == NULL) p = (drawitem *)t->data;
-if (level > 20) draw_error(84, " ", t);
+if (level > 20) draw_error(ERR84, " ", t);
 
 while (p->d.val != dr_end)
   {
-  int errornumber = 0;
-  if (out_drawstackptr > DRAW_STACKSIZE - 4) 
-    errornumber = 121;
+  int errornumber = -1;
+  if (out_drawstackptr > DRAW_STACKSIZE - 4)
+    errornumber = ERR121;
   else
     {
-    int cc = 0; 
-    unsigned int xx = stack_rqd[p->d.val];
+    int cc = 0;
+    usint xx = stack_rqd[p->d.val];
     while (xx != 0)
       {
       cc++;
-      if (out_drawstackptr < cc) 
-        { 
-        errornumber = 71;    /* Insufficient items on stack */
-        break; 
+      if (out_drawstackptr < cc)
+        {
+        errornumber = ERR71;    /* Insufficient items on stack */
+        break;
         }
       if ((xx & 0x0f) != dd_any &&
           (xx & 0x0f) != draw_stack[out_drawstackptr - cc].dtype)
         {
-        errornumber = 6;     /* Wrong type of item on stack */
+        errornumber = ERR6;     /* Wrong type of item on stack */
         break;
-        }    
+        }
       xx >>= 4;
-      }        
+      }
     }
 
-  if (errornumber != 0)
+  if (errornumber >= 0)
     {
     int i;
     uschar *s = US"???";
@@ -920,7 +929,7 @@ while (p->d.val != dr_end)
         { s = US draw_operators[i].name; break; }
     draw_error(errornumber, CS s, t);
     do_pstack("** Draw stack contents when error detected:\n", "\n");
-    return FALSE; 
+    return FALSE;
     }
 
   switch ((p++)->d.val)
@@ -932,7 +941,7 @@ while (p->d.val != dr_end)
     case dr_bra:
       {
       int count = 1;
-      draw_stack[out_drawstackptr].dtype = dd_code; 
+      draw_stack[out_drawstackptr].dtype = dd_code;
       draw_stack[out_drawstackptr++].d.ptr = p;
       while (p->d.val != dr_end && (p->d.val != dr_ket || --count > 0))
         {
@@ -942,7 +951,7 @@ while (p->d.val != dr_end)
         else if (type == dr_draw || type == dr_varref || type == dr_number ||
           type == dr_text || type == dr_varname) p++;
         }
-      if (p->d.val != dr_ket) draw_error(116, " ", t); else p++;
+      if (p->d.val != dr_ket) draw_error(ERR116, " ", t); else p++;
       }
     break;
 
@@ -952,7 +961,7 @@ while (p->d.val != dr_end)
     case dr_if:
       {
       drawitem *pp = draw_stack[--out_drawstackptr].d.ptr;
-      if (!check_ptr(pp, t)) return FALSE; 
+      if (!check_ptr(pp, t)) return FALSE;
       if (draw_stack[--out_drawstackptr].d.val != 0 &&
         !sub_draw(t, pp, x, y, c, overflag)) return FALSE;
       }
@@ -970,17 +979,17 @@ while (p->d.val != dr_end)
     break;
 
     case dr_number:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = (p++)->d.val;
     break;
 
     case dr_text:
-    draw_stack[out_drawstackptr].dtype = dd_text; 
+    draw_stack[out_drawstackptr].dtype = dd_text;
     draw_stack[out_drawstackptr++].d.ptr = (p++)->d.ptr;
     break;
 
     case dr_varname:
-    draw_stack[out_drawstackptr].dtype = dd_varname; 
+    draw_stack[out_drawstackptr].dtype = dd_varname;
     draw_stack[out_drawstackptr++].d.val = (p++)->d.val;
     break;
 
@@ -989,12 +998,12 @@ while (p->d.val != dr_end)
     break;
 
     case dr_accleft:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = n_maxaccleft;
     break;
 
     case dr_add:
-    draw_stack[out_drawstackptr-2].d.val += 
+    draw_stack[out_drawstackptr-2].d.val +=
       draw_stack[out_drawstackptr-1].d.val;
     out_drawstackptr--;
     break;
@@ -1015,7 +1024,7 @@ while (p->d.val != dr_end)
       format_sprintf(buff, "%b", out_bar);
       sscanf(CS buff, "%d.%d", &a, &b);
       if (b < 10) b *= 100; else if (b < 100) b *= 10;
-      draw_stack[out_drawstackptr].dtype = dd_number; 
+      draw_stack[out_drawstackptr].dtype = dd_number;
       draw_stack[out_drawstackptr++].d.val = a * 1000 + b;
       }
     break;
@@ -1023,10 +1032,10 @@ while (p->d.val != dr_end)
     case dr_copy:
       {
       int count = draw_stack[--out_drawstackptr].d.val / 1000;
-      if (out_drawstackptr < count) 
-        { 
-        draw_error(71, "copy", t); 
-        break; 
+      if (out_drawstackptr < count)
+        {
+        draw_error(ERR71, "copy", t);
+        break;
         }
       memcpy(draw_stack + out_drawstackptr,
         draw_stack + out_drawstackptr - count, count * sizeof(drawitem));
@@ -1035,27 +1044,27 @@ while (p->d.val != dr_end)
     break;
 
     case dr_currentgray:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = gray;
     break;
 
     case dr_currentlinewidth:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = draw_thickness;
     break;
 
     case dr_currentpoint:
-    if (cp <= 0) draw_error(72, "currentpoint", t);
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    if (cp <= 0) draw_error(ERR72, "currentpoint", t);
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = x[xp-1] - draw_ox;
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = y[yp-1] - draw_oy;
     break;
 
     case dr_curveto:
       {
       int i;
-      if (!currentpoint) draw_error(72, "curveto", t);
+      if (!currentpoint) draw_error(ERR72, "curveto", t);
       for (i = 6; i >= 2; i -= 2)
         {
         x[xp++] = draw_ox + draw_stack[out_drawstackptr - i].d.val;
@@ -1070,9 +1079,9 @@ while (p->d.val != dr_end)
       {
       drawtextstr *d = (drawtextstr *)(draw_stack[--out_drawstackptr].d.ptr);
       int n = draw_stack[--out_drawstackptr].d.val;
-      if (d->ident != magic) draw_error(82, "cvs", t);
+      if (d->ident != magic) draw_error(ERR82, "cvs", t);
       format_sprintf(d->text, "%f", n);
-      draw_stack[out_drawstackptr].dtype = dd_text; 
+      draw_stack[out_drawstackptr].dtype = dd_text;
       draw_stack[out_drawstackptr++].d.ptr = d;
       }
     break;
@@ -1080,7 +1089,7 @@ while (p->d.val != dr_end)
     case dr_def:
       {
       int n = draw_stack[out_drawstackptr-2].d.val;
-      if (n < 0 || n > MAX_DRAW_VARIABLE) draw_error(101, "", t);
+      if (n < 0 || n > MAX_DRAW_VARIABLE) draw_error(ERR101, "", t);
         else draw_variables[n] = draw_stack[out_drawstackptr-1];
       }
     out_drawstackptr -= 2;
@@ -1088,7 +1097,7 @@ while (p->d.val != dr_end)
 
     case dr_div:
     if (draw_stack[out_drawstackptr-1].d.val == 0)
-      { draw_error(95, "", t); }
+      { draw_error(ERR95, "", t); }
     else
      {
      draw_stack[out_drawstackptr-2].d.val =
@@ -1111,7 +1120,7 @@ while (p->d.val != dr_end)
 
     case dr_eq:
     draw_stack[out_drawstackptr-2].d.val =
-      (draw_stack[out_drawstackptr-2].d.val == 
+      (draw_stack[out_drawstackptr-2].d.val ==
         draw_stack[out_drawstackptr-1].d.val)?
           1000 : 0;
     out_drawstackptr--;
@@ -1129,7 +1138,7 @@ while (p->d.val != dr_end)
     return FALSE;
 
     case dr_fill:
-    if (!currentpoint) draw_error(72, "fill", t);
+    if (!currentpoint) draw_error(ERR72, "fill", t);
     c[cp++] = path_end;
     if (overflag) setup_overdraw(-1, x, y, c); else ps_path(x, y, c, -1);
     cp = xp = yp = 0;
@@ -1137,7 +1146,7 @@ while (p->d.val != dr_end)
     break;
 
     case dr_fillretain:
-    if (!currentpoint) draw_error(72, "fillretain", t);
+    if (!currentpoint) draw_error(ERR72, "fillretain", t);
     c[cp++] = path_end;
     if (overflag) setup_overdraw(-1, x, y, c); else ps_path(x, y, c, -1);
     break;
@@ -1145,20 +1154,20 @@ while (p->d.val != dr_end)
     case dr_fontsize:
       {
       drawtextstr *d = (drawtextstr *)(draw_stack[--out_drawstackptr].d.ptr);
-      if (d->ident != magic) draw_error(82, "fontsize", t);
-      draw_stack[out_drawstackptr].dtype = dd_number; 
+      if (d->ident != magic) draw_error(ERR82, "fontsize", t);
+      draw_stack[out_drawstackptr].dtype = dd_number;
       draw_stack[out_drawstackptr++].d.val =
         ((curmovt->fontsizes)->fontsize_text)[d->size];
       }
     break;
 
     case dr_false:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = 0;
     break;
 
     case dr_gaptype:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = draw_gap;
     break;
 
@@ -1171,82 +1180,82 @@ while (p->d.val != dr_end)
 
     case dr_gt:
     draw_stack[out_drawstackptr-2].d.val =
-      (draw_stack[out_drawstackptr-2].d.val > 
+      (draw_stack[out_drawstackptr-2].d.val >
         draw_stack[out_drawstackptr-1].d.val)?
           1000 : 0;
     out_drawstackptr--;
     break;
 
     case dr_headbottom:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val =
       (n_minpitch - 128)*main_stavemagn - cuegrace_scale(2*main_stavemagn);
     break;
 
     case dr_headleft:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = n_invertleft?
       cuegrace_scale(6*main_stavemagn) : 0;
     break;
 
     case dr_headright:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val =
       cuegrace_scale((n_invertright? 12 : 6)*main_stavemagn);
     break;
 
     case dr_headtop:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val =
       (n_maxpitch - 128)*main_stavemagn + cuegrace_scale(2*main_stavemagn);
     break;
 
     case dr_le:
     draw_stack[out_drawstackptr-2].d.val =
-      (draw_stack[out_drawstackptr-2].d.val <= 
+      (draw_stack[out_drawstackptr-2].d.val <=
         draw_stack[out_drawstackptr-1].d.val)?
           1000 : 0;
     out_drawstackptr--;
     break;
 
     case dr_leftbarx:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr].d.val = out_lastbarlinex - draw_ox;
     if (out_startlinebar) draw_stack[out_drawstackptr].d.val -= 6000;
     out_drawstackptr++;
     break;
 
     case dr_linebottom:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
-    draw_stack[out_drawstackptr++].d.val = 
+    draw_stack[out_drawstackptr].dtype = dd_number;
+    draw_stack[out_drawstackptr++].d.val =
       (n_minpitch & 2)? 2*main_stavemagn : 0;
     break;
 
     case dr_linegapx:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = draw_lgx;
     break;
 
     case dr_linegapy:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = draw_lgy;
     break;
 
     case dr_linelength:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = curmovt->linelength;
     break;
 
     case dr_lineto:
-    if (!currentpoint) draw_error(72, "lineto", t);
+    if (!currentpoint) draw_error(ERR72, "lineto", t);
     y[yp++] = draw_oy + draw_stack[--out_drawstackptr].d.val;
     x[xp++] = draw_ox + draw_stack[--out_drawstackptr].d.val;
     c[cp++] = path_line;
     break;
 
     case dr_linetop:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
-    draw_stack[out_drawstackptr++].d.val = 
+    draw_stack[out_drawstackptr].dtype = dd_number;
+    draw_stack[out_drawstackptr++].d.val =
       (n_maxpitch & 2)? 2*main_stavemagn : 0;
     break;
 
@@ -1261,14 +1270,14 @@ while (p->d.val != dr_end)
 
     case dr_lt:
     draw_stack[out_drawstackptr-2].d.val =
-      (draw_stack[out_drawstackptr-2].d.val < 
+      (draw_stack[out_drawstackptr-2].d.val <
         draw_stack[out_drawstackptr-1].d.val)?
           1000 : 0;
     out_drawstackptr--;
     break;
 
     case dr_magnification:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = main_magnification;
     break;
 
@@ -1288,14 +1297,14 @@ while (p->d.val != dr_end)
 
     case dr_ne:
     draw_stack[out_drawstackptr-2].d.val =
-      (draw_stack[out_drawstackptr-2].d.val != 
+      (draw_stack[out_drawstackptr-2].d.val !=
         draw_stack[out_drawstackptr-1].d.val)?
           1000 : 0;
     out_drawstackptr--;
     break;
 
     case dr_neg:
-    draw_stack[out_drawstackptr-1].d.val = 
+    draw_stack[out_drawstackptr-1].d.val =
       -draw_stack[out_drawstackptr-1].d.val;
     break;
 
@@ -1314,29 +1323,29 @@ while (p->d.val != dr_end)
 
     case dr_origin:
     case dr_originx:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = draw_ox;
     break;
 
     case dr_originy:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = draw_oy;
     break;
 
     case dr_pagelength:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = main_pagelength;
     break;
 
     case dr_pagenumber:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = curpage->number * 1000;
     break;
 
     case dr_topleft:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = - draw_ox;
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val =   out_ystave - draw_oy;
     break;
 
@@ -1345,13 +1354,13 @@ while (p->d.val != dr_end)
     break;
 
     case dr_pstack:
-    do_pstack("Draw stack contents (pstack command):\n", ""); 
+    do_pstack("Draw stack contents (pstack command):\n", "");
     break;
 
     case dr_rcurveto:
       {
       int i;
-      if (!currentpoint) draw_error(72, "rcurveto", t);
+      if (!currentpoint) draw_error(ERR72, "rcurveto", t);
       for (i = 0; i <= 2; i++)
         {
         x[xp+i] = x[xp-1] + draw_stack[out_drawstackptr + 2*i - 6].d.val;
@@ -1374,7 +1383,7 @@ while (p->d.val != dr_end)
     break;
 
     case dr_rlineto:
-    if (!currentpoint) draw_error(72, "rlineto", t);
+    if (!currentpoint) draw_error(ERR72, "rlineto", t);
     y[yp] = y[yp-1] + draw_stack[--out_drawstackptr].d.val;
     yp++;
     x[xp] = x[xp-1] + draw_stack[--out_drawstackptr].d.val;
@@ -1383,7 +1392,7 @@ while (p->d.val != dr_end)
     break;
 
     case dr_rmoveto:
-    if (!currentpoint) draw_error(72, "rmoveto", t);
+    if (!currentpoint) draw_error(ERR72, "rmoveto", t);
     y[yp] = y[yp-1] + draw_stack[--out_drawstackptr].d.val;
     yp++;
     x[xp] = x[xp-1] + draw_stack[--out_drawstackptr].d.val;
@@ -1397,7 +1406,7 @@ while (p->d.val != dr_end)
       int amount = (draw_stack[--out_drawstackptr].d.val)/1000;
       int count = (draw_stack[--out_drawstackptr].d.val)/1000;
 
-      if (out_drawstackptr < count) draw_error(71, "roll", t);
+      if (out_drawstackptr < count) draw_error(ERR71, "roll", t);
 
       if (amount > 0) for (i = 0; i < amount; i++)
         {
@@ -1435,7 +1444,7 @@ while (p->d.val != dr_end)
       int boxring = 0;
 
       drawtextstr *d = draw_stack[--out_drawstackptr].d.ptr;
-      if (d->ident != magic) draw_error(82, "show", t);
+      if (d->ident != magic) draw_error(ERR82, "show", t);
       unscaled_fontsize = ((curmovt->fontsizes)->fontsize_text)[d->size];
       fontsize = mac_muldiv(main_stavemagn, unscaled_fontsize, 1000);
       matrix = ((curmovt->fontsizes)->fontmatrix_text)[d->size];
@@ -1448,9 +1457,9 @@ while (p->d.val != dr_end)
 
       if (p[-1].d.val == dr_stringwidth)
         {
-        draw_stack[out_drawstackptr].dtype = dd_number; 
+        draw_stack[out_drawstackptr].dtype = dd_number;
         draw_stack[out_drawstackptr++].d.val = width;
-        draw_stack[out_drawstackptr].dtype = dd_number; 
+        draw_stack[out_drawstackptr].dtype = dd_number;
         draw_stack[out_drawstackptr++].d.val = font_stringheight;
         }
 
@@ -1458,7 +1467,7 @@ while (p->d.val != dr_end)
 
       else
         {
-        if (!currentpoint) draw_error(72, "show", t);
+        if (!currentpoint) draw_error(ERR72, "show", t);
         xx = x[xp-1] + d->xdelta;
         yy = y[yp-1] + d->ydelta;
 
@@ -1508,39 +1517,39 @@ while (p->d.val != dr_end)
     break;
 
     case dr_stavesize:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = main_stavemagn;
     break;
 
     case dr_stavestart:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = (out_stave < 0)? 0 :
       (out_sysblock->startxposition + out_sysblock->xjustify - draw_ox);
     break;
 
     case dr_stavespace:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
-    draw_stack[out_drawstackptr++].d.val = 
+    draw_stack[out_drawstackptr].dtype = dd_number;
+    draw_stack[out_drawstackptr++].d.val =
       out_sysblock->stavespacing[out_stave];
     break;
 
     case dr_stembottom:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr].d.val = (n_minpitch - 130)*1000;
     if ((n_flags & (nf_stem | nf_stemup)) == nf_stem)
-      draw_stack[out_drawstackptr].d.val -= 
+      draw_stack[out_drawstackptr].d.val -=
         cuegrace_scale(12000 + n_stemlength);
-    draw_stack[out_drawstackptr].d.val =  
+    draw_stack[out_drawstackptr].d.val =
       mac_muldiv(draw_stack[out_drawstackptr].d.val, main_stavemagn, 1000);
     out_drawstackptr++;
     break;
 
     case dr_stemtop:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr].d.val = (n_maxpitch - 126)*1000;
     if ((n_flags & (nf_stem | nf_stemup)) == (nf_stem | nf_stemup))
       draw_stack[out_drawstackptr].d.val += cuegrace_scale(12000+ n_stemlength);
-    draw_stack[out_drawstackptr].d.val = 
+    draw_stack[out_drawstackptr].d.val =
       mac_muldiv(draw_stack[out_drawstackptr].d.val, main_stavemagn, 1000);
     out_drawstackptr++;
     break;
@@ -1550,7 +1559,7 @@ while (p->d.val != dr_end)
     ***/
 
     case dr_stroke:
-    if (!currentpoint) draw_error(72, "stroke", t);
+    if (!currentpoint) draw_error(ERR72, "stroke", t);
     c[cp++] = path_end;
     if (overflag) setup_overdraw(draw_thickness, x, y, c);
       else ps_path(x, y, c, draw_thickness);
@@ -1559,12 +1568,12 @@ while (p->d.val != dr_end)
     break;
 
     case dr_systemdepth:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = out_sysblock->systemdepth;
     break;
 
     case dr_sub:
-    draw_stack[out_drawstackptr-2].d.val -= 
+    draw_stack[out_drawstackptr-2].d.val -=
       draw_stack[out_drawstackptr-1].d.val;
     out_drawstackptr--;
     break;
@@ -1575,7 +1584,7 @@ while (p->d.val != dr_end)
     break;
 
     case dr_true:
-    draw_stack[out_drawstackptr].dtype = dd_number; 
+    draw_stack[out_drawstackptr].dtype = dd_number;
     draw_stack[out_drawstackptr++].d.val = 1000;
     break;
 
