@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: April 2020 */
 
 
 /* This file contains code for outputting non-note items in a bar */
@@ -81,6 +81,7 @@ switch (p->type)
     is processed, as a result of the slurstarted flag. However, put in some
     defaults for the mad case when the slur does not cross any notes. */
 
+    s->moff = out_moff;
     s->x = out_barx + out_findXoffset(out_moff);
     s->y = L_3L;
 
@@ -101,7 +102,7 @@ switch (p->type)
     {
     slurstr *s = misc_getendslur(p);
     if (s == NULL)
-      error_moan(62, ((b_endslurstr *)p)->id, out_bar, out_stave);
+      error_moan(ERR62, ((b_endslurstr *)p)->id, out_bar, out_stave);
     else
       misc_drawslur(s, n_x, n_pitch, FALSE);
     }
@@ -128,13 +129,13 @@ switch (p->type)
       }
 
     if (s == NULL)
-      { error_moan(62, pg->id, out_bar, out_stave); break; }
+      { error_moan(ERR62, pg->id, out_bar, out_stave); break; }
 
     if ((s->slur->flags & sflag_l) == 0)
       {
       if (p->type == b_linegap)
         {
-        error_moan(93, out_bar, out_stave);
+        error_moan(ERR93, out_bar, out_stave);
         break;
         }
       }
@@ -142,7 +143,7 @@ switch (p->type)
       {
       if (p->type == b_slurgap)
         {
-        error_moan(119, out_bar, out_stave);
+        error_moan(ERR119, out_bar, out_stave);
         break;
         }
       }
@@ -169,10 +170,10 @@ switch (p->type)
     b_hairpinstr *h = (b_hairpinstr *)p;
     if (h->opt != 0)
       out_setstarthairpin(h, out_barx + out_findXoffset(out_moff));
-    else if (out_moff != 0)
-      out_drawhairpin(h, n_x + 6*main_stavemagn);
-    else
+    else if (out_moff == 0)
       out_drawhairpin(h, out_barx - 4*main_stavemagn);
+    else
+      out_drawhairpin(h, n_x + 6*main_stavemagn);
     }
   break;
 
@@ -617,7 +618,7 @@ switch (p->type)
   else
     {
     if (out_textqueueptr >= TEXT_QUEUE_SIZE)
-      error_moan(131, TEXT_QUEUE_SIZE);
+      error_moan(ERR131, TEXT_QUEUE_SIZE);
     out_textXqueue[out_textqueueptr] = out_textX;
     out_textqueue[out_textqueueptr++] = (b_textstr *)p;
     }
