@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2016 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: June 2016 */
+/* This file last modified: November 2020 */
 
 
 /* This file contains code for outputting text items */
@@ -206,7 +206,7 @@ uschar buff[256];
 DEBUG(("out_repeatstring() start\n"));
 
 while (--htype > 0 && h != NULL) h = h->next;
-if (h == NULL) error_moan(91, out_bar, out_stave);  /* Hard error */
+if (h == NULL) error_moan(ERR91, out_bar, out_stave);  /* Hard error */
 
 unscaled_fontsize = ((curmovt->fontsizes)->fontsize_text)[h->size1];
 fontsize = mac_muldiv(main_stavemagn, unscaled_fontsize, 1000);
@@ -288,6 +288,7 @@ BOOL above = (flags & text_above) != 0;
 BOOL endalign = (flags & text_endalign) != 0;
 BOOL postscript = (flags & text_ps) != 0;
 BOOL rehearse = (flags & text_rehearse) != 0;
+int stringwidth;
 int six = 6*main_stavemagn;
 int x = n_x - out_Xadjustment;
 int y = above? 20000 : -10000;
@@ -298,21 +299,43 @@ int *matrix = rehearse? (curmovt->fontsizes)->fontmatrix_rehearse :
 
 DEBUG(("out_text()%s: %s\n", ((flags & text_ul) != 0)? " underlay" :"", s));
 
-/* If the "halfway" value is set, adjust the x coordinate if there is a note
-that follows in the bar. */
-
-if (out_textX != NULL && out_textX->halfway != 0)
-  {
-  if (out_moff < out_poslast->moff)
-    x += (endalign? 0 : ((flags & text_centre) != 0)? six/2 : six) +
-      mac_muldiv(out_barx + out_findXoffset(out_moff + n_length) -
-        n_x - six, out_textX->halfway, 1000);
-  }
-
-/* If font is transformed, set the matrix */
+/* If font is transformed, set the matrix, and then find the complete string
+width before any rotation (used for centring and right-aligning). */
 
 if (matrix != NULL) memcpy(font_transform, matrix, 4*sizeof(int));
+stringwidth = string_width(s, p->font, fontsize);
 
+/* Handle special positioning and rotation. If the "halfway" value is set,
+adjust the x coordinate if there is a note that follows in the bar. Otherwise,
+if a note-related offset is set, adjust the x coordinate by looking for the
+relevant musical offset. */
+
+if (out_textX != NULL)
+  {
+  if (out_textX->halfway != 0)
+    {
+    if (out_moff < out_poslast->moff)
+      x += (endalign? 0 : ((flags & text_centre) != 0)? six/2 : six) +
+        mac_muldiv(out_barx + out_findXoffset(out_moff + n_length) -
+          n_x - six, out_textX->halfway, 1000);
+    }
+
+  /* The offset value units are crotchets. */
+
+  else if (out_textX->offset != 0)
+    {
+    int offset = mac_muldiv(len_crotchet, out_textX->offset, 1000);
+    x = out_barx + out_findAoffset(out_moff + offset);
+    }
+
+  /* No rotation for underlay or overlay */
+
+  if ((flags & text_ul) == 0 && out_textX->rotate != 0)
+    font_rotate(out_textX->rotate);
+  }
+
+
+/* ======================= Underlay and Overlay ======================= */
 
 /* {Und,Ov}erlay text is centred. If it ends in '=' the position is remembered
 for starting the extender; if it ends in '-' the position is remembered for
@@ -557,13 +580,17 @@ if ((flags & text_ul) != 0)
   s = ss;
   }
 
+
+/* ======================= General text ======================= */
+
 /* Deal with not underlay. Adjust the x position if end or bar alignment is
 required, and adjust level according to the pitch of the just printed
 note/chord, if any, for the first text for any given note. Otherwise, the
-position is above or below the previous. Non-underlay text is permitted to be
-rotated. */
+position is above or below the previous. Non-underlay text can be rotated;
+this was handled above, but we retained the string width before rotating the
+font so that centring and end-alignment are done first. */
 
-else
+else if ((flags & text_followon) == 0)
   {
   BOOL baralign = (flags & text_baralign) != 0;
   BOOL timealign = (flags & text_timealign) != 0;
@@ -596,28 +623,23 @@ else
       }
     }
 
-  /* Handle /e and /c */
+  /* Handle /e and /c. Note that stringwidth is the horizontal width, ignoring
+  any font rotation. This means that rotation occurs after the text is
+  positioned. */
 
   if (endalign)
-    x += ((atbar || startbar)? 1000 : six) - string_width(s, p->font, fontsize);
-
+    x += ((atbar || startbar)? 1000 : six) - stringwidth;
   else if ((flags & text_centre) != 0)
-    x += ((atbar || startbar)? 500 : six/2) -
-      string_width(s, p->font, fontsize)/2;
-
-  /* Deal with rotation */
-
-  if (out_textX != NULL && out_textX->rotate != 0)
-    font_rotate(out_textX->rotate);
-
+    x += ((atbar || startbar)? 500 : six/2) - stringwidth/2;
 
   /* At the bar end, we adjust as for the last note or rest if end-aligned,
-  because usually such text sticks back as far as the last note. */
+  because usually such text sticks back as far as the last note. Don't adjust
+  for the next note when printing a rehearsal mark at the left of the stave. */
 
   if (above)     /* text above */
     {
     if (out_textnextabove) y = out_textnextabove;
-    else if (!atbar || endalign)
+    else if ((!atbar || endalign) && (!rehearse || !curmovt->rehearsallsleft))
       {
       int pt  = misc_ybound(FALSE, n_nexttie, TRUE, TRUE);  /* ties going out */
       int ppt = misc_ybound(FALSE, n_prevtie, TRUE, TRUE);  /* ties coming in */
@@ -683,6 +705,23 @@ else
     }
   }
 
+
+/* ======================= Generate follow-on text ======================= */
+
+/* This outputs at the ending point of the previous text string, which has
+already been through the stave magnification, though any adjustment must be so
+scaled. */
+
+else
+  {
+  out_string(s, p->font, fontsize, out_string_endx + p->x,
+    out_string_endy - (p->y * main_stavemagn)/1000, 0);
+  s = US"";   /* Stops normal output */
+  }
+
+
+/* ======================= Generate output ======================= */
+
 /* Parameters are now set up -- print, but omit null strings (to keep the
 PostScript smaller). */
 
@@ -697,9 +736,42 @@ if (*s != 0)
     int style = curmovt->rehearsalstyle;
     int yextra = (style == text_box)? 2000 : (style == text_ring)? 4000 : 0;
 
-    if (out_moff == 0) x = out_startlinebar?
-      (out_sysblock->firstnoteposition + out_sysblock->xjustify) :
-        out_lastbarlinex;
+    /* At the start of a bar, unless an offset is provided, if we are at the
+    start of a line, align with the very start if rehearsallsleft is set or
+    else with the first note. If not the start of a line, align with the
+    previous bar line. Relative p->x is then added. */
+
+    if (out_moff == 0 && (out_textX == NULL || out_textX->offset == 0))
+      {
+      if (out_startlinebar)
+        {
+        if (curmovt->rehearsallsleft)
+          {
+          x = out_sysblock->startxposition + out_sysblock->xjustify;
+          switch (bar_cont->clef)
+            {
+            case clef_trebledescant:
+            x += 15000;
+            /* Fall through */
+            case clef_trebletenor:
+            case clef_trebletenorB:
+            case clef_treble:
+            yextra += 3000;
+            if (style == text_box) yextra += 1000;
+            break;
+
+            case clef_soprabass:
+            x += 9000;
+            yextra += 1000;
+            break;
+
+            default: break;
+            }
+          }
+        else x = out_sysblock->firstnoteposition + out_sysblock->xjustify;
+        }
+      else x = out_lastbarlinex;
+      }
 
     out_string(s, p->font,
       mac_muldiv(main_stavemagn,
