@@ -2,10 +2,10 @@
 *    The PMW Music Typesetter - 3rd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2018 */
+/* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: May 2018 */
+/* This file last modified: November 2020 */
 
 
 /* This file contains the code for generating an image of one individual note
@@ -265,8 +265,10 @@ show_note(int x)
 {
 uschar buff[100];
 uschar *p;
+BOOL positioned = FALSE;
 BOOL ledgered = FALSE;
 BOOL inverted = (n_flags & nf_invert) != 0;
+int noteheadstyle;
 int top = P_6L;
 int bot = P_0L;
 int fontsize = (n_fontsize*main_stavemagn)/1000;
@@ -275,6 +277,12 @@ int yy;
 
 DEBUG(("show_note() start\n"));
 
+/* Set the notehead style */
+
+noteheadstyle = ((n_flags & nf_nhharmonic) != 0)? nh_harmonic :
+                ((n_flags & nf_nhcross) != 0)? nh_cross :
+                bar_cont->noteheadstyle;
+
 /* Set up for coupled notes */
 
 if ((n_flags & nf_coupleU) != 0)
@@ -293,7 +301,7 @@ below, as there seems no tidy way of combining it owing to the requirement for
 <= or >= comparisons. We can optimize into a single music-font string if the
 size is standard. The existence of breves makes this messy! */
 
-if (n_pitch <= bot && out_stavelines >= 5 && bar_cont->noteheadstyle != nh_none)
+if (n_pitch <= bot && out_stavelines >= 5 && noteheadstyle != nh_none)
   {
   int breve_right = 0;
   int xx = x;
@@ -305,7 +313,7 @@ if (n_pitch <= bot && out_stavelines >= 5 && bar_cont->noteheadstyle != nh_none)
     breve_right = mac_muldiv(2*curmovt->breveledgerextra, fontsize, 10000);
     }
 
-  ledgered = TRUE;
+  ledgered = positioned = TRUE;
   yy = out_ystave - (bot - 130)*main_stavemagn - n_pcorrection;
   if (n_fontsize == 10000 && !inverted)
     {
@@ -352,8 +360,7 @@ if (n_pitch <= bot && out_stavelines >= 5 && bar_cont->noteheadstyle != nh_none)
     }
   }
 
-else if (n_pitch >= top && out_stavelines >= 5 &&
-         bar_cont->noteheadstyle != nh_none)
+else if (n_pitch >= top && out_stavelines >= 5 && noteheadstyle != nh_none)
   {
   int breve_right = 0;
   int xx = x;
@@ -365,7 +372,7 @@ else if (n_pitch >= top && out_stavelines >= 5 &&
     breve_right = mac_muldiv(2*curmovt->breveledgerextra, fontsize, 10000);
     }
 
-  ledgered = TRUE;
+  ledgered = positioned = TRUE;
   yy = out_ystave - (top - 130)*main_stavemagn - n_pcorrection;
   if (n_fontsize == 10000 && !inverted)
     {
@@ -417,16 +424,15 @@ else if (n_pitch >= top && out_stavelines >= 5 &&
 the music font. */
 
 p = buff;
-if (n_notetype < dsquaver && n_stemlength == 0 &&
-  bar_cont->noteheadstyle == nh_normal &&
+if (n_notetype < dsquaver && n_stemlength == 0 && noteheadstyle == nh_normal &&
     (n_flags & (nf_invert|nf_stem|nf_smallhead)) == nf_stem)
-      {
-      if ((n_flags & nf_appogg) != 0) *p++ = n_upflag? 129 : 130;
-      *p++ = common[n_notetype + n_upflag*6];
-      *p = 0;
-      ps_musstring(buff, fontsize, x, y);
-      return ledgered;
-      }
+  {
+  if ((n_flags & nf_appogg) != 0) *p++ = n_upflag? 129 : 130;
+  *p++ = common[n_notetype + n_upflag*6];
+  *p = 0;
+  ps_musstring(buff, fontsize, x, y);
+  return ledgered;
+  }
 
 /* Deal with rarer cases, first dealing with stems & tails. We impose a minimum
 stemlength at this point. */
@@ -444,6 +450,7 @@ if ((n_flags & nf_stem) != 0)
 
   yy = y + (direction*n_stemlength*main_stavemagn)/1000;
   p += sprintf(CS p, "%s", tailstrings[n_notetype + n_upflag*8]);
+  positioned = TRUE;
 
   /* Notes with stems up */
 
@@ -451,7 +458,7 @@ if ((n_flags & nf_stem) != 0)
     {
     if (yy <= y)    /* stem is lengthened */
       {
-      int stemch = (bar_cont->noteheadstyle == nh_cross)? 'o' : 'J';
+      int stemch = (noteheadstyle == nh_cross)? 'o' : 'J';
       int z = yy;
       while (z <= y)
         {
@@ -463,7 +470,7 @@ if ((n_flags & nf_stem) != 0)
       ps_musstring(buff, fontsize, x, yy);
       p = buff;
       if (z < y + font10) *p++ = stemch;
-      if (bar_cont->noteheadstyle == nh_harmonic) *p++ = 'q';
+      if (noteheadstyle == nh_harmonic) *p++ = 'q';
       }
 
     else            /* stem is shortened */
@@ -488,7 +495,7 @@ if ((n_flags & nf_stem) != 0)
     {
     if (yy >= y)    /* stem is lengthened */
       {
-      int stemch = (bar_cont->noteheadstyle == nh_cross)? 'p' : 'K';
+      int stemch = (noteheadstyle == nh_cross)? 'p' : 'K';
       int z = yy;
       while (z >= y)
         {
@@ -500,7 +507,7 @@ if ((n_flags & nf_stem) != 0)
       ps_musstring(buff, fontsize, x, yy);
       p = buff;
       if (z > y - font10) *p++ = stemch;
-      if (bar_cont->noteheadstyle == nh_harmonic) *p++ = 'r';
+      if (noteheadstyle == nh_harmonic) *p++ = 'r';
       }
 
     else            /* stem is shortened */
@@ -522,7 +529,7 @@ if ((n_flags & nf_stem) != 0)
 
 /* Now add the note head */
 
-if (bar_cont->noteheadstyle != nh_none)
+if (noteheadstyle != nh_none)
   {
   if (inverted)
     {
@@ -550,7 +557,7 @@ if (bar_cont->noteheadstyle != nh_none)
 
   if ((n_flags & nf_smallhead) == 0)
     {
-    *p++ = headchars[n_notetype + 8*(bar_cont->noteheadstyle)];
+    *p++ = headchars[n_notetype + 8*noteheadstyle];
 
     /* When printing right-to-left, we put some redundant spacing *after*
     inverted noteheads. This is just a fudge to fool the x-coordinate adjusting
@@ -566,26 +573,36 @@ if (bar_cont->noteheadstyle != nh_none)
 *p = 0;
 ps_musstring(buff, fontsize, x, y);
 
-/* In the special case of a small note head, the note head was skipped above,
-but the printing position should be in the correct place for a full size note
-head. Arrange to output one at the cue size, with a relative position adjusted
-to allow for the head size. */
+/* In the special case of a small note head, the note head was skipped above.
+The printing position should be in the correct place for a full size note head
+if a stem or ledger lines were output above. Arrange to output the notehead at
+the cue size, with a relative position adjusted to allow for the head size. */
 
 if ((n_flags & nf_smallhead) != 0)
   {
+  int cue_fontsize = (curmovt->fontsizes)->fontsize_cue;
+  int sm_fontsize = (cue_fontsize * main_stavemagn)/1000;
+
   p = buff;
-  x = y = 0;
-  fontsize = ((curmovt->fontsizes)->fontsize_cue * main_stavemagn)/1000;
-
-  if ((n_upflag && (n_flags & nf_invert) == 0) ||
-      (!n_upflag && (n_flags & nf_invert) != 0))
-    x += 6000 - 3 * ((curmovt->fontsizes)->fontsize_cue / 5);
-
-  y += 2000 - (curmovt->fontsizes)->fontsize_cue / 5;
-
-  *p++ = headchars[n_notetype + 8*(bar_cont->noteheadstyle)];
+  *p++ = headchars[n_notetype + 8*noteheadstyle];
   *p = 0;
-  ps_relmusstring(buff, fontsize, x, y);
+
+  if (positioned && (n_flags & nf_stem) != 0)
+    {
+    x = 0;
+    if ((n_upflag && (n_flags & nf_invert) == 0) ||
+        (!n_upflag && (n_flags & nf_invert) != 0))
+      x += 6000 - 3 * (cue_fontsize / 5);
+    y = 2000 - cue_fontsize / 5;
+    ps_relmusstring(buff, sm_fontsize, x, y);
+    }
+
+  else
+    {
+    x += (((n_notetype == breve)? 19:11)*main_stavemagn)/10;
+    y -= (5*main_stavemagn)/10;
+    ps_musstring(buff, sm_fontsize, x, y);
+    }
   }
 
 /* Return whether ledger lines or not */
@@ -657,12 +674,23 @@ else yoffset += 4000 + n_pcorrection;
 
 y = out_ystave - (yoffset*main_stavemagn)/1000;
 
-/* Output a normal rest */
+/* Output a single rest, taking care of the special cases that are used for
+conventional repeat signs. */
 
 if (notetype >= 0)
-  ps_musstring(reststrings[n_notetype], fontsize, x, y);
+  {
+  uschar *s;
+  if ((n_flags & nf_restrep) == 0) s = reststrings[n_notetype];
+  else if (n_notetype == crotchet) s = US"\217";
+  else
+    {
+    s = US"\220\217";
+    y += 4*main_stavemagn;
+    }
+  ps_musstring(s, fontsize, x, y);
+  }
 
-/* Output a coded multirest */
+/* Output a multirest sign */
 
 else if (curmovt->codemultirests && out_manyrest < 9)
   {
@@ -737,10 +765,14 @@ BOOL ledgered = FALSE;
 
 DEBUG(("out_shownote() start\n"));
 
-/* If the note is invisible, skip printing; just show accents, etc. */
+/* If the note is invisible, we skip printing and just show accents, etc.
+below. This test is for notes on and this note not hidden. */
 
 if ((bar_cont->flags & cf_notes) != 0 && (n_flags & nf_hidden) == 0)
   {
+  int x_acc = 0;
+  int x_dot = 0;
+
   /* If printing a breve, move left to align with semibreve position */
 
   if (n_pitch != 0 && n_notetype == breve) xn -= (23*main_stavemagn)/10;
@@ -757,7 +789,8 @@ if ((bar_cont->flags & cf_notes) != 0 && (n_flags & nf_hidden) == 0)
       offset += ((n_acc == ac_sharp)? (curmovt->hsharpstyle == 0) :
         (curmovt->hflatstyle == 0))? 18:36;
       }
-    ps_muschar(xn - mac_muldiv(n_accleft, n_fontsize, 10000),
+    x_acc = xn - mac_muldiv(n_accleft, n_fontsize, 10000);
+    ps_muschar(x_acc,
       out_ystave - (n_pitch - 130)*main_stavemagn - n_pcorrection,
         out_acctable[offset], fontsize);
      }
@@ -780,6 +813,7 @@ if ((bar_cont->flags & cf_notes) != 0 && (n_flags & nf_hidden) == 0)
 
   if ((n_flags & (nf_dot+nf_plus)) != 0)
     {
+    BOOL dotplus = (n_flags & nf_plus) != 0;
     int dotpos = 84;
     int dotlevel;
 
@@ -818,14 +852,63 @@ if ((bar_cont->flags & cf_notes) != 0 && (n_flags & nf_hidden) == 0)
         dotpos = mac_muldiv(dotpos, n_fontsize, 10000);
       }
 
-    /* Output the dot(s) */
+    /* Output the dot(s). The '+' character is 135 in the music font; we have
+    to specify this as a UTF-8 string. */
 
-    ps_musstring(((n_flags & nf_plus) == 0)? US"?" : US"\302\207", fontsize,
-      xn + dotpos, out_ystave - dotlevel - n_pcorrection);
+    x_dot = xn + dotpos;
+    ps_musstring(dotplus? US"\302\207" : US"?", fontsize, x_dot,
+      out_ystave - dotlevel - n_pcorrection);
 
     if ((n_flags & nf_dot2) != 0)
-      ps_musstring(US"?", fontsize, xn + dotpos + (35*main_stavemagn)/10,
-        out_ystave - dotlevel - n_pcorrection);
+      {
+      x_dot += (35*main_stavemagn)/10;
+      ps_musstring(US"?", fontsize, x_dot, out_ystave - dotlevel - n_pcorrection);
+      }
+    if (dotplus) x_dot += 4*main_stavemagn;  /* Extra for ) */
+    }
+
+  /* Deal with bracketed notehead */
+
+  if ((n_flags & nf_headbra) != 0)
+    {
+    int adjustbra = 0;
+    int adjustket = 0;
+    int bfontsize = (6*fontsize)/10;
+    int yb = out_ystave - (n_pitch - 130)*main_stavemagn -
+      (9*main_stavemagn/10);
+    uschar *bra = main_righttoleft? US"\302\216" : US"\302\215";
+    uschar *ket = main_righttoleft? US"\302\215" : US"\302\216";
+
+    if (x_acc == 0) x_acc = xn + main_stavemagn;
+    if (x_dot == 0) x_dot = xn + 5*main_stavemagn;
+
+    if (n_notetype == breve) adjustket += 5*main_stavemagn;
+
+    if ((n_flags & nf_smallhead) != 0)
+      {
+      if ((n_flags & nf_stem) != 0)
+        {
+        if (n_upflag) adjustbra += 2*main_stavemagn;
+          else adjustket -= 2*main_stavemagn;
+        }
+      else
+        {
+        adjustbra += 2*main_stavemagn;
+        adjustket -= 2*main_stavemagn;
+        }
+
+      if (ledgered)
+        {
+        adjustbra -= (15*main_stavemagn)/10;
+        adjustket += main_stavemagn;
+        }
+      }
+
+    ps_musstring(bra, bfontsize,
+      x_acc - (35*main_stavemagn)/10 + adjustbra, yb);
+
+    ps_musstring(ket, bfontsize,
+      x_dot + (40*main_stavemagn)/10 + adjustket, yb);
     }
   }
 
@@ -1008,11 +1091,11 @@ if ((n_acflags & af_dynoutside) != 0)
   int yextra = 0;
   int upflag = acc_upflag;
 
-  /* Sanitizing compilers may complain about shifting negative numbers and/or
-  shifting top bits off words, so be pedantic here. Start by getting the first
-  relevant bit to the top position. */
+  /* Sanitizing compilers may complain about shifting top bits off words, so be
+  pedantic here. Start by getting the first relevant bit to the top position.
+  */
 
-  unsigned int f = (unsigned int)(n_acflags & 0x1fffffff) << 3;
+  usint f = (n_acflags & 0x1fffffff) << 3;
 
   /* The order is: > v V ' down up. We assume that only one of the first four
   will exist, possibly with one of the last two. */
