@@ -2,10 +2,10 @@
 *    The PMS Music Typesetter - 2nd incarnation  *
 *************************************************/
 
-/* Copyright (c) Philip Hazel, 1991 - 2019 */
+/* Copyright (c) Philip Hazel, 1991 - 2020 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: July 2020 */
 
 
 /* This file contains code for reading and processing strings. */
@@ -57,7 +57,7 @@ static esctabstr esctab[] = {
   { ('C' << 8) + '\'',   0x0106 },  /* Cacute */
   { ('C' << 8) + ')',    0x00a9 },  /* Copyright */
   { ('C' << 8) + ',',    0x00c7 },  /* Ccedilla */
-  { ('C' << 8) + '@',    0x0108 },  /* Ccircumflex */ 
+  { ('C' << 8) + '@',    0x0108 },  /* Ccircumflex */
   { ('C' << 8) + 'v',    0x010c },  /* Ccaron */
 
   { ('D' << 8) + '-',    0x0110 },  /* Dcroat */
@@ -71,10 +71,10 @@ static esctabstr esctab[] = {
   { ('E' << 8) + 'u',    0x0114 },  /* Ebreve */
   { ('E' << 8) + 'v',    0x011a },  /* Ecaron */
 
-  { ('G' << 8) + '@',    0x011c },  /* Gcircumflex */ 
+  { ('G' << 8) + '@',    0x011c },  /* Gcircumflex */
   { ('G' << 8) + 'u',    0x011e },  /* Gbreve */
 
-  { ('H' << 8) + '@',    0x0124 },  /* Hcircumflex */ 
+  { ('H' << 8) + '@',    0x0124 },  /* Hcircumflex */
 
   { ('I' << 8) + '\'',   0x00cd },  /* Iacute */
   { ('I' << 8) + '-',    0x012a },  /* Imacron */
@@ -84,7 +84,7 @@ static esctabstr esctab[] = {
   { ('I' << 8) + 'u',    0x012C },  /* Ibreve */
   { ('I' << 8) + '~',    0x0128 },  /* Itilde */
 
-  { ('J' << 8) + '@',    0x0134 },  /* Jcircumflex */ 
+  { ('J' << 8) + '@',    0x0134 },  /* Jcircumflex */
 
   { ('L' << 8) + '\'',   0x0139 },  /* Lacute */
   { ('L' << 8) + '/',    0x0141 },  /* Lslash */
@@ -108,7 +108,7 @@ static esctabstr esctab[] = {
 
   { ('S' << 8) + '\'',   0x015a },  /* Sacute */
   { ('S' << 8) + ',',    0x015e },  /* Scedilla */
-  { ('S' << 8) + '@',    0x015c },  /* Scircumflex */ 
+  { ('S' << 8) + '@',    0x015c },  /* Scircumflex */
   { ('S' << 8) + 'v',    0x0160 },  /* Scaron */
 
   { ('T' << 8) + ',',    0x0162 },  /* Tcedilla */
@@ -123,11 +123,11 @@ static esctabstr esctab[] = {
   { ('U' << 8) + 'u',    0x016c },  /* Ubreve */
   { ('U' << 8) + '~',    0x0168 },  /* Utilde */
 
-  { ('W' << 8) + '@',    0x0174 },  /* Wcircumflex */ 
+  { ('W' << 8) + '@',    0x0174 },  /* Wcircumflex */
 
   { ('Y' << 8) + '\'',   0x00dd },  /* Yacute */
   { ('Y' << 8) + '.',    0x0178 },  /* Ydieresis */
-  { ('Y' << 8) + '@',    0x0176 },  /* Ycircumflex */ 
+  { ('Y' << 8) + '@',    0x0176 },  /* Ycircumflex */
 
   { ('Z' << 8) + '\'',   0x0179 },  /* Zacute */
   { ('Z' << 8) + 'v',    0x017d },  /* Zcaron */
@@ -144,7 +144,7 @@ static esctabstr esctab[] = {
   { ('c' << 8) + '\'',   0x0107 },  /* cacute */
   { ('c' << 8) + ')',    0x00a9 },  /* copyright */
   { ('c' << 8) + ',',    0x00e7 },  /* ccedilla */
-  { ('c' << 8) + '@',    0x0109 },  /* ccircumflex */ 
+  { ('c' << 8) + '@',    0x0109 },  /* ccircumflex */
   { ('c' << 8) + 'v',    0x010d },  /* ccaron */
 
   { ('d' << 8) + '-',    0x0111 },  /* dcroat */
@@ -158,10 +158,10 @@ static esctabstr esctab[] = {
   { ('e' << 8) + 'u',    0x0115 },  /* ebreve */
   { ('e' << 8) + 'v',    0x011b },  /* ecaron */
 
-  { ('g' << 8) + '@',    0x011d },  /* gcircumflex */ 
+  { ('g' << 8) + '@',    0x011d },  /* gcircumflex */
   { ('g' << 8) + 'u',    0x011f },  /* gbreve */
 
-  { ('h' << 8) + '@',    0x0125 },  /* hcircumflex */ 
+  { ('h' << 8) + '@',    0x0125 },  /* hcircumflex */
 
   { ('i' << 8) + '\'',   0x00ed },  /* iacute */
   { ('i' << 8) + '-',    0x012b },  /* imacron */
@@ -171,7 +171,7 @@ static esctabstr esctab[] = {
   { ('i' << 8) + 'u',    0x012d },  /* ibreve */
   { ('i' << 8) + '~',    0x0129 },  /* itilde */
 
-  { ('j' << 8) + '@',    0x0135 },  /* jcircumflex */ 
+  { ('j' << 8) + '@',    0x0135 },  /* jcircumflex */
 
   { ('l' << 8) + '\'',   0x013a },  /* Lacute */
   { ('l' << 8) + '/',    0x0142 },  /* Lslash */
@@ -195,7 +195,7 @@ static esctabstr esctab[] = {
 
   { ('s' << 8) + '\'',   0x015b },  /* sacute */
   { ('s' << 8) + ',',    0x015f },  /* scedilla */
-  { ('s' << 8) + '@',    0x015d },  /* scircumflex */ 
+  { ('s' << 8) + '@',    0x015d },  /* scircumflex */
   { ('s' << 8) + 'v',    0x0161 },  /* scaron */
 
   { ('t' << 8) + ',',    0x0163 },  /* tcedilla */
@@ -210,11 +210,11 @@ static esctabstr esctab[] = {
   { ('u' << 8) + 'u',    0x016d },  /* ubreve */
   { ('u' << 8) + '~',    0x0169 },  /* utilde */
 
-  { ('w' << 8) + '@',    0x0175 },  /* wcircumflex */ 
+  { ('w' << 8) + '@',    0x0175 },  /* wcircumflex */
 
   { ('y' << 8) + '\'',   0x00fd },  /* yacute */
   { ('y' << 8) + '.',    0x00ff },  /* ydieresis */
-  { ('y' << 8) + '@',    0x0177 },  /* ycircumflex */ 
+  { ('y' << 8) + '@',    0x0177 },  /* ycircumflex */
 
   { ('z' << 8) + '\'',   0x017a },  /* zacute */
   { ('z' << 8) + 'v',    0x017e },  /* zcaron */
@@ -312,7 +312,7 @@ uschar *s;
 sigch();
 if (read_ch != '\"')
   {
-  error_moan(10, "String in quotes");
+  error_moan(ERR10, "String in quotes");
   return NULL;
   }
 
@@ -368,7 +368,7 @@ while (read_ch != '\"' && read_ch != EOF)
   }
 
 s[p] = 0;
-if (read_ch == EOF) { error_moan(23); return NULL; }
+if (read_ch == EOF) { error_moan(ERR23); return NULL; }
 next_ch();
 return s;
 }
@@ -380,8 +380,8 @@ return s;
 
 /* This is called for all but PostScript strings on reading, so that any errors
 are given at that time. We set a flag (in_string_check) while handling escapes
-to ensure that the \a^ -> \a@ fudge works correctly. Character codes are 
-checked for supported values, and those that are not supported in standardly 
+to ensure that the \a^ -> \a@ fudge works correctly. Character codes are
+checked for supported values, and those that are not supported in standardly
 encoded fonts are converted to use the Symbol font where possible.
 
 Another job of this function is to look for note letter transpositions in the
@@ -446,7 +446,7 @@ if ((tt = Ustrstr(s, "\\t")) != NULL)
     /* If we are not in a stave, the appropriate values for the transposing
     function should have been fudged up by the heading reading code. */
 
-    if (stave_transpose < max_transpose)
+    if (stave_transpose != no_transpose)
       abspitch = transpose_note(
         abspitch,    /* input absolute pitch */
         &pitch,      /* for output new pitch */
@@ -511,7 +511,7 @@ if ((tt = Ustrstr(s, "\\t")) != NULL)
  ss[p] = 0;
  s = ss;
  }
- 
+
 /* Now do an escape check on the (new) string, but don't make any changes to
 it. This gets any errors early. The setting of in_string_check is tested in
 string_escape() to cause an error for \@, which is used internally. */
@@ -549,7 +549,7 @@ string_integer(uschar **p)
 uschar *s = *p;
 int n = 0;
 while (isdigit(*s)) n = (n * 10) + (*s++ - '0');
-if (n < 32 || n > 255) { error_moan(53); n = 32; }
+if (n < 32 || n > 255) { error_moan(ERR53); n = 32; }
 *p = s;
 return n;
 }
@@ -560,7 +560,7 @@ return n;
 *             Decode string escape               *
 *************************************************/
 
-/* This function is called during the checking process when a string is first 
+/* This function is called during the checking process when a string is first
 read (to catch escape errors early), in which case in_string_check is TRUE. It
 is also called again later when the string is processed for real. On entry, the
 pointer is past the initial '\'; return it past the whole escape. The data
@@ -591,7 +591,7 @@ uschar *escape_uschar = s - 1;
 uschar *error_argS = NULL;
 uschar escbuffer[8];
 int error_argN = 0;
-int error_number = 54;
+int error_number = ERR54;
 int unicode = -1;
 int bot, top, ee;
 
@@ -816,8 +816,8 @@ else switch (*s)
     }
   if (*s != '\\')
     {
-    error_number = 27;
-    error_argN = *s; 
+    error_number = ERR27;
+    error_argN = *s;
     }
   else s++;
   break;
@@ -830,16 +830,16 @@ else switch (*s)
   case 'd':
   case 'e':
   case 'g':
-  case 'h': 
+  case 'h':
   case 'i':
-  case 'j': 
+  case 'j':
   case 'l':
   case 'n':
   case 'o':
   case 'r':
   case 't':
   case 'u':
-  case 'w': 
+  case 'w':
   case 'y':
   case 'z':
   case 'A':
@@ -848,7 +848,7 @@ else switch (*s)
   case 'E':
   case 'G':
   case 'H':
-  case 'J':  
+  case 'J':
   case 'I':
   case 'L':
   case 'N':
@@ -857,7 +857,7 @@ else switch (*s)
   case 'S':
   case 'T':
   case 'U':
-  case 'W': 
+  case 'W':
   case 'Y':
   case 'Z':
 
@@ -867,15 +867,15 @@ else switch (*s)
 
   /* There is a problem with escapes using circumflex, because this is also
   used to split up underlay strings for centering. This is dealt with as
-  follows: All strings are checked as they are read. If we are in the 
+  follows: All strings are checked as they are read. If we are in the
   string_check() function, \@ is an error, because it is not defined for the
-  user to use. After that test, if we see \^ we turn it into \@. Thus, when the 
+  user to use. After that test, if we see \^ we turn it into \@. Thus, when the
   string is interpreted for escapes when actually being processed for display,
   it will see \@ and not \^. */
 
   if (*(++s) == '@' && in_string_check)
     {
-    error_number = 125;
+    error_number = ERR125;
     error_argS = s - 1;
     break;
     }
@@ -909,7 +909,7 @@ else switch (*s)
 
   if (unicode < 0)
     {
-    error_number = 125;
+    error_number = ERR125;
     Ustrncpy(escbuffer, s - 3, 3);
     escbuffer[3] = 0;
     error_argS = escbuffer;
@@ -927,8 +927,8 @@ else switch (*s)
     }
   else
     {
-    error_number = 40;
-    error_argN = *s; 
+    error_number = ERR40;
+    error_argN = *s;
     }
   break;
   }
@@ -946,7 +946,7 @@ if (error_argS != NULL || error_argN != 0)
   while (*s) { *s++ = '/'; error_ptr_correction++; }
   error_ptr_correction += 1;
   if (error_argS != NULL) error_moan(error_number, error_argS);
-  if (error_argN != 0) error_moan(error_number, error_argN); 
+  if (error_argN != 0) error_moan(error_number, error_argN);
   }
 
 /* If there is a value in unicode, convert it to a string */
@@ -976,6 +976,7 @@ string_stavestring(BOOL rehearse)
 BOOL hadab = FALSE;
 int rotate = 0;
 int halfway = 0;
+int offset = 0;
 int htype = 0;
 int size = -1;
 int flags = stave_textflags;
@@ -1058,6 +1059,12 @@ while (read_ch == '/' && *read_chptr != '/')
     next_ch();
     break;
 
+    case 'F':
+    if (!stave_string_followOK || rehearse) error_moan(ERR141);
+      else flags |= text_followon;
+    next_ch();
+    break;
+
     case 'f':
     next_ch();
     if (read_ch == 'b')
@@ -1065,7 +1072,7 @@ while (read_ch == '/' && *read_chptr != '/')
       flags &= ~(text_ul | text_above | text_ps | text_middle | text_atulevel);
       flags |= text_fb;
       }
-    else error_moan(37, "/fb");
+    else error_moan(ERR37, "/fb");
     next_ch();
     break;
 
@@ -1076,7 +1083,12 @@ while (read_ch == '/' && *read_chptr != '/')
     break;
 
     case 'l':
-    adjustx -= read_movevalue();
+    if (*read_chptr == 'c')
+      {
+      next_ch();
+      offset -= read_movevalue();
+      }
+    else adjustx -= read_movevalue();
     break;
 
     case 'm':
@@ -1089,7 +1101,7 @@ while (read_ch == '/' && *read_chptr != '/')
     next_ch();
     if (read_ch == 'c') flags &= ~text_centre;
     else if (read_ch == 'e') flags &= ~text_endalign;
-    else error_moan(37, "/nc or /ne");
+    else error_moan(ERR37, "/nc or /ne");
     next_ch();
     break;
 
@@ -1100,7 +1112,7 @@ while (read_ch == '/' && *read_chptr != '/')
       flags &= ~(text_fb | text_ps);
       flags |= text_ul | text_above;
       }
-    else error_moan(37, "/ol");
+    else error_moan(ERR37, "/ol");
     next_ch();
     break;
 
@@ -1112,7 +1124,7 @@ while (read_ch == '/' && *read_chptr != '/')
         text_atulevel | text_absolute);
       flags |= text_ps;
       }
-    else error_moan(37, "/ps");
+    else error_moan(ERR37, "/ps");
     next_ch();
     break;
 
@@ -1132,6 +1144,11 @@ while (read_ch == '/' && *read_chptr != '/')
       next_ch();
       read_expect_integer(&rotate, TRUE, TRUE);
       }
+    else if (*read_chptr == 'c')
+      {
+      next_ch();
+      offset += read_movevalue();
+      }
     else adjustx += read_movevalue();
     break;
 
@@ -1140,14 +1157,14 @@ while (read_ch == '/' && *read_chptr != '/')
     if (read_expect_integer(&size, FALSE, FALSE))
       {
       if (--size < 0 || size >= MaxFontSizes)
-        { error_moan(39, MaxFontSizes); size = 0; }
+        { error_moan(ERR39, MaxFontSizes); size = 0; }
       }
     break;
 
     case 't':
     next_ch();
     if (read_ch == 's') flags |= text_timealign;
-      else error_moan(37, "/ts");
+      else error_moan(ERR37, "/ts");
     next_ch();
     break;
 
@@ -1160,13 +1177,13 @@ while (read_ch == '/' && *read_chptr != '/')
         flags &= ~(text_fb | text_above | text_ps);
         flags |= text_ul;
         }
-      else error_moan(37, "/u<number> or /ul");
+      else error_moan(ERR37, "/u<number> or /ul");
       next_ch();
       }
     break;
 
     default:
-    error_moan(37, "/a, /ao, /b, /bar, /box, /bu, /d, /e, /fb, "
+    error_moan(ERR37, "/a, /ao, /b, /bar, /box, /bu, /d, /e, /F, /fb, "
       "/h, /l, /m, /ol, /ps, /r, /ring, /s, /u or /ul");
     next_ch();
     break;
@@ -1222,7 +1239,7 @@ while (read_ch == '/' && *read_chptr != '/')
             if (read_expect_integer(sizeptr, FALSE, FALSE))
               {
               if ((*sizeptr -= 1) < 0 || *sizeptr >= MaxFontSizes)
-                { error_moan(39, MaxFontSizes); *sizeptr = 0; }
+                { error_moan(ERR39, MaxFontSizes); *sizeptr = 0; }
               }
             break;
 
@@ -1231,13 +1248,13 @@ while (read_ch == '/' && *read_chptr != '/')
             break;
 
             case '\"':
-            if (s2 != NULL) error_moan(92);
+            if (s2 != NULL) error_moan(ERR92);
             s2 = string_read();
             sizeptr = &size2;
             break;
 
             default:
-            error_moan(37, "/d, /s, /u, or string");
+            error_moan(ERR37, "/d, /s, /u, or string");
             next_ch();
             break;
             }
@@ -1246,7 +1263,7 @@ while (read_ch == '/' && *read_chptr != '/')
         /* Only applies to {und,ov}erlay. Set up a hyphen-type block, or point
         to an identical one. */
 
-        if ((flags & text_ul) == 0) error_moan(90); else
+        if ((flags & text_ul) == 0) error_moan(ERR90); else
           {
           int font = ((flags & text_above) == 0)?
             stave_ulfont : stave_olfont;
@@ -1294,9 +1311,37 @@ while (read_ch == '/' && *read_chptr != '/')
 
 ENDSTRING:
 
-/* Unless this is a PostScript string, check its escapes */
+/* If this is a follow-on string, allow only relative positioning options.
+Follow-ons are forbidden for rehearsal strings above. */
 
-if ((flags & text_ps) == 0) s = string_check(s);
+if ((flags & text_followon) != 0)
+  {
+  if (hadab || halfway != 0 || offset != 0 ||
+       ((flags & (text_baralign|text_box|text_centre|text_endalign|text_fb|
+         text_middle|text_ps|text_ring|text_timealign|text_ul)) != 0))
+    {
+    error_moan(ERR142,
+      ((flags & text_ps) != 0)? "PostScript string" :
+      ((flags & (text_box|text_ring)) != 0)? "boxed or ringed string" :
+      "string with explicit positioning");
+    flags &= ~text_followon;
+    }
+  }
+
+/* Unless this is a PostScript string, check its escapes. If the next
+significant character is double quote, the next string can be a follow-on,
+unless this string is boxed or ringed. */
+
+stave_string_followOK = FALSE;
+if ((flags & text_ps) == 0)
+  {
+  s = string_check(s);
+  if ((flags & (text_box|text_ring)) == 0)
+    {
+    sigch();
+    stave_string_followOK = read_ch == '\"';
+    }
+  }
 
 /* If the absolute flag is set and we have not had /a or /b, then add in the
 default absolute position. */
@@ -1335,15 +1380,20 @@ preceded by an extras block if required. */
 if ((flags & text_ul) == 0)
   {
   b_textstr *p;
-  if (rotate || halfway)
+  if (rotate != 0 || halfway != 0 || offset != 0)
     {
     b_textXstr *pp = store_getitem(b_textX);
     pp->rotate = rotate;
     pp->halfway = halfway;
+    pp->offset = offset;
+    if (halfway != 0 && offset != 0) error_moan(ERR140);  /* Warning */
     }
   p = store_getitem(b_text);
-  p->size = size;
   p->font = fontid;
+  p->size = size;
+  p->htype = 0;
+  p->ulevel = 0;
+  p->ulen = 0;
   p->flags = flags;
   p->string = s;
   p->x = adjustx;
@@ -1362,7 +1412,7 @@ else
   ulaypend *p = stave_pendulay;
   ulaypend *q;
 
-  if (rotate) error_moan(98, "supported", "for underlay or overlay");
+  if (rotate) error_moan(ERR98, "supported", "for underlay or overlay");
 
   while (p != NULL)
     {
@@ -1389,6 +1439,7 @@ else
   q->x = adjustx;
   q->y = adjusty;
   q->halfway = halfway;
+  q->offset = offset;
   }
 }
 
@@ -1429,7 +1480,7 @@ if (f >= font_sc)
   }
 fs = &(font_List[font_table[f]]);
 
-while ((c = *s++) != 0)
+while (*s != 0)
   {
   int nf, r;
   uschar xs[80];
@@ -1445,9 +1496,15 @@ while ((c = *s++) != 0)
     pp = ss;
     }
 
-  /* Until we hit the escape character, simply copy bytes, except that we must
-  turn ` and ' into open and close quotes, and fi into the ligature, for fonts
-  with standard encoding. */
+  /* Pick up the next character. For backwards compatibility, bytes with a
+  value >= 0x80 but < 0xc0 are taken as character values. */
+
+  if (*s > 127 && (*s & 0x40) == 0) c = *s; else { GETCHARINC(c, s); }
+
+  /* Until we hit the escape character, just copy the character, except that we
+  must turn ` and ' into open and close quotes, and fi into the ligature, for
+  fonts with standard encoding. We have to do this by character rather than
+  just copying bytes because of the buffer overflow handling above. */
 
   if (c != '\\')
     {
