@@ -5,7 +5,7 @@
 /* Copyright (c) Philip Hazel, 1991 - 2019 */
 
 /* Written by Philip Hazel, starting November 1991 */
-/* This file last modified: January 2019 */
+/* This file last modified: November 2020 */
 
 
 /* This file contains all the major tables used by the program; a few that are
@@ -43,14 +43,14 @@ const uschar utf8_table4[] = {
   1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
   2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
   3,3,3,3,3,3,3,3,4,4,4,4,5,5,5,5 };
-  
+
 
 
 /*************************************************
 *               Miscellaneous                    *
 *************************************************/
 
-unsigned int main_bit[] = {
+usint main_bit[] = {
  0x00000001u, 0x00000002u, 0x00000004u, 0x00000008u,
  0x00000010u, 0x00000020u, 0x00000040u, 0x00000080u,
  0x00000100u, 0x00000200u, 0x00000400u, 0x00000800u,
@@ -70,7 +70,10 @@ uschar *font_IdStrings[] = {
   US"xf9", US"xf10", US"xf11", US"xf12" };
 
 
-/* Key signatures table giving number of sharps or flats */
+/* Key signatures table giving number of sharps or flats; positive for sharps
+and negative for flats. A value of -100 is used for unsupported key signatures.
+The final value is for the special "N" key signature, which is like C major but
+is never transposed. */
 
 int main_keysigtable[] = {
   /* a     b     c     d     e     f     g    */
@@ -79,7 +82,8 @@ int main_keysigtable[] = {
     -4,   -2,   -7,   -5,   -3, -100,   -6,  /* flattened  */
      0,    2,   -3,   -1,    1,   -4,   -2,  /* minor keys */
      7, -100,    4,    6, -100,    3,    5,  /* sharpened  */
-    -7,   -5, -100, -100,   -6, -100, -100   /* flattened  */
+    -7,   -5, -100, -100,   -6, -100, -100,  /* flattened  */
+     0                                       /* N */
 };
 
 
@@ -111,16 +115,16 @@ int main_flatorder[] = {
 
 /* Table giving offsets into above for all the clefs */
 
-int main_clefoffset[] = { 
-   0,     /* treble */ 
+int main_clefoffset[] = {
+   0,     /* treble */
    7,     /* soprano */
   14,     /* mezzo */
   21,     /* alto */
   28,     /* tenor */
-  35,     /* Cbaritone */ 
-  42,     /* (F)baritone */ 
+  35,     /* Cbaritone */
+  42,     /* (F)baritone */
   49,     /* bass */
-  56,     /* deepbass */ 
+  56,     /* deepbass */
    0,     /* hclef */
    0,     /* none */
    0,     /* trebledescant */
@@ -134,13 +138,13 @@ int main_clefoffset[] = {
 the table of widths in the movement structure. It indicates which entries in
 the table are for which kinds of clef shape. */
 
-int main_cleftypes[] = { clef_treble, 
-  clef_alto, clef_alto, clef_alto, clef_alto, clef_alto, 
-  clef_bass, clef_bass, clef_bass, 
+int main_cleftypes[] = { clef_treble,
+  clef_alto, clef_alto, clef_alto, clef_alto, clef_alto,
+  clef_bass, clef_bass, clef_bass,
   clef_h, clef_none,
   clef_treble, clef_treble, clef_treble,
-  clef_bass, clef_bass };   
-  
+  clef_bass, clef_bass };
+
 
 /* Table of pitch values for the seven note names */
 
@@ -148,10 +152,10 @@ uschar read_basicpitch[] = { 9, 11, 0, 2, 4, 5, 7 };
 
 /* Table giving pitch increments/decrements indexed by accidental number. The
 values are kept with 2 added so that they will fit into the baraccs tables as
-bytes. The first value shouldn't be used (it represents "no accidental"), but
-set it as a natural just in case. */                      
+bytes. The first value represents "no accidental" so it is set the same as a
+natural. */
 
-                        /* - ##  $ $$  %  # */ 
+                        /* - ##  $ $$  %  # */
 uschar read_accpitch[] = { 2, 4, 1, 0, 2, 3 };
 
 
@@ -180,9 +184,9 @@ int out_acctable[] = {
 int read_headingsizes[] = { 17000, 12000, 10000, 8000 };
 
 
-/* Tables for accent and ornament decoding. The values of the or_xx variables 
-are all less than 256, whereas the flag bits defined by the af_xx variables are 
-all greater. The ordering of strings that start with the same character is 
+/* Tables for accent and ornament decoding. The values of the or_xx variables
+are all less than 256, whereas the flag bits defined by the af_xx variables are
+all greater. The ordering of strings that start with the same character is
 important! */
 
 accent accent_chars[] = {
@@ -196,32 +200,32 @@ accent accent_chars[] = {
   { US"u",   af_up },
   { US"o",   af_ring },
   { US"'",   af_vline },
-  { US"!",   af_opposite },        
-  { US"///", or_trem3 }, 
-  { US"//",  or_trem2 }, 
-  { US"/",   or_trem1 }, 
+  { US"!",   af_opposite },
+  { US"///", or_trem3 },
+  { US"//",  or_trem2 },
+  { US"/",   or_trem1 },
   { US"~~|", or_dmord },
   { US"~~",  or_dimord },
   { US"~|",  or_mord },
   { US"~",   or_imord },
-  { US"aru", or_arpu }, 
-  { US"ard", or_arpd }, 
-  { US"ar",  or_arp }, 
-  { US"sp",  or_spread }, 
+  { US"aru", or_arpu },
+  { US"ard", or_arpd },
+  { US"ar",  or_arp },
+  { US"sp",  or_spread },
   { US"tr#", or_trsh },
   { US"tr$", or_trfl },
   { US"tr%", or_trnat },
-  { US"tr",  or_tr },     
-  { US"t|",  or_iturn }, 
-  { US"t",   or_turn }, 
+  { US"tr",  or_tr },
+  { US"t|",  or_iturn },
+  { US"t",   or_turn },
   { US"f",   or_ferm },
   { NULL, 0}
-  }; 
+  };
 
-/* This table lists the accents in user numerical order, for a<n>, followed by 
+/* This table lists the accents in user numerical order, for a<n>, followed by
 af_opposite for the "accent on opposite side" flag, which must be last. */
 
-unsigned int accent_list[] = {
+usint accent_list[] = {
   0, af_staccato, af_bar, af_gt, af_wedge, af_tp, af_down, af_up, af_ring,
   af_vline, af_staccatiss, af_opposite };
 
@@ -249,11 +253,11 @@ int stave_tops[] =    { 8000, 8000, 12000, 16000, 12000, 16000, 20000 };
 int stave_bottoms[] = { 8000, 8000,  4000,     0,     0,     0,     0 };
 
 
-/* Data for long rest bars; [0] is for general symbol, [2-8] for special 
+/* Data for long rest bars; [0] is for general symbol, [2-8] for special
 symbols. */
 
-int longrest_barwidths[] = { 50, 0, 30, 35, 30, 35, 35, 45, 40 }; 
-int longrest_widths[]    = { 30000, 0, 4000, 11500, 4000, 11500, 10700, 18200, 
+int longrest_barwidths[] = { 50, 0, 30, 35, 30, 35, 35, 45, 40 };
+int longrest_widths[]    = { 30000, 0, 4000, 11500, 4000, 11500, 10700, 18200,
                              10700 };
 
 
@@ -328,30 +332,30 @@ static mfstr df1           = { NULL, 3900,  0, '\'' };
 static mfstr doubleflat    = { &df1, 0,     0, '\'' };
 
 /* Round-bracketed accidentals */
-                           
+
 static mfstr rnatural      = { NULL,  0,  0, (142<<16)+('('<<8)+141 };
 static mfstr rsharp        = { NULL,  0,  0, (142<<16)+('%'<<8)+141 };
 static mfstr rdsharp       = { NULL,  0,  0, (142<<16)+('&'<<8)+141 };
-                           
+
 static mfstr rf2           = { NULL,  8000,2000,  142 };
 static mfstr rf1           = { &rf2,  3500,   0, '\'' };
 static mfstr rflat         = { &rf1,     0,2000,  141 };
-                           
+
 static mfstr rdf3          = { NULL, 11900,2000,  142 };
 static mfstr rdf2          = { &rdf3, 7400,   0, '\'' };
 static mfstr rdf1          = { &rdf2, 3500,   0, '\'' };
 static mfstr rdflat        = { &rdf1,    0,2000,  141 };
 
 /* Square-bracket accidentals */
-                           
+
 static mfstr snatural      = { NULL,  0,  0, (140<<16)+('('<<8)+139 };
 static mfstr ssharp        = { NULL,  0,  0, (140<<16)+('%'<<8)+139 };
 static mfstr sdsharp       = { NULL,  0,  0, (140<<16)+('&'<<8)+139 };
-                           
+
 static mfstr sf2           = { NULL,  8000,2000,  140 };
 static mfstr sf1           = { &sf2,  3500,   0, '\'' };
 static mfstr sflat         = { &sf1,     0,2000,  139 };
-                           
+
 static mfstr sdf3          = { NULL, 11900,2000,  140 };
 static mfstr sdf2          = { &sdf3, 7400,   0, '\'' };
 static mfstr sdf1          = { &sdf2, 3500,   0, '\'' };
@@ -368,28 +372,28 @@ static mfstr halfflat2     = { NULL, 0, 0, 192 };
 
 static mfstr hrsharp1      = { NULL,  0,  0, (142<<16)+(189<<8)+141 };
 static mfstr hrsharp2      = { NULL,  0,  0, (142<<16)+(190<<8)+141 };
-                           
+
 static mfstr hrf12         = { NULL,   8000, 2000,  142 };
 static mfstr hrf11         = { &hrf12, 3500,    0,  191 };
 static mfstr hrflat1       = { &hrf11,    0, 2000,  141 };
-                           
+
 static mfstr hrf22         = { NULL,   8000, 2000,  142 };
 static mfstr hrf21         = { &hrf22, 3500,    0,  192 };
 static mfstr hrflat2       = { &hrf21,    0, 2000,  141 };
 
 /* Square-bracketed half-accidentals */
-                           
+
 static mfstr hssharp1      = { NULL,  0,  0, (140<<16)+(189<<8)+139 };
 static mfstr hssharp2      = { NULL,  0,  0, (140<<16)+(190<<8)+139 };
 
 static mfstr hsf12         = { NULL,   8000, 2000, 140 };
 static mfstr hsf11         = { &hsf12, 3500,    0, 191 };
 static mfstr hsflat1       = { &hsf11,    0, 2000, 139 };
-                           
+
 static mfstr hsf22         = { NULL,   8000, 2000, 140 };
 static mfstr hsf21         = { &hsf22, 3500,    0, 192 };
 static mfstr hsflat2       = { &hsf21,    0, 2000, 139 };
-                           
+
 /* Miscllaneous */
 
 static mfstr common        = { NULL, 0, 0, '^' };
@@ -398,25 +402,25 @@ static mfstr hclef         = { NULL, 0, 0, 173 };
 static mfstr longrest      = { NULL, 0, 0, '0' };
 
 
-/* This table is indexed by "virtual music font" character. These are defined 
-as mc_xxx enums in outhdr.h and must be kept in step with this table. The order 
+/* This table is indexed by "virtual music font" character. These are defined
+as mc_xxx enums in outhdr.h and must be kept in step with this table. The order
 is arbitrary. */
 
 mfstr *out_mftable_ps[] =
   {
-  &treble,       &trebletenor,  &trebledescant, &bass,      
-  &contrabass,   &soprabass,    &cbaritone,     &tenor,       
-  &alto,         &mezzo,        &soprano,       &natural,  
+  &treble,       &trebletenor,  &trebledescant, &bass,
+  &contrabass,   &soprabass,    &cbaritone,     &tenor,
+  &alto,         &mezzo,        &soprano,       &natural,
   &sharp,        &doublesharp,  &flat,          &doubleflat,
-  &rnatural,     &rsharp,       &rdsharp,       &rflat, 
-  &rdflat,       &snatural,     &ssharp,        &sdsharp,     
+  &rnatural,     &rsharp,       &rdsharp,       &rflat,
+  &rdflat,       &snatural,     &ssharp,        &sdsharp,
   &sflat,        &sdflat,       &common,        &cut,
-  &longrest,     &trebletenorB, &hclef,         &baritone,   
-  &deepbass,     &oldbass,      &oldcontrabass, &oldsoprabass, 
-  &oldcbaritone, &oldtenor,     &oldalto,       &oldmezzo,   
-  &oldsoprano,   &oldbaritone,  &olddeepbass,   &halfsharp1, 
-  &hrsharp1,     &hssharp1,     &halfsharp2,    &hrsharp2, 
-  &hssharp2,     &halfflat1,    &hrflat1,       &hsflat1, 
+  &longrest,     &trebletenorB, &hclef,         &baritone,
+  &deepbass,     &oldbass,      &oldcontrabass, &oldsoprabass,
+  &oldcbaritone, &oldtenor,     &oldalto,       &oldmezzo,
+  &oldsoprano,   &oldbaritone,  &olddeepbass,   &halfsharp1,
+  &hrsharp1,     &hssharp1,     &halfsharp2,    &hrsharp2,
+  &hssharp2,     &halfflat1,    &hrflat1,       &hsflat1,
   &halfflat2,    &hrflat2,      &hsflat2
   };
 
@@ -497,6 +501,14 @@ static fontsizestr init_fontsizes = {
     10000,     /* user font size 10 */
     10000,     /* user font size 11 */
     10000,     /* user font size 12 */
+    10000,     /* user font size 13 */
+    10000,     /* user font size 14 */
+    10000,     /* user font size 15 */
+    10000,     /* user font size 16 */
+    10000,     /* user font size 17 */
+    10000,     /* user font size 18 */
+    10000,     /* user font size 19 */
+    10000,     /* user font size 20 */
     11800,     /* tsfont */
     10000,     /* ulay */
     10000,     /* olay */
@@ -564,8 +576,8 @@ uschar init_play_volume[] =
    15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15,
    15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15,
    15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15 };
-   
-/* Default midi channels; must keep in step with STAVE_BITVEC_SIZE */ 
+
+/* Default midi channels; must keep in step with STAVE_BITVEC_SIZE */
 
 uschar init_midi_channel[] =
   { 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
@@ -573,8 +585,8 @@ uschar init_midi_channel[] =
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
     1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1 };
 
-/* Default midi forced pitch; must keep in step with STAVE_BITVEC_SIZE. 
-Zero means use pitch of note (i.e. don't force). */ 
+/* Default midi forced pitch; must keep in step with STAVE_BITVEC_SIZE.
+Zero means use pitch of note (i.e. don't force). */
 
 uschar init_midi_note[] =
   { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
@@ -584,15 +596,15 @@ uschar init_midi_note[] =
 
 /* Default midi voice allocation to midi channels; 128 => don't set */
 
-uschar init_midi_voice[] = 
-  { 128, 128, 128, 128, 128, 128, 128, 128, 
-    128, 128, 128, 128, 128, 128, 128, 128 }; 
-    
+uschar init_midi_voice[] =
+  { 128, 128, 128, 128, 128, 128, 128, 128,
+    128, 128, 128, 128, 128, 128, 128, 128 };
+
 /* Default midi channel volumes */
 
 uschar init_midi_volume[] =
-  { 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15 }; 
-   
+  { 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15, 15 };
+
 
 /* Default stave zero copy block */
 
@@ -612,13 +624,13 @@ movtstr init_movtstr = {
   &init_fontsizes,    /* fontsizes */
   NULL,               /* posvector */
   &init_startline,    /* startline */
-  &init_zcopy,        /* zcopy */ 
+  &init_zcopy,        /* zcopy */
 
   NULL,               /* bracelist - default is none */
   &default_stave_list, /* bracketlist - default is all */
-  &default_stave_list, /* joinlist - default is  all */ 
+  &default_stave_list, /* joinlist - default is  all */
   NULL,               /* joindottedlist - default is none */
-  NULL,               /* thinbracketlist - default is none */ 
+  NULL,               /* thinbracketlist - default is none */
 
   NULL,               /* footing */
   NULL,               /* heading */
@@ -631,17 +643,17 @@ movtstr init_movtstr = {
   US"-",              /* hyphenstring */
   US"\\mf\\\\136\\",  /* trill string */
   init_play_volume,   /* play volumes */
-  
+
   init_midi_channel,  /* midi channel for each stave */
   init_midi_note,     /* midi note forced for each stave */
-  init_midi_voice,    /* voices for each MIDI channel */ 
-  init_midi_volume,   /* volumes for each MIDI channel */ 
+  init_midi_voice,    /* voices for each MIDI channel */
+  init_midi_volume,   /* volumes for each MIDI channel */
 
   init_accadjusts,    /* accadjusts */
   init_accspacing,    /* accspacing */
   init_notespacing,   /* notespacing */
-  NULL,               /* layout */ 
-  NULL,               /* midi_start */ 
+  NULL,               /* layout */
+  NULL,               /* midi_start */
   NULL,               /* play_tempo_changes */
   init_stavesizes,    /* stavesizes */
   NULL,               /* stave_ensure */
@@ -649,26 +661,26 @@ movtstr init_movtstr = {
   init_stemswaplevel, /* stemswaplevel */
 
   0,                  /* barcount */
-  0,                  /* barlinesize */ 
+  0,                  /* barlinesize */
   0x80000000,         /* barlinespace */
   0,                  /* barlinestyle */
   0,                  /* barno_interval */
-  0,                  /* barno_level */ 
+  0,                  /* barno_level */
   0,                  /* barno_textflags */
   0,                  /* baroffset */
   1800,               /* beamdepth */
-  5000,               /* beamflaglength */ 
+  5000,               /* beamflaglength */
   0,                  /* botmargin */
-  0,                  /* bracestyle */ 
+  0,                  /* bracestyle */
   2300,               /* breveledgerextra */
   0,                  /* caesurastyle */
   1000,               /* clefsize */
-  0,                  /* clefstyle */ 
+  0,                  /* clefstyle */
   1200,               /* dotspacefactor */
   0,                  /* endlinesluradjust */
-  0,                  /* endlineslurstyle */ 
+  0,                  /* endlineslurstyle */
   0,                  /* endlinetieadjust */
-  0,                  /* endlinetiestyle */    
+  0,                  /* endlinetiestyle */
   0,                  /* extenderlevel */
   font_rm,            /* bar number font */
   font_bf,            /* long rest font */
@@ -676,68 +688,69 @@ movtstr init_movtstr = {
   font_rm,            /* repeat bar numbers font */
   font_bf,            /* time signature font */
   font_rm,            /* triplet font */
-  4000,               /* footnotesep */ 
+  4000,               /* footnotesep */
   { 6000,6000 },      /* gracespacing */
   0,                  /* gracestyle */
-  0,                  /* hflatstyle */ 
-  0,                  /* hsharpstyle */ 
+  0,                  /* hflatstyle */
+  0,                  /* hsharpstyle */
   200,                /* hairpinlinewidth */
   7000,               /* hairpinwidth */
   50000,              /* hyphenthreshold */
   just_all,           /* justify */
   2,                  /* key default = C major */
-  0,                  /* keyspacing */ 
+  0,                  /* keyspacing */
   0,                  /* laststave */
-  '=',                /* ledger character */ 
+  '=',                /* ledger character */
   -1,                 /* leftmargin */
   -1,                 /* linelength */
   500,                /* maxbarcount */
-  310,                /* maxbeamslope1 */ 
-  330,                /* maxbeamslope2 */ 
+  310,                /* maxbeamslope1 */
+  330,                /* maxbeamslope2 */
   movt_default,       /* movt_opt */
-  0,                  /* movement number */ 
+  0,                  /* movement number */
   11000,              /* overlaydepth */
   120,                /* play_tempo */
   text_box,           /* rehearsalstyle */
   0,                  /* repeatstyle */
-  0,                  /* shorten (stems) */ 
-  700,                /* smallcapsize */ 
+  0,                  /* shorten (stems) */
+  700,                /* smallcapsize */
   0,                  /* startbracketbar */
   stemswap_default,   /* stemswaptype */
   44000,              /* systemgap */
   0x00010404,         /* time default = 1*4/4 */
-  0,                  /* timespacing */ 
+  0,                  /* timespacing */
   10000,              /* topmargin */
   0,                  /* totalnocount */
   0,                  /* transpose */
   300,                /* tripletlinewidth */
   480000,             /* truelinelength */
   11000,              /* underlaydepth */
-  0,                  /* underlaystyle */ 
+  0,                  /* underlaystyle */
 
   { 0, 0 },           /* breakbarlines */   /*** must be kept in ***/
   { -1, -1 },         /* staves */          /*** step with ***/
   { 0, 0 },           /* suspend */         /*** STAVE_BITVEC_SIZE ***/
-  
+
   { 13, 15, 15, 15, 15, 15, 16, 16, 16, 15, 0, 13, 13, 13, 16, 16 },  /* clefwidths */
-  { 0, 0, 0, 0, 0, 0, 2000, 4000 },         /* tailadjusts */ 
+  { 0, 0, 0, 0, 0, 0, 2000, 4000 },         /* tailadjusts */
 
   FALSE,              /* beamendrests */
   FALSE,              /* breverests */
   TRUE,               /* check */
   TRUE,               /* checkdoublebars */
-  FALSE,              /* codemultirests */ 
+  FALSE,              /* codemultirests */
   TRUE,               /* fullbarend */
   FALSE,              /* incipitjoin */
   TRUE,               /* keydoublebar */
   TRUE,               /* keywarn */
+  FALSE,              /* rehearsallsleft */
   TRUE,               /* showtime */
   TRUE,               /* showtimebase */
-  FALSE,              /* tiesoverwarnings */ 
+  FALSE,              /* tiesoverwarnings */
   TRUE,               /* timewarn */
   FALSE,              /* startnotime */
   TRUE,               /* spreadunderlay */
-  TRUE,               /* underlayextenders */ 
+  TRUE,               /* underlayextenders */
   FALSE,              /* unfinished */
 
   { 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,   /* playtranspose; must keep in */
@@ -781,8 +794,8 @@ uschar length_table [] = {
   ROUND(sizeof(b_endpletstr)),
   ROUND(sizeof(b_endslurstr)),
   ROUND(sizeof(b_ensstr)),
-  ROUND(sizeof(b_ensurestr)), 
-  ROUND(sizeof(b_footnotestr)), 
+  ROUND(sizeof(b_ensurestr)),
+  ROUND(sizeof(b_footnotestr)),
   ROUND(sizeof(b_hairpinstr)),
   ROUND(sizeof(b_ibarstr)),
   ROUND(sizeof(b_justifystr)),
@@ -805,12 +818,12 @@ uschar length_table [] = {
   ROUND(sizeof(b_olevelstr)),
   ROUND(sizeof(b_olherestr)),
   ROUND(sizeof(b_ornamentstr)),
-  ROUND(sizeof(b_pagestr)), 
+  ROUND(sizeof(b_pagestr)),
   ROUND(sizeof(b_pagebotsstr)),
   ROUND(sizeof(b_pagetopsstr)),
-  ROUND(sizeof(b_playchangestr)), 
+  ROUND(sizeof(b_playchangestr)),
   ROUND(sizeof(b_pletstr)),
-  ROUND(sizeof(b_prevbarstr)), 
+  ROUND(sizeof(b_prevbarstr)),
   ROUND(sizeof(b_resetstr)),
   ROUND(sizeof(b_resumestr)),
   ROUND(sizeof(b_rrepeatstr)),
@@ -822,11 +835,12 @@ uschar length_table [] = {
   ROUND(sizeof(b_sgstr)),
   ROUND(sizeof(b_slopestr)),
   ROUND(sizeof(b_slurstr)),
-  ROUND(sizeof(b_linegapstr)), 
+  ROUND(sizeof(b_linegapstr)),
   ROUND(sizeof(b_slurmodstr)),
   ROUND(sizeof(b_spacestr)),
   ROUND(sizeof(b_ssstr)),
   ROUND(sizeof(b_ssstr)),
+  ROUND(sizeof(b_ssstr)),
   ROUND(sizeof(b_suspendstr)),
   ROUND(sizeof(b_textstr)),
   ROUND(sizeof(b_textXstr)),
@@ -838,7 +852,7 @@ uschar length_table [] = {
   ROUND(sizeof(b_ulevelstr)),
   ROUND(sizeof(b_ulherestr)),
   ROUND(sizeof(b_unbreakbarlinestr)),
-  ROUND(sizeof(b_zcopystr)) 
+  ROUND(sizeof(b_zcopystr))
 };
 
 /* End of tables.c */
